/* 
 *	Jeti, a Java Jabber client, Copyright (C) 2003 E.S. de Boer  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	For questions, comments etc, 
 *	use the website at http://jeti.jabberstudio.org
 *  or mail me at eric@jeti.tk
 *
 *	Created on 28-dec-2003
 */
 
package nu.fw.jeti.applet;


import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.util.Iterator;

import javax.swing.*;

import nu.fw.jeti.backend.LoginInfo;
import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.elements.DiscoItem;
import nu.fw.jeti.ui.LoginStatusWindow;
import nu.fw.jeti.util.I18N;
import nu.fw.jeti.util.Preferences;
import nu.fw.jeti.util.QueryServers;

/**
 * @author E.S. de Boer
 * @author Published under the terms and conditions of the
 *    		GNU General Public License
 */

public class LoginWindow extends JPanel
{
	private JTextField txtUser = new JTextField();
	private JTextField txtPassword = new JPasswordField();
	private JComboBox cmbServer = new JComboBox();
	private JTextField txtResource = new JTextField();
	private JPanel jPanel1 = new JPanel();
    private JCheckBox chkSSL = new JCheckBox();
    private JSpinner spinner = new JSpinner(new SpinnerNumberModel(0,-128,127,1));
    private JPanel jPanel2 = new JPanel();
    private JTextField txtPort = new JTextField("5222");
	private JButton btnLogin = new JButton();
	private JButton btnRegister = new JButton();
	private Backend backend;
	private JFrame window;

	public LoginWindow(Backend backend,LoginInfo info)
	{//wrong password
		this.backend = backend;
		txtUser.setText(info.getUsername());
		txtResource.setText(info.getResource());
		try
		{
			jbInit();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		cmbServer.setSelectedItem(info.getServer());
		createWindow();
		window.setTitle(I18N.gettext("main.login.Wrong_Password_Try_again"));
	}
		
	public LoginWindow(Backend backend)
	{
		this(backend,true);
		
	}
	
	public LoginWindow(Backend backend,boolean createWindow)
	{	
		
		this.backend = backend; 
		try
		{
			jbInit();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		if (Jeti.loginInfo!=null)
		{
			txtUser.setText(Jeti.loginInfo.getUsername());
			txtResource.setText(Jeti.loginInfo.getResource());
			cmbServer.setSelectedItem(Jeti.loginInfo.getServer());
			txtPort.setText(""+Jeti.loginInfo.getPort());
			chkSSL.setSelected(Jeti.loginInfo.isSSl());
		}
		if(createWindow)
		{
			createWindow();
			window.setTitle(I18N.gettext("main.login.Login"));
		}
	}

	private void createWindow()
	{
		window = new JFrame();
		window.getContentPane().add(this);
		window.setIconImage(nu.fw.jeti.images.StatusIcons.getImageIcon("jeti").getImage());
		window.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		window.setResizable(false);
		window.pack();
		window.setLocationRelativeTo(null);
		window.getRootPane().setDefaultButton(btnLogin);
		window.setVisible(true);
	}
	
	
	private void jbInit() throws Exception
	{
		I18N.setTextAndMnemonic("main.EditProfile.Register",btnRegister);
		btnRegister.setToolTipText(I18N.gettext("main.EditProfile.Request_new_account"));
		txtResource.setText("JetiApplet");
				
		setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));
				
		JLabel label = new JLabel();
  
       
        if(!Preferences.getBoolean("jeti", "hideServerSelect", false))
        {
        	I18N.setTextAndMnemonic("main.EditProfile.Jabber_Server",label);
    		label.setLabelFor(cmbServer);
            add(label, BorderLayout.NORTH);
            add(cmbServer);
	        cmbServer.addItem("");
	    	cmbServer.setAlignmentX((float) 0.0);		
        }
        
        cmbServer.setEditable(true);
//		load server list from servers.xml
		for(Iterator i = QueryServers.getServers().iterator();i.hasNext();)
		{
			DiscoItem item = (DiscoItem)i.next();
			cmbServer.addItem(item.getJID().getDomain());
		}
       	        
		label = new JLabel();
		I18N.setTextAndMnemonic("main.EditProfile.Username",label);
		label.setLabelFor(txtUser);
		add(label);
		add(txtUser);
		label = new JLabel();
		I18N.setTextAndMnemonic("main.EditProfile.Password",label);
		label.setLabelFor(txtPassword);
		add(label);
		add(txtPassword);
		if(!Preferences.getBoolean("jeti", "hideResourceInput", false))
		{
			JPanel panel =  new JPanel(new GridLayout(2,2));
			label = new JLabel();
			I18N.setTextAndMnemonic("main.EditProfile.Resource",label);
			label.setLabelFor(txtResource);
			panel.add(label);
			
			label = new JLabel();
			I18N.setTextAndMnemonic("main.EditProfile.Priority",label);
			label.setLabelFor(spinner);
			panel.add(label);
			
			panel.add(txtResource, null);
			
			
			panel.add(spinner);
			panel.setAlignmentX(0.0f);
			add(panel);
		}
		
		if(!Preferences.getBoolean("jeti", "hidePortSelect", false))
		{
			I18N.setTextAndMnemonic("main.EditProfile.Use_SSL",chkSSL);
	        label = new JLabel();
	        I18N.setTextAndMnemonic("main.EditProfile.Port",label);
	        jPanel2.add(label);
	        label.setLabelFor(txtPort);
	        jPanel2.add(txtPort);
	        jPanel2.add(chkSSL);
	        jPanel2.setAlignmentX((float) 0.0);
	        txtPort.setPreferredSize(new Dimension(80, 21));       
			add(jPanel2);
		}
		       
        btnRegister.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                btnRegister_actionPerformed(e);
            }
        });
       // btnLogin.setMnemonic('L'); // TODO: Mnemonic here
		btnLogin.setText(I18N.gettext("main.login.Login"));
		
		btnLogin.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                btnLogin_actionPerformed(e);
            }
        });
		jPanel1.setAlignmentX((float) 0.0);
		if(!Preferences.getBoolean("jeti", "hideRegisterButton", false))jPanel1.add(btnRegister);
		jPanel1.add(btnLogin);
		add(jPanel1);
	}

    void btnLogin_actionPerformed(ActionEvent e)
    {
    	if(!(txtUser.getText().equals("") || cmbServer.getSelectedItem().equals("")))
		{
    		int port;
			try
			{
				port = Integer.parseInt(txtPort.getText());
			}
			catch (NumberFormatException ex)
			{
				if(chkSSL.isSelected()) port = 5223;
				else port = 5222;
			}
			String resource = txtResource.getText();
			if(resource.equals("")) resource = "JetiApplet";
			LoginInfo info = new LoginInfo((String)cmbServer.getSelectedItem(),null,
					txtUser.getText(),txtPassword.getText(),resource,port,
					chkSSL.isSelected(),((Number)spinner.getValue()).intValue());
			new LoginStatusWindow(info,backend,1);
			if(window!=null)window.dispose();
		}
	}
	   
    void btnRegister_actionPerformed(ActionEvent e)
    {
    	String server = (String)cmbServer.getSelectedItem();
    	if(server==null || server.equals("")) server = JOptionPane.showInputDialog(this, I18N.gettext("main.EditProfile.Jabber_Server"), I18N.gettext("main.EditProfile.Create_New_Account"), JOptionPane.QUESTION_MESSAGE);
		if (server == null || server.equals("")) return;
		//setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		int port =5222;
		if(Jeti.loginInfo!=null)port = Jeti.loginInfo.getPort();
		backend.newAccount(server,port,txtUser.getText(),txtPassword.getText());
    }
       
}
/*
 * Overrides for emacs
 * Local variables:
 * tab-width: 4
 * End:
 */
