/* 
 *	Jeti, a Java Jabber client, Copyright (C) 2001 E.S. de Boer  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	For questions, comments etc, 
 *	use the website at http://jeti.jabberstudio.org
 *  or mail/IM me at jeti@jabber.org
 */

package nu.fw.jeti.images;

import java.io.*;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.*;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import nu.fw.jeti.backend.Start;
import nu.fw.jeti.jabber.JIDStatus;
import nu.fw.jeti.jabber.elements.Presence;
import nu.fw.jeti.util.Preferences;

import org.xml.sax.SAXException;

/**
 * @author E.S. de Boer
 */

public class StatusIcons
{
	private static Map statusIcons;
	private SAXParser parser;

	public StatusIcons(SAXParser parser)
    {
		if(statusIcons ==null)
		{
			try{
				new Icons(parser,"rostericons");
			}catch(IOException e)
			{
				e.printStackTrace();
			}
			statusIcons = new HashMap();
			loadDefaultPics();
			//loadWebstartPics();
			List iconList = Preferences.getPlugable("rostericons");
			for (Iterator i = iconList.iterator();i.hasNext();)
			{
				Object[] temp = (Object[])i.next();
				if(((Boolean)temp[1]).booleanValue())
				{
					loadRosterIcon((String)temp[4],(String)temp[3]);
				}
			}
		}
	}
	
	public StatusIcons(){}
	
	private void loadParser()
	{
		try{parser = SAXParserFactory.newInstance().newSAXParser();}
		catch (FactoryConfigurationError ex){ex.printStackTrace();}
		catch (SAXException ex){ex.printStackTrace();}
		catch (ParserConfigurationException ex){ex.printStackTrace();}
	}
	
	protected void reloadRosterIcon(String name,String type)
	{
		if(!statusIcons.containsKey(type))loadRosterIcon(name, type);
	}
	
	protected void loadRosterIcon(String filename,String type)
	{
		String name2 = filename.substring(0, filename.lastIndexOf("."));
		InputStream stream = null;
		URL url = null;
		try
		{
			url = getClass().getResource("/" + name2 + "/");
			if(url==null) url =  new URL ("jar:" + Start.dataURL + 
					"plugins/rostericons/" + filename +"!/" + name2 + "/");
			stream =  new URL(url,"icondef.xml").openStream();
		}
		catch (IOException ex)
		{
			//ex.printStackTrace();
		}
		if(stream==null)
		{
			System.out.println(filename + " not found");
			return;
		}
		try
		{
			if(parser == null) loadParser();
			if(parser != null) parser.parse(stream ,new RosterIconsHandler(url,statusIcons,type));
		}
		catch (SAXException ex)
		{
			ex.getException().printStackTrace();
		}
		catch (IOException ex)
		{
			ex.printStackTrace();
		}
	}

	protected void unloadRosterIcon(String name)
	{
		statusIcons.remove(name);
	}
	

	static public ImageIcon getOfflineIcon()
	{
		return getStatusIcon(Presence.UNAVAILABLE ,"default");
	}
	
	static public ImageIcon getStatusIcon(int show){return getStatusIcon(show,"");}

	/** verander
	 * calls getImageIcon(show) whith the default imageset
	 * @param show
	 * @return ImageIcon
	 */
	static public ImageIcon getImageIcon(String show){return getStatusIcon(show,"images");}


	/**
	 * calls getStatusIcon(show,type) whith the default imageset
	 * @param show
	 * @return ImageIcon
	 */
	//static public ImageIcon getImageIcon(int image){return getStatusIcon(image,"images");}

	//TODO change all calls with jidstatus
	/**
	 * returns a status icon
	 * @param show one of chat,available,dnd,away,xa or unavailable
	 * @param type which image set
	 * @return ImageIcon
	 */
	static public ImageIcon getStatusIcon(int show,String type)
	{
		switch (show)
		{
			case Presence.FREE_FOR_CHAT: return  getStatusIcon("status/chat",type);
			case Presence.AWAY: return  getStatusIcon("status/away",type);
			case Presence.XA: return  getStatusIcon("status/xa",type);
			case Presence.DND: return  getStatusIcon("status/dnd",type);
			case Presence.UNAVAILABLE: return  getStatusIcon("status/offline",type);
			case Presence.INVISIBLE: return  getStatusIcon("status/invisible",type);
			case Presence.NONE: return  getStatusIcon("status/ask",type);
			default: return getStatusIcon("status/online",type);
		}	
	}
	
	static public ImageIcon getStatusIcon(JIDStatus jidStatus)
	{
		return getStatusIcon(jidStatus.getShow(),jidStatus.getType());
	}
	
	static private ImageIcon getStatusIcon(String show,String type)
	{
		if(!statusIcons.containsKey(type)) type = "default";
		ImageIcon icon = (ImageIcon)((Map)statusIcons.get(type)).get(show);
		if (icon == null) icon = (ImageIcon)((Map)statusIcons.get("default")).get(show);
		return icon;
	}

	public void loadDefaultPics()
	{
		//load default smilies
	//	readSmilies(new BufferedReader(new InputStreamReader(PlgEmoticons.class.getResourceAsStream("emoticons/msn/info.cfg"))),null);
		// Now, search for and get the URL for this class.
		String urlString = getClass().getResource("StatusIcons.class").toString();
		//String urlString = url.toString();
		//System.out.println(url);
		try{
			urlString = urlString.substring(0,urlString.lastIndexOf("StatusIcons.class"));
			readImages(new BufferedReader(new InputStreamReader(getClass().getResourceAsStream("status.cfg"))),new URL(urlString));
			readImages(new BufferedReader(new InputStreamReader(getClass().getResourceAsStream("images.cfg"))),new URL(urlString));
		}catch(MalformedURLException e){e.printStackTrace(); }
		catch(NullPointerException e){System.err.println("nu/fw/jeti/images/Status.cfg or images.cfg missing");}
	}
	
	
//	public static ImageIcon getImage(String location)
//	{
//		//get url for images
//		URL url = null;
//		try
//		{
//			if (Start.remoteURL != null)
//			{
//
//				url = new URL("jar:" + Start.remoteURL + location);
//			}
//			else url = new URL("jar:" + Start.localURL + location);
//		}
//		catch (MalformedURLException e)
//		{
//			e.printStackTrace();
//		}
//		return new ImageIcon(url);
//	}
	

	public void readImages(BufferedReader data,URL file)
	{
	    try
		{
			Map map = new HashMap();
			statusIcons.put(data.readLine(),map);
			while(true)
			{
				readSmilie(data,file,map);
			}
		}
		catch(EOFException e)
		{
		    //System.out.println(smilies);
		}
		catch (IOException e2)
		{
		    e2.printStackTrace();
		}
	}

	private void readSmilie(BufferedReader data,URL file,Map current) throws IOException
	{
		StringBuffer buffer=new StringBuffer();
		while(true)
		{
			int in2 = data.read();
			if(in2 ==-1) throw new EOFException();
			char ca =(char)in2;
			if(ca ==' ')
			{
				current.put(new String(buffer),readImage(data,file));
				return;
			}
			buffer.append(ca);
		}
	}

	private Icon readImage(BufferedReader data,URL file) throws IOException
	{
		//Icon icon=null;
		java.net.URL picURL=null;
		String resource = data.readLine();
		picURL =new URL(file + resource);
		//System.out.println(picURL);
		//if(picURL ==null) System.err.println(resource +" not found");
		return new ImageIcon(picURL);
	}
/*	
	private void loadWebstartPics()
	{
		try
		{
			Enumeration m=getClass().getClassLoader().getResources("info.cfg");
			for(Enumeration s =m;s.hasMoreElements();)
			{
				try
				{
					//URL urlJar = new URL("jar:"+url+"!/");
					URL urlJar = (URL )s.nextElement();
					//JarFile jarFile = new JarFile(  ""  );
					//BufferedReader data =new BufferedReader(new InputStreamReader(jarFile.getInputStream(jarFile.getEntry("info.cfg"))));

					BufferedReader data = null;
					data =new BufferedReader(new InputStreamReader(urlJar.openStream()));
					String url = urlJar.toString(); 
					url = url.substring(0,url.length()-8);
					//System.out.println(url);
					readImages(data,new URL(url));
				}
				catch(IOException e)
				{
					//System.out.println(currentFile.getName() + " is not a valid jar file");
					System.err.println(e.toString());
				}
			}
			
		} catch (IOException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
*/
}
/*
 * Overrides for emacs
 * Local variables:
 * tab-width: 4
 * End:
 */
