package nu.fw.jeti.plugins.alertwindow;

import java.awt.*;
import java.text.MessageFormat;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.*;
import javax.swing.border.EmptyBorder;

import nu.fw.jeti.events.MessageListener;
import nu.fw.jeti.events.PresenceListener;
import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.jabber.JIDStatus;
import nu.fw.jeti.jabber.elements.Message;
import nu.fw.jeti.jabber.elements.Presence;
import nu.fw.jeti.plugins.Avatars;
import nu.fw.jeti.plugins.NativeUtils;
import nu.fw.jeti.plugins.Plugins;
import nu.fw.jeti.plugins.PluginsInfo;
import nu.fw.jeti.util.I18N;
import nu.fw.jeti.util.Preferences;
import nu.fw.jeti.util.Utils;


/**
 * @author E.S. de Boer
 * @version 1.0
 */

public class Plugin extends JWindow implements PresenceListener,MessageListener,Plugins
{
	public final static String VERSION = "2.0";
	public final static String DESCRIPTION = "alertwindow.Shows_an_alert_window_when_a_message_arrives";
	public final static String MIN_JETI_VERSION = "0.5.3";
	public final static String NAME = "alertwindow";
	public final static String ABOUT = "by E.S. de Boer";
	public final static String PARENT = "Notifiers";
	//msn alert window make with 1.4 focus things
	private JLabel lblName;
	private JLabel label;
	private JLabel icon;
	private Timer timer = new Timer(true);
	private TimerTask tt;
	private int time;
	private static Plugin plugin;
	private NativeUtils util;
	private static Point pntLowerRight;// save if demo is pressed
	private int alpha;
	//private Window owner;
	private Avatars avatars;
	static boolean supportsAlpha;
	private Color[] colors = new Color[5];
	private final static int MESSAGE=0;
	private final static int ONLINE=1;
	private final static int OFFLINE=2;
	private final static int AWAY=3;
	private final static int DND=4;
		
	public Plugin() 
	{
		super((Window)null);
		getContentPane().setBackground(Color.BLACK);
		//this.owner = owner;
		getContentPane().setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		
		c.gridx =0;
		c.gridy=0;
		c.weightx=1;
		c.weighty=1;
		c.gridheight = 2;
		c.fill = GridBagConstraints.BOTH;
		c.insets = new Insets(1,1,1,0);
		icon  = new JLabel();
		icon.setOpaque(true);
		getContentPane().add(icon,c);
		
		c.gridx =1;
		c.gridy=0;
		c.weightx=1;
		c.insets = new Insets(1,0,0,1);
		c.gridheight = 1;
		lblName = new JLabel();
		lblName.setBorder(new EmptyBorder(4,4,4,4));
		lblName.setOpaque(true);
		lblName.setFont((Font)UIManager.get("TextArea.font"));
		getContentPane().add(lblName,c);
		label = new JLabel();
		label.setBorder(new EmptyBorder(0,4,4,4));
		label.setOpaque(true);
		label.setFont((Font)UIManager.get("TextArea.font"));
		c.gridx =1;
		c.gridy=1;
		c.weightx=1;
		c.insets = new Insets(0,0,1,1);
		getContentPane().add(label,c);
		enableInputMethods(false);
		setSize(1,1);
		setVisible(true);
		setFocusableWindowState(false);
		setFocusable(false);
		int xOffset=Preferences.getInteger("alertwindow","XOffset", 0); 
		int yOffset=Preferences.getInteger("alertwindow","YOffset", 0); 
		caculateLowerRightPoint(xOffset, yOffset);
	}
	
    public static void init(Backend backend)
    {
    	plugin = new Plugin();
    	plugin.initColors();
    	backend.addListener(MessageListener.class,plugin);
		backend.addListener(PresenceListener.class,plugin);
		Color foreground = new Color(Preferences.getInteger("alertwindow","foregroundcolor",SystemColor.controlText.getRGB()));
		plugin.label.setForeground(foreground);
		plugin.lblName.setForeground(foreground);
		plugin.time =Preferences.getInteger("alertwindow","popuptime", 5)*1000;
	}
	
    private void initColors()
    {
    	plugin.colors[MESSAGE]= new Color(Preferences.getInteger("alertwindow","message.backgroundcolor",SystemColor.info.getRGB()));
    	plugin.colors[ONLINE]= new Color(Preferences.getInteger("alertwindow","online.backgroundcolor",Color.GREEN.getRGB()));
    	plugin.colors[OFFLINE]= new Color(Preferences.getInteger("alertwindow","offline.backgroundcolor",Color.GRAY.getRGB()));
    	plugin.colors[AWAY]= new Color(Preferences.getInteger("alertwindow","away.backgroundcolor",Color.RED.getRGB()));
    	plugin.colors[DND]= new Color(Preferences.getInteger("alertwindow","do_not_disturb.backgroundcolor",Color.ORANGE.getRGB()));
    }
    
	private void loadWindowsUtils() 
	{
		if(util==null)
		{
			if(PluginsInfo.isPluginLoaded("windowsutils"))
			{
				util =(NativeUtils)PluginsInfo.newPluginInstance("windowsutils");
				supportsAlpha = util.supportsAlpha();
			 	if(util!=null)
				{
					try
					{//make window before always on top & alpha
						if(Utils.isAtleastJava(5))
						{
							util.windowAlwaysOnTop(plugin,true);
						}
						if(supportsAlpha)
						{	
							alpha = Preferences.getInteger("alertwindow","alpha", 100);
							if(alpha<100)util.setWindowAlpha(plugin,alpha);
						}
					}
					catch (Throwable ex)
					{
						ex.printStackTrace();
					}
				}
			}
		}
		else 
	 	{/* 2004-09-07 - Juergen Ulbts - added the else section for OS/2 and eCS to call the native windowAlwaysOnTop function */
			if ( System.getProperty("os.name").equals("OS/2") ) {
				util.windowAlwaysOnTop(plugin, true);
			}
		}
	}
	
    public void unload() 
    {
    	if(util!=null)
    	{
    		try{
    			if(util.supportsAlpha())util.setWindowAlpha(this,0);
    			util.windowAlwaysOnTop(this,false);
    		}catch(Exception e)
			{
    			//util already unloaded
    		}
    	}
       	dispose();
       //	owner.dispose();
    }
    
	public static void unload(Backend backend)
	{
		backend.removeListener(MessageListener.class,plugin);
		backend.removeListener(PresenceListener.class,plugin);
		plugin.unload();
		plugin = null;
	}
	
	private Avatars getAvatar()
	{
		if(avatars!=null)return avatars;
		if(PluginsInfo.isPluginLoaded("ichatavatars"))
		{
			avatars =  (Avatars)PluginsInfo.getPluginInstance("ichatavatars");
			return avatars;
		}
		return null;
	}

	public void presenceChanged(final Presence presence)
	{
		loadWindowsUtils();
		Runnable updateAComponent = new Runnable() {
			public void run()
			{
				if(presence.getShow()==Presence.UNAVAILABLE)
				{
					if(!Preferences.getBoolean("alertwindow","offline.enabled", true))return;
					setBackgroundColor(colors[OFFLINE]);
				}
				else if(presence.getShow()==Presence.AVAILABLE || presence.getShow()==Presence.FREE_FOR_CHAT )
				{
					if(!Preferences.getBoolean("alertwindow","online.enabled", true))return;
					setBackgroundColor(colors[ONLINE]);
				}
				else if(presence.getShow()==Presence.DND)
				{
					if(!Preferences.getBoolean("alertwindow","dnd.enabled", true))return;
					setBackgroundColor(colors[DND]);
				}
				else
				{
					if(!Preferences.getBoolean("alertwindow","away.enabled", true))return;
					setBackgroundColor(colors[AWAY]);
				}
				
				setIcon(presence.getFrom());
				lblName.setText(getNick(presence.getFrom()));
				String status = presence.getStatus();
				if(status==null)status = Presence.toLongShow(presence.getShow());
				label.setText(status);
				place();
			}
		};
		SwingUtilities.invokeLater(updateAComponent);
	}

	public void message(final Message e)
	{
		if(!Preferences.getBoolean("alertwindow","message.enabled", true))return;
		loadWindowsUtils();
		Runnable updateAComponent = new Runnable() {
			public void run()
			{
				setBackgroundColor(colors[MESSAGE]);
				JID jid = e.getFrom();
				setIcon(jid);
				String nick;
				if(e.getType().equals("groupchat")) nick = jid.getResource();
				else nick = getNick(jid); 
				lblName.setText(nick);
				label.setText(e.getBody());
				place();
			}
		};
		SwingUtilities.invokeLater(updateAComponent);
	}
	
	private synchronized void setIcon(JID from)
	{
		icon.setIcon(null);
		Avatars avatar = getAvatar();
		if(avatar!=null)
		{
			JIDStatus jidStatus = Backend.getJIDStatus(from);
			if(jidStatus!=null)
			{
				ImageIcon icon2  = jidStatus.getAvatar();
				if(icon2!=null)
				{
					if(icon2.getIconHeight()>64 || icon2.getIconWidth()>64)
					{
					icon2 = new ImageIcon(
							icon2.getImage().getScaledInstance(64,64,Image.SCALE_DEFAULT));
					}
					icon.setIcon(icon2);
				}
			}
		}
	}
	
	private String getNick(JID jid)
	{
		String nick = jid.getUser();
		JIDStatus jidStatus = Backend.getJIDStatus(jid);
		if(jidStatus!=null) nick = jidStatus.getNick();
		return nick;
	}
	
	private void setBackgroundColor(Color color)
	{
		plugin.label.setBackground(color);
		plugin.lblName.setBackground(color);
		plugin.icon.setBackground(color);
	}
		
	public void demo(Color color,Color foreground,final int alpha,final int time,int xOffset,int yOffset)
	{
		initColors();
		this.time = time *1000;
		label.setBackground(color);
		lblName.setBackground(color);
		icon.setBackground(color);
		label.setForeground(foreground);
		lblName.setForeground(foreground);
		plugin.time = this.time;
		setBackgroundColor(color);
		plugin.label.setForeground(foreground);
		plugin.lblName.setForeground(foreground);
		plugin.alpha=alpha;
		loadWindowsUtils();
		caculateLowerRightPoint(xOffset, yOffset);
		if(util!=null && supportsAlpha)
		{
			try
			{
				util.setWindowAlpha(this,alpha);
				util.setWindowAlpha(plugin,alpha);
			}
			catch (Throwable ex)
			{
				ex.printStackTrace();
			}
		}
		Runnable updateAComponent = new Runnable() {
			public void run()
			{
				lblName.setText(I18N.gettext("alertwindow.This_is_a_Demonstration_PopupWindow"));
				label.setText(MessageFormat.format(I18N.gettext("alertwindow.Alpha_is_{0},_Time_is_{1}"),new Object[]{String.valueOf(alpha),String.valueOf(time)}));
				place();
			}
		};
		SwingUtilities.invokeLater(updateAComponent);
	}

	private void caculateLowerRightPoint(int xOffset, int yOffset)
	{
		GraphicsEnvironment env = GraphicsEnvironment.getLocalGraphicsEnvironment();
		Rectangle bounds = env.getMaximumWindowBounds();
		pntLowerRight = new Point(bounds.x + bounds.width - xOffset,bounds.y + bounds.height - yOffset);
	}

	private void place()
	{
		if(tt!=null) tt.cancel();
		tt = (new TimerTask()
		{
			public void run()
			{
				setSize(1,1);
				setLocation(pntLowerRight);
				if(util!=null && supportsAlpha && alpha <100)
				{//remove alpha because otherwise half of the window becomes black	
					try
					{
						util.setWindowAlpha(Plugin.this,100);
					}
					catch (Throwable ex)
					{
						ex.printStackTrace();
					}
				}
			}
		});
		pack();
		//int screenX = (int)Toolkit.getDefaultToolkit().getScreenSize().getWidth();
		//int screenY = (int)Toolkit.getDefaultToolkit().getScreenSize().getHeight();
		//setLocation(screenX-getWidth(),screenY-getHeight()-40);
		setLocation(pntLowerRight.x-getWidth(),pntLowerRight.y-getHeight());
		//toFront();
		//show();
		
		timer.schedule(tt,time);
		if(util!=null && supportsAlpha && alpha<100)
		{	
			try
			{
				util.setWindowAlpha(this,alpha);
			}
			catch (Throwable ex)
			{
				ex.printStackTrace();
			}
		}
		
		if(Utils.isAtleastJava(5))
		{
 			setAlwaysOnTop(true);
 		}
	}
}
/*
 * Overrides for emacs
 * Local variables:
 * tab-width: 4
 * End:
 */
