package nu.fw.jeti.plugins.buddyspacemaps;

/*
 * BSMapsPlugin.java
 *
 * Project: BuddySpace
 * (C) Copyright Knowledge Media Institute 2002
 *
 *
 * Created on 27 September 2002, 11:53
 * 
 * E.S. de Boer
 * converted to jeti plugin
 */

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileFilter;
import java.io.FilenameFilter;
import java.util.Vector;

import javax.swing.*;
import javax.swing.event.MenuEvent;
import javax.swing.event.MenuListener;

import nu.fw.jeti.backend.Start;
import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.plugins.Plugins;
import nu.fw.jeti.plugins.buddyspacemaps.core.BSMapBean;
import nu.fw.jeti.plugins.buddyspacemaps.gui.BSMapMenuItem;
import nu.fw.jeti.plugins.buddyspacemaps.gui.BSMapWinManager;
import nu.fw.jeti.plugins.buddyspacemaps.xml.MapTagHandler;


/**
 * <code>BSMapsPlugin</code> is main class of maping plugin to BuddySpace.
 * It provides methods for plugin loading and user interaction.
 *
 * @author  Jiri Komzak, Knowledge Media Institute, Open University, United Kingdom
 */
public class Plugin implements Plugins, ActionListener, MenuListener {
    
	public final static String VERSION = "0.2";
	public final static String DESCRIPTION = "buddyspacemaps.Shows_contacts_on_a_map";
	public final static String MIN_JETI_VERSION = "0.6.5";
	public final static String NAME = "buddyspacemaps";
	public final static String ABOUT = "by Knowledge Media Institute, Open University, http://buddyspace.sourceforge.net/";
	
    final static String DEF_MAP_PATH                 = "./maps";
    public static final String PREF_NAME_MAP_PATH    = "mapPath";
    public static final String PREF_NAME_PUBSUB_JID  = "mapPubsubJid";
    public static final String PREF_NAME_PUBSUB_NODE = "mapPubsubNode";
    public static String mapPath                     = DEF_MAP_PATH;
    public static JID pubsubJID                      = null;
    public static String pubsubNode                  = null;
    
    private BSMapBean mapBean;
    private BSMapWinManager mapWinMan;
    //private BSMapPubsubCore mapPubsub;
    
    private JMenu mapsMenu;
    private JMenuItem getPubsubMapsMenuItem;
    
    private JTextField mapPathTextField;
    private JTextField pubsubJidTextField;
    private JTextField pubsubNodeTextField;
    
    public static void init(final Backend backend)
	{
		backend.addExtensionHandler("http://jabber.open.ac.uk/tags/map",new MapTagHandler());
		new Plugin(backend);
	}
    
    public Plugin(Backend backend)
    {
    	mapPath = Start.path  + File.separator + "maps";
    	System.out.println(mapPath);
    	loadPreferences();
	    initCore(backend);
	    initGUI(backend);
	    addGUIControls(backend);
	    connectBeans();
    }
    
    public static void unload(Backend backend)
	{
		backend.getMain().removeFromRosterMenu("Maps");
		backend.removeExtensionHandler("http://jabber.open.ac.uk/tags/map");
	}
	
	public void unload()
	{
		
	}
    
    /** Inits plugin core */
    protected void initCore(Backend backend) {
        mapBean = new BSMapBean(mapPath,backend);
        //mapBean.setProxy(mainFrame.proxy, mainFrame.proxyPort);
        //mapBean.setFileServer(mainFrame.fileServer, mainFrame.fileServerPath, mainFrame.fileServerPort);
        
//        mapPubsub = new BSMapPubsubCore(mainFrame, mapBean);
//        mapPubsub.setPubsubJID(pubsubJID);
//        mapPubsub.setPubsubNode(pubsubNode);
    }
    
    /** Inits plugin GUI */
    protected void initGUI(Backend backend ) {
        mapWinMan = new BSMapWinManager(backend,mapPath);
        mapWinMan.setMapBean(mapBean);
    }
    
    protected void loadPreferences() {
    	/*
    	String[] names = {PREF_NAME_MAP_PATH};
        String[] defaults = {DEF_MAP_PATH};
        Vector prefs = mainFrame.loadPreferences(names, defaults);
        if (prefs != null && prefs.size() == 1)
            mapPath = (String) prefs.elementAt(0);
        
        names = new String[] {PREF_NAME_PUBSUB_JID, PREF_NAME_PUBSUB_NODE};
        defaults = new String[] {"",""};
        prefs = mainFrame.loadPreferences(names, defaults);
        if (prefs != null && prefs.size() == 2) {
            String pubsubJidStr = (String) prefs.elementAt(0);
            if (pubsubJidStr == null || "".equals(pubsubJidStr)) pubsubJID = null;
            else pubsubJID = JID.fromString(pubsubJidStr);
            pubsubNode = (String) prefs.elementAt(1);
        }
        if (mapPubsub != null) {
            mapPubsub.setPubsubJID(pubsubJID);
            mapPubsub.setPubsubNode(pubsubNode);
        }
        */
    }
    
    /** Adds plugin GUI controls */
    protected void addGUIControls(Backend backend) {
        /*testMenuItem = new JMenuItem("Test local maps");
        testMenuItem.addActionListener(this);
        //testMenuItem.setEnabled(false);
        mainFrame.addActionMenuItem(testMenuItem);*/
        
        mapsMenu = new JMenu("Maps");
        mapsMenu.setMnemonic('m');
        mapsMenu.addMenuListener(this);
        backend.getMain().addToMenu(mapsMenu);
        //mainFrame.addMenu(mapsMenu);
        
        /*
        getPubsubMapsMenuItem = new JMenuItem("Get published maps");
        getPubsubMapsMenuItem.setEnabled(false);
        getPubsubMapsMenuItem.addActionListener(this);
        mapsMenu.add(getPubsubMapsMenuItem);
        */
    }
    
    /** Connects underlying beans */
    protected void connectBeans() {
        mapWinMan.setMapBean(mapBean);
        /*
        if (core != null) {
            
            mapWinMan.setPresenceBean(core.getPresenceBean());
            mapWinMan.setRosterBean(core.getRosterBean());
            
            BSInfoQueryBean iqBean = core.getInfoQueryBean();
            if (iqBean != null)
                mapBean.setIQBean(iqBean.getIQBean());
            
            BSMessengerBean msgBean = core.getMessengerBean();
            if (msgBean != null)
                mapBean.setMessengerBean(msgBean.getMessengerBean());
        }
        */
    }
    
    /** Handles connection change to connected */
    public void connected() {
    	/*
        super.connected();
        //joinMenuItem.setEnabled(true);
        //mapPubsub.conected();
        
        Vector namespaces = new Vector();
        mainFrame.getSupportedNamespaces(namespaces, new Vector());
        mapWinMan.setSupportPlans(namespaces.contains("http://jabber.open.ac.uk/tags/plan#get"));
        
        getPubsubMapsMenuItem.setEnabled(true);
        */
    }

    
    /** Handles connection change to disconnected */
    public void disconnected() {
        //mapWinMan.closeAllWindows();
        //confBean.disconnected();
        //joinMenuItem.setEnabled(false);
     //   getPubsubMapsMenuItem.setEnabled(false);
    }
    
    /** Adds menu items into map menu */
    private void addMapsMenuItems() {
        mapsMenu.removeAll();
        
       // mapsMenu.add(getPubsubMapsMenuItem);
       // mapsMenu.add(new JSeparator());
        
        // gets all subdirs in mapPath
        File mapDir = new File(mapPath);
        File dirs[] = mapDir.listFiles(new FileFilter() {
            public boolean accept(File file) {
                return file.isDirectory();
            }
        });
        // for all subdirs
        if (dirs == null) return;
        for (int i=0; i<dirs.length; i++) {
            // adds submenu for subdir
            //JMenu submenu = new JMenu(dirs[i].getName());
            String dirName = dirs[i].getName();
            JMenu submenu = new JMenu();
            String submenuName;
            submenuName = dirName;
            /*
            // if this is a directory of maps received from a pubsub
            if (dirName.startsWith(BSMapPubsubCore.PUBSUB_DIR_PREFIX))
            {
                submenu.setToolTipText(dirName.substring(BSMapPubsubCore.PUBSUB_DIR_PREFIX.length()));
                submenuName = BSMapPubsubCore.getFriendlyOrigin(dirName);
            }
            */
            submenu.setText(submenuName);
            mapsMenu.add(submenu);
            // gets all map files in subdir
            File maps[] = dirs[i].listFiles(new FilenameFilter() {
                public boolean accept(File dir, String name) {
                    return name.endsWith(".ygf");
                }
            });
            // adds menuItems for all maps
            if (maps == null) return;
            for (int j=0; j<maps.length; j++) {
                // adds menuItem for map
                String label = maps[j].getName();
                if (label.length() >= 4) label = label.substring(0, label.length()-4);
                JMenuItem menuItem = new BSMapMenuItem(label, maps[j].getName(), dirs[i].getName());
                menuItem.addActionListener( new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        loadMapForItem((BSMapMenuItem)e.getSource());
                    }
                });
                submenu.add(menuItem);
            }
        }
    }
    
    /** Loads map for given menu item */
    private void loadMapForItem(BSMapMenuItem i) {
        String mapName = i.getMapName();
        String originID = i.getOriginID();
        mapBean.loadMap(mapName, originID);
    }
    
    /** Handles actions from GUI controls */
    public void actionPerformed(ActionEvent evt) {
    	/*
        if (evt.getSource() == getPubsubMapsMenuItem) {
            mapPubsub.getPublishedMaps();
        }
        */
        // a map menu item - load the map
        if (evt.getSource() instanceof BSMapMenuItem) {
            BSMapMenuItem i = (BSMapMenuItem) evt.getSource();
            String mapName = i.getMapName();
            String originID = i.getOriginID();
            mapBean.loadMap(mapName, originID);
        }
    }
    
    /** Fills the map menu on its selection */
    public void menuSelected(MenuEvent evt) {
        Object source = evt.getSource();
        if (source == mapsMenu) {
            addMapsMenuItems();
        }
    }
    
    /** Empty */
    public void menuCanceled(MenuEvent menuEvent) { }
    /** Empty */
    public void menuDeselected(MenuEvent menuEvent) { }
    
    
    /** Returns components and their names for display in preferences dialog */
    public void getPreferencesTab(Vector components, Vector names) {
        GridBagConstraints gbc;
        
        JPanel prefsPanel = new JPanel(new GridBagLayout());
        
        JLabel label = new JLabel("Local path for maps:");
        gbc = new GridBagConstraints();
        gbc.gridy = 1;
        gbc.anchor = GridBagConstraints.WEST;
        prefsPanel.add(label, gbc);
        mapPathTextField = new JTextField(mapPath);
        gbc = new GridBagConstraints();
        gbc.gridy = 2;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.weightx = 1;
        prefsPanel.add(mapPathTextField, gbc);
        label = new JLabel("JID of pubsub server component for maps:");
        gbc = new GridBagConstraints();
        gbc.gridy = 3;
        gbc.anchor = GridBagConstraints.WEST;
        prefsPanel.add(label, gbc);
        pubsubJidTextField = new JTextField(((pubsubJID == null)? "" : pubsubJID.toString()));
        gbc = new GridBagConstraints();
        gbc.gridy = 4;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.weightx = 1;
        prefsPanel.add(pubsubJidTextField, gbc);
        label = new JLabel("Maps node in the pubsub server component (e.g. group/maps):");
        gbc = new GridBagConstraints();
        gbc.gridy = 5;
        gbc.anchor = GridBagConstraints.WEST;
        prefsPanel.add(label, gbc);
        pubsubNodeTextField = new JTextField(((pubsubNode != null)? pubsubNode : ""));
        gbc = new GridBagConstraints();
        gbc.gridy = 6;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.weightx = 1;
        prefsPanel.add(pubsubNodeTextField, gbc);
        
        label = new JLabel("\nSome of the changes may take effect" + 
                                  " after restart of BuddySpace");
        gbc = new GridBagConstraints();
        gbc.gridy = 7;
        gbc.anchor = GridBagConstraints.WEST;
        prefsPanel.add(label, gbc);
        
        components.add(prefsPanel);
        names.add("Mapping");
    }
    
    /** Stores preferences from preferences tab.
     */
    public boolean storePreferences() {
        mapPath = mapPathTextField.getText();
        /*
        String pubsubJidStr = pubsubJidTextField.getText();
        if (pubsubJidStr == null || "".equals(pubsubJidStr)) pubsubJID = null;
        else pubsubJID = JID.fromString(pubsubJidStr);
        pubsubNode = pubsubNodeTextField.getText();
        String[] names = {PREF_NAME_MAP_PATH,
                          PREF_NAME_PUBSUB_JID,
                          PREF_NAME_PUBSUB_NODE};
        String[] values = {mapPath, 
                           pubsubJidStr, 
                           pubsubNode};
        mainFrame.addAndSavePreferences(names, values);
        
        mapPubsub.setPubsubJID(pubsubJID);
        mapPubsub.setPubsubNode(pubsubNode);
        */
        return true;
    }
    
    /** Updates LAF of all plug-ins  */
    public void updateLAF() {
        mapWinMan.updateLAF();
    }
    
    
    /** 
     * Performs action within given namespace for given JID.
     * Returns if action was performed.
     */
    public boolean performAction(JID jid, String namespace) {
        if (namespace != null && namespace.equals("http://jabber.open.ac.uk/tags/map")) {
            if (mapWinMan == null) return false;
            return mapWinMan.openMap(jid);
        }
        else
            return false;
    }
    
    
   
    /*
    /** Opens windows specified in vector. *
    public void openTheWindows(Vector openWindows, boolean connected) { 
        if (mapWinMan != null)
            mapWinMan.openTheWindows(openWindows, connected);
    }
    
    
    /** Add its open windows into the vector (for opening when starting next time). *
    public void addOpenWindows(Vector openWindows, boolean connected) { 
        if (mapWinMan != null)
            mapWinMan.addOpenWindows(openWindows, connected);
    }
    */
    
}
