package nu.fw.jeti.plugins.buddyspacemaps.core;

/*
 * BSMapBean.java
 *
 * Project: BuddySpace
 * (C) Copyright Knowledge Media Institute 2002
 *
 *
 * Created on 16 August 2002, 10:20
 * 
 * E.S. de Boer
 * converted to jeti plugin 
 */

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.JOptionPane;

import nu.fw.jeti.backend.XMLData;
import nu.fw.jeti.events.MessageListener;
import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.jabber.elements.Extension;
import nu.fw.jeti.jabber.elements.IQXOOB;
import nu.fw.jeti.jabber.elements.Message;
import nu.fw.jeti.jabber.elements.MessageBuilder;
import nu.fw.jeti.plugins.buddyspacemaps.xml.ImgTag;
import nu.fw.jeti.plugins.buddyspacemaps.xml.LayerTag;
import nu.fw.jeti.plugins.buddyspacemaps.xml.MapTag;
import nu.fw.jeti.plugins.buddyspacemaps.xml.XMLFileIO;



/**
 * <code>BSMapBean</code> provides map plugin back-end.
 * It relies on <code>BSMessengerBean</code> and <code>BSInfoQueryBean</code>, 
 * which must be set after each reconnection.
 *
 * @author  Jiri Komzak, Knowledge Media Institute, Open University, United Kingdom
 */
public class BSMapBean implements MessageListener {
    protected String name = "Map";
    protected Hashtable maps = null;
    protected Vector mapListeners = null;
    private String mapPath;
    private Backend backend;
    private String proxy = null;
    private String proxyPort = null;
    private String fileServer = null;
    private String fileServerPath = null;
    private String fileServerPort = null;
    
    private boolean recursiveMapsSend = true;
    
    /**
     * Constructor
     */
    public BSMapBean(String mapPath, Backend backend) {
        maps = new Hashtable();
        mapListeners = new Vector();
        this.mapPath = mapPath;
        this.backend = backend;
        backend.addListener(MessageListener.class,this);
    }
    /*
    /**
     * Constructor, which sets existing and connected <code>IQBean</code>.
     * Then this is registered as listener for IQ packets.
     *
    public BSMapBean(String mapPath, IQBean iqBean) {
        maps = new Hashtable();
        mapListeners = new Vector();
        this.mapPath = mapPath;
        setIQBean(iqBean);
    }
    
    /**
     * Constructor, which sets existing and connected <code>MessengerBean</code>.
     * Then this is registered as listener for message packets.
     *
    public BSMapBean(String mapPath, MessengerBean msgBean) {
        maps = new Hashtable();
        mapListeners = new Vector();
        this.mapPath = mapPath;
        setMessengerBean(msgBean);
    }
    
    /**
     * Sets existing and connected <code>IQBean</code>.
     * Then this is registered as listener for IQ packets.
     *
    public void setIQBean(IQBean iqBean) {
        if (this.iqBean != null)
            this.iqBean.delPacketListener(this);
        this.iqBean = iqBean;
        if (iqBean != null)
            iqBean.addPacketListener(this);
        //servedID = null;
    }
    
    /**
     * Sets existing and connected <code>MessengerBean</code>.
     * Then this is registered as listener for message packets.
     *
    public void setMessengerBean(MessengerBean msgBean) {
        if (this.msgBean != null)
            this.msgBean.delPacketListener(this);
        this.msgBean = msgBean;
        if (msgBean != null)
            msgBean.addPacketListener(this);
    }
    */
    
    /** Sets proxy for sending map files */
    public void setProxy(String proxy, String proxyPort) {
        this.proxy = proxy;
        this.proxyPort = proxyPort;
    }
    
    /** Sets file server for sending map files */
    public void setFileServer(String fileServer, String fileServerPath, String fileServerPort) {
        this.fileServer = fileServer;
        this.fileServerPath = fileServerPath;
        this.fileServerPort = fileServerPort;
    }
    
   
    /**
     * Frees all object bindings to allow object destroy
     */
    public void prepareToDestroy() {
        removeAllMapListeners();
        backend.removeListener(MessageListener.class,this);
        //msgBean = null;
        //iqBean = null;
    }
         
    /**
     * Handles <code>Message</code> packet.
     */
    public void message(Message msg) {
        
        if ("error".equals(msg.getType()))
            return;
        if(!msg.hasExtensions())return;
        Iterator extensions;
        
        Vector curMaps = new Vector();
        extensions = msg.getExtensions();
        // for all extensions
        while (extensions.hasNext()) {
            Extension ext = (Extension) extensions.next();
            // if map extension
            if (ext instanceof MapTag) {
                MapTag map = (MapTag) ext;
               // logMap(map);
                // adds map into temporary hashtable
                curMaps.add(map);
                JID jid = msg.getFrom();
                String completeID = new String(jid.getUser() + "@" +
                                               jid.getDomain() + "/" + map.getID());
                maps.put(completeID, map);
            }
        }
        
        // if that was not map message ends
        if (curMaps.isEmpty())
            return;
        
        // if it was map message, looks for included map images
        extensions = msg.getExtensions();
        while (extensions.hasNext()) {
            Extension ext = (Extension) extensions.next();
            // for all included files
            if (ext instanceof IQXOOB) {
            	IQXOOB oob = (IQXOOB) ext;
                JID jid = msg.getFrom();
                try {
                    handleMapOOB(oob, jid);
                } catch (IOException e) {}
            }
        }
        
        // for all just received maps 
        Enumeration mapEnum = curMaps.elements();
        while (mapEnum.hasMoreElements()) {
            JID jid = msg.getFrom();
            String originID = new String(jid.getUser() + "@" + jid.getDomain());
            MapTag map = (MapTag) mapEnum.nextElement();
            // add references to referenced maps
            processMap(map, originID);
            // saves map into local cache
            saveMap(map, originID);
            fireMapReceived(map, originID);
        }
        curMaps.clear();
    }
    
    /** Handles received OOB extension. 
     * Downloads it and stores in maps directory for given JID. 
     */
    protected void handleMapOOB(IQXOOB oob, JID jid) 
                                    throws IOException {
        String originID = new String(jid.getUser() + "@" + jid.getDomain());
        handleMapOOB(oob, originID);
    }
    
    /** Handles received OOB extension. 
     * Downloads it and stores in maps directory for given originID. 
     */
    public void handleMapOOB(IQXOOB oob, String originID) 
                                    throws IOException {
        URL url;
        File file;
        String filename = oob.getDescription();
        url = oob.getURL();
        //file = new File(jidDir);
        file = new File(mapPath + "/" + originID);
        file.mkdirs();
        //file = new File(jidDir + "/" + filename);
        file = new File(mapPath + "/" + originID + "/" + filename);
        try {
            URLFileTransfer.downloadURL(url, file, proxy, proxyPort);
        }
        catch (IOException e) {
            throw e;
        }
    }
    
    /*
    /** Debug dump of map tag *
    protected void logMap(MapTag map) {
        BSCore.logEvent(name, "map id=" + map.getID() + " recieved");
        Enumeration layers = map.layers();
        while (layers.hasMoreElements()) {

            LayerTag layer = (LayerTag) layers.nextElement();
            BSCore.logEvent(name, "layer id=" + layer.getID() +
                                       " offset_x=" + layer.getOffsetX() +
                                       " offset_y=" + layer.getOffsetY() +
                                       " scale=" + layer.getScale() +
                                       " src=" + layer.getSrc() +
                                       " priority=" + layer.getPriority());

            ImgTag img = layer.getImg();
            if (img != null)
                BSCore.logEvent(name, "  img src=" + img.getSrc() +
                                           " width=" + img.getWidth() +
                                           " height=" + img.getHeight());

            Enumeration items = layer.getItems();
            while (items.hasMoreElements()) {
                Object o = items.nextElement();
                if (o instanceof ItemTag) {
                    ItemTag i = (ItemTag) o;
                    BSCore.logEvent(name, "  item jid=" + i.getJID().toString() +
                                                " lat=" + i.getLat() +
                                                " lon=" + i.getLon());
                }
                else if (o instanceof ClusterTag) {
                    ClusterTag c = (ClusterTag) o;
                    BSCore.logEvent(name, "  cluster size=" + c.getSize() +
                                                " lat=" + c.getLat() +
                                                " lon=" + c.getLon());
                }
            }
        }
    }
    */
    
    /** Saves map into map directory for originID */
    public void saveMap(MapTag map, String originID) {
        File file = new File(mapPath + "/" + originID);
        file.mkdirs();
        String filename = new String(mapPath + "/" + originID + "/" + 
                                     map.getID());
        try {
            XMLFileIO.saveTag(map, filename);
        } catch (IOException e) {}
    }
    
    /** Saves map into file with given name. Changes the map ID to match
     * the filename. */
    public boolean saveMap(String originID, String mapID,
                           String newOriginID, String newMapID) {
                               
        if (mapID == null || originID == null || newMapID == null)
            return false;
        
        // gets the map tag
        String completeID = new String(originID + "/" + mapID);
        MapTag map = (MapTag) maps.get(completeID);
        if (map == null) return false;
        
        // gets the filename and creates path for it
        String newCompleteID = new String(newOriginID + "/" + newMapID);
        String filename = new String(mapPath + "/" + newCompleteID);
        File parentFile = new File(filename).getAbsoluteFile().getParentFile();
        if (parentFile != null)
            parentFile.mkdirs();
        
        // sets new ID and origin and re-inserts the map under the new IDs
        map.setID(newMapID);
        maps.remove(completeID);
        maps.put(newCompleteID, map);
        
        try {
            XMLFileIO.saveTag(map, filename);
        } catch (IOException e) {
            return false;
        }
        
        return true;
    }
    
    /**
     * Connects references to maps to MapTags if exist otherwise downloads maps.
     */
    protected void processMap(MapTag map, String originID) {
        Enumeration layers = map.layers();
        while (layers.hasMoreElements()) {
            LayerTag layer = (LayerTag) layers.nextElement();
            String src = layer.getSrc();
            
            // loads all referenced maps
            if (src != null) {
                MapTag refdMap = (MapTag) maps.get(src);
                
                // if map not in memory
                if (refdMap == null) {
                    refdMap = loadMapTag(src, originID);
                }
                    
                // otherwise downloads it
                if (refdMap == null) {
                }
                
                // sets the referenced map (possibly null)
                layer.setSrcMap(refdMap);
                
                // processes also loaded map
                if (refdMap != null) {
                    processMap(refdMap, originID);
                }
                
            } //loads referenced map
            
        } //while
    }
    
    /** Sends map and OOB for necessary files inside a message 
     * and uploads the files on web server.
     */
    public boolean sendMap(String mapID, String originID, JID jid, String subject, String body) {
        if (mapID == null || originID == null || jid == null)
            return false;
        
        MessageBuilder msgBuilder = new MessageBuilder();
        
        if (!addMapIntoMessage(msgBuilder, mapID, originID, true))
            return false;
        
        msgBuilder.subject =subject;
        msgBuilder.body = body;
        msgBuilder.to = jid;
        backend.send(msgBuilder.build());
        
        return true;
    }
    
    /** Adds map and OOB for necessary files into the message 
     * and uploads the files on web server.
     */
    private boolean addMapIntoMessage(MessageBuilder msgBuilder, String mapID, 
                                      String originID, boolean mainMap) {
                                          
        String completeMapPath = new String(mapPath + "/" + originID);
        String completeID = new String(originID + "/" + mapID);
        
        MapTag map = (MapTag) maps.get(completeID);
        if (map == null) return false;
                
        // first goes through the mapTag and looks for map images
        // all images are uploaded and OOB is added
        Enumeration layers = map.layers();
        while (layers.hasMoreElements()) {
            LayerTag l = (LayerTag) layers.nextElement();
            ImgTag img = l.getImg();
            if (img != null) {
                File imgFile = new File(completeMapPath + "/" + img.getSrc());
                String urlStr = new String("http://" + fileServer + ":" + 
                                fileServerPort + fileServerPath + "/" + img.getSrc());
                URL url;
                try {
                    url = new URL(urlStr);
                    URLFileTransfer.uploadURL(imgFile, url, proxy, proxyPort);
                    msgBuilder.addExtension(new IQXOOB(url,img.getSrc()));
                } catch (MalformedURLException e) {
                    break;
                } catch (IOException e) {
                    break;
                }
            }
            MapTag insetMap = l.getSrcMap();
            if (insetMap != null && recursiveMapsSend) {
                addMapIntoMessage(msgBuilder, insetMap.getID(), originID, false);
            }
        }
        
        msgBuilder.addExtension(map);
        
        return true;
    }
    
    /** Copies all necessary files for given map into specified dir. */
    public boolean copyFilesForMap(MapTag map, String sourceOriginID, 
                                               String destOriginID) {
                                                   
        if (map == null) return true;
        
        String sourcePath = new String(mapPath + "/" + sourceOriginID);
        String destPath = new String(mapPath + "/" + destOriginID);
        
        // for all layers
        Enumeration layers = map.layers();
        while (layers.hasMoreElements()) {
            LayerTag l = (LayerTag) layers.nextElement();
            // copies images
            ImgTag img = l.getImg();
            if (img != null) {
                if (!FileCopier.copyFile(new File(sourcePath + "/" + img.getSrc()),
                                         new File(destPath + "/" + img.getSrc())))
                    return false;
            }
            // for nested maps
            MapTag insetMap = l.getSrcMap();
            if (insetMap != null) {
                // copies map files
                if (!FileCopier.copyFile(new File(sourcePath + "/" + insetMap.getID()),
                                         new File(destPath + "/" + insetMap.getID())))
                    return false;
                // and all necessary files referenced
                if (!copyFilesForMap(insetMap, sourceOriginID, destOriginID))
                    return false;
            }
        }
        
        return true;
    }
    
    /** Loads map */
    public void loadMap(String mapID, String originID) {
        MapTag newMap = loadMapTag(mapID, originID);
        if (newMap == null) return;
        // pridat do hashtable
        String completeID = originID + "/" + mapID;
        maps.put(completeID, newMap);
        processMap(newMap, originID);
        // zavolat manager, at to zobrazi
        fireMapReceived(newMap, originID);
    }
    
    /** Load map tag from disk */
    private MapTag loadMapTag(String mapID, String originID) {
        MapTag result = loadMapImpl(mapID,  originID);
        if (result == null)
            JOptionPane.showMessageDialog(null, "Cannot load map " + mapID,
                                      "Error", JOptionPane.ERROR_MESSAGE);
        return result;
    }
    
    /** Implementation of load map tag from disk */
    private MapTag loadMapImpl(String mapID, String originID) {
        String completeID = new String(originID + "/" + mapID);
        String filename = new String(mapPath + "/" + completeID);
        try {
            XMLFileIO mapLoader = new XMLFileIO();
            XMLData d = mapLoader.readFile(filename);
            if (d == null)
                return null;
            if (d instanceof MapTag) {
                maps.put(completeID, d);
                return (MapTag) d;
            }
            else
                return null;
        } 
        catch (IOException e) {return null;}
    }
    
    /** Returns <code>MapTag</code> either from cache or loaded from disk */
    public MapTag getMap(String originID, String mapID) {
        String completeID = new String(originID + "/" + mapID);
        
        MapTag map = (MapTag) maps.get(completeID);
        if (map != null) return map;
        
        map = loadMapImpl(mapID, originID);
        if (map != null) processMap(map, originID);
        return map;
    }
      
    
    // *** map listeners ***
    
    /**
     * Adds <code>MapListener</code> from listeners notified when
     * map arrives.
     *
     * @see #removeMapListener
     * @see #removeAllMapListeners
     * @see #fireMapReceived
     */
    public void addMapListener(BSMapListener listener) {
        //assert listener != null : listener;
        if (listener == null) return;
        if (!mapListeners.contains(listener)) {
            mapListeners.addElement(listener);
        }
    }

    /**
     * Removes <code>MapListener</code> from listeners notified when
     * map arrives.
     *
     * @see #addMapListener
     * @see #removeAllMapListeners
     * @see #fireMapReceived
     */
    public void removeMapListener(BSMapListener listener) {
        //assert listener != null : listener;
        if (listener == null) return;
        mapListeners.removeElement(listener);
    }

    /**
     * Removes all listeners notified when map arrives.
     * This can be used before to free dependencies and allow dispose of
     * all objects.
     *
     * @see #addMapListener
     * @see #removeMapListener
     * @see #fireMapReceived
     */
    public void removeAllMapListeners() {
        mapListeners.clear();
    }

    /**
     * Notifies map listeners that  
     * <code>map</code> was received.
     *
     * @see #addMapListener
     * @see #removeMapListener
     * @see #removeAllMapListeners
     */
    protected void fireMapReceived(MapTag map, String originID) {
        for (Enumeration e = mapListeners.elements(); e.hasMoreElements(); ) {
            BSMapListener listener = (BSMapListener) e.nextElement();
            listener.mapReceived(map, originID);
        }
    }
    
}