package nu.fw.jeti.plugins.buddyspacemaps.editor;

/*
 * BSMapEditJIDLabel.java
 *
 * Project: BuddySpace
 * (C) Copyright Knowledge Media Institute 2002
 *
 *
 * Created on 16 December 2002, 10:17
 * 
 * E.S. de Boer
 * converted to jeti plugin
 */

import java.awt.Color;

import javax.swing.Icon;
import javax.swing.border.LineBorder;

import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.plugins.buddyspacemaps.xml.ItemTag;
import nu.fw.jeti.plugins.buddyspacemaps.xml.LayerTag;



/**
 * <code>BSMapEditJIDLabel</code> is a icon label for specified JID 
 * with references to the source <map> tag to allow editting. 
 * It is meant to display presence specific icon and tooltip including
 * JID's nick name and JID string representation.
 *
 * @author  Jiri Komzak, Knowledge Media Institute, Open University, United Kingdom
 */
public class BSMapEditJIDLabel extends BSJIDLabel {
    
    private ItemTag tag = null;
    private LayerTag layerTag = null;
    private boolean editable;
    
    /** Constructor */
    public BSMapEditJIDLabel(JID jid, String nick, Icon img, 
                             ItemTag tag, LayerTag layerTag, boolean editable) {
        super(jid, nick, img);
        this.tag = tag;
        this.layerTag = layerTag;
        this.editable = editable;
    }
    
    /** Returns label's tag */
    public ItemTag getTag() {
        return tag;
    }
    
    /** Returns label's layer tag */
    public LayerTag getLayerTag() {
        return layerTag;
    }
    
    /** Displays the label as (un)selected */
    public void setAsSelected(boolean selected) {
        setBorder(selected? new LineBorder(Color.yellow, 2) : null);
    }
    
    /** Displays the label as (un)edited */
    public void setAsEdited(boolean edited) {
        setBorder((edited && editable)? new LineBorder(Color.green, 2) : null);
    }
    
    /** Returns if the label is editable */
    public boolean isEditable() {
        return editable;
    }
}