package nu.fw.jeti.plugins.buddyspacemaps.editor;

/*
 * BSMapEditView.java
 *
 * Project: BuddySpace
 * (C) Copyright Knowledge Media Institute 2002
 *
 *
 * Created on 19 August 2002, 9:54
 * 
 * E.S. de Boer
 * converted to jeti plugin
 */

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.*;
import javax.swing.border.LineBorder;
import javax.swing.event.MouseInputListener;

import nu.fw.jeti.backend.roster.Roster;
import nu.fw.jeti.images.StatusIcons;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.jabber.JIDStatus;
import nu.fw.jeti.jabber.elements.Presence;
import nu.fw.jeti.plugins.buddyspacemaps.xml.*;




/**
 * <code>BSMapEditView</code> is view of map represented by a <code>MapTag</code>.
 *
 * @author  Jiri Komzak, Knowledge Media Institute, Open University, United Kingdom
 */
public class BSMapEditView extends JScrollPane
                           implements MouseInputListener, ActionListener {
    
    
    
    private BSMapEditWindow mapPanel = null;
    
    private JPanel helpPanel = null;
    private JLayeredPane mainPanel = null;
    private int layerNum = Integer.MAX_VALUE;
    
    private ImageIcon lurkerIcon = null;
    private ImageIcon clusterIcon = null;
    
    private int w = 10, h = 10;   // icons width and height
    
    private JPopupMenu jidPopupMenu;
    private JMenuItem deleteMenuItem;
    private JMenuItem addMenuItem;
    private JMenuItem jidMenuItem;
    private JPopupMenu clusterPopupMenu;
    private JMenuItem setClusterNameMenuItem;
    private JMenuItem deleteClusterMenuItem;
    private JPopupMenu imgPopupMenu;
    private JMenuItem changeMenuItem;
    private JPopupMenu layerPopupMenu;
    private JMenuItem resizeMenuItem;
    private JMenuItem deleteLayerMenuItem;
    private JMenuItem addLayerMenuItem;
    private JMenuItem layerMenuItem;
    private JMenuItem infoMenuItem;
    
    private String originID = null;
    private String mapID = null;
    private MapTag map = null;
    
    private String mapPath;
    
    private Object selectedComponent = null;
    private LayerTag selectedMapLayer = null;
    private int posInSelectedX;
    private int posInSelectedY;
    
    private boolean refreshOnRelease = false;
    
    private Hashtable layersComponents = new Hashtable();
    
    /** Constructor */
    public BSMapEditView(BSMapEditWindow mapPanel, MapTag map, String mapID, String originID,
                         String mapPath) {
        super();
        
        this.mapPanel = mapPanel;
        this.mapPath = mapPath;
               
        
        lurkerIcon = StatusIcons.getStatusIcon(Presence.NONE);
        clusterIcon = new ImageIcon(getClass().getResource("map_cluster.gif"));
        
        jidPopupMenu = new JPopupMenu();
        clusterPopupMenu = new JPopupMenu();
        deleteMenuItem = new JMenuItem("Delete buddy");
        deleteMenuItem.addActionListener(this);
        addMenuItem = new JMenuItem("Add buddy");
        addMenuItem.addActionListener(this);
        jidMenuItem = new JMenuItem("Set JID");
        jidMenuItem.addActionListener(this);
        deleteClusterMenuItem = new JMenuItem("Delete cluster");
        deleteClusterMenuItem.addActionListener(this);
        setClusterNameMenuItem = new JMenuItem("Set cluster name");
        setClusterNameMenuItem.addActionListener(this);
        imgPopupMenu = new JPopupMenu();
        changeMenuItem = new JMenuItem("Change image");
        changeMenuItem.addActionListener(this);
        layerPopupMenu = new JPopupMenu();
        resizeMenuItem = new JMenuItem("Resize inset");
        resizeMenuItem.addActionListener(this);
        layerMenuItem = new JMenuItem("Set layer properties");
        layerMenuItem.addActionListener(this);
        addLayerMenuItem = new JMenuItem("Add layer");
        addLayerMenuItem.addActionListener(this);
        deleteLayerMenuItem = new JMenuItem("Delete layer");
        deleteLayerMenuItem.addActionListener(this);
        infoMenuItem = new JMenuItem("Info");
    
        this.mapID = mapID;
        this.originID = originID;
        this.map = map;
        
        helpPanel = new JPanel();
        //helpPanel.setLayout(new BorderLayout());
        mainPanel = new JLayeredPane();
        mainPanel.setDoubleBuffered(true);
        mainPanel.setBorder(new LineBorder(Color.black, 1));
        //mainPanel.setLayout(null);
        
        layersComponents.clear();
        addMap(map, 0, 0, 1, true, null);
        
        /*Insets insets = mainPanel.getInsets();
        JLabel lblnnn = new JLabel("nazdarek");
        mainPanel.add(lblnnn, new Integer(Integer.MAX_VALUE));
        lblnnn.setBounds(50 + insets.left, 50 + insets.top, 30, 30);*/
        
        setSelectedLayer(null);
        
        helpPanel.add(mainPanel);
        //helpPanel.add(mainPanel, BorderLayout.CENTER);
        setViewportView(helpPanel);
    }
    
    /** Returns map id */
    public String getMapID() {
        return mapID;
    }
    
    /** Returns origin id */
    public String getOriginID() {
        return originID;
    }
    
    public void setMapID(String mapID) {
        this.mapID = mapID;
    }
    
    public void setOriginID(String originID) {
        this.originID = originID;
    }
    
    
    // *** map view construction functions ***
    
    /** Adds <code>map</code> into the view.
     * This is used for inserting insets during the view construction.
     */
    protected void addMap(MapTag map, int offsetX, int offsetY, float scale, 
                          boolean editable, LayerTag topLevelLayer) {
        Enumeration layers;
        
        // creates temp vector
        Vector tmpLayers = new Vector();
        layers = map.layers();
        while (layers.hasMoreElements()) {
            tmpLayers.add(layers.nextElement());
        }
        
        // goes through all layers and inserts them according to their priority
        int lastPriority = Integer.MAX_VALUE;
        int curPriority = getNextHighestPriority(tmpLayers, lastPriority);
        while (curPriority != -1) {
            Vector tmp2Layers = (Vector) tmpLayers.clone();
            layers = tmp2Layers.elements();
            while (layers.hasMoreElements()) {
                LayerTag l = (LayerTag) layers.nextElement();
                if (l.getPriorityInt() == curPriority) {
                    if (editable) {
                        topLevelLayer = l;
                        layersComponents.put(l, new Vector());
                    }
                    addLayer(l, offsetX, offsetY, scale, editable, topLevelLayer);
                    tmpLayers.remove(l);
                }
            }
            lastPriority = curPriority;
            curPriority = getNextHighestPriority(tmpLayers, lastPriority);
        }
        
        invalidate();
        repaint();
    }
    
    /**
     * Returns the highest priority in vector lower than lastPriority.
     */
    protected int getNextHighestPriority(Vector layersVector, int lastPriority) {
        int newPriority = -1;
        Enumeration layers = layersVector.elements();
        while (layers.hasMoreElements()) {
            int priority = ((LayerTag) layers.nextElement()).getPriorityInt();
            if (priority < lastPriority && priority > newPriority)
                newPriority = priority;
        }
        return newPriority;
    }
    
    /** Adds <code>layer</code> into the view.
     *  This is used during the view construction.
     */
    protected void addLayer(LayerTag layer, int offsetX, int offsetY, float scale, 
                            boolean editable, LayerTag topLevelLayer) {
        
        Insets insets = mainPanel.getInsets();
        
        // sets the new scale and offset inside the layer
        float newScale = scale * layer.getScaleFloat();
        int newOffsetX = Math.round(offsetX + layer.getOffsetXInt() * scale);
        int newOffsetY = Math.round(offsetY + layer.getOffsetYInt() * scale);
        
        // adds all the items
        Enumeration items = layer.getItems();
        while (items.hasMoreElements()) {
            Object o = items.nextElement();
            // if it's item
            if (o instanceof ItemTag) {
                ItemTag i = (ItemTag) o;
                JID jid = i.getJID();
                if (jid == null) break;
                String nick = null;
                JIDStatus j = Roster.getJIDStatus(jid);
                if (j!= null) nick = j.getNick(); 
                if(nick == null) nick = "?";
                BSMapEditJIDLabel jidLabel = new BSMapEditJIDLabel(jid, nick, lurkerIcon, (ItemTag)o, topLevelLayer, editable);
                ((Vector)layersComponents.get(topLevelLayer)).add(jidLabel);
                jidLabel.setOpaque(false);
                if (lurkerIcon != null) {
                    w = lurkerIcon.getIconWidth();
                    h = lurkerIcon.getIconHeight();
                }
                int x = Math.round(i.getLonFloat()*newScale + newOffsetX - w/2);
                int y = Math.round(i.getLatFloat()*newScale + newOffsetY - h/2);
                //mainPanel.add(jidLabel, new AbsoluteConstraints(x, y, w, h));
                //mainPanel.add(jidLabel, JLayeredPane.DEFAULT_LAYER, layerNum);
                mainPanel.add(jidLabel, new Integer(layerNum));
                jidLabel.setBounds(x + insets.left, y + insets.top, w, h);
                jidLabel.addMouseListener(this);
                jidLabel.addMouseMotionListener(this);
            }
            // else if it's cluster
            else if (o instanceof ClusterTag) {
                ClusterTag c = (ClusterTag) o;
                //BSClusterLabel clusterLabel = new BSClusterLabel(c, clusterIcon);
                BSMapEditClusterLabel clusterLabel = new BSMapEditClusterLabel(c, clusterIcon, topLevelLayer, editable);
                ((Vector)layersComponents.get(topLevelLayer)).add(clusterLabel);
                clusterLabel.setOpaque(false);
                if (clusterIcon != null) {
                    w = clusterIcon.getIconWidth();
                    h = clusterIcon.getIconHeight();
                }
                int x = Math.round(c.getLonFloat()*newScale + newOffsetX - w/2);
                int y = Math.round(c.getLatFloat()*newScale + newOffsetY - h/2);
                //mainPanel.add(jidLabel, new AbsoluteConstraints(x, y, w, h));
                //mainPanel.add(clusterLabel, JLayeredPane.DEFAULT_LAYER, layerNum);
                mainPanel.add(clusterLabel, new Integer(layerNum));
                clusterLabel.setBounds(x + insets.left, y + insets.top, w, h);
                clusterLabel.addMouseListener(this);
                clusterLabel.addMouseMotionListener(this);
            }
        }
        layerNum--;
        
        String completeMapPath = new String(mapPath + "/" + originID);
        // loads image of map
        ImgTag imgTag = layer.getImg();
        if (imgTag != null) {
            //Image image = getImage(ClassLoader.getSystemResource("images/" + imgTag.getSrc()));
            /*Image image = Toolkit.getDefaultToolkit().getImage(
                            ClassLoader.getSystemResource(completeMapPath + "/" + imgTag.getSrc()));*/
        	
        	 MediaTracker media = new MediaTracker(this);
        	 Image image = Toolkit.getDefaultToolkit().getImage(completeMapPath + "/" + imgTag.getSrc());
        	 media.addImage(image, 0);
        	 try {
        	 media.waitForID(0);
        	 }
        	 catch (Exception e) {}
        	 
            BSMapEditImagePanel imgPanel = new BSMapEditImagePanel(
                                    image, newScale, mainPanel, 
                                    imgTag.getWidthInt(), imgTag.getHeightInt(),
                                    imgTag, topLevelLayer, editable);
            ((Vector)layersComponents.get(topLevelLayer)).add(imgPanel);
            //mainPanel.add(imgPanel, JLayeredPane.DEFAULT_LAYER, layerNum);
            mainPanel.add(imgPanel, new Integer(layerNum));
            imgPanel.addMouseListener(this);
            imgPanel.addMouseMotionListener(this);
            imgPanel.setBounds(newOffsetX + insets.left, newOffsetY + insets.top,
                               imgPanel.getWidth(), imgPanel.getHeight());
            imgPanel.maybeResizeParent();
        }
        layerNum--;
        
        // loads map given in layer's src attribute (when the map is only referenced)
        MapTag map = layer.getSrcMap();
        if (map != null) {
            addMap(map, newOffsetX, newOffsetY, newScale, false, topLevelLayer);
        }
    }
    
    /** Shows popup menu if supposed to */
    private void maybeShowPopup(MouseEvent e) {
        // if that was context popup event
        if (e.isPopupTrigger()) {
        //if (e.getClickCount() == 1 && SwingUtilities.isRightMouseButton(e)) {
            // gets the clicked label
            Object src = e.getSource();
            
            // if we clicked on the group node
            if (src instanceof BSMapEditJIDLabel) {
                BSMapEditJIDLabel label = (BSMapEditJIDLabel) src;
                JID jid = label.getJID();
                jidPopupMenu.removeAll();
                
                if (label.isEditable()) {
                    jidPopupMenu.add(addMenuItem);
                    jidPopupMenu.add(deleteMenuItem);
                    //jidPopupMenu.add(jidMenuItem);
                    jidPopupMenu.add(new JSeparator());
                }
                
                jidPopupMenu.add(layerMenuItem);
                jidPopupMenu.add(addLayerMenuItem);
                jidPopupMenu.add(deleteLayerMenuItem);
                jidPopupMenu.add(new JSeparator());
                infoMenuItem.setText(jid.toString());
                jidPopupMenu.add(infoMenuItem);
                
                jidPopupMenu.show(e.getComponent(), e.getX(), e.getY());
                //adjustPopupPosition(jidPopupMenu);*/
            }
            
            // else if we clicked on image panel
            else if (src instanceof BSMapEditImagePanel) {
                BSMapEditImagePanel imgPanel = (BSMapEditImagePanel) src;
                
                imgPopupMenu.removeAll();
                
                if (imgPanel.isEditable()) {
                    imgPopupMenu.add(changeMenuItem);
                    imgPopupMenu.add(new JSeparator());
                    imgPopupMenu.add(addMenuItem);
                }
                
                imgPopupMenu.add(layerMenuItem);
                imgPopupMenu.add(addLayerMenuItem);
                imgPopupMenu.add(deleteLayerMenuItem);
                imgPopupMenu.add(new JSeparator());
                infoMenuItem.setText(imgPanel.getTag().getSrc());
                imgPopupMenu.add(infoMenuItem);
                
                imgPopupMenu.show(e.getComponent(), e.getX(), e.getY());
                //adjustPopupPosition(jidPopupMenu);*/
            }
            
            // else if we clicked on cluster label
            else if (src instanceof BSMapEditClusterLabel) {
                
                BSMapEditClusterLabel label = (BSMapEditClusterLabel) src;
                clusterPopupMenu.removeAll();
                
                if (label.isEditable()) {
                    clusterPopupMenu.add(addMenuItem);
                    clusterPopupMenu.add(deleteClusterMenuItem);
                    clusterPopupMenu.add(setClusterNameMenuItem);
                    //jidPopupMenu.add(jidMenuItem);
                    clusterPopupMenu.add(new JSeparator());
                }
                
                clusterPopupMenu.add(layerMenuItem);
                clusterPopupMenu.add(addLayerMenuItem);
                clusterPopupMenu.add(deleteLayerMenuItem);
                clusterPopupMenu.add(new JSeparator());
                
                Enumeration jids = label.getJIDs();
                // for all jids in cluster
                while (jids.hasMoreElements()) {
                    JID j = (JID) jids.nextElement();
                    clusterPopupMenu.add(new JMenuItem(j.toString()));
                }
                
                clusterPopupMenu.show(e.getComponent(), e.getX(), e.getY());
                //adjustPopupPosition(jidPopupMenu);*/
                
            } // else(cluster)
        } //if(popup)
    }
    
    
    // *** presence listener stuff ***
    
    /** <code>MouseListener</code> method - catches double-clicks */
    public void mouseClicked(MouseEvent e) {
        // handles only double clicks on JID nodes
        if ( e.getClickCount() == 2 ) {
            //maybeOpenChatWindow(e);
            //tree.repaint();
        }
    }
    
    /** <code>MouseListener</code> method - catches popup-menu clicks */
    public void mousePressed(MouseEvent e) {
        selectComponent(e);
        maybeShowPopup(e);
    }
    
    /** <code>MouseListener</code> method - catches popup-menu clicks */
    public void mouseReleased(MouseEvent e) {
        //selectComponent(e);
        if (refreshOnRelease) {
            refresh();
            refreshOnRelease = false;
        }
        else {
            maybeShowPopup(e);
        }
    }
    
    /** <code>MouseListener</code> method - empty implementation */
    public void mouseEntered(MouseEvent e) { }
    /** <code>MouseListener</code> method - empty implementation */
    public void mouseExited(MouseEvent e) { }
    
    public void mouseDragged(MouseEvent e) {
        if (selectedComponent == null)
            moveLayer(e);
        else
            moveComponent(e);
    }
    
    public void mouseMoved(MouseEvent e) { }
    
    
    /** Selects component */
    protected void selectComponent(MouseEvent e) {
        unselectComponent();
        Object component = e.getSource();
        posInSelectedX = e.getX();
        posInSelectedY = e.getY();
        
        if (component instanceof BSMapEditJIDLabel) {
            BSMapEditJIDLabel label = (BSMapEditJIDLabel) component;
            mapPanel.setSelectedLayer(label.getLayerTag().getID());
            if (label.isEditable()) {
                selectedComponent = component;
                label.setAsEdited(true);
            }
        }
        else if (component instanceof BSMapEditClusterLabel) {
            BSMapEditClusterLabel label = (BSMapEditClusterLabel) component;
            mapPanel.setSelectedLayer(label.getLayerTag().getID());
            if (label.isEditable()) {
                selectedComponent = component;
                label.setAsEdited(true);
            }
        }
        else if (component instanceof BSMapEditImagePanel) {
            BSMapEditImagePanel imgPanel = (BSMapEditImagePanel) component;
            mapPanel.setSelectedLayer(imgPanel.getLayerTag().getID());
            if (imgPanel.isEditable()) {
                selectedComponent = component;
                imgPanel.setAsEdited(true);
            }
        }
    }
    
    
    /** Unselects selected component */
    protected void unselectComponent() {
        if (selectedComponent == null) return;
        
        if (selectedComponent instanceof BSMapEditJIDLabel)
            ((BSMapEditJIDLabel) selectedComponent).setAsEdited(false);
        else if (selectedComponent instanceof BSMapEditClusterLabel)
            ((BSMapEditClusterLabel) selectedComponent).setAsEdited(false);
        else if (selectedComponent instanceof BSMapEditImagePanel)
            ((BSMapEditImagePanel) selectedComponent).setAsEdited(false);
        
        selectedComponent = null;
    }
    
    
    /** Moves component */
    protected void moveComponent(MouseEvent e) {
        Object component = e.getSource();
        
        if (component != selectedComponent) return;
        
        if (component instanceof BSMapEditJIDLabel) {
            BSMapEditJIDLabel label = (BSMapEditJIDLabel) component;
            //ImgTag img = label.getLayerTag().getImg();
            Point p = label.getLocation();
            p.translate(e.getX()-posInSelectedX, e.getY()-posInSelectedY);
            label.setLocation(p);
            float lon = (float)(p.getX() + label.getWidth()/2);
            float lat = (float)(p.getY() + label.getHeight()/2);
            LayerTag layer = label.getLayerTag();
            lon = (lon - layer.getOffsetXInt())/layer.getScaleFloat();
            lat = (lat - layer.getOffsetYInt())/layer.getScaleFloat();
            ItemTag tag = label.getTag();
            tag.setLon(lon);
            tag.setLat(lat);
        }
        else if (component instanceof BSMapEditClusterLabel) {
            BSMapEditClusterLabel label = (BSMapEditClusterLabel) component;
            //ImgTag img = label.getLayerTag().getImg();
            Point p = label.getLocation();
            p.translate(e.getX()-posInSelectedX, e.getY()-posInSelectedY);
            label.setLocation(p);
            float lon = (float)(p.getX() + label.getWidth()/2);
            float lat = (float)(p.getY() + label.getHeight()/2);
            LayerTag layer = label.getLayerTag();
            lon = (lon - layer.getOffsetXInt())/layer.getScaleFloat();
            lat = (lat - layer.getOffsetYInt())/layer.getScaleFloat();
            ClusterTag tag = label.getTag();
            tag.setLonFloat(lon);
            tag.setLatFloat(lat);
        }
        else if (component instanceof BSMapEditImagePanel) {
            moveLayer(e);
        }
    }
    
    
    /** Moves whole layer */
    protected void moveLayer(MouseEvent e) {
        
        if (selectedMapLayer == null) return;
        
        Vector components = (Vector) layersComponents.get(selectedMapLayer);
        if (components == null) return;
        
        int moveX = e.getX()-posInSelectedX;
        int moveY = e.getY()-posInSelectedY;
        Enumeration compEnum = components.elements();
        while (compEnum.hasMoreElements()) {
            Component c = (Component) compEnum.nextElement();
            Point p = c.getLocation();
            p.translate(moveX, moveY);
            c.setLocation(p);
        }
        selectedMapLayer.setOffsetX(Integer.toString(
                                    selectedMapLayer.getOffsetXInt() + moveX));
        selectedMapLayer.setOffsetY(Integer.toString(
                                    selectedMapLayer.getOffsetYInt() + moveY));
        refreshOnRelease = true;
    }
    
    
    /** Handles actions from GUI controls */
    public void actionPerformed(ActionEvent e) {
        Object src = e.getSource();
        
        // adds buddy into cluster
        if (src == addMenuItem) {
            /*JID j = chatMenuItem.getJID();
            if (j != null)
                mapPanel.openChat(j);*/
            if (selectedComponent instanceof BSMapEditImagePanel) {
                mapPanel.addBuddy();
            }
            if (selectedComponent instanceof BSMapEditClusterLabel) {
                mapPanel.addBuddy();
            }
            else if (selectedComponent instanceof BSMapEditJIDLabel) {
                mapPanel.addBuddy();
            }
        }
        
        // deletes buddy
        else if (src == deleteMenuItem) {
            if (selectedComponent == null)
                return;
            if (selectedComponent instanceof BSMapEditJIDLabel) {
                BSMapEditJIDLabel label = (BSMapEditJIDLabel) selectedComponent;
                label.getLayerTag().removeItem(label.getTag());
                mainPanel.remove(label);
                ((Vector)layersComponents.get(label.getLayerTag())).remove(label);
            }
            mainPanel.repaint();
        }
        
        // deletes cluster
        else if (src == deleteClusterMenuItem) {
            if (selectedComponent == null)
                return;
            if (selectedComponent instanceof BSMapEditClusterLabel) {
                BSMapEditClusterLabel label = (BSMapEditClusterLabel) selectedComponent;
                label.getLayerTag().removeCluster(label.getTag());
                mainPanel.remove(label);
                ((Vector)layersComponents.get(label.getLayerTag())).remove(label);
            }
            mainPanel.repaint();
        }
        
        // sets name of cluster
        else if (src == setClusterNameMenuItem) {
            if (selectedComponent == null)
                return;
            if (selectedComponent instanceof BSMapEditClusterLabel) {
                BSMapEditClusterLabel label = (BSMapEditClusterLabel) selectedComponent;
                ClusterTag tag = label.getTag();
                if (tag != null)
                    mapPanel.setClusterName(tag.getName());
                else 
                    mapPanel.setClusterName(null);
            }
        }
        
        // changes buddy (JID)
        else if (src == jidMenuItem) {
            /*JID j = chatMenuItem.getJID();
            if (j != null)
                mapPanel.sendSubscriptionRequest(j);*/
        }
        
        // changes image source file
        else if (src == changeMenuItem) {
            mapPanel.setImg();
        }
        
        // changes layer properties
        else if (src == layerMenuItem) {
            mapPanel.setLayer();
        }
        
        // adds layer
        else if (src == addLayerMenuItem) {
            mapPanel.addLayer();
        }
        
        // deletes layer
        else if (src == deleteLayerMenuItem) {
            mapPanel.deleteLayer();
        }
        
        // a resource must be specified !!!
        /*else if (src == sendFileMenuItem) {
            JID j = chatMenuItem.getJID();
            if (j != null)
                mapPanel.sendFile(j);
        }*/
    }
    
    public Enumeration getLayers() {
        if (map != null)
            return map.layers();
        else return new Vector().elements();
    }
    
    public boolean isSelectedLayerEditable() {
        return selectedMapLayer != null && selectedMapLayer.getSrc() == null;
    }
    
    public void addBuddy(JID jid) {
        
        if (selectedComponent == null || selectedMapLayer == null) {
            JOptionPane.showMessageDialog(null, "No layer selected", 
                       "Add buddy error", 
                       JOptionPane.ERROR_MESSAGE);
            return;
        }
            
        if (selectedMapLayer.getSrc() != null) {
            JOptionPane.showMessageDialog(null, "Cannot change referenced map", 
                                          "Add buddy error", 
                                          JOptionPane.ERROR_MESSAGE);
            return;
        }
        
        ItemTagBuilder itemBuilder = new ItemTagBuilder();
        itemBuilder.setJID(jid);
        ItemTag newItem;
        //try {
        	newItem = new ItemTag(itemBuilder); 
         // }        catch (InstantiationException e) { return; }

        if (selectedComponent instanceof BSMapEditImagePanel) {
            float scale = selectedMapLayer.getScaleFloat();
            float lon, lat;
            if (scale != 0) {
                lon = posInSelectedX / scale;
                lat = posInSelectedY / scale;
            }
            else {
                lon = lurkerIcon.getIconWidth()/2;
                lat = lurkerIcon.getIconHeight()/2;
            }
            newItem.setLon(lon);
            newItem.setLat(lat);
            selectedMapLayer.addItem(newItem);
        }
        else if (selectedComponent instanceof BSMapEditClusterLabel) {
            // gets the cluster
            BSMapEditClusterLabel label = (BSMapEditClusterLabel) selectedComponent;
            ClusterTag cluster = label.getTag();
            // sets coordinates for the new one
            newItem.setLon(cluster.getLonFloat());
            newItem.setLat(cluster.getLatFloat());
            // adds item into the cluster
            cluster.addItem(newItem);
        }
        else if (selectedComponent instanceof BSMapEditJIDLabel) {
            // gets the old item
            BSMapEditJIDLabel label = (BSMapEditJIDLabel) selectedComponent;
            ItemTag oldItem = label.getTag();
            // sets coordinates for the new one
            newItem.setLon(oldItem.getLonFloat());
            newItem.setLat(oldItem.getLatFloat());
            // creates new cluster
            ClusterTagBuilder clusterBuilder = new ClusterTagBuilder();
            clusterBuilder.setSize(Integer.toString(0));
            clusterBuilder.setLon(oldItem.getLon());
            clusterBuilder.setLat(oldItem.getLat());
            clusterBuilder.addItem(newItem);
            clusterBuilder.addItem(oldItem);
            ClusterTag clusterTag;
            //try {
            	clusterTag = new ClusterTag(clusterBuilder); 
            //	}         catch (InstantiationException e) { return; }
            // removes the old item from the layer
            selectedMapLayer.removeItem(oldItem);
            // adds cluster into the layer
            selectedMapLayer.addCluster(clusterTag);
        }

        reloadMap();
    }
    
    
    public void setClusterName(String name) {
        
        if (selectedComponent == null) {
            JOptionPane.showMessageDialog(null, "No cluster selected", 
                       "Set cluster name error", 
                       JOptionPane.ERROR_MESSAGE);
            return;
        }
        
        if (selectedComponent instanceof BSMapEditClusterLabel) {
            BSMapEditClusterLabel label = (BSMapEditClusterLabel) selectedComponent;
            ClusterTag tag = label.getTag();
            if (tag == null) 
                return;
            tag.setName(name);
        }
        reloadMap();
    }
    
    
    public void setSelectedLayer(String layerName) {
        Enumeration layersEnum = getLayers();
        if (layerName == null) {
            if (layersEnum.hasMoreElements()) {
                if (selectedMapLayer != null)
                    selectLayerComponents(selectedMapLayer, false);
                selectedMapLayer = (LayerTag) layersEnum.nextElement();
                if (selectedMapLayer != null)
                    mapPanel.setSelectedLayer(selectedMapLayer.getID());
                selectLayerComponents(selectedMapLayer, true);
            }
            else
                selectedMapLayer = null;
        }
        else {
            if (selectedMapLayer != null)
                selectLayerComponents(selectedMapLayer, false);
            selectedMapLayer = null;
            while (layersEnum.hasMoreElements()) {
                LayerTag curLayer = (LayerTag)layersEnum.nextElement();
                String curLayerName = curLayer.getID();
                if (layerName.equals(curLayerName)) {
                    selectedMapLayer = curLayer;
                    mapPanel.setSelectedLayer(layerName);
                    selectLayerComponents(selectedMapLayer, true);
                    break;
                }
            }
        }
    }
    
    public void selectLayerComponents(LayerTag layer, boolean select) {
        if (layer == null) return;
        Vector comps = (Vector) layersComponents.get(layer);
        if (comps == null) return;
        Enumeration compsEnum = comps.elements();
        while (compsEnum.hasMoreElements()) {
            Object o = compsEnum.nextElement();
            if (o instanceof BSMapEditJIDLabel) ((BSMapEditJIDLabel)o).setAsSelected(select);
            else if (o instanceof BSMapEditClusterLabel) ((BSMapEditClusterLabel)o).setAsSelected(select);
            else if (o instanceof BSMapEditImagePanel) ((BSMapEditImagePanel)o).setAsSelected(select);
        }
    }
    
    public void reloadMap() {
        selectedComponent = null;
        mainPanel.removeAll();
        layersComponents.clear();
        this.addMap(map, 0, 0, 1, true, null);
    }
    
    public void refresh() {
        reloadMap();
        revalidate();
        repaint();
        mainPanel.revalidate();
        mainPanel.repaint();
    }
    
    public String getMapPath() {
        return mapPath;
    }
    
    public void setImg(String imgSrc, String imgWidth, String imgHeight) {
        if (selectedMapLayer == null)
            JOptionPane.showMessageDialog(null, "No layer selected", 
                       "Set bitmap error", 
                       JOptionPane.ERROR_MESSAGE);
        else if (selectedMapLayer.getSrc() != null)
            JOptionPane.showMessageDialog(null, "Cannot change referenced map", 
                       "Set bitmap error", 
                       JOptionPane.ERROR_MESSAGE);
        else {
            //String imgPath = new String(mapPath + "/" + originID + "/" + imgSrc);
            //Icon tmpIcon = new ImageIcon(imgPath);
            ImgTag imgTag = selectedMapLayer.getImg();
            if (imgTag == null) {
                ImgTagBuilder imgBuilder = new ImgTagBuilder();
                //try { 
                	imgTag = new ImgTag(imgBuilder); 
                //	}    catch (InstantiationException e) { return; }
            }
            imgTag.setSrc(imgSrc);
            //imgTag.setWidth(imgWidth);
            //imgTag.setHeight(imgHeight);
            imgTag.setWidth(Integer.toString(0));
            imgTag.setHeight(Integer.toString(0));
            //imgTag.setWidth(Integer.toString(tmpIcon.getIconWidth()));
            //imgTag.setHeight(Integer.toString(tmpIcon.getIconHeight()));
            selectedMapLayer.setImg(imgTag);
            refresh();
        }
    }
    
    public LayerTag getSelectedLayer() {
        return selectedMapLayer;
    }
    
    public void setLayerProperties(int offsetX, int offsetY, float scale, int priority) {
        if (selectedMapLayer != null && isTopLevelLayer(selectedMapLayer)) {
            selectedMapLayer.setOffsetX(Integer.toString(offsetX));
            selectedMapLayer.setOffsetY(Integer.toString(offsetY));
            selectedMapLayer.setScale(Float.toString(scale));
            selectedMapLayer.setPriority(Integer.toString(priority));
            refresh();
        }
    }
    
    public void deleteLayer() {
        if (selectedMapLayer != null && isTopLevelLayer(selectedMapLayer)) {
            map.deleteLayer(selectedMapLayer);
            refresh();
        }
    }
    
    public boolean isTopLevelLayer(LayerTag layer) {
        if (map == null) return false;
        Enumeration layersEnum = map.layers();
        while(layersEnum.hasMoreElements())
            if (layersEnum.nextElement() == layer)
                return true;
        return false;
    }
    
    public void addMap(MapTag newMapTag) {
        if (map == null || newMapTag == null) return;
        LayerTagBuilder lb = new LayerTagBuilder();
        lb.setSrc(newMapTag.getID());
        lb.setOffsetX(Integer.toString(0));
        lb.setOffsetY(Integer.toString(0));
        lb.setPriority(Integer.toString(10));
        lb.setScale(Float.toString(1));
        lb.setID("inset_" + newMapTag.getID());
        LayerTag layer;
       // try {
            layer = new LayerTag(lb);
       // } catch(InstantiationException e) { return; }
        layer.setSrcMap(newMapTag);
        map.addLayer(layer);
        refresh();
    }
    
    public void addLayerWithImg(String imgSrc, String imgWidth, String imgHeight) {
        if (map == null) return;
        LayerTagBuilder lb = new LayerTagBuilder();
        //lb.setSrc(newMapTag.getID());
        lb.setOffsetX(Integer.toString(0));
        lb.setOffsetY(Integer.toString(0));
        lb.setPriority(Integer.toString(10));
        lb.setScale(Float.toString(1));
        Enumeration layerEnum = map.layers();
        int i;
        for (i=1; layerEnum.hasMoreElements(); i++)
            layerEnum.nextElement();
        lb.setID("inset_" + Integer.toString(i));
        LayerTag layer;
       // try {
            layer = new LayerTag(lb);
      //  } catch(InstantiationException e) { return; }
        map.addLayer(layer);
        
        selectedMapLayer = layer;
        setImg(imgSrc, imgWidth, imgHeight);
        
        refresh();
    }
    
}