package nu.fw.jeti.plugins.buddyspacemaps.editor;

/*
 * BSMapEditWindow.java
 *
 * Project: BuddySpace
 * (C) Copyright Knowledge Media Institute 2002
 *
 *
 * Created on 13 December 2002, 14:46
 * 
 * E.S. de Boer
 * converted to jeti plugin
 */

//import java.util.*;
import java.awt.BorderLayout;
import java.awt.Image;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.util.Enumeration;

import javax.swing.*;

import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.plugins.buddyspacemaps.core.FileCopier;
import nu.fw.jeti.plugins.buddyspacemaps.gui.BSMapWinManager;
import nu.fw.jeti.plugins.buddyspacemaps.gui.BSNameDialog;
import nu.fw.jeti.plugins.buddyspacemaps.gui.DockableWindow;
import nu.fw.jeti.plugins.buddyspacemaps.xml.LayerTag;
import nu.fw.jeti.plugins.buddyspacemaps.xml.MapTag;



/**
 * <code>BSMapEditWindow</code> provides map editing window GUI.
 * It contains <code>BSMapEditView</code> which is the actual map view and several
 * control components. All communication is done through 
 * <code>BSMapWinManager</code>.
 *
 * @author  Jiri Komzak, Knowledge Media Institute, Open University, United Kingdom
 */
public class BSMapEditWindow extends DockableWindow 
                             implements ActionListener, ItemListener {
                          
    static final String TEMP_DIR = "temp";
    
    private JPanel mainPanel;
    
    private BSMapEditView mapView = null;
    //private JPanel mapPanel = null;
    //private JPanel helpPanel = null;
    //private JScrollPane mapScrollPane = null;
    private JPanel buttonPanel = null;
    
    private JButton closeButton = null;
    private JButton saveButton = null;
    private JButton saveAsButton = null;

    private JButton dockButton;
    private JComboBox layersComboBox = null;
    /*private JButton addBuddyButton;
    private JButton setImgButton;
    private JButton setLayerButton;
    private JButton addMapButton;*/
    private JButton addLayerButton;
    private JButton refreshButton;
    private JPanel checkBoxesPanel;
    private JFrame mainFrame;
    
    private boolean dontSetLayer = false;
    
    /** Constructor */
    public BSMapEditWindow(BSMapWinManager winMan, String title,
                    Image icon, MapTag map, String mapID, String originID,
                    String mapPath,  boolean docked) {
                   
        super(originID + "/" + mapID, title, icon, docked, winMan);
        
        saveMapAs(originID, mapID, TEMP_DIR, mapID);
        mapView = new BSMapEditView(this, map, mapID, TEMP_DIR, mapPath);
        setTitle(TEMP_DIR + "/" + mapID);
        
        initComponents();
        
        if (!docked)
        {
            frame.pack();
            frame.setLocationRelativeTo(null);
        }
    }
    
    protected void initComponents() {
        mainPanel = new JPanel(new BorderLayout());
        mainPanel.add(mapView, BorderLayout.CENTER);
        
        //JPanel topPanel = new JPanel(new GridBagLayout());
        
        JPanel checkBoxesPanel = new JPanel();
        
        Icon icon = new ImageIcon(getClass().getResource(!docked?
                                "dock.gif" : "float.gif"));
        dockButton = new JButton(icon);
        dockButton.setToolTipText(docked? "Float" : "Dock");
        dockButton.addActionListener(this);
        checkBoxesPanel.add(dockButton);
              
        layersComboBox = new JComboBox();
        initLayersComboBox();
        layersComboBox.addItemListener(this);
        checkBoxesPanel.add(layersComboBox);
        
        /*addBuddyButton = new JButton("Add buddy");
        addBuddyButton.addActionListener(this);
        checkBoxesPanel.add(addBuddyButton);
        
        setImgButton = new JButton("Set bitmap");
        setImgButton.addActionListener(this);
        checkBoxesPanel.add(setImgButton);
        
        setLayerButton = new JButton("Set layer");
        setLayerButton.addActionListener(this);
        checkBoxesPanel.add(setLayerButton);
        
        addMapButton = new JButton("Add inset map");
        addMapButton.addActionListener(this);
        checkBoxesPanel.add(addMapButton);*/
        
        addLayerButton = new JButton("Add layer");
        addLayerButton.addActionListener(this);
        checkBoxesPanel.add(addLayerButton);
        
        refreshButton = new JButton("Refresh");
        refreshButton.addActionListener(this);
        checkBoxesPanel.add(refreshButton);
        
        mainPanel.add(checkBoxesPanel, BorderLayout.NORTH);
        
        buttonPanel = new JPanel();
        
        saveButton = new JButton();
        saveButton.setText("Save");
        saveButton.addActionListener(this);
        buttonPanel.add(saveButton);
        
        saveAsButton = new JButton();
        saveAsButton.setText("Save as");
        saveAsButton.addActionListener(this);
        buttonPanel.add(saveAsButton);
        
        closeButton = new JButton();
        closeButton.setText("Close");
        closeButton.addActionListener(this);
        buttonPanel.add(closeButton);
        
        mainPanel.add(buttonPanel, BorderLayout.SOUTH);
        
        setLayout(new BorderLayout());
        add(mainPanel, BorderLayout.CENTER);
    }
    
    public void initLayersComboBox() {
        layersComboBox.removeAllItems();
        Enumeration layersEnum = mapView.getLayers();
        while (layersEnum.hasMoreElements()) {
            LayerTag layer = (LayerTag) layersEnum.nextElement();
            String layerName = layer.getID();
            layersComboBox.addItem(layerName);
        }
    }
   
    
    /** Handles actions from GUI controls */
    public void actionPerformed(java.awt.event.ActionEvent evt) {
        if (evt.getSource() == closeButton) {
            winMan.closeWindow(this);
        }
        else if (evt.getSource() == saveButton) {
            ((BSMapWinManager)winMan).saveMap(mapView.getOriginID(), mapView.getMapID(),
                           mapView.getOriginID(), mapView.getMapID());
        }
        
        else if (evt.getSource() == saveAsButton) {
            
            JFileChooser fileChooser = new JFileChooser(mapView.getMapPath());
            fileChooser.setFileFilter(new BSMapFilter());
            fileChooser.setDialogTitle("Save map");
        
            if (JFileChooser.APPROVE_OPTION != 
                fileChooser.showSaveDialog(docked? mainFrame : frame))
                return;
                
            File mapFile = fileChooser.getSelectedFile().getAbsoluteFile();
            String mapPath = mapFile.getAbsolutePath();
            if (!mapPath.endsWith(".ygf")) {
                mapPath = mapPath + ".ygf";
                mapFile = new File(mapPath);
            }
            String mapName = mapFile.getName();
            String mapOrigin = mapFile.getParentFile().getName();
            
            saveMapAs(mapView.getOriginID(), mapView.getMapID(),
                           mapOrigin, mapName);
            
            mapView.setMapID(mapName);
            mapView.setOriginID(mapOrigin);
            this.setTitle(mapOrigin + "/" + mapName);
        }
        
        else if (evt.getSource() == dockButton) {
            if (winMan != null) {
                winMan.setWindowDocked(this, !docked);
                Icon icon = new ImageIcon(getClass().getResource(!docked?
                        "dock.gif" : "float.gif"));
                dockButton.setIcon(icon);
                dockButton.setToolTipText(docked? "Float" : "Dock");
                winMan.selectWindow(this);
            }
        }
        
        /*else if (evt.getSource() == addBuddyButton) {
            if (!mapView.isSelectedLayerEditable()) {
                JOptionPane.showMessageDialog(docked? winMan.mainFrame : frame,
                        "Cannot change referenced map", 
                        "Add buddy error", 
                        JOptionPane.ERROR_MESSAGE);
                return;
            }
            addBuddy();
        }
        
        else if (evt.getSource() == setImgButton) {
            setImg();
        }
        
        else if (evt.getSource() == setLayerButton) {
            setLayer();
        }
        
        else if (evt.getSource() == addMapButton) {
            addLayer();
        }*/
        
        else if (evt.getSource() == addLayerButton) {
            addLayer();
        }
        
        else if (evt.getSource() == refreshButton) {
            mapView.refresh();
        }
    }
    
    public void addBuddy() {
    	
        BSChooseJIDDialog dlg = new BSChooseJIDDialog(
                                    docked? mainFrame : frame, 
                                    "Add buddy");
        dlg.setVisible(true);
        if(dlg.jid!=null)mapView.addBuddy(dlg.jid);
        
    }
    
    public void setClusterName(String defaultName) {
        BSNameDialog dlg = new BSNameDialog(
                                    docked? mainFrame : frame, 
                                    "Set cluster name", defaultName);
        dlg.setVisible(true);
        if (dlg.name != null)
            mapView.setClusterName(dlg.name);
    }
    
    protected String getImgName() {
        JFileChooser fileChooser = new JFileChooser(mapView.getMapPath());
        fileChooser.setFileFilter(new ImageFilter());
        fileChooser.setDialogTitle("Choose image");
        if (JFileChooser.APPROVE_OPTION ==
            fileChooser.showOpenDialog(docked? mainFrame : frame)) {
            File imgFile = fileChooser.getSelectedFile();
            String imgName = imgFile.getName();
            if (!FileCopier.copyFile(imgFile, 
                new File(mapView.getMapPath() + "/" + mapView.getOriginID()
                                                        + "/" + imgName))) {
                JOptionPane.showMessageDialog(docked? mainFrame : frame,
                    "Copying image file failed", 
                    "Set bitmap error", 
                    JOptionPane.ERROR_MESSAGE);
                return null;
            }
            return imgName;
        }
        else
            return null;
    }
    
    public void setImg() {
        if (!mapView.isSelectedLayerEditable()) {
            JOptionPane.showMessageDialog(docked? mainFrame : frame,
                    "Cannot change referenced map", 
                    "Set bitmap error", 
                    JOptionPane.ERROR_MESSAGE);
            return;
        }
        String imgName = getImgName();
        if (imgName == null)
            return;
        
        mapView.setImg(imgName, "100", "100");
    }
    
    public void setLayer() {
        /*if (!mapView.isSelectedTopLevelLayer()) {
            JOptionPane.showMessageDialog(docked? winMan.mainFrame : frame,
                    "Cannot change referenced map", 
                    "Set priority error", 
                    JOptionPane.ERROR_MESSAGE);
            return;
        }*/
        LayerTag layer = mapView.getSelectedLayer();
        if (layer == null) return;
        BSLayerDialog dlg = new BSLayerDialog(
                                  docked? mainFrame : frame,
                                  layer.getOffsetXInt(), layer.getOffsetYInt(),
                                  layer.getScaleFloat(),
                                  0, 20, layer.getPriorityInt());
        dlg.show();
        if (dlg.confirmed)
            mapView.setLayerProperties(dlg.offsetX, dlg.offsetY, dlg.scale, 
                                       dlg.priority);
    }
    
    public void deleteLayer() {
        LayerTag layer = mapView.getSelectedLayer();
        if (layer == null) return;
        /*if (!mapView.isSelectedTopLevelLayer()) {
            JOptionPane.showMessageDialog(docked? winMan.mainFrame : frame,
                    "Cannot change referenced map", 
                    "Set priority error", 
                    JOptionPane.ERROR_MESSAGE);
            return;
        }*/
        mapView.deleteLayer();
    }
    
    public void addLayer() {
        Object[] options = {"Add inline layer", 
                            "Add referenced map",
                            "Cancel"};
        int result = JOptionPane.showOptionDialog(
                        this,
                        "Do you want to define a new layer directly in this map\n " 
                        + "or add a layer referencing an existing map file?",
                        "Add new layer", 
                        JOptionPane.YES_NO_CANCEL_OPTION,
                        JOptionPane.QUESTION_MESSAGE,
                        null, options, options[1]);
        if (result == 1)
            addInsetMap();
        else if (result == 0)
            addInlineLayer();
    }
    
    public void addInsetMap() {
        // gets the file of map for the new layer
        JFileChooser fileChooser = new JFileChooser(mapView.getMapPath());
        fileChooser.setFileFilter(new BSMapFilter());
        fileChooser.setDialogTitle("Open map");
        
        if (JFileChooser.APPROVE_OPTION ==
            fileChooser.showOpenDialog(docked? mainFrame : frame)) {
            File mapFile = fileChooser.getSelectedFile();
            String mapName = mapFile.getName();
            
            // copies the map file into working directory
            if (!FileCopier.copyFile(mapFile, 
                new File(mapView.getMapPath() + "/" + mapView.getOriginID()
                                                        + "/" + mapName))) {
                JOptionPane.showMessageDialog(docked? mainFrame : frame,
                    "Copying map file failed", 
                    "Add map error", 
                    JOptionPane.ERROR_MESSAGE);
                return;
            }
            
            // reads the inserted map
            String mapOrigin = mapFile.getParentFile().getName();
            MapTag newMap = ((BSMapWinManager)winMan).getMap(mapOrigin, mapName);
            if (newMap == null) {
                JOptionPane.showMessageDialog(docked? mainFrame : frame,
                    "Specified file is not valid map", 
                    "Add map error", 
                    JOptionPane.ERROR_MESSAGE);
                return;
            }
            
            // copies all necessary referenced files into working directory
            if (!((BSMapWinManager)winMan).copyFilesForMap(newMap, mapOrigin, mapView.getOriginID())) {
                JOptionPane.showMessageDialog(docked? mainFrame : frame,
                    "Could not copy all referenced files", 
                    "Add map error", 
                    JOptionPane.ERROR_MESSAGE);
                return;
            }
            
            // adds the new map into the editted one
            mapView.addMap(newMap);
            
            initLayersComboBox();
        }
    }
    
    /** Adds a new inline defined layer. */
    public void addInlineLayer() {
        String imgName = getImgName();
        if (imgName == null)
            return;
        
        mapView.addLayerWithImg(imgName, "100", "100");
        
        initLayersComboBox();
    }
    
    /** Sends the map to given JID. Calls BSMapWinManager method. */
    public void sendMap(JID jid, String subject, String body) {
        ((BSMapWinManager)winMan).sendMap(mapView.getMapID(), mapView.getOriginID(), jid, subject, body);
    }
    
    /** Saves the map using given new originID and mapID. 
      * Calls BSMapWinManager method. */
    public void saveMapAs(String originID, String mapID,
                          String newOriginID, String newMapID) {
                              
        ((BSMapWinManager)winMan).saveMap(originID, mapID, newOriginID, newMapID);
            
        MapTag map = ((BSMapWinManager)winMan).getMap(newOriginID, newMapID);
        // copies all necessary referenced files into working directory
        if (!((BSMapWinManager)winMan).copyFilesForMap(map, originID, newOriginID)) {
            JOptionPane.showMessageDialog(docked? mainFrame : frame,
                "Could not copy all referenced files", 
                "Add map error", 
                JOptionPane.ERROR_MESSAGE);
            return;
        }
    }
    
    /** Opens chat with given JID. Calls BSMapWinManager method. */
    public void openChat(JID jid) {
        ((BSMapWinManager)winMan).openChat(jid);
    }
    
    /** Sends file to JID. Calls BSMapWinManager method. */
    public void sendFile(JID jid) {
        ((BSMapWinManager)winMan).sendFile(jid);
    }
    
    /** Sends subscription request to JID. Calls BSMapWinManager method. */
    public void sendSubscriptionRequest(JID jid) {
        ((BSMapWinManager)winMan).sendSubscriptionRequest(jid);
    }
    
    /** Returns ID of the map */
    public String getMapID() {
        if (mapView == null) return "";
        return mapView.getMapID();
    }
    
    /** Returns origin ID of the map */
    public String getOriginID() {
        if (mapView == null) return "";
        return mapView.getOriginID();
    }
    
     public void cancelListening() {
         /*if (mapView != null)
             mapView.cancelListening();*/
     }
     
     public void itemStateChanged(ItemEvent itemEvent) {
         if (itemEvent.getStateChange() == ItemEvent.SELECTED) {
            String layerName = (String) itemEvent.getItem();
            if (!dontSetLayer)
                setSelectedLayer(layerName);
        }
     }
     
     
     public void setSelectedLayer(String layerName) {
        if (!dontSetLayer) {
            dontSetLayer = true;
            if (layersComboBox != null)
                layersComboBox.setSelectedItem(layerName);
            if (mapView != null)
                mapView.setSelectedLayer(layerName);
            dontSetLayer = false;
        }
     }
    
     public void setDocked(boolean docked, boolean select) {
         super.setDocked(docked, select);
         if (!docked)
             frame.pack();
     }
     
}