package nu.fw.jeti.plugins.buddyspacemaps.gui;

/*
 * BSClusterLabel.java
 *
 * Project: BuddySpace
 * (C) Copyright Knowledge Media Institute 2002
 *
 *
 * Created on 30 Sempember 2002, 16:06
 * E.S. de Boer
 * converted to jeti plugin
 */

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JLabel;

import nu.fw.jeti.backend.roster.Roster;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.jabber.JIDStatus;
import nu.fw.jeti.jabber.UnknownJIDStatus;
import nu.fw.jeti.plugins.buddyspacemaps.xml.ClusterTag;
import nu.fw.jeti.plugins.buddyspacemaps.xml.ItemTag;



/**
 * <code>BSClusterLabel</code> is a icon label for specified cluster. 
 * It is meant to display presence specific icon and tooltip.
 *
 * @author  Jiri Komzak, Knowledge Media Institute, Open University, United Kingdom
 */
public class BSClusterLabel extends JLabel {
    
    protected ClusterTag cluster;
    private Hashtable presences;
    //private String toolTipText = null;
    
    /** presence icons */
    private static Icon allOfflineIcon = new ImageIcon(BSClusterLabel.class.getResource("map_allOffline.gif"));
    private static Icon allOnlineIcon = new ImageIcon(BSClusterLabel.class.getResource("map_allOnline.gif"));
    private static Icon moreOfflineIcon = new ImageIcon(BSClusterLabel.class.getResource("map_moreOffline.gif"));
    private static Icon moreOnlineIcon = new ImageIcon(BSClusterLabel.class.getResource("map_moreOnline.gif"));
    
    /** Constructor */
    public BSClusterLabel(ClusterTag cluster, Icon img) {
        super(img);
        this.cluster = cluster;
        presences = new Hashtable();
        
        /*String toolTip = "cluster of";
        Enumeration jids = getJIDs();
        while (jids.hasMoreElements()) {
            JID j = (JID) jids.nextElement();
            toolTip += j.toString() + "\n";
        }
        setToolTipText(toolTip);*/
        
        if (cluster != null && cluster.getName() != null)
            setToolTipText(cluster.getName());
        else
            setToolTipText("cluster");
    }
    
    /** Sets label icon */
    public void setImg(Icon img) {
        setIcon(img);
    }
    
    /** Returns all JIDs in cluster */
    public Enumeration getJIDs() {
        Vector jids = new Vector();
        addClusterJIDs(jids, cluster);
        return jids.elements();
    }
    
    // recursivelly adds jids of cluster into the vector
    private void addClusterJIDs(Vector jids, ClusterTag cluster) {
        Enumeration items = cluster.getItemsAndClusters();
        while (items.hasMoreElements()) {
            Object o = items.nextElement();
            if (o instanceof ItemTag)
                jids.add(((ItemTag)o).getJID());
            else if (o instanceof ClusterTag)
                addClusterJIDs(jids, (ClusterTag)o);
        }
    }
    
    /** Refreshes presences and updates label icon */
    public void refreshPresences() {
        presences.clear();
        Enumeration jids = getJIDs();
        while(jids.hasMoreElements()) {
            JID j = (JID) jids.nextElement();
            JIDStatus pi = Roster.getJIDStatus(j);
            //String str = BSPresenceBean.getJIDHashString(j, true);
            if (pi == null) {
                pi = new UnknownJIDStatus(j);
            }
            //presences.put(str, pi);
            presences.put(j, pi);
        }
        
        updateIcon();
    }
    
    /** Sets presence */
    public void setPresence(JIDStatus pi) {
        if (pi == null) return;
        //String str = BSPresenceBean.getJIDHashString(pi.getJID(), true);
        JID key = pi.getJID();
        //if (presences.get(str) != null) {
        if (presences.get(key) != null) {
            //presences.put(str, pi);
            presences.put(key, pi);
        
            updateIcon();
        }
    }
    
    /** Clears all presences */
    public void clearPresences() {
        presences.clear();
        Enumeration jids = getJIDs();
        while(jids.hasMoreElements()) {
            JID j = (JID) jids.nextElement();
            JIDStatus pi = new UnknownJIDStatus(j);
             //String str = BSPresenceBean.getJIDHashString(j, false);
            //presences.put(str, pi);
            presences.put(j, pi);
        }
        
        updateIcon();
    }
    
    /** Updates label icon according to presence */
    private void updateIcon() {
        Enumeration piEnum = presences.elements();
        int online = 0;
        int offline = 0;
        while(piEnum.hasMoreElements()) {
           JIDStatus pi = (JIDStatus) piEnum.nextElement();
            if (pi.isOnline()) online++;
            else offline++;
        }
        
        if (online == 0) {
            setIcon(allOfflineIcon);
        } else if (online < offline) {
            setIcon(moreOfflineIcon);
        } else if (offline != 0) {
            setIcon(moreOnlineIcon);
        } else {
            setIcon(allOnlineIcon);
        }
    }
    
}