package nu.fw.jeti.plugins.buddyspacemaps.gui;

/*
 * BSMapView.java
 *
 * Project: BuddySpace
 * (C) Copyright Knowledge Media Institute 2002
 *
 *
 * Created on 19 August 2002, 9:54
 * 
 * E.S. de Boer
 * converted to jeti plugin
 */

import java.awt.Dimension;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.*;

import nu.fw.jeti.backend.roster.Roster;
import nu.fw.jeti.events.PresenceListener;
import nu.fw.jeti.events.StatusChangeListener;
import nu.fw.jeti.images.StatusIcons;
import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.jabber.JIDStatus;
import nu.fw.jeti.jabber.UnknownJIDStatus;
import nu.fw.jeti.jabber.elements.Presence;
import nu.fw.jeti.plugins.buddyspacemaps.editor.BSJIDLabel;
import nu.fw.jeti.plugins.buddyspacemaps.xml.*;



/**
 * <code>BSMapView</code> is view of map represented by a <code>MapTag</code>.
 *
 * @author  Jiri Komzak, Knowledge Media Institute, Open University, United Kingdom
 */
public class BSMapView extends JScrollPane
                      implements PresenceListener, 
                                 MouseListener, ActionListener {
    
    private Hashtable jidLabels = null;
    private Vector clusterLabels = null;
    private Backend backend;   
    private BSMapWindow mapPanel = null;
    
    private JPanel helpPanel = null;
    private JLayeredPane mainPanel = null;
    private int layerNum = Integer.MAX_VALUE;
    
    
    private ImageIcon clusterIcon = null;
    /*
    private ImageIcon onlineIcon = null;
    private ImageIcon offlineIcon = null;
    private ImageIcon awayIcon = null;
    private ImageIcon xaIcon = null;
    private ImageIcon dndIcon = null;
    private ImageIcon chatIcon = null;
    private ImageIcon busyIcon = null;
    private ImageIcon elseIcon = null;
    private ImageIcon myselfIcon = null;
    */
    private int w = 10, h = 10;   // icons width and height
    
    private JPopupMenu jidPopupMenu;
    private BSJIDActionMenuItem chatMenuItem;
    private BSJIDActionMenuItem msgMenuItem;
    private BSJIDActionMenuItem sendFileMenuItem;
    private BSJIDActionMenuItem subscribeMenuItem;
    private BSJIDActionMenuItem getPlanMenuItem;
    private JMenuItem presenceMenuItem;
    private JPopupMenu clusterPopupMenu;
    
    private String CHAT_MENU_STR = "Chat";
    private String MSG_MENU_STR = "Message";
    private String FILE_MENU_STR = "Send file";
    private String SUBSCRIBE_MENU_STR = "Subscribe to presence";
    private String GET_PLAN_MENU_STR = "Get plan";
    
    private String originID = null;
    private String mapID = null;
    
    private String mapPath;
    
    /** Constructor */
    public BSMapView(BSMapWindow mapPanel, MapTag map, String mapID, String originID,
                     String mapPath,Backend backend) {
        super();
        
        jidLabels = new Hashtable();
        clusterLabels = new Vector();
        
        this.mapPanel = mapPanel;
        this.mapPath = mapPath;
        this.backend = backend;
      
        
        backend.addListener(PresenceListener.class, this);
        backend.addListener(StatusChangeListener.class, new StatusChangeListener()
		{
			/* (non-Javadoc)
			 * @see nu.fw.jeti.events.StatusChangeListener#connectionChanged(boolean)
			 */
			public void connectionChanged(boolean online)
			{
			 refreshNicks();

			}

			/* (non-Javadoc)
			 * @see nu.fw.jeti.events.StatusChangeListener#ownPresenceChanged(int, java.lang.String)
			 */
			public void ownPresenceChanged(int show, String status)
			{
			// TODO Auto-generated method stub

			}

			/* (non-Javadoc)
			 * @see nu.fw.jeti.events.StatusChangeListener#exit()
			 */
			public void exit()
			{
			// TODO Auto-generated method stub

			}
		});
        clusterIcon = new ImageIcon(getClass().getResource("map_cluster.gif"));
       /* 
        onlineIcon = new ImageIcon(ClassLoader.getSystemResource("images/map_online.gif"));
        offlineIcon = new ImageIcon(ClassLoader.getSystemResource("images/map_offline.gif"));
        awayIcon = new ImageIcon(ClassLoader.getSystemResource("images/map_away.gif"));
        lurkerIcon = new ImageIcon(ClassLoader.getSystemResource("images/map_lurker.gif"));
        xaIcon = new ImageIcon(ClassLoader.getSystemResource("images/map_xa.gif"));
        dndIcon = new ImageIcon(ClassLoader.getSystemResource("images/map_dnd.gif"));
        chatIcon = new ImageIcon(ClassLoader.getSystemResource("images/map_chat.gif"));
        busyIcon = new ImageIcon(ClassLoader.getSystemResource("images/map_busy.gif"));
        elseIcon = new ImageIcon(ClassLoader.getSystemResource("images/map_swoosh.gif"));
        myselfIcon = new ImageIcon(ClassLoader.getSystemResource("images/map_myself.gif"));
        */
        jidPopupMenu = new JPopupMenu();
        clusterPopupMenu = new JPopupMenu();
        
        chatMenuItem = new BSJIDActionMenuItem(CHAT_MENU_STR);
        chatMenuItem.addActionListener(this);
        msgMenuItem = new BSJIDActionMenuItem(MSG_MENU_STR);
        msgMenuItem.addActionListener(this);
        sendFileMenuItem = new BSJIDActionMenuItem(FILE_MENU_STR);
        sendFileMenuItem.addActionListener(this);
        subscribeMenuItem = new BSJIDActionMenuItem(SUBSCRIBE_MENU_STR);
        subscribeMenuItem.addActionListener(this);
        getPlanMenuItem = new BSJIDActionMenuItem(GET_PLAN_MENU_STR);
        getPlanMenuItem.addActionListener(this);
        presenceMenuItem = new JMenuItem("");
        
        
        this.mapID = mapID;
        this.originID = originID;
        
        helpPanel = new JPanel();
        //helpPanel.setLayout(new BorderLayout());
        mainPanel = new JLayeredPane();
        mainPanel.setDoubleBuffered(true);
        //mainPanel.setLayout(null);
        
        addMap(map, 0, 0, 1);
        
        refreshPresences();
        helpPanel.add(mainPanel);
        //helpPanel.add(mainPanel, BorderLayout.CENTER);
        setViewportView(helpPanel);
    }
      
    
    /** Returns map id */
    public String getMapID() {
        return mapID;
    }
    
    /** Returns origin id */
    public String getOriginID() {
        return originID;
    }
    
    
    /** Refreshes nicks according to current roster bean */
    public void refreshNicks() {
        // for all jid labels
        Enumeration allLabels = BSMapView.this.jidLabels.elements();
        while (allLabels.hasMoreElements()) {
            BSJIDLabelList labels = (BSJIDLabelList) allLabels.nextElement();
            JID jid = labels.getJID();
            JIDStatus j = Roster.getJIDStatus(jid);
            String nick=null;
            if(j!=null) nick = j.getNick();
            if (nick == null) nick = "?";
            labels.setNick(nick);
        }
    }
    
    
    // *** map view construction functions ***
    
    /** Adds <code>map</code> into the view.
     * This is used for inserting insets during the view construction.
     */
    protected void addMap(MapTag map, int offsetX, int offsetY, float scale) {
        Enumeration layers;
        
        // creates temp vector
        Vector tmpLayers = new Vector();
        layers = map.layers();
        while (layers.hasMoreElements()) {
            tmpLayers.add(layers.nextElement());
        }
        
        // goes through all layers and inserts them according to their priority
        int lastPriority = Integer.MAX_VALUE;
        int curPriority = getNextHighestPriority(tmpLayers, lastPriority);
        while (curPriority != -1) {
            Vector tmp2Layers = (Vector) tmpLayers.clone();
            layers = tmp2Layers.elements();
            while (layers.hasMoreElements()) {
                LayerTag l = (LayerTag) layers.nextElement();
                if (l.getPriorityInt() == curPriority) {
                    addLayer(l, offsetX, offsetY, scale);
                    tmpLayers.remove(l);
                }
            }
            lastPriority = curPriority;
            curPriority = getNextHighestPriority(tmpLayers, lastPriority);
        }
        
        
        
        invalidate();
        repaint();
    }
    
    /*public void repaint() {
        super.repaint();
        if (mapPanel != null)
            mapPanel.repaint();
    }
    
    public void invalidate() {
        super.invalidate();
        if (mapPanel != null)
            mapPanel.invalidate();
    }*/
    
    /**
     * Returns the highest priority in vector lower than lastPriority.
     */
    protected int getNextHighestPriority(Vector layersVector, int lastPriority) {
        int newPriority = -1;
        Enumeration layers = layersVector.elements();
        while (layers.hasMoreElements()) {
            int priority = ((LayerTag) layers.nextElement()).getPriorityInt();
            if (priority < lastPriority && priority > newPriority)
                newPriority = priority;
        }
        return newPriority;
    }
    
    /** Adds <code>layer</code> into the view.
     *  This is used during the view construction.
     */
    protected void addLayer(LayerTag layer, int offsetX, int offsetY, float scale) {
        
        Insets insets = mainPanel.getInsets();
        
        // sets the new scale and offset inside the layer
        float newScale = scale * layer.getScaleFloat();
        int newOffsetX = Math.round(offsetX + layer.getOffsetXInt() * scale);
        int newOffsetY = Math.round(offsetY + layer.getOffsetYInt() * scale);
        
        // adds all the items
        Enumeration items = layer.getItems();
        while (items.hasMoreElements()) {
            Object o = items.nextElement();
            // if it's item
            if (o instanceof ItemTag) {
                ItemTag i = (ItemTag) o;
                JID jid = i.getJID();
                if (jid == null) break;
                String nick = null;
                JIDStatus j = Roster.getJIDStatus(jid);
                if (j != null) nick = j.getNick();
                if (nick == null) nick = "?";
                
                BSJIDLabel jidLabel = new BSJIDLabel(jid, nick, getIconForPresence(0));
                jidLabel.setOpaque(false);
                Icon onlineIcon = getIconForPresence(Presence.AVAILABLE);
                if (onlineIcon != null) {
                    w = onlineIcon.getIconWidth();
                    h = onlineIcon.getIconHeight();
                }
                int x = Math.round(i.getLonFloat()*newScale + newOffsetX - w/2);
                int y = Math.round(i.getLatFloat()*newScale + newOffsetY - h/2);
                //mainPanel.add(jidLabel, new AbsoluteConstraints(x, y, w, h));
                //mainPanel.add(jidLabel, JLayeredPane.DEFAULT_LAYER, layerNum);
                mainPanel.add(jidLabel, new Integer(layerNum));
                jidLabel.setBounds(x + insets.left, y + insets.top, w, h);
                jidLabel.addMouseListener(this);
                registerJIDLabel(jidLabel);
            }
            // else if it's cluster
            else if (o instanceof ClusterTag) {
                ClusterTag c = (ClusterTag) o;
                BSClusterLabel clusterLabel = new BSClusterLabel(c, clusterIcon);
                clusterLabel.setOpaque(false);
                Icon onlineIcon = getIconForPresence(Presence.AVAILABLE);
                if (onlineIcon != null) {
                    w = onlineIcon.getIconWidth();
                    h = onlineIcon.getIconHeight();
                }
                int x = Math.round(c.getLonFloat()*newScale + newOffsetX - w/2);
                int y = Math.round(c.getLatFloat()*newScale + newOffsetY - h/2);
                //mainPanel.add(jidLabel, new AbsoluteConstraints(x, y, w, h));
                //mainPanel.add(clusterLabel, JLayeredPane.DEFAULT_LAYER, layerNum);
                mainPanel.add(clusterLabel, new Integer(layerNum));
                clusterLabel.setBounds(x + insets.left, y + insets.top, w, h);
                clusterLabel.addMouseListener(this);
                registerClusterLabel(clusterLabel);
            }
        }
        layerNum--;
        
        String completeMapPath = new String(mapPath + "/" + originID);
        // loads image of map
        ImgTag imgTag = layer.getImg();
        if (imgTag != null) {
            //Image image = getImage(ClassLoader.getSystemResource("images/" + imgTag.getSrc()));
            /*Image image = Toolkit.getDefaultToolkit().getImage(
                            ClassLoader.getSystemResource(completeMapPath + "/" + imgTag.getSrc()));*/
            Image image = Toolkit.getDefaultToolkit().getImage(completeMapPath + "/" + imgTag.getSrc());
            ImagePanel imgPanel = new ImagePanel(image, newScale, mainPanel, imgTag.getWidthInt(), imgTag.getHeightInt());
            //mainPanel.add(imgPanel, JLayeredPane.DEFAULT_LAYER, layerNum);
            mainPanel.add(imgPanel, new Integer(layerNum));
            imgPanel.setBounds(newOffsetX + insets.left, newOffsetY + insets.top,
                               imgPanel.getWidth(), imgPanel.getHeight());
            imgPanel.maybeResizeParent();
            Dimension d = mainPanel.getMinimumSize();
            setMinimumSize(new Dimension(d.width + 30, d.height + 30));
        }
        layerNum--;
        
        // loads map given in layer's src attribute (when the map is only referenced)
        MapTag map = layer.getSrcMap();
        if (map != null) {
            addMap(map, newOffsetX, newOffsetY, newScale);
        }
    }
    
    /** Registeres jid label to receive presence changes */
    protected void registerJIDLabel(BSJIDLabel lbl) {
        JID jid = lbl.getJID();
        if (jid == null) return;
        
        //String str = BSPresenceBean.getJIDHashString(jid, true);
        //BSJIDLabelList labels = (BSJIDLabelList) jidLabels.get(str);
        BSJIDLabelList labels = (BSJIDLabelList) jidLabels.get(jid);
        
        if (labels == null) {
            labels = new BSJIDLabelList(jid);
            //jidLabels.put(str, labels);
            jidLabels.put(jid, labels);
        }
        
        labels.addLabel(lbl);
    }
    
    /** Registers cluster label to receive presence changes */
    protected void registerClusterLabel(BSClusterLabel lbl) {
        clusterLabels.add(lbl);
    }
    
    /** Refreshes presences and icon of labels */
    protected void refreshPresences() {
                
        // for all jid labels
        Enumeration labelLists = jidLabels.elements();
        while (labelLists.hasMoreElements()) {
            BSJIDLabelList labels = (BSJIDLabelList) labelLists.nextElement();
            JID jid = labels.getJID();
            // *** what about people who are not in roster !!!
            //BSPresenceInfo pi = presenceBean.getResourcePresence(jid);
            JIDStatus j = Roster.getJIDStatus(jid);
            int show=0;
            if(j!=null) show = j.getShow();
            else show = Presence.NONE;
            Icon img = getIconForPresence(show);
            labels.setImg(img);
        }
        
        // for all cluster labels
        Enumeration clusterEnum = clusterLabels.elements();
        while (clusterEnum.hasMoreElements()) {
            BSClusterLabel label = (BSClusterLabel) clusterEnum.nextElement();
            label.refreshPresences();
        }
    }
    
    
    /** Returns icon for given presence */
    protected Icon getIconForPresence(int pi) {
        //if (pi == null)  return StatusIcons.getOfflineIcon();
        return StatusIcons.getStatusIcon(pi);
        
        /*
        else if (pi.isMyself())
            return myselfIcon;
        else if (!pi.isOnline())
            return offlineIcon;
        
        else if (pi.getShow() == null ||
                 BSPresenceInfo.SHOW_ONLINE.equals(pi.getShow())) {
            if (BSMainFrame.STATUS_BUSY_STR.equals(pi.getStatus()))
                return busyIcon;
            else if (BSMainFrame.STATUS_ELSE_STR.equals(pi.getStatus()))
                return elseIcon;
            else
                return onlineIcon;
        }
        
        else if (BSPresenceInfo.SHOW_CHAT.equals(pi.getShow()))
            return chatIcon;
        else if (BSPresenceInfo.SHOW_AWAY.equals(pi.getShow()))
            return awayIcon;
        else if (BSPresenceInfo.SHOW_XA.equals(pi.getShow()))
            return xaIcon;
        else if (BSPresenceInfo.SHOW_DND.equals(pi.getShow()))
            return dndIcon;
        else
            return lurkerIcon;
            */
    }
    
    
    /** Opens chat if clicked on jid label */
    private void maybeOpenChatWindow(MouseEvent e) {
        Object src = e.getSource();
        if (src instanceof BSJIDLabel)
            mapPanel.openChat(((BSJIDLabel)src).getJID());
    }
    
    /** Shows popup menu if supposed to */
    private void maybeShowPopup(MouseEvent e) {
        // if that was context popup event
        if (e.isPopupTrigger()) {
        //if (e.getClickCount() == 1 && SwingUtilities.isRightMouseButton(e)) {
            // gets the clicked label
            Object src = e.getSource();
            
            boolean connected = backend.isLoggedIn();
            
            // if we clicked on the group node
            if (src instanceof BSJIDLabel) {
                JID jid = ((BSJIDLabel)src).getJID();
                jidPopupMenu.removeAll();
                
                if (connected) {
                    chatMenuItem.setJID(jid);
                    jidPopupMenu.add(chatMenuItem);

                    msgMenuItem.setJID(jid);
                    jidPopupMenu.add(msgMenuItem);

                    //sendFileMenuItem.setJID(((BSJIDLabel)src).getJID());
                    //jidPopupMenu.add(sendFileMenuItem);

                    subscribeMenuItem.setJID(jid);
                    jidPopupMenu.add(subscribeMenuItem);

                    if (mapPanel.supportsPlans()) {
                        getPlanMenuItem.setJID(jid);
                        jidPopupMenu.add(getPlanMenuItem);
                    }

                    jidPopupMenu.add(new JSeparator());
                } //if connected
                
                //BSPresenceInfo pi = presenceBean.getResourcePresence(jid);
                //if (pi == null) pi = new BSPresenceInfo(jid, false, null, null);
                JIDStatus j = Roster.getJIDStatus(jid);
                int show=0;
                String status=null;
                String nick=null;
                if(j!=null)
                {
                	show = j.getShow();
                	status = j.getStatus();
                	nick = j.getNick();
                }
                else show = Presence.NONE;
                Icon img = getIconForPresence(show);
                presenceMenuItem.setIcon(img);
               
                if (status == null) status = "";
                if(nick==null)nick ="?";
                presenceMenuItem.setText(nick + " - " + status);
                jidPopupMenu.add(presenceMenuItem);
                
                jidPopupMenu.show(e.getComponent(), e.getX(), e.getY());
                //adjustPopupPosition(jidPopupMenu);
            }
            
            // else if we clicked on cluster label
            else if (src instanceof BSClusterLabel) {
                
                clusterPopupMenu.removeAll();
                Enumeration jids = ((BSClusterLabel)src).getJIDs();
                
                // for all jid in cluster
                while (jids.hasMoreElements()) {
                    JID j = (JID) jids.nextElement();
                    //BSPresenceInfo pi = presenceBean.getResourcePresence(j);
                    
                    JIDStatus js = Roster.getJIDStatus(j);
                    int show=0;
                    String nick=null;
                    if(js!=null)
                    {
                    	show = js.getShow();
                    	nick = js.getNick();
                    }
                    else show = Presence.NONE;
                    Icon img = getIconForPresence(show);
                    if(nick==null)nick ="?";
                                     
                    JMenu jidSubmenu = new JMenu(nick + " (" + j.toString() + ")");
                    jidSubmenu.setIcon(img);
                    clusterPopupMenu.add(jidSubmenu);
                    
                    if (connected) {
                        BSJIDActionMenuItem mi;
                        mi = new BSJIDActionMenuItem(CHAT_MENU_STR, j);
                        mi.addActionListener(new ActionListener() {
                            public void actionPerformed(ActionEvent e) {
                                mapPanel.openChat(((BSJIDActionMenuItem)e.getSource()).getJID());
                            }
                        });
                        jidSubmenu.add(mi);

                        mi = new BSJIDActionMenuItem(MSG_MENU_STR, j);
                        mi.addActionListener(new ActionListener() {
                            public void actionPerformed(ActionEvent e) {
                                mapPanel.sendMessage(((BSJIDActionMenuItem)e.getSource()).getJID());
                            }
                        });
                        jidSubmenu.add(mi);

                        mi = new BSJIDActionMenuItem(SUBSCRIBE_MENU_STR, j);
                        mi.addActionListener(new ActionListener() {
                            public void actionPerformed(ActionEvent e) {
                                mapPanel.sendSubscriptionRequest(((BSJIDActionMenuItem)e.getSource()).getJID());
                            }
                        });
                        jidSubmenu.add(mi);
/*
                        if (mapPanel.supportsPlans()) {
                            mi = new BSJIDActionMenuItem(GET_PLAN_MENU_STR, j);
                            mi.addActionListener(new ActionListener() {
                                public void actionPerformed(ActionEvent e) {
                                    mapPanel.getPlan(((BSJIDActionMenuItem)e.getSource()).getJID());
                                }
                            });
                            jidSubmenu.add(mi);
                        }
                        */

                        jidSubmenu.add(new JSeparator());
                    } // if connected
                    
                    if (js == null) js = new UnknownJIDStatus(j);
                    String status = js.getStatus();
                    if (status == null) status = "";
                    JMenuItem presenceMI = new JMenuItem(Presence.toLongShow(js.getShow()) + " - " + status);
                    presenceMI.setIcon(img);
                    jidSubmenu.add(presenceMI);
                } //while
                
                clusterPopupMenu.show(e.getComponent(), e.getX(), e.getY());
                //adjustPopupPosition(clusterPopupMenu);
                
            } // else(cluster)
        } //if(popup)
    }
    
    
    // *** presence listener stuff ***
    
    /** <code>BSPresenceListener</code> method. */
    public void presenceChanged(Presence pi) {
        if (pi == null || pi.getFrom() == null)
            return;
        /* TODO add best resource check
        JID jid = pi.getFrom();
        // sets resource-specific presence
        if (jid.getResource() != null && jid.getResource() != "") {
            pi = presenceBean.getResourcePresence(jid);
            if (pi == null)
                pi = new BSPresenceInfo(jid, false, null, null);
            updatePresence(pi);
        }
        
        // sets best presence for JID without resource
        //JID jidWORes = BSPresenceBean.getJidWithoutRes(jid);
        JID jidWORes = jid.getSimpleJID();
        pi = presenceBean.getResourcePresence(jidWORes);
        if (pi == null)
            pi = new BSPresenceInfo(jidWORes, false, null, null);
        else {
            boolean isMyself = pi.isMyself();
            pi = new BSPresenceInfo(jidWORes, pi.isOnline(), pi.getShow(), 
                                    pi.getStatus());
            pi.setIsMyself(isMyself);
        }
        */
        updatePresence(pi);
    }
    
    /** Sets given presence to clusters and labels */
    protected void updatePresence(Presence pi) {
        if (pi == null || pi.getFrom() == null)
            return;
        
        // for all clusters
        Enumeration clusterEnum = clusterLabels.elements();
        while (clusterEnum.hasMoreElements()) {
            BSClusterLabel label = (BSClusterLabel) clusterEnum.nextElement();
            // sets presence and updates icon
            UnknownJIDStatus s= new UnknownJIDStatus(pi.getFrom());
            s.updatePresence(pi);
            label.setPresence(s);
        }
        
        // for jid labels for given jid
        //String str = BSPresenceBean.getJIDHashString(pi.getJID(), true);
        JID key = pi.getFrom();
        //BSJIDLabelList labels = (BSJIDLabelList) jidLabels.get(str);
        BSJIDLabelList labels = (BSJIDLabelList) jidLabels.get(key);
        if (labels == null)
            return;
        // sets the right icon
        Icon img = getIconForPresence(pi.getShow());
        labels.setImg(img);
    }
    
    /** <code>BSPresenceListener</code> method. */
    public void presencesCleared() {
        // for all clusters
        Enumeration clusterEnum = clusterLabels.elements();
        while (clusterEnum.hasMoreElements()) {
            BSClusterLabel label = (BSClusterLabel) clusterEnum.nextElement();
            // sets presence and updates icon
            label.clearPresences();
        }
        
        // for all jid labels
        Enumeration allLabels = jidLabels.elements();
        while (allLabels.hasMoreElements()) {
            BSJIDLabelList labels = (BSJIDLabelList) allLabels.nextElement();
            if (labels == null)
                return;
            // sets the right icon
            Icon img = getIconForPresence(Presence.NONE);
            labels.setImg(img);
        }
    }
    
    /** Empty */
    public void subscriptionApproved(JID jid) { }
    /** Empty */
    public void subscriptionRequested(JID jid) { }
    
    /** <code>MouseListener</code> method - catches double-clicks */
    public void mouseClicked(MouseEvent e) {
        // handles only double clicks on JID nodes
        if ( e.getClickCount() == 2 ) {
            maybeOpenChatWindow(e);
            //tree.repaint();
        }
    }
    
    /** <code>MouseListener</code> method - catches popup-menu clicks */
    public void mousePressed(MouseEvent e) {
        maybeShowPopup(e);
    }
    
    /** <code>MouseListener</code> method - catches popup-menu clicks */
    public void mouseReleased(MouseEvent e) {
        maybeShowPopup(e);
    }
    
    /** <code>MouseListener</code> method - empty implementation */
    public void mouseEntered(MouseEvent e) { }
    /** <code>MouseListener</code> method - empty implementation */
    public void mouseExited(MouseEvent e) { }
    
    /** Handles actions from GUI controls */
    public void actionPerformed(ActionEvent e) {
        Object src = e.getSource();
        
        // opens chat
        if (src == chatMenuItem) {
            JID j = chatMenuItem.getJID();
            if (j != null)
                mapPanel.openChat(j);
        }
        
        // sends message
        if (src == msgMenuItem) {
            JID j = msgMenuItem.getJID();
            if (j != null)
                mapPanel.sendMessage(j);
        }
        /*
        // subscribes for presence
        else if (src == subscribeMenuItem) {
            JID j = subscribeMenuItem.getJID();
            if (j != null)
                mapPanel.sendSubscriptionRequest(j);
        }
        */
      
        
        // a resource must be specified !!!
        /*else if (src == sendFileMenuItem) {
            JID j = chatMenuItem.getJID();
            if (j != null)
                mapPanel.sendFile(j);
        }*/
    }
    
    
    public void cancelListening() {
        backend.removeListener(PresenceListener.class,this);
    }
    
}