package nu.fw.jeti.plugins.buddyspacemaps.gui;

/*
 * BSMapWinManager.java
 *
 * Project: BuddySpace
 * (C) Copyright Knowledge Media Institute 2002
 *
 *
 * Created on 31 October 2002, 10:39
 * 
 * E.S. de Boer
 * converted to jeti plugin
 */

import java.awt.Image;
import java.awt.Toolkit;

import javax.swing.JTabbedPane;

import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.plugins.buddyspacemaps.core.BSMapBean;
import nu.fw.jeti.plugins.buddyspacemaps.core.BSMapListener;
import nu.fw.jeti.plugins.buddyspacemaps.editor.BSMapEditWindow;
import nu.fw.jeti.plugins.buddyspacemaps.xml.MapTag;
import nu.fw.jeti.ui.SendMessage;
import nu.fw.jeti.util.Popups;





/**
 * <code>BSMapWinManager</code> manages map windows.
 * It relies on <code>BSMapBean</code> and forwards events to particular
 * map windows.
 *
 * @author  Jiri Komzak, Knowledge Media Institute, Open University, United Kingdom
 */
public class BSMapWinManager extends WinManager
                             implements BSMapListener {
                                 
    protected BSMapBean mapBean = null;
  // protected BSPresenceBean presenceBean = null;
    
    
   // public BSMainFrame mainFrame;
    protected String mapPath;
    protected Image mapImage = null;
    private Backend backend;
    
    // wheter the support for .plan (BSPlansPlugin) is present
    public boolean supportPlans = false;
    
    /** Constructor */
    public BSMapWinManager(Backend backend, String mapPath) {
                               
        super(new JTabbedPane());
        this.backend = backend;
        this.mapPath = mapPath;
        mapImage = Toolkit.getDefaultToolkit().getImage(getClass().getResource("map.gif"));
    }
    
    /** Sets map bean */
    public void setMapBean(BSMapBean mapBean) {
        if (this.mapBean != null)
            this.mapBean.removeMapListener(this);
        this.mapBean = mapBean;
        if (mapBean != null)
            mapBean.addMapListener(this);
    }
       
    /** Sets wether support for plans is present */
    public void setSupportPlans(boolean present) {
        supportPlans = present;
    }
    /*
    /** Returns main roster elements *
    public Enumeration getRoster() {
        if (rosterBean != null)
            return rosterBean.entries();
        else {
            return new Vector().elements();
        }
    }
    */
    
    /** Opens given map */
    public boolean openMap(JID jid) {
        if (jid == null) return false;
        String mapName = jid.getResource();
        String originID = ((jid.getUser()!=null)? (jid.getUser() + "@") : "")
                         + jid.getDomain();
        return openMap(mapName, originID);
    }
    
    
    /** Opens given map */
    public boolean openMap(String mapName, String originID) {
        if (mapName == null || "".equals(mapName) || "".equals(originID)) {
        	Popups.errorPopup("Invalid map identification.",
                    "Error");
            return false;
        }
        mapBean.loadMap(mapName, originID);
        return true;
    }
    
    
    /** Selects window with given map */
    public void selectMap(String completeMapID) {
        if (completeMapID == null) return;
        
        DockableWindow win = (DockableWindow) getWindow(completeMapID);
        if (win == null) return;
        
        selectWindow(win);
    }
    
    /** Called when map received. Opens a new window for it. */
    public void mapReceived(MapTag map, String originID) {
        if (map == null || originID == null) return;
        
        String mapID = map.getID();
        //String completeMapID = new String(originID + "/" + mapID);
        String windowTitle = originID;//BSMapPubsubCore.getFriendlyOrigin(originID);
        windowTitle += "/" + mapID.substring(0, mapID.length()-4);
        boolean dock =false; // mainFrame.isDockingWindows();
        BSMapWindow win = new BSMapWindow(this, windowTitle, mapImage, 
                                map, mapID, originID, mapPath, dock,backend);
        addWindow(win, dock);
        showWindow(win, true, true);
        selectWindow(win);
        win.repaint();
    }
    
    /** Starts editting of given map. 
     *  Closes normal map window and opens editting one.
     */
    public void edit(DockableWindow win) {
        if (win == null || (win instanceof BSMapEditWindow)) return;
        
        String mapID = ((BSMapWindow)win).getMapID();
        String originID = ((BSMapWindow)win).getOriginID();
        String completeMapID = new String(originID + "/" + mapID);
        boolean dock = win.isDocked();
        MapTag map = mapBean.getMap(originID, mapID);
        
        closeWindow(win);
        BSMapEditWindow newWin = new BSMapEditWindow(this, completeMapID, mapImage, 
                                map, mapID, originID, mapPath,dock);
       // newWin.setPresenceBean(presenceBean);
        addWindow(newWin, dock);
        showWindow(newWin, true, true);
        selectWindow(newWin);
        
        newWin.repaint();
    }
    
    /** Sends the map to the JID */
    public void sendMap(String mapID, String originID, JID jid, String subject, String body) {
        if (mapBean != null) {
            if (!mapBean.sendMap(mapID, originID, jid, subject, body))
                Popups.errorPopup("Error occured while sending map",
                                              "Error");
        }
        else 
        	Popups.errorPopup("Cannot send map - no connected","Error");
        
    }
    
    /** Saves the map into file with given filename */
    public void saveMap(String mapID, String originID, 
                        String newMapID, String newOriginID) {
                            
        if (mapBean != null) {
            if (!mapBean.saveMap(mapID, originID, newMapID, newOriginID))
            	Popups.errorPopup("Error occured while saving map","Error");
        }
        else 
        	Popups.errorPopup("Cannot save map","Error");
        
        String completeMapID = new String(originID + "/" + mapID);
        DockableWindow win = (DockableWindow) openWindows.get(completeMapID);
        if (win != null) {
            openWindows.remove(completeMapID);
            completeMapID = new String(newOriginID + "/" + newMapID);
            openWindows.put(completeMapID, win);
        }
    }
        
    /** Opens chat with the JID */
    public void openChat(JID jid) {
        backend.getMain().startChat(jid);
    
    }
    
    /** Sends message to JID */
    public void sendMessage(JID jid) {
    	new SendMessage(backend,jid,jid.toString()).setVisible(true);
    }
    
    /** Sends file to the jid */
    public void sendFile(JID jid) {
//        if (mainFrame != null) {
//            mainFrame.sendFile(jid);
//        }
    }
    
    /** Sends subscription request to the jid */
    public void sendSubscriptionRequest(JID jid) {
//        if (mainFrame != null) {
//            mainFrame.sendSubscriptionRequest(jid);
//        }
    }
       
    
    /** Returns the <code>MapTag</code> obtained from <code>MapBean</code> */
    public MapTag getMap(String originID, String mapID) {
        if (mapBean == null) return null;
        return mapBean.getMap(originID, mapID);
    }
    
    /** Copies all necessary files for given map into specified dir. */
    public boolean copyFilesForMap(MapTag map, String sourceOriginID, 
                                               String destOriginID) {
        if (mapBean == null) return false;
        return mapBean.copyFilesForMap(map, sourceOriginID, destOriginID);
    }
    
//    public boolean addBookmark(String originID, String mapID) {
//        if (mainFrame == null || originID == null || mapID == null) return false;
//        JID jid = JID.fromString(originID + "/" + mapID);
//        if (jid == null) return false;
//        /*return mainFrame.addBookmark(jid, "http://jabber.open.ac.uk/tags/map", 
//                                     "Map " + jid.getResource() + " from " +
//                                     (new JID(jid.getUsername(), jid.getServer(), null)).toString());*/
//        return mainFrame.addBookmark(jid, "http://jabber.open.ac.uk/tags/map", 
//                                     "Map " + mapID.substring(0,mapID.length()-4) + " from " +
//                                     BSMapPubsubCore.getFriendlyOrigin(originID));
    //}
    
    /*
    /** Opens windows specified in vector. *
    public void openTheWindows(Vector openWindows, boolean connected) { 
        if (openWindows == null) return;
        if (connected) return;
        
        Enumeration owEnum = openWindows.elements();
        while (owEnum.hasMoreElements()) {
            BSOpenWindows.BSOpenWindow ow = (BSOpenWindows.BSOpenWindow) owEnum.nextElement();
            if ("http://jabber.open.ac.uk/tags/map".equals(ow.namespace) && ow.jid != null) {
                BSOpenWindows.adjustWindowToScreen(ow);
                openMap(ow.jid);
                String mapName = ow.jid.getResource();
                String originID = ((ow.jid.getUsername()!=null)? (ow.jid.getUsername() + "@") : "")
                                 + ow.jid.getServer();
                DockableWindow win = (DockableWindow) getWindow(originID + "/" + mapName);
                if (win != null) {
                    win.setFrameSize(ow.width, ow.height);
                    win.setFrameLocation(ow.x, ow.y);
                    setWindowDocked(win, ow.docked);
                }
            } //if
        } //while
    }
    
    
    /** Add its open windows into the vector (for opening when starting next time). *
    public void addOpenWindows(Vector openWindows, boolean connected) { 
        if (openWindows == null) return;
        if (connected) return;

        Enumeration owEnum = this.openWindows.elements();
        while (owEnum.hasMoreElements()) {
            BSMapWindow win = (BSMapWindow) owEnum.nextElement();
            JID jid = JID.fromString(win.getOriginID() + "/" + win.getMapID());
            if (jid != null) {
                BSOpenWindows.BSOpenWindow ow = new BSOpenWindows.BSOpenWindow(jid, 
                                        "http://jabber.open.ac.uk/tags/map", 
                                        win.getFrameX(), win.getFrameY(), 
                                        win.getFrameWidth(), win.getFrameHeight(), 
                                        win.isDocked());
                openWindows.add(ow);
            }
        } //while
    }
    */
    
}