package nu.fw.jeti.plugins.buddyspacemaps.gui;

/*
 * BSMapWindow.java
 *
 * Project: BuddySpace
 * (C) Copyright Knowledge Media Institute 2002
 *
 *
 * Created on 31 October 2002, 10:15
 * 
 * E.S. de Boer
 * converted to jeti plugin
 */

//import java.util.*;
import java.awt.BorderLayout;
import java.awt.Image;
import java.awt.Window;
import java.awt.event.ActionListener;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;

import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.plugins.buddyspacemaps.xml.MapTag;



/**
 * <code>BSMapWindow</code> provides map window GUI. 
 * It contains <code>BSMapView</code> which is the actual map view and several
 * control components. All communication is done through 
 * <code>BSMapWinManager</code>.
 *
 * @author  Jiri Komzak, Knowledge Media Institute, Open University, United Kingdom
 */
public class BSMapWindow extends DockableWindow 
                         implements ActionListener {
                          
    private JPanel mainPanel;
    
    private BSMapView mapView = null;
    //private JPanel mapPanel = null;
    //private JPanel helpPanel = null;
    //private JScrollPane mapScrollPane = null;
    private JPanel buttonPanel = null;
    
    private JButton closeButton = null;
    private JButton sendButton = null;

    private JButton dockButton;
    private JButton bookmarkButton;
    private JButton editButton;
    private JPanel checkBoxesPanel;
    
    /** Constructor */
    BSMapWindow( BSMapWinManager winMan, String title,
                Image icon, MapTag map, String mapID, String originID,
               String mapPath, boolean docked,Backend backend) {
                   
        super(originID + "/" + mapID, title, icon, docked, winMan);
        
        mapView = new BSMapView(this, map, mapID, originID, mapPath,backend);
        
        mainPanel = new JPanel(new BorderLayout());
        mainPanel.add(mapView, BorderLayout.CENTER);
        
        addButtons();
        
        if (!docked)
        {
            frame.pack();
            frame.setLocationRelativeTo(null);
        }
    }
    
    /** Adds buttons */
    protected void addButtons() {
        buttonPanel = new JPanel();
        
        sendButton = new JButton();
        sendButton.setText("Send locations");
        sendButton.addActionListener(this);
        buttonPanel.add(sendButton);
        
        closeButton = new JButton();
        closeButton.setText("Close");
        closeButton.addActionListener(this);
        buttonPanel.add(closeButton);
        
        JPanel checkBoxesPanel = new JPanel();
        Icon icon = new ImageIcon(getClass().getResource(!docked?
                                "dock.gif" : "float.gif"));
        dockButton = new JButton(icon);
        dockButton.setToolTipText(docked? "Float" : "Dock");
        dockButton.addActionListener(this);
        checkBoxesPanel.add(dockButton);
      
        icon = new ImageIcon(getClass().getResource("bookmark.gif"));
        bookmarkButton = new JButton(icon);
        bookmarkButton.setToolTipText("Add bookmark");
        bookmarkButton.addActionListener(this);
        checkBoxesPanel.add(bookmarkButton);
        editButton = new JButton("Edit");
        editButton.addActionListener(this);
        checkBoxesPanel.add(editButton);
        
        mainPanel.add(checkBoxesPanel, BorderLayout.NORTH);
        mainPanel.add(buttonPanel, BorderLayout.SOUTH);
        
        setLayout(new BorderLayout());
        add(mainPanel, BorderLayout.CENTER);
    }
    
    
    /** Handles actions from GUI controls */
    public void actionPerformed(java.awt.event.ActionEvent evt) {
        if (evt.getSource() == closeButton) {
            winMan.closeWindow(this);
        }
        else if (evt.getSource() == sendButton) {
        	
            BSMapSendDialog dlg = new BSMapSendDialog(frame);
            dlg.setVisible(true);
            if (dlg.jid != null)
                sendMap(dlg.jid, dlg.subject, dlg.body);
            
        }
        else if (evt.getSource() == dockButton) {
            if (winMan != null) {
                winMan.setWindowDocked(this, !docked);
                Icon icon = new ImageIcon(getClass().getResource(!docked?
                        "dock.gif" : "float.gif"));
                dockButton.setIcon(icon);
                dockButton.setToolTipText(docked? "Float" : "Dock");
                winMan.selectWindow(this);
            }
        }
        else if (evt.getSource() == editButton) {
            if (winMan != null) {
                ((BSMapWinManager)winMan).edit(this);
            }
        }
        /*
        else if (evt.getSource() == bookmarkButton) {
            if (winMan != null) {
                ((BSMapWinManager)winMan).addBookmark(mapView.getOriginID(), mapView.getMapID());
            }
        }
        */
    }
    
    /** Sends the map to given JID. Calls BSMapWinManager method. */
    public void sendMap(JID jid, String subject, String body) {
        ((BSMapWinManager)winMan).sendMap(mapView.getMapID(), mapView.getOriginID(), jid, subject, body);
    }
      
    
    /** Opens chat with given JID. Calls BSMapWinManager method. */
    public void openChat(JID jid) {
        ((BSMapWinManager)winMan).openChat(jid);
    }
    
    /** Sends message. Calls BSMapWinManager method. */
    public void sendMessage(JID jid) {
        ((BSMapWinManager)winMan).sendMessage(jid);
    }
    
    /** Sends file to JID. Calls BSMapWinManager method. */
    public void sendFile(JID jid) {
        ((BSMapWinManager)winMan).sendFile(jid);
    }
    
    /** Sends subscription request to JID. Calls BSMapWinManager method. */
    public void sendSubscriptionRequest(JID jid) {
        ((BSMapWinManager)winMan).sendSubscriptionRequest(jid);
    }
    
  
    
    /** Returns ID of the map */
    public String getMapID() {
        if (mapView == null) return "";
        return mapView.getMapID();
    }
    
    /** Returns origin ID of the map */
    public String getOriginID() {
        if (mapView == null) return "";
        return mapView.getOriginID();
    }
    
    /** Returns if the .plan support is present */
    public boolean supportsPlans() {
        return ((BSMapWinManager)winMan).supportPlans;
    }
    
     public void cancelListening() {
         if (mapView != null)
             mapView.cancelListening();
     }
     
     
     public void setDocked(boolean docked, boolean select) {
         super.setDocked(docked, select);
         if (!docked) {
             //setPreferredSize(mapView.getPreferredSize());
             doLayout();
             setPreferredSize(getMinimumSize());
             frame.pack();
         }
     }
          
}