/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: jeti $
 * @version $Revision: 1.1 $ 
 *
 * j.komzak
 * Changed into ClusterTagHandler
 * 
 * E.S. de Boer
 * converted to jeti plugin
 */

package nu.fw.jeti.plugins.buddyspacemaps.xml;

/*
 * ClusterTagHandler.java
 *
 * Project: BuddySpace
 * (C) Copyright Knowledge Media Institute 2002
 *
 *
 * Created on 30 September 2002, 11:25
 */

import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.jabber.elements.Extension;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

/**
 * Handler class for &lt;cluster&gt tag.
 *
 * @author  Jiri Komzak, Knowledge Media Institute, Open University, United Kingdom
 */
public class ClusterTagHandler
{
    /** builder to construct the product extension object */
    ClusterTagBuilder builder;
    ItemTagBuilder itemBuilder;
    ClusterTagHandler clusterTagHandler;

    /**
     * Creates a new <code>ClusterTagHandler</code> instance.
     */
    public ClusterTagHandler() 
    {
	     builder = new ClusterTagBuilder();
        itemBuilder = new ItemTagBuilder();
    }


    /**
     * Gets called when the underlying engine decides to pass an entity and
     * all sub-entities off to your subhandler.<p>
     *
     * Upon seeing the element that this subhandler handles, we call this
     * constructor, passing in the attributes.
     *
     * @param name name of the element which we are handling.
     * @param attributes list of attributes on this element
     */
    protected final void startHandler(String name,Attributes attributes)
	{
	  	builder.reset();
        
        builder.setLat(attributes.getValue("lat"));
        builder.setLon(attributes.getValue("lon"));
        builder.setSize(attributes.getValue("size"));
        builder.setName(attributes.getValue("name"));
    }
        
    /**
     * <code>handleStartElement</code> is overloaded by the new class to
     * provide logic to handle the element code.
     *
     * @param name a <code>String</code> value
     * @param attributes an <code>AttributeList</code> value
     * @exception SAXException if an error occurs
     */
    protected final void handleStartElement(String name, 
				      Attributes attributes)
	{
    	if(clusterTagHandler!=null)
    	{
    		clusterTagHandler.handleStartElement(name, attributes);
    	}
    	else if (name.equals("item")) {
            itemBuilder.reset();
            itemBuilder.setJID(JID.jidFromString(attributes.getValue("jid")));
            itemBuilder.setLat(attributes.getValue("lat"));
            itemBuilder.setLon(attributes.getValue("lon"));
        }
        else if (name.equals("cluster")) 
       	{
        	clusterTagHandler= new ClusterTagHandler();
        	clusterTagHandler.startHandler(name, attributes);
       	}
    }
    
    public boolean isInCluster()
    {
    	return clusterTagHandler!=null;
    }
    
    /**
     * <code>handleEndElement</code> is overloaded by the new class to
     * provide logic to handle element code.
     *
     * @param name a <code>String</code> value
     * @exception SAXException if an error occurs
     */
    protected final void handleEndElement(String name,String data)
    {
    	if (name.equals("cluster")) {
    		if(clusterTagHandler.isInCluster())clusterTagHandler.handleEndElement(name,data);
    		else
    		{
	    		builder.addCluster((ClusterTag)clusterTagHandler.build());
	    		clusterTagHandler=null;
    		}
    	}
    	else if(clusterTagHandler!=null)
    	{
    		clusterTagHandler.handleEndElement(name,data);
    	}
    	else if (name.equals("item")) {
            //itemBuilder.setInfo(data);
          
                builder.addItem((ItemTag)itemBuilder.build());
         
        }
    }
    
    /**
     * Stophandler is the same as end element, except that it is called saying
     * that the subhandler is no longer in scope.
     *
     * @param Object a value being returned to the parent - the parent is 
     * meant to interpret this result.
     */
    protected final Extension build()
	{
	    return builder.build();
	
    }
  
}

