/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: jeti $
 * @version $Revision: 1.1 $
 *
 * j.komzak
 * Changed into ImgTag
 * 
 * E.S. de Boer
 * converted to jeti plugin
 */

package nu.fw.jeti.plugins.buddyspacemaps.xml;

/*
 * ImgTag.java
 *
 * Project: BuddySpace
 * (C) Copyright Knowledge Media Institute 2002
 *
 *
 * Created on 16 August 2002, 10:20
 */

import nu.fw.jeti.jabber.elements.Extension;

/**
 * <code>ImgTag</code> contains &lt;img&gt tag.
 *
 * @author  Jiri Komzak, Knowledge Media Institute, Open University, United Kingdom
 */
public class ImgTag extends  Extension
{
    private String src = null;
    private String width = null;
    private String height = null;

    /**
     * Creates a new <code>ImgTag</code> instance, based on the builder
     * state.
     *
     * @param builder an <code>ImgTagBuilder</code> value
     * @exception InstantiationException if malformed or insufficient data is
     * in the builder.
     */
    public ImgTag(ImgTagBuilder builder)
    {
        src = builder.getSrc();
        width = builder.getWidth();
        height = builder.getHeight();
    }
    
    /**
     * returns src attribute
     */
    public String getSrc()
    {
        return src;
    }
    
    /**
     * Sets src attribute
     */
    public void setSrc(String src)
    {
        this.src = src;
    }
    
    /**
     * returns width attribute
     */
    public String getWidth()
    {
        return width;
    }
    
    /**
     * Sets width attribute
     */
    public void setWidth(String width)
    {
        this.width = width;
    }
    
    /**
     * returns width attribute in int format
     */
    public int getWidthInt() {
        try {
            String str = getWidth();
            int i = Integer.parseInt(str);
            return i;
        } catch (NumberFormatException e) {
            return 0;
        }
    }
    
    /**
     * returns height attribute
     */
    public String getHeight()
    {
        return height;
    }


    /**
     * Sets height attribute
     */
    public void setHeight(String height)
    {
        this.height = height;
    }
    
    /**
     * returns height attribute in int format
     */
    public int getHeightInt() {
        try {
            String str = getHeight();
            int i = Integer.parseInt(str);
            return i;
        } catch (NumberFormatException e) {
            return 0;
        }
    }
    
    /**
     * <code>appendItem</code> converts this packet to XML. 
     * It then appends it to a StringBuffer.
     *
     * @param retval The <code>StringBuffer</code> to append to
     */
    public void appendToXML(StringBuffer retval)
    {
	retval.append("<img src=\"" + src + 
                          "\" width=\"" + width +
                          "\" height=\"" + height + "\"/>\n");
    }
}
