/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: jeti $
 * @version $Revision: 1.1 $
 *
 * j.komzak
 * Changed into ImgTagBuilder
 * 
 * E.S. de Boer
 * converted to jeti plugin
 */

package nu.fw.jeti.plugins.buddyspacemaps.xml;

/*
 * ImgTagBuilder.java
 *
 * Project: BuddySpace
 * (C) Copyright Knowledge Media Institute 2002
 *
 *
 * Created on 16 August 2002, 10:20
 */

import nu.fw.jeti.jabber.elements.Extension;
import nu.fw.jeti.jabber.elements.ExtensionBuilder;

/**
 * <code>ImgTagBuilder</code> is used to construct ImgTag objects
 *
 * @author  Jiri Komzak, Knowledge Media Institute, Open University, United Kingdom
 */
public class ImgTagBuilder implements ExtensionBuilder
{
    private String src = null;
    private String width = null;
    private String height = null;

    /** Creates a new <code>ImgTagBuilder</code> instance. */
    public ImgTagBuilder()
    { reset(); }
    
    /**
     * <code>reset</code> the builder to a default state, so it can be reused.
     */
    public void reset()
    { 
        src = null;
        width = null;
        height = null;
    }
    
    public void setSrc(String newSrc) {
        src = newSrc;
    }
    
    public void setWidth(String newWidth) {
        width = newWidth;
    }
    
    public void setHeight(String newHeight) {
        height = newHeight;
    }

    public String getSrc() {
        return src;
    }
    
    public String getWidth() {
        return width;
    }
    
    public String getHeight() {
        return height;
    }
    
    /**
     * <code>build</code> a new ImgTag object
     *
     * @return an <code>Extension</code> value
     * @exception InstantiationException if insufficient or incorrect data
     * was proviced.
     */
    public Extension build()
    { return new ImgTag(this); }
}
