/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: jeti $
 * @version $Revision: 1.1 $
 *
 * j.komzak
 * Changed into ItemTag
 * 
 * E.S. de Boer
 * converted to jeti plugin
 */

package nu.fw.jeti.plugins.buddyspacemaps.xml;

/*
 * ItemTag.java
 *
 * Project: BuddySpace
 * (C) Copyright Knowledge Media Institute 2002
 *
 *
 * Created on 16 August 2002, 10:20
 */

import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.jabber.elements.Extension;

/**
 * <code>ItemTag</code> contains &lt;item&gt tag.
 *
 * @author  Jiri Komzak, Knowledge Media Institute, Open University, United Kingdom
 */
public class ItemTag extends  Extension
{
    private JID jid = null;
    private String lat = null;
    private String lon = null;

    /**
     * Creates a new <code>ItemTag</code> instance, based on the builder
     * state.
     *
     * @param builder an <code>ItemTagBuilder</code> value
     * @exception InstantiationException if malformed or insufficient data is
     * in the builder.
     */
    public ItemTag(ItemTagBuilder builder)
    {
        jid = builder.getJID();
        lat = builder.getLat();
        lon = builder.getLon();
    }
    
    /**
     * returns jid attribute.
     */
    public JID getJID()
    {
        return jid;
    }
    
    /**
     * returns lat attribute.
     */
    public String getLat()
    {
        return lat;
    }
    
    /**
     * returns lon attribute.
     */
    public String getLon()
    {
        return lon;
    }
    
    /**
     * Sets lat attribute.
     */
    public void setLat(float lat)
    {
        this.lat = Float.toString(lat);
    }
    
    /**
     * Sets lon attribute.
     */
    public void setLon(float lon)
    {
        this.lon = Float.toString(lon);
    }

    /**
     * returns lat attribute in float form.
     */
    public float getLatFloat() {
        try {
            String str = getLat();
            float f = Float.parseFloat(str);
            return f;
        } catch (NumberFormatException e) {
            return 0;
        }
    }
    
    /**
     * returns lon attribute in float form.
     */
    public float getLonFloat() {
        try {
            String str = getLon();
            float f = Float.parseFloat(str);
            return f;
        } catch (NumberFormatException e) {
            return 0;
        }
    }

    /**
     * <code>appendItem</code> converts this packet to XML. 
     * It then appends it to a StringBuffer.
     *
     * @param retval The <code>StringBuffer</code> to append to
     */
    public void appendToXML(StringBuffer retval)
    {
	retval.append("<item jid=\"" + jid.toString() + 
                          "\" lat=\"" + lat +
                          "\" lon=\"" + lon + "\"/>\n");
    }
}
