/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: jeti $
 * @version $Revision: 1.1 $
 *
 * j.komzak
 * Changed into LayerTag
 * 
 * E.S. de Boer
 * converted to jeti plugin
 */

package nu.fw.jeti.plugins.buddyspacemaps.xml;

/*
 * LayerTag.java
 *
 * Project: BuddySpace
 * (C) Copyright Knowledge Media Institute 2002
 *
 *
 * Created on 16 August 2002, 10:20
 */

import java.util.Enumeration;
import java.util.Vector;

import nu.fw.jeti.backend.XMLData;
import nu.fw.jeti.jabber.elements.Extension;

/**
 * <code>LayerTag</code> contains &lt;layer&gt tag.
 *
 * @author  Jiri Komzak, Knowledge Media Institute, Open University, United Kingdom
 */
public class LayerTag extends  Extension
{
    /** Vector of ItemTag and ClusterTag objects contained. */
    private Vector itemList;
    
    private ImgTag img = null;
    
    private String id = null;
    private String offsetX = null;
    private String offsetY = null;
    private String scale = null;
    private String priority = null;
    
    private String src = null;
    /** not included in XML - only used to link maps together */
    private MapTag srcMap = null;

    /**
     * Creates a new <code>LayerTag</code> instance, based on the builder
     * state.
     *
     * @param builder an <code>LayerTagBuilder</code> value
     * @exception InstantiationException if malformed or insufficient data is
     * in the builder.
     */
    public LayerTag(LayerTagBuilder builder)
    {
        itemList = (Vector) builder.getItemsAndClusters().clone();
        img = builder.getImg();
        id = builder.getID();
        offsetX = builder.getOffsetX();
        offsetY = builder.getOffsetY();
        scale = builder.getScale();
        priority = builder.getPriority();
        src = builder.getSrc();
    }

    /**
     * returns an enumeration of <code>items and clusters</code> contained within this
     * object.
     *
     * @return an <code>Enumeration</code> value
     */
    public Enumeration itemsAndClusters()
    {
        return itemList.elements();
    }


    /**
     * <code>appendItem</code> converts this packet to XML.
     * It then appends it to a StringBuffer.
     *
     * @param retval The <code>StringBuffer</code> to append to
     */
    public void appendToXML(StringBuffer retval)
    {
	retval.append("<layer id=\"" + id + 
                          "\" offset_x=\"" + offsetX +
                          "\" offset_y=\"" + offsetY +
                          "\" scale=\"" + scale + "\"");
        if (priority != null)
            retval.append(" priority=\"" + priority + "\"");
        
        if (src != null)
            retval.append(" src=\"" + src + "\"");
        
        if (!(itemsAndClusters().hasMoreElements()) && img == null) {
            retval.append("/>\n");
            return;
        }
        
        retval.append(">\n");
        
        img.appendToXML(retval);
        
        Enumeration items = itemsAndClusters();
        while (items.hasMoreElements())
            ((XMLData)items.nextElement()).appendToXML(retval);
        
        retval.append("</layer>\n");
    }
    
    public String getID() {
        return id;
    }
    
    public void setOffsetX(String newOffsetX) {
        offsetX = newOffsetX;
    }
    
    public String getOffsetX() {
        return offsetX;
    }
    
    public void setOffsetY(String newOffsetY) {
        offsetY = newOffsetY;
    }
    
    public String getOffsetY() {
        return offsetY;
    }
    
    public void setScale(String newScale) {
        scale = newScale;
    }
    
    public String getScale() {
        return scale;
    }
    
    public void setPriority(String newPriority) {
        priority = newPriority;
    }
    
    public String getPriority() {
        return (priority == null)? new String("0"): priority;
    }
    
    public String getSrc() {
        return src;
    }
    
    public ImgTag getImg() {
        return img;
    }
    
    public void setImg(ImgTag imgTag) {
        this.img = imgTag;
    }
    
    public Enumeration getItems() {
        return itemList.elements();
    }
    
    public void addItem(ItemTag item) {
        if (itemList != null)
            itemList.add(item);
    }
    
    public void removeItem(ItemTag item) {
        if (itemList != null)
            itemList.remove(item);
    }

    public void addCluster(ClusterTag cluster) {
        if (itemList != null)
            itemList.add(cluster);
    }
    
    public void removeCluster(ClusterTag cluster) {
        if (itemList != null)
            itemList.remove(cluster);
    }

    /** not included in XML - only used to link maps together */
    public MapTag getSrcMap() {
        return srcMap;
    }
    
    /** not included in XML - only used to link maps together */
    public void setSrcMap(MapTag newSrcMap) {
        srcMap = newSrcMap;
    }
    
    public int getOffsetXInt() {
        try {
            String str = getOffsetX();
            int i = Integer.parseInt(str);
            return i;
        } catch (NumberFormatException e) {
            return 0;
        }
    }
    
    public int getOffsetYInt() {
        try {
            String str = getOffsetY();
            int i = Integer.parseInt(str);
            return i;
        } catch (NumberFormatException e) {
            return 0;
        }
    }
    
    public float getScaleFloat() {
        try {
            String str = getScale();
            float f = Float.parseFloat(str);
            return f;
        } catch (NumberFormatException e) {
            return 1;
        }
    }
    
    public int getPriorityInt() {
        try {
            String str = getPriority();
            int i = Integer.parseInt(str);
            return i;
        } catch (NumberFormatException e) {
            return 0;
        }
    }
    
}
