/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 *	 Shawn Wilton
 *
 *   Changes
 *
 * @author  Shawn Wilton <a href="mailto:shawn@black9.net">
 *                      <i>&lt;shawn@black9.net&gt;</i></a>
 *
 * @author  $Author: jeti $
 * @version $Revision: 1.1 $
 *
 * j.komzak
 * Changed into MapTagHandler
 * 
 * E.S. de Boer
 * converted to jeti plugin
 */

package nu.fw.jeti.plugins.buddyspacemaps.xml;

/*
 * MapTagHandler.java
 *
 * Project: BuddySpace
 * (C) Copyright Knowledge Media Institute 2002
 *
 *
 * Created on 16 August 2002, 10:20
 */
import nu.fw.jeti.jabber.elements.Extension;
import nu.fw.jeti.jabber.handlers.ExtensionHandler;

import org.xml.sax.Attributes;

/**
 * Handler class to build &lt;map&gt objects
 *
 * @author  Jiri Komzak, Knowledge Media Institute, Open University, United Kingdom
 */
public class MapTagHandler extends ExtensionHandler
{
	/** builder for MapTag objects */
	private MapTagBuilder builder;
	private LayerTagHandler layerTagHandler;


	/*
	 * Creates a new <code>MapTagHandler</code> instance.
	 */
	public MapTagHandler()
	{
		super();
		builder = new MapTagBuilder();
	}
	
	/**
         * Gets called when the underlying engine decides to pass an entity and
         * all sub-entities off to your subhandler.<p>
         *
         * Upon seeing the element that this subhandler handles, we call this
         * constructor, passing in the attributes.
         *
         * @param name name of the element which we are handling.
         * @param attributes list of attributes on this element
         */
	public void startHandling(Attributes attributes)
    {
        builder.reset();
        builder.setID(attributes.getValue("id"));
    }
    
        /*
	 * <code>handleStartElement</code> is overloaded by the new class to
	 * provide logic to handle the element code.
	 *
	 * @param name a <code>String</code> value
	 * @param attributes an <code>AttributeList</code> value
	 * @exception SAXException if an error occurs
	 */
	public void startElement(String name,Attributes attributes)
	{			
		//Start new handler for items that *may* be containers, but never for namespaces "ns".
		if(layerTagHandler!=null)layerTagHandler.startElement(name, attributes);
		else
		{
			if(!"layer".equals(name))  return;
	        layerTagHandler = new LayerTagHandler();
	        layerTagHandler.startHandler(name, attributes);
		}
	}


	/*
	 * Stophandler is the same as end element, except that it is called saying
	 * that the subhandler is no longer in scope.
	 *
	 * @param Object a value being returned to the parent - the parent is 
	 * meant to interpret this result.
	 */
	public Extension build()
	{
		Extension e = builder.build();
		builder.reset();
		return e;
	}
	
	
	/*
	 * <code>receiveChildData</code> is called when a child handler exits,
	 * returning control to this code. The now-defunct handler along with the
	 * data object are both returned.
	 *
	 * @param subHandler a <code>SubHandler</code> value
	 * @param o an <code>Object</code> value
	 */
	public void endElement(String name)
	{
		if(name.equals("layer"))
		{
		//Add the returned object to the child vector
		builder.addLayer((LayerTag)layerTagHandler.build());
		layerTagHandler=null;
		}
		else if(layerTagHandler!=null)
		{
			layerTagHandler.handleEndElement(name,getText());
		}
		clearCurrentChars();
	}
}
