// Created on 14-sep-2003
package nu.fw.jeti.plugins.compression;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.util.Map;

import nu.fw.jeti.backend.Connect;
import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.elements.Packet;
import nu.fw.jeti.jabber.handlers.PacketHandler;
import nu.fw.jeti.plugins.Plugins;
import nu.fw.jeti.plugins.compression.jcraft.jzlib.JZlib;
import nu.fw.jeti.plugins.compression.jcraft.jzlib.ZInputStream;
import nu.fw.jeti.plugins.compression.jcraft.jzlib.ZOutputStream;
import nu.fw.jeti.plugins.xmpp.Features;


/**
 * @author E.S. de Boer
 *
 */
public class Plugin implements Plugins
{
	public final static String VERSION = "0.2";
	public final static String DESCRIPTION = "compression.Compresses_the_jabber_stream_to_save_bandwidth";
	public final static String MIN_JETI_VERSION = "0.5.1";
	public final static String NAME = "compression";
	public final static String ABOUT = "by E.S. de Boer";
				
	public static void init(final Backend backend)
	{
		backend.addExtensionHandler(CompressionFeature.NAMESPACE,new CompressionFeatureHandler());
	}
	
	public static boolean checkFeature(Features features,Connect connect)
	{
		if(features.hasFeature(CompressionFeature.class)){
			if(((CompressionFeature)features.getFeature(CompressionFeature.class)).getMethods().contains("zlib"))
			{
				connect.sendWhileConnecting(new Compress("compress","zlib"));
				return true;
			}
		}
		return false;
	}
	
	public static void addHandler (Map<String,PacketHandler> handlers)
	{
		handlers.put("compressed",new CompressHandler("compressed"));
	}
	
	public static  void receivePackets(Packet packet,final Connect connect,final nu.fw.jeti.plugins.xmpp.Plugin plugin)
	{//TODO add errors
		if(packet instanceof Compress)
		{
			if(((Compress)packet).getType().equals("compressed"))
			{
				Socket socket = connect.getSocket();
				try {
					connect.startCompressed(plugin,getCompressedInputStream(socket.getInputStream()),getCompressedOutputStream(socket.getOutputStream()));
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
		
			}
		}
	}
	
	public static OutputStream getCompressedOutputStream(OutputStream out)
	{
		ZOutputStream zOut = new ZOutputStream(out,JZlib.Z_BEST_COMPRESSION);
		zOut.setFlushMode(JZlib.Z_PARTIAL_FLUSH);
		return zOut;
	}
	
	public static InputStream getCompressedInputStream(InputStream in)
	{
		ZInputStream zIn = new ZInputStream(in);
		zIn.setFlushMode(JZlib.Z_PARTIAL_FLUSH);
		return zIn;
	}
	
	
	public static void unload(Backend backend)
	{
		backend.removeExtensionHandler(CompressionFeature.NAMESPACE);
	}
	
	public void unload()
	{
		
	}
}
