package nu.fw.jeti.plugins.drawing;

import nu.fw.jeti.jabber.elements.Extension;
import nu.fw.jeti.jabber.handlers.ExtensionHandler;
import nu.fw.jeti.plugins.drawing.elements.BackgroundSettingMessage;
import nu.fw.jeti.plugins.drawing.elements.ColorSettingMessage;
import nu.fw.jeti.plugins.drawing.elements.CreationMessage;
import nu.fw.jeti.plugins.drawing.elements.CurrentUsersMessage;
import nu.fw.jeti.plugins.drawing.elements.DeletionMessage;
import nu.fw.jeti.plugins.drawing.elements.DisplayMessage;
import nu.fw.jeti.plugins.drawing.elements.PictureMessage;
import nu.fw.jeti.plugins.drawing.elements.PictureMessageBuilder;
import nu.fw.jeti.plugins.drawing.elements.TranslationMessage;

import org.xml.sax.Attributes;

/**
 * This class implements the handler for the picture message.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class PictureMessageHandler extends ExtensionHandler {

	private PictureMessageBuilder builder;
	
	public PictureMessageHandler() {
		builder = new PictureMessageBuilder();
	}
	
	public void startHandling(Attributes attr)
	{
		((PictureMessageBuilder)builder).time = attr.getValue(PictureMessage.TIME_TAG);
		((PictureMessageBuilder)builder).counter = attr.getValue(PictureMessage.COUNTER_TAG);
	}
	
	/**
	 * It looks on the first tag and tries to match it with the available tags.
	 * If the type of the message is already known, it does nothing.
	 */
	public void startElement(String name, Attributes attr) {
		if (builder.messageType == 
					PictureMessageBuilder.UNKNOWN_MESSAGE) {
			if (name.equals(CreationMessage.TYPE_TAG)) {
				((PictureMessageBuilder)builder).messageType = PictureMessageBuilder.CREATION_MESSAGE;
			} else if (name.equals(DisplayMessage.TYPE_TAG)) {
				((PictureMessageBuilder)builder).messageType = PictureMessageBuilder.DISPLAY_MESSAGE;
			} else if (name.equals(DeletionMessage.TYPE_TAG)) {
				((PictureMessageBuilder)builder).messageType = PictureMessageBuilder.DELETION_MESSAGE;
			} else if (name.equals(TranslationMessage.TYPE_TAG)) {
				((PictureMessageBuilder)builder).messageType = PictureMessageBuilder.TRANSLATION_MESSAGE;
			} else if (name.equals(ColorSettingMessage.TYPE_TAG)) {
				((PictureMessageBuilder)builder).messageType = PictureMessageBuilder.COLOR_CHANGE_MESSAGE;
			} else if (name.equals(BackgroundSettingMessage.TYPE_TAG)) {
				((PictureMessageBuilder)builder).messageType = PictureMessageBuilder.BACKGROUND_COLOR_SETTING_MESSAGE;
			} else if (name.equals(CurrentUsersMessage.TYPE_TAG)) {
				((PictureMessageBuilder)builder).messageType = PictureMessageBuilder.CURRENT_USERS_MESSAGE;
			}
			
			// else if...
		}
	}
	
	/**
	 * It sets all the elements according to the message type.
	 */
	public void endElement(String name)
	{
		if (name.equals("error")){
		} else if (name.equals(PictureMessage.ACTIONS_AUTHOR_TAG)) {
			((PictureMessageBuilder)builder).actionsAuthor = getText();
		} else if (name.equals(PictureMessage.THREAD_TAG)) {
			((PictureMessageBuilder)builder).thread = getText();
		} else if (name.equals(PictureMessage.SUBJECT_TAG)) {
			((PictureMessageBuilder)builder).subject = getText();
		} else if (name.equals(PictureMessage.SHAPE_ID_TAG)) {
			((PictureMessageBuilder)builder).shapeIdList = getText(); 
		} else switch (((PictureMessageBuilder)builder).messageType) {
		
			case PictureMessageBuilder.CREATION_MESSAGE:
				if (name.equals(CreationMessage.COLOR_TAG)) {
					((PictureMessageBuilder)builder).color = getText();
				} else if (name.equals(CreationMessage.AUTHOR_TAG)) {
					((PictureMessageBuilder)builder).shapesAuthor = getText();
				} else if (Plugin.shapeFactoryContains(name)) {
					// this is the name of a shape and its coordinates
					((PictureMessageBuilder)builder).shapeName = name;
					((PictureMessageBuilder)builder).coord = getText();
				} else if (name.equals(CreationMessage.HEIGHT_TAG)) {
					((PictureMessageBuilder)builder).shapeDisplayHeight = getText();
				} else if (name.equals(CreationMessage.IS_FILLED_TAG)) {
					((PictureMessageBuilder)builder).isFilled = true;
				} else if (name.equals(CreationMessage.TYPE_TAG)) {
				} else {
					notParsed(name, getText());
				}
				break;
				
			case PictureMessageBuilder.DISPLAY_MESSAGE:
				if (name.equals(DisplayMessage.TYPE_TAG)) {
				} else {
					notParsed(name, getText());
				}
				break;
				
			case PictureMessageBuilder.DELETION_MESSAGE:
				if (name.equals(DeletionMessage.TYPE_TAG)) {				
				} else {
					notParsed(name, getText());
				}				
				break;
				
			case PictureMessageBuilder.TRANSLATION_MESSAGE:
				if (name.equals(TranslationMessage.TYPE_TAG)) {
				} else if (name.equals(TranslationMessage.VECTOR_TAG)) {
					((PictureMessageBuilder)builder).transVector = getText();
				} else {
					notParsed(name, getText());
				}
				break;
				
			case PictureMessageBuilder.COLOR_CHANGE_MESSAGE:
				if (name.equals(ColorSettingMessage.COLOR_TAG)) {
					((PictureMessageBuilder)builder).color = getText();
				} else if (name.equals(ColorSettingMessage.TYPE_TAG)) {
				} else {
					notParsed(name, getText());
				}
				break;
				
			case PictureMessageBuilder.BACKGROUND_COLOR_SETTING_MESSAGE:
				if (name.equals(BackgroundSettingMessage.COLOR_TAG)) {
					((PictureMessageBuilder)builder).color = getText();
				} else if (name.equals(BackgroundSettingMessage.OLD_COLOR_TAG)) {
					((PictureMessageBuilder)builder).oldColor = getText();
				} else if (name.equals(BackgroundSettingMessage.TYPE_TAG)) {
				} else {
					notParsed(name, getText());
				}
				break;				
				
			case PictureMessageBuilder.CURRENT_USERS_MESSAGE:
				if (name.equals(CurrentUsersMessage.TYPE_TAG)) {
					((PictureMessageBuilder)builder).users = getText();
				} else {
					notParsed(name, getText());
				}
				break;
				
			default:
				// there was not an apropriate tag
				notParsed(name, getText());
				break;
		} 
			
		clearCurrentChars();
	}
	
	public Extension build()
	{
		clearCurrentChars();
		Extension e = builder.build();
		builder.reset();
		return e;
	}

	private void notParsed(String name, String text) {
		nu.fw.jeti.util.Log.notParsedXML("PictureMessageHandler: Unknown tag "
				+ name + " with text: " + text);		
	}

}
