
package nu.fw.jeti.plugins.drawing;

import java.awt.Color;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Random;

import nu.fw.jeti.backend.roster.JIDStatusGroup;
import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.jabber.JIDStatus;
import nu.fw.jeti.plugins.Plugins;
import nu.fw.jeti.plugins.RosterMenuListener;
import nu.fw.jeti.plugins.drawing.shapes.Ellipse;
import nu.fw.jeti.plugins.drawing.shapes.EllipseFactory;
import nu.fw.jeti.plugins.drawing.shapes.Line;
import nu.fw.jeti.plugins.drawing.shapes.LineFactory;
import nu.fw.jeti.plugins.drawing.shapes.PictureChat;
import nu.fw.jeti.plugins.drawing.shapes.Polygon;
import nu.fw.jeti.plugins.drawing.shapes.PolygonFactory;
import nu.fw.jeti.plugins.drawing.shapes.Polyline;
import nu.fw.jeti.plugins.drawing.shapes.PolylineFactory;
import nu.fw.jeti.plugins.drawing.shapes.Rectangle;
import nu.fw.jeti.plugins.drawing.shapes.RectangleFactory;
import nu.fw.jeti.plugins.drawing.shapes.Shape;
import nu.fw.jeti.plugins.drawing.shapes.ShapeFactory;
import nu.fw.jeti.plugins.drawing.shapes.TextShape;
import nu.fw.jeti.plugins.drawing.shapes.TextShapeFactory;
import nu.fw.jeti.util.I18N;



public class Plugin implements Plugins
{
	public final static String VERSION = "0.1";
	public final static String DESCRIPTION = "drawing.whiteboard";
	public final static String MIN_JETI_VERSION = "0.7.6";
	public final static String NAME = "drawing";
	public final static String ABOUT = "by Elabieta Iwaszkiewicz";
	private static HashMap<String, ShapeFactory> shapeFactories;
	private static Random rand = new Random();
	private static Map<String, PictureChat> pictureChats;
	private static Backend backend;

	public static void init(Backend backend)
	{
		backend.addExtensionHandler("http://jeti.sf.net/protocol/whiteboard",new PictureMessageHandler());
		backend.getMain().addToOnlineRosterMenu(I18N.getText("drawing","Drawing"),new RosterMenuListener ()
		{
			public void actionPerformed(JIDStatus jidStatus,JIDStatusGroup group)
			{
				addPictureChat(jidStatus.getJID());
			}
		});
		Plugin.backend = backend;
		shapeFactories = new HashMap<String, ShapeFactory>();
    	shapeFactories.put(Line.NAME, new LineFactory());
    	shapeFactories.put(Polyline.NAME, new PolylineFactory());
    	shapeFactories.put(Rectangle.NAME, new RectangleFactory());
    	shapeFactories.put(Ellipse.NAME, new EllipseFactory());
    	shapeFactories.put(Polygon.NAME, new PolygonFactory());
    	shapeFactories.put(TextShape.NAME, new TextShapeFactory());
	}
	

	    
	    public static Shape createShape(String shapeName, long id, JID author, 
	    		String coordinates, Color color, int height, boolean isFilled) {
	    	return (shapeFactories.get(shapeName)).createShape(
	    			id, author, coordinates, color, height, isFilled);
	    }
	    
	    public static boolean shapeFactoryContains(String shapeName) {
	    	return shapeFactories.containsKey(shapeName);
	    }
	    
	    public static long getRandLong() {
	    	return rand.nextLong();
	    }
	    
	    /**
	     * If there has not been a drawingChat with this person, we should create one.
	     * In the other case, it should just display it.
	     * If this is a multi-user chat, there is no jid in the chat (there are 
	     * many users on this chat and none is priviledged).
	     * @param jid
	     */
	    public static void addPictureChat(JID jid) {
	    	if (pictureChats == null) {
	    		pictureChats = Collections.synchronizedMap(new HashMap<String, PictureChat>());
	    	}
	    	Collection col = pictureChats.values();    	
	    	for (Iterator iter = col.iterator(); iter.hasNext();) {
	    		PictureChat chat = (PictureChat)iter.next();
	    		if (jid != null && jid.equals(chat.getJID())) {
	    			chat.showWindow();
	    			return;
	    		}
	    	}
	    	String thread = backend.createThread();
	    	pictureChats.put(thread, new PictureChat(backend, jid, thread));    	
	    }
	    
	    public static PictureChat getPictureChat(JID jid, String thread) {
	    	if (pictureChats == null) {
	    		pictureChats = Collections.synchronizedMap(new HashMap<String, PictureChat>());
	    	}
	    	if (pictureChats.get(thread) == null) { 
	    		pictureChats.put(thread, new PictureChat(backend, jid, thread));
	    	}
	    	return pictureChats.get(thread);
	    }
		
	public void unload(){}
	
	public static void unload(Backend backend)
	{
		backend.getMain().removeFromRosterMenu(I18N.getText("drawing","Drawing"));
		backend.removeExtensionHandler("http://jeti.sf.net/protocol/whiteboard");
	}
}

/*
 * Overrides for emacs
 * Local variables:
 * tab-width: 4
 * End:
 */
