package nu.fw.jeti.plugins.drawing.elements;

import java.awt.Color;

import nu.fw.jeti.jabber.JID;

/**
 * This class represents the message for setting the color of the picture's
 * background.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class BackgroundSettingMessage extends PictureMessage {
	private Color newColor;

	private Color oldColor;

	/**
	 * Tag used to send this message.
	 */
	public static final String TYPE_TAG = "background";

	/**
	 * Tag used to send an old color.
	 */
	public static final String OLD_COLOR_TAG = "old_color";

	/**
	 * Constructs the message from the builder.
	 * 
	 * @param pmb
	 *            builder where all needed fields are set
	 */
	public BackgroundSettingMessage(PictureMessageBuilder pmb) {
		super(pmb);
		newColor = stringToColor(pmb.color);
		oldColor = stringToColor(pmb.oldColor);
	}

	/**
	 * Constructs the message and sets all the fields.
	 * 
	 * @param to
	 *            adresser of the message
	 * @param newC
	 *            color to be set
	 * @param oldC
	 *            old background color
	 * @param thread
	 *            thread of the chat
	 * @param time
	 *            time of the message
	 * @param author
	 * 			  action's author
	 * @param counter
	 *            counter of the message
	 */
	public BackgroundSettingMessage(JID to, Color newC, Color oldC,
			String thread, String time, String author, int counter) {
		super(to, author, null, thread, time, counter);
		newColor = newC;
		oldColor = oldC;
	}

	public void appendToXML(StringBuffer xml) {
		startAppendingToXMLWithoutShapeId(xml, TYPE_TAG);
		appendElement(xml, COLOR_TAG, colorToString(newColor));
		appendElement(xml, OLD_COLOR_TAG, colorToString(oldColor));
		endAppendingToXML(xml, TYPE_TAG);
	}

	/**
	 * @return Returns the newColor.
	 */
	public Color getColor() {
		return newColor;
	}

	/**
	 * @return Returns the oldColor.
	 */
	public Color getOldColor() {
		return oldColor;
	}

}
