package nu.fw.jeti.plugins.drawing.elements;

import java.awt.Color;
import java.util.List;

import nu.fw.jeti.jabber.JID;

/**
 * This class represents the message for setting the color of the selected
 * shapes.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 * 
 */
public class ColorSettingMessage extends PictureMessage {
	private Color color;

	/**
	 * Tag used to send this message.
	 */
	public static final String TYPE_TAG = "color_set";

	/**
	 * Constructs the message from the builder.
	 * 
	 * @param pmb
	 *            builder where all needed fields are set
	 */
	public ColorSettingMessage(PictureMessageBuilder pmb) {
		super(pmb);
		color = stringToColor(pmb.color);
	}

	/** Constructs the message and sets all the fields.
	 * 
	 * @param to
	 *            adresser of the message
	 * @param idList
	 *            list of selected shapes' id; their color are going to be set
	 * @param c
	 *            color to be set
	 * @param thread
	 *            thread of the chat
	 * @param author
	 * 			  action's author
	 * @param time
	 *            time of the message
	 */
	public ColorSettingMessage(JID to, List<Long> idList, Color c,
			String thread, String time, String author, int counter) {
		super(to, author, idList, thread, time, counter);
		color = c;
	}

	public void appendToXML(StringBuffer xml) {
		startAppendingToXML(xml, TYPE_TAG);
		appendElement(xml, COLOR_TAG, colorToString(color));
		endAppendingToXML(xml, TYPE_TAG);
	}

	/**
	 * @return returns the color.
	 */
	public Color getColor() {
		return color;
	}

}
