package nu.fw.jeti.plugins.drawing.elements;

import java.awt.Color;

import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.plugins.drawing.shapes.actions.Action;

/**
 * This class represents the message for creation of the shape. It sends all
 * needed information about the shape.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class CreationMessage extends PictureMessage {
	private String shapeName;

	private String coordinates;

	private Color color;

	private JID author;

	private int shapeDisplayHeight;

	private boolean isFilled;

	/**
	 * Tag used to send this message.
	 */
	public static final String TYPE_TAG = "create";

	/**
	 * Tag used to send the shape's author.
	 */
	public static final String AUTHOR_TAG = "author";

	/**
	 * Tag used to send the shape's height in the picture's array.
	 */
	public static final String HEIGHT_TAG = "height";

	/**
	 * Tag used to send the information if the shape is filled.
	 */
	public static final String IS_FILLED_TAG = "is_filled";

	/**
	 * Constructs the message from the builder. There has to be only one shape
	 * sent by this message.
	 * 
	 * @param pmb
	 *            builder where all needed fields are set
	 */
	public CreationMessage(PictureMessageBuilder pmb) {
		super(pmb);
		shapeName = pmb.shapeName;
		coordinates = pmb.coord;
		color = stringToColor(pmb.color);
		author = stringToJID(pmb.shapesAuthor);
		shapeDisplayHeight = Integer.parseInt(pmb.shapeDisplayHeight);
		isFilled = pmb.isFilled;
	}

	/**
	 * Constructs the message and sets all the fields.
	 * 
	 * @param to
	 *            adresser of the message
	 * @param id
	 *            shape's id
	 * @param name
	 *            shape's name
	 * @param coord
	 *            shape's coordinates converted to String
	 * @param col
	 *            shape's color
	 * @param filled
	 *            <tt>true</tt> if the shape is filled
	 * @param auth
	 *            shape's author
	 * @param thread
	 *            thread of the chat
	 * @param displayHeight
	 *            shape's height in the picture's array
	 * @param time
	 *            time of the message
	 * @param actionsAuthor
	 * 			  action's author
	 * @param counter
	 *            counter of the message
	 */
	public CreationMessage(JID to, long id, String name, String coord,
			Color col, boolean filled, JID auth, String thread,
			int displayHeight, String time, String actionsAuthor, int counter) {
		super(to, actionsAuthor, Action.createShapeIdList(id), thread, time, counter);
		shapeName = name;
		coordinates = coord;
		color = col;
		isFilled = filled;
		shapeDisplayHeight = displayHeight;
		author = auth;
	}

	public void appendToXML(StringBuffer xml) {
		startAppendingToXML(xml, TYPE_TAG);
		appendElement(xml, shapeName, coordinates);
		appendElement(xml, COLOR_TAG, colorToString(color));
		appendElement(xml, HEIGHT_TAG, shapeDisplayHeight);
		if (isFilled) {
			appendElement(xml, IS_FILLED_TAG, "");
		}
		appendElement(xml, AUTHOR_TAG, author);
		endAppendingToXML(xml, TYPE_TAG);
	}

	public JID getAuthor() {
		return author;
	}

	public Color getColor() {
		return color;
	}

	public boolean isFilled() {
		return isFilled;
	}

	public String getCoordinates() {
		return coordinates;
	}

	public String getShapeName() {
		return shapeName;
	}

	public int getShapeHeight() {
		return shapeDisplayHeight;
	}
}
