package nu.fw.jeti.plugins.drawing.elements;

import java.util.ArrayList;
import java.util.List;

import nu.fw.jeti.jabber.JID;

/**
 * This class represents the message that sends all the current users in the
 * chat.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class CurrentUsersMessage extends PictureMessage {
	private List<JID> users;

	/**
	 * Tag used to send this message.
	 */
	public static final String TYPE_TAG = "current_users";

	/**
	 * Constructs the message and sets all the fields.
	 * 
	 * @param to
	 *            adresser of the message
	 * @param usersList
	 *            list of JID of the current users
	 * @param t
	 *            thread of the chat
	 * @param time
	 *            time of the message
	 * @param author
	 * 			  action's author
	 * @param counter
	 *            counter of the message
	 */
	public CurrentUsersMessage(JID to, List<JID> usersList, String t,
			String time, String author, int counter) {
		super(to, author, null, t, time, counter);
		users = usersList;
	}

	/**
	 * Constructs the message from the builder.
	 * 
	 * @param builder
	 *            builder where all needed fields are set
	 */
	public CurrentUsersMessage(PictureMessageBuilder builder) {
		super(builder);
		users = stringToJIDList(builder.users);
	}

	public void appendToXML(StringBuffer xml) {
		startAppendingToXMLWithoutShapeId(xml, TYPE_TAG);
		xml.append(jidListToString(users));
		endAppendingToXML(xml, TYPE_TAG);

	}

	public List<JID> getUsers() {
		return users;
	}

	private String jidListToString(List<JID> usersList) {
		String s = "";
		for (JID jid : usersList) {
			s = s + jid + " ";
		}
		return s;
	}

	private List<JID> stringToJIDList(String usersList) {
		List<JID> jidList = new ArrayList<JID>();
		String[] result = usersList.split("\\s");
		for (int i = 0; i < result.length; i++) {
			try {
				jidList.add(JID.checkedJIDFromString(result[i]));
			} catch (InstantiationException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return jidList;
	}

}
