package nu.fw.jeti.plugins.drawing.elements;

import nu.fw.jeti.jabber.elements.Extension;

/**
 * This class represents the message builder for
 * {@link nu.fw.jeti.jabber.elements.drawing.PictureMessage}s.  Never all the
 * fields are set. It depends on the message.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class PictureMessageBuilder  {
	/**
	 * Type of a message.
	 */
	public int messageType = UNKNOWN_MESSAGE;

	/**
	 * List of shapes' ids.
	 */
	public String shapeIdList;

	/**
	 * Shape's color.
	 */
	public String color;

	/**
	 * Shape's author.
	 */
	public String shapesAuthor;
	
	/**
	 * Action's author.
	 */
	public String actionsAuthor;

	/**
	 * <tt>True</tt> if the shape is filled.
	 */
	public boolean isFilled;

	/**
	 * Where the shape should be displayed at the picture (under shapes with
	 * bigger height and above these with smaller).
	 */
	public String shapeDisplayHeight;

	/**
	 * Shape's name.
	 */
	public String shapeName;

	/**
	 * Shape's coordinates converted to the String.
	 */
	public String coord;

	/**
	 * Vector of the translation of the Shapes.
	 */
	public String transVector;

	/**
	 * Old color.
	 */
	public String oldColor;

	/**
	 * Current users of the chat.
	 */
	public String users;

	/**
	 * Chat's thread.
	 */
	public String thread;

	/**
	 * Chat's subject.
	 */
	public String subject;

	/**
	 * Message's time.
	 */
	public String time;

	/**
	 * Message's counter.
	 */
	public String counter;

	public static final int UNKNOWN_MESSAGE = -1;

	public static final int CREATION_MESSAGE = 1;

	public static final int DISPLAY_MESSAGE = 2;

	public static final int DELETION_MESSAGE = 3;

	public static final int COLOR_CHANGE_MESSAGE = 4;

	public static final int BACKGROUND_COLOR_SETTING_MESSAGE = 5;

	public static final int TRANSLATION_MESSAGE = 6;

	public static final int CURRENT_USERS_MESSAGE = 15;

	/**
	 * Creates an appropriate message.
	 */
	public Extension build() {
		switch (messageType) {
		case CREATION_MESSAGE:
			return new CreationMessage(this);
		case DISPLAY_MESSAGE:
			return new DisplayMessage(this);
		case TRANSLATION_MESSAGE:
			return new TranslationMessage(this);
		case DELETION_MESSAGE:
			return new DeletionMessage(this);
		case COLOR_CHANGE_MESSAGE:
			return new ColorSettingMessage(this);
		case BACKGROUND_COLOR_SETTING_MESSAGE:
			return new BackgroundSettingMessage(this);
		case CURRENT_USERS_MESSAGE:
			return new CurrentUsersMessage(this);

		default:
			return null;
		}

	}

	/**
	 * Sets all the fields to null.
	 */
	public void reset() {
		messageType = UNKNOWN_MESSAGE;
		shapeIdList = null;
		color = null;
		shapesAuthor = null;
		actionsAuthor = null;
		shapeName = null;
		coord = null;
		shapeDisplayHeight = null;
		isFilled = false;
		transVector = null;
		oldColor = null;
		users = null;
		thread = null;
		subject = null;
		time = null;
		counter = null;
	}

}
