package nu.fw.jeti.plugins.drawing.elements;

import java.util.List;

import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.plugins.drawing.geometry.Vector;

/**
 * This class represents the message for translation (move) of shapes.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class TranslationMessage extends PictureMessage {
	private Vector vector;

	/**
	 * Tag used to send this message.
	 */
	public static final String TYPE_TAG = "move";

	/**
	 * Tag used to send the vector.
	 */
	public static final String VECTOR_TAG = "vector";

	/**
	 * Constructs the message from the builder.
	 * 
	 * @param pmb
	 *            builder where all needed fields are set
	 */
	public TranslationMessage(PictureMessageBuilder pmb) {
		super(pmb);
		vector = stringToVector(pmb.transVector);
	}

	/**
	 * Constructs the message and sets all the fields.
	 * 
	 * @param to
	 *            adresser of the message
	 * @param idList
	 *            list of shapes' id that will be moved
	 * @param v
	 *            vector of the translation
	 * @param thread
	 *            thread of the chat
	 * @param time
	 *            time of the message
	 * @param author
	 * 			  action's author
	 * @param counter
	 *            counter of the message
	 */
	public TranslationMessage(JID to, List<Long> idList, Vector v,
			String thread, String time, String author, int counter) {
		super(to, author, idList, thread, time, counter);
		vector = v;
	}

	public void appendToXML(StringBuffer xml) {
		startAppendingToXML(xml, TYPE_TAG);
		appendElement(xml, VECTOR_TAG, vector);
		endAppendingToXML(xml, TYPE_TAG);
	}

	private Vector stringToVector(String s) {
		String[] result = s.split("\\s");
		return new Vector(Double.parseDouble(result[0]),
				Double.parseDouble(result[1]));
	}

	public Vector getVector() {
		return vector;
	}
}
