package nu.fw.jeti.plugins.drawing.geometry;

import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.List;

/**
 * This class provides operations on class Point2D.Double.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 * 
 */
public class PointsOperations {

	/**
	 * Checks if the point is inside the rectangle. The rectangle is created by
	 * two points: upper left corner and bottom right corner.
	 * 
	 * @param myPoint
	 *            point we check
	 * @param upperLeft
	 *            upper left corner of the rectangle
	 * @param bottomRight
	 *            bottom right corner of the rectangle
	 * @return <tt>true</tt> if the point is inside the rectangle
	 */
	public static boolean pointIsInsideRect(Point2D.Double myPoint, Point2D.Double upperLeft,
			Point2D.Double bottomRight) {
		return (myPoint.x > upperLeft.x && myPoint.y > upperLeft.y
				&& myPoint.x < bottomRight.x && myPoint.y < bottomRight.y);
	}

	/**
	 * Determines the upper left corner of the rectangle determined by the line
	 * a-b.
	 * 
	 * @param a
	 *            first point determining the line
	 * @param b
	 *            second point determining the line
	 * @return the upper left corner of the rectangle determined by the line a-b
	 */
	public static Point2D.Double upperLeftPoint(Point2D.Double a, Point2D.Double b) {
		return new Point2D.Double(Math.min(a.x, b.x), Math.min(a.y, b.y));
	}

	/**
	 * Determines the bottom right corner of the rectangle determined by the
	 * line a-b.
	 * 
	 * @param a
	 *            first point determining the line
	 * @param b
	 *            second point determining the line
	 * @return the bottom right corner of the rectangle determined by the line
	 *         a-b
	 */
	public static Point2D.Double bottomRightPoint(Point2D.Double a, Point2D.Double b) {
		return new Point2D.Double(Math.max(a.x, b.x), Math.max(a.y, b.y));
	}

	/**
	 * Gets the mediana of the Points array.
	 * 
	 * @param points
	 *            array of Points
	 * @return the mediana of given Points
	 */
	public static Point2D.Double getMiddlePoint(Point2D.Double... points) {
		double x = 0;
		double y = 0;
		int counter = 0;
		for (Point2D.Double point : points) {
			x += point.x;
			y += point.y;
			counter++;
		}
		return new Point2D.Double(x / counter, y / counter);
	}

	/**
	 * Gets list of points from the String containing a list of Points.
	 * 
	 * @param string
	 *            String containing a list of Points
	 * @return the list of Points
	 */
	public static List<Point2D.Double> getPointsFromString(String string) {
		String[] result = (string.split("\\s"));
		List<Point2D.Double> points = new ArrayList<Point2D.Double>();

		for (int i = 0; i < result.length; i = i + 2) {
			points.add(new Point2D.Double(Double.parseDouble(result[i]), 
					Double.parseDouble(result[i + 1])));
		}
		return points;
	}

	/**
	 * Converts list of Points to String containing this list.
	 * 
	 * @param points
	 *            list of points
	 * @return String conatining the list of Points
	 */
	public static String pointsToString(List<Point2D.Double> points) {
		String s = "";
		for (Point2D.Double p : points) {
			s = s + p.x + " " + p.y + " ";
		}
		return s;
	}

	/**
	 * Conversts the array of Points to String containing this array.
	 * 
	 * @param a
	 *            array of Points
	 * @return String containing the array of points
	 */
	public static String pointsToString(Point2D.Double... a) {
		String s = "";
		for (Point2D.Double point : a) {
			s = s + point.x + " " + point.y + " ";
		}
		return s;
	}

	/**
	 * Converts the arrays of coordinates to String containing Points created
	 * from this arrays.
	 * 
	 * @param pointsX
	 *            array of x coordinates
	 * @param pointsY
	 *            array of xy coordinates
	 * @return String containing the array of Points
	 */
	public static String pointsToString(double[] pointsX, double[] pointsY) {
		String coord = "";
		for (int i = 0; i < pointsX.length; i++) {
			coord = coord + pointsX[i] + " " + pointsY[i] + " ";
		}
		return coord;
	}
}
