/*
 * Created on 2006-02-25
 *
 */
package nu.fw.jeti.plugins.drawing.shapes;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.List;

import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.plugins.drawing.geometry.PointsOperations;
import nu.fw.jeti.plugins.drawing.geometry.Vector;

/**
 * This class represents the line.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class Line extends Shape {
	/**
	 * Tag used to send this shape.
	 */
	public static final String NAME = "line";

	private Point2D.Double start, end;

	/**
	 * Constructs a new line and sets all the fields.
	 * 
	 * @param id
	 *            shape's id
	 * @param a
	 *            shape's author
	 * @param points
	 *            shape's coordinates converted to String
	 * @param c
	 *            shape's color
	 * @param height
	 *            shape's height in the picture's array
	 */
	public Line(long id, JID a, String points, Color c, int height) {
		super(id, a, c, height);
		setCoordinates(points);
	}

	/**
	 * Constructs a new line and sets all the fields.
	 * 
	 * @param s
	 *            first  of the line
	 * @param e
	 *            second point of the line
	 * @param c
	 *            shape's color
	 * @param a
	 *            shape's author
	 */
	public Line(Point2D.Double s, Point2D.Double e, Color c, JID a) {
		super(generateId(), a, c, INCORRECT_HEIGHT);
		start = s;
		end = e;
	}

	/**
	 * @param coords
	 *            coordinates in String: first Point2D.Double of the line, second Point2D.Double
	 *            of the line
	 */
	public void setCoordinates(String coords) {

		List<Point2D.Double> points = PointsOperations.getPointsFromString(coords);
		start = points.get(0);
		end = points.get(1);
	}

	public void draw(Graphics g, int xScale, int yScale) {
		int newStartX = ceil(start.x, xScale);
		int newStartY = ceil(start.y, yScale);
		int newEndX = ceil(end.x, xScale);
		int newEndY = ceil(end.y, yScale);
		
		g.setColor(color);
		g.drawLine(newStartX, newStartY, newEndX, newEndY);
	}

	public String getCoordinates() {
		ArrayList<Point2D.Double> points = new ArrayList<Point2D.Double>();
		points.add(start);
		points.add(end);
		return PointsOperations.pointsToString(points);
	}

	public String getName() {
		return NAME;
	}

	public void drawAsSelected(Graphics g, int xScale, int yScale) {
		draw(g, xScale, yScale);
		int newStartX = ceil(start.x, xScale);
		int newStartY = ceil(start.y, yScale);
		int newEndX = ceil(end.x, xScale);
		int newEndY = ceil(end.y, yScale);
		
		g.setColor(selectionColor);
		g.drawRect(newStartX - 2, newStartY - 2, 4, 4);
		g.drawRect(newEndX - 2, newEndY - 2, 4, 4);

	}

	public boolean isInsideTheRect(Point2D.Double externalUpperLeft,
			Point2D.Double externalBottomRight) {
		return (PointsOperations.pointIsInsideRect(start, externalUpperLeft,
				externalBottomRight) && PointsOperations.pointIsInsideRect(end,
				externalUpperLeft, externalBottomRight));
	}

	public boolean isPointInShapesNeighbourhood(Point2D.Double p, int scaleX, int scaleY) {
		return isPointInLinesNeighbourhood(start, end, p, scaleX, scaleY);
	}

	public Point2D.Double getDisplayPoint() {
		return PointsOperations.getMiddlePoint(start, end);
	}

	public void changeCoordinates(Vector v) {
		start = v.addVectorToPoint(start);
		end = v.addVectorToPoint(end);
	}

	public Point2D.Double getTheMostUpperLeftPoint() {
		return new Point2D.Double(Math.min(start.x, end.x), Math.min(start.y, end.y));
	}

	public Point2D.Double getTheMostBottomRightPoint() {
		return new Point2D.Double(Math.max(start.x, end.x), Math.max(start.y, end.y));
	}

	@Override
	public boolean isPointInShapesInterior(Point2D.Double p, int scaleX, int scaleY) {
		return false;
	}
}
