package nu.fw.jeti.plugins.drawing.shapes;

import java.awt.Color;
import java.awt.geom.Point2D;

import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.plugins.drawing.geometry.PointsOperations;
import nu.fw.jeti.plugins.drawing.geometry.Vector;

public abstract class PolyPointsShape extends Shape {
	protected double[] pointsX;

	protected double[] pointsY;
	
	protected int[] scaledPointsX;
	
	protected int[] scaledPointsY;

	public PolyPointsShape(long id, JID a, Color c, int height) {
		super(id, a, c, height);
	}


	public String getCoordinates() {
		return PointsOperations.pointsToString(pointsX, pointsY);
	}
	
	/**
	 * @param points
	 *            coordinates in String: x coordinate of the first Point, y
	 *            coordinate of the first Point, x coordinate of the second
	 *            Point, y coordinate of the second Point...
	 */
	public void setCoordinates(String points) {
		String[] result = (points.split("\\s"));
		pointsX = new double[result.length / 2];
		pointsY = new double[result.length / 2];
		for (int i = 0; i < result.length / 2; i++) {
			pointsX[i] = Double.parseDouble(result[2 * i]);
			pointsY[i] = Double.parseDouble(result[2 * i + 1]);
		}
	}

	public boolean isInsideTheRect(Point2D.Double upperLeft, Point2D.Double bottomRight) {
		return isPolylineInsideRect(pointsX, pointsY, upperLeft, bottomRight);
	}

	public Point2D.Double getDisplayPoint() {
		return getDisplayPointForPolyline(pointsX, pointsY);
	}

	public Point2D.Double getTheMostUpperLeftPoint() {
		return getTheMostUpperLeftPoint(pointsX, pointsY);
	}

	public Point2D.Double getTheMostBottomRightPoint() {
		return getTheMostBottomRightPoint(pointsX, pointsY);
	}
	public void changeCoordinates(Vector v) {
		for (int i = 0; i < pointsX.length; i++) {
			Point2D.Double p = v.addVectorToPoint(new Point2D.Double(pointsX[i], pointsY[i]));
			pointsX[i] = p.x;
			pointsY[i] = p.y;
		}
	}
	
	/**
	 * @param xScale
	 * @param yScale
	 */
	protected void ceilPoints(int xScale, int yScale) {
		scaledPointsX = new int[pointsX.length];
		scaledPointsY = new int[pointsY.length];
		for (int i = 0; i < pointsX.length; i++) {
			scaledPointsX[i] = ceil(pointsX[i], xScale);
			scaledPointsY[i] = ceil(pointsY[i], yScale);
		}
	}

	protected boolean isPolylineInsideRect(double[] pointsX, double[] pointsY,
			Point2D.Double upperLeft, Point2D.Double bottomRight) {
		for (int i = 0; i < pointsX.length; i++) {
			if (!(PointsOperations.pointIsInsideRect(new Point2D.Double(pointsX[i],
					pointsY[i]), upperLeft, bottomRight))) {
				return false;
			}
		}
		return true;
	}

	protected boolean isPointInPolylinesNeighbourhood(double[] pointsX,
			double[] pointsY, Point2D.Double p, int scaleX, int scaleY) {
		for (int i = 0; i < pointsX.length - 1; i++) {
			if (isPointInLinesNeighbourhood(new Point2D.Double(pointsX[i], pointsY[i]),
					new Point2D.Double(pointsX[i + 1], pointsY[i + 1]), p, scaleX, scaleY)) {
				return true;
			}
		}
		// check for the last pair of points
		if (pointsX.length - 2 >= 0) {
			return (isPointInLinesNeighbourhood(new Point2D.Double(
					pointsX[pointsX.length - 2], pointsY[pointsX.length - 2]),
					new Point2D.Double(pointsX[pointsX.length - 1],
							pointsY[pointsX.length - 1]), p, scaleX, scaleY));
		} else {
			// TODO sprawdz, czy jeden punkt mozna zaznaczyc
			return false;
		}
	}

	protected Point2D.Double getDisplayPointForPolyline(double[] pointsX, double[] pointsY) {
		if (pointsX.length == 1) {
			return new Point2D.Double(pointsX[0], pointsY[0]);
		} else {
			int i = pointsX.length / 2;
			return PointsOperations.getMiddlePoint(new Point2D.Double(pointsX[i - 1],
					pointsY[i - 1]), new Point2D.Double(pointsX[i], pointsY[i]));
		}
	}
}
