package nu.fw.jeti.plugins.drawing.shapes;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.geom.Point2D;

import nu.fw.jeti.jabber.JID;

/**
 * This class represents the ellipse.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class Polygon extends PolyPointsShape {

	/**
	 * Tag used to send this shape.
	 */
	public static final String NAME = "polygon";

	/**
	 * Constructs a new polygon and sets all the fields.
	 * 
	 * @param id
	 *            shape's id
	 * @param author
	 *            shape's author
	 * @param coords
	 *            shape's coordinates converted to String
	 * @param color
	 *            shape's color
	 * @param height
	 *            shape's height in the picture's array
	 * @param filled
	 *            <tt>true</tt> if the shape is filled
	 */
	public Polygon(long id, JID author, String coords, Color color, int height,
			boolean filled) {
		super(id, author, color, height);
		setCoordinates(coords);
		isFilled = filled;
	}

	/**
	 * Constructs a new polygon and sets all the fields.
	 * 
	 * @param points
	 *            Vector of Points determining the polygon
	 * @param c
	 *            shape's color
	 * @param a
	 *            shape's author
	 * @param filled
	 *            <tt>true</tt> if the shape is filled
	 */
	public Polygon(java.util.Vector<Point2D.Double> points, Color c, JID a,
			boolean filled) {
		super(generateId(), a, c, INCORRECT_HEIGHT);
		if (pointsX == null || pointsY == null) {
			pointsX = new double[points.size()];
			pointsY = new double[points.size()];
		}
		for (int i = 0; i < points.size(); i++) {
			pointsX[i] = (points.get(i)).x;
			pointsY[i] = (points.get(i)).y;
		}
		isFilled = filled;
	}

	public void draw(Graphics g, int xScale, int yScale) {		
		ceilPoints(xScale, yScale);
		
		if (isFilled) {
			g.setColor(color);
			g.fillPolygon(scaledPointsX, scaledPointsY, scaledPointsX.length);
		}
		if (isFilled) {
			g.setColor(borderColor);
		} else {
			g.setColor(color);
		}
		g.drawPolygon(scaledPointsX, scaledPointsY, scaledPointsX.length);

	}

	public void drawAsSelected(Graphics g, int xScale, int yScale) {
		draw(g, xScale, yScale);
		g.setColor(selectionColor);
		for (int i = 0; i < scaledPointsX.length; i++) {
			g.drawRect(scaledPointsX[i] - 2, scaledPointsY[i] - 2, 4, 4);
		}
	}

	public String getName() {
		return NAME;
	}

	public boolean isPointInShapesNeighbourhood(Point2D.Double p, int scaleX, int scaleY) {
		if (isFilled) {
			return isPointInShapesInterior(p, scaleX, scaleY);
		} else {
			return isPointInPolylinesNeighbourhood(pointsX, pointsY, p, scaleX, scaleY);		
		}
	}

	@Override
	public boolean isPointInShapesInterior(Point2D.Double p, int scaleX, int scaleY) {
		int scale = 1000;
		ceilPoints(scale, scale);
		Point scaledPoint = ceil(p, scale, scale);
		java.awt.Polygon polygon = new java.awt.Polygon(scaledPointsX, scaledPointsY, scaledPointsX.length);
		return polygon.contains(scaledPoint);
	}
}
