/*
 * Created on 2006-02-28
 *
 */
package nu.fw.jeti.plugins.drawing.shapes;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.geom.Point2D;
import java.util.Vector;

import nu.fw.jeti.jabber.JID;

/**
 * This class represents the polyline.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class Polyline extends PolyPointsShape {
	
	/**
	 * Tag used to send this shape.
	 */
	public static final String NAME = "polyline";

	/**
	 * Constructs a new polyline and sets all the fields.
	 * 
	 * @param id
	 *            shape's id
	 * @param a
	 *            shape's author
	 * @param points
	 *            shape's coordinates converted to String
	 * @param c
	 *            shape's color
	 * @param height
	 *            shape's height in the picture's array
	 */
	public Polyline(long id, JID a, String points, Color c, int height) {
		super(id, a, c, height);
		setCoordinates(points);
	}

	/**
	 * Constructs a new polyline and sets all the fields.
	 * 
	 * @param id
	 *            shape's id
	 * @param a
	 *            shape's author
	 * @param x
	 *            array of x coordinates of the Points
	 * @param y
	 *            array of y coordinates of the Points
	 * @param c
	 *            shape's color
	 */
	public Polyline(long id, JID a, double[] x, double[] y, Color c) {
		super(id, a, c, INCORRECT_HEIGHT);
		pointsX = x;
		pointsY = y;
	}

	/**
	 * Constructs a new polyline and sets all the fields.
	 * 
	 * @param points
	 *            Voctor of Point2D.Double that determine the polyline
	 * @param c
	 *            shape's color
	 * @param a
	 *            shape's author
	 */
	public Polyline(Vector<Point2D.Double> points, Color c, JID a) {
		super(generateId(), a, c, INCORRECT_HEIGHT);
		if (pointsX == null || pointsY == null) {
			pointsX = new double[points.size()];
			pointsY = new double[points.size()];
		}
		for (int i = 0; i < points.size(); i++) {
			pointsX[i] = (points.get(i)).x;
			pointsY[i] = (points.get(i)).y;
		}
	}

	public void draw(Graphics g, int xScale, int yScale) {
		ceilPoints(xScale, yScale);
		
		g.setColor(color);
		g.drawPolyline(scaledPointsX, scaledPointsY, scaledPointsX.length);
	}

	public String getName() {
		return NAME;
	}

	public void drawAsSelected(Graphics g, int xScale, int yScale) {
		draw(g, xScale, yScale);		
		g.setColor(selectionColor);
		g.drawRect(scaledPointsX[0] - 2, scaledPointsY[0] - 2, 4, 4);
		int lastSelectedX = scaledPointsX[0];
		int lastSelectedY = scaledPointsY[0];
		for (int i = 0; i < scaledPointsX.length - 1; i++) {
			if ((Math.abs(scaledPointsX[i] - lastSelectedX) > 10 || Math
					.abs(scaledPointsY[i] - lastSelectedY) > 10)) {
				lastSelectedX = scaledPointsX[i];
				lastSelectedY = scaledPointsY[i];
				g.drawRect(lastSelectedX - 2, lastSelectedY - 2, 4, 4);
			}
		}
		g.drawRect(scaledPointsX[scaledPointsX.length - 1] - 2,
				scaledPointsY[scaledPointsY.length - 1] - 2, 4, 4);
	}


	public boolean isPointInShapesNeighbourhood(Point2D.Double p, int scaleX, int scaleY) {
		return isPointInPolylinesNeighbourhood(pointsX, pointsY, p, scaleX, scaleY);
	}

	@Override
	public boolean isPointInShapesInterior(Point2D.Double p, int scaleX, int scaleY) {
		return false;
	}
}
