package nu.fw.jeti.plugins.drawing.shapes;

import java.util.Vector;


/**
 * This class represents the priotiry stack for shapes.  It manages adding
 * the shapes in a good order (given by the shape's height).
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class ShapePriorityStack {
	Vector<Shape> stack;
	
	/**
	 * Creates an empty priority stack.
	 *
	 */
	public ShapePriorityStack() {
		stack = new Vector<Shape>();
	}
	
	/**
	 * Adds the shape keeping the order given by shape's height.
	 * 
	 * @param s shape which will be added
	 */
	public void add(Shape s) {
		if (s != null && !stack.contains(s)) {
			for (int i = stack.size() - 1; i >= 0; i--) {
				if (s.isHigherThan(stack.get(i))) {
					stack.add(i+1, s);
					return;
				}
			}
			// we did not add the shape yet
			stack.add(0, s);
		}
	}
	
	/**
	 * Removes the given shape.
	 * 
	 * @param s shape that will be romoved
	 */
	public void remove(Shape s) {
		stack.remove(s);
	}
	
	/**
	 * Gets the stack's size.
	 * 
	 * @return stack's size
	 */
	public int size() {
		return stack.size();
	}
	
	/**
	 * Returns the element at the specified position in this stack.
	 * 
	 * @param i index of element to return
	 * @return shape at the specified index
	 */
	public Shape get(int i) {
		return stack.get(i);
	}

}
