package nu.fw.jeti.plugins.drawing.shapes;

import java.util.Stack;

import nu.fw.jeti.plugins.drawing.shapes.actions.Action;

/**
 * This class manages undoing and redoing {@link Action}s.  It stores actions
 * done by me and gets the action that can be undone and redone.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class UndoRedoStackRepresentation {
	private Stack<Action> doneStack;
	private Stack<Action> undoneStack;
	
	/**
	 * Creates an empty stack.
	 */
	public UndoRedoStackRepresentation() {
		doneStack = new Stack<Action>();
		undoneStack = new Stack<Action>();
	}
	
	/**
	 * Takes the action that should be undone.
	 * 
	 * @return action that should be undone<br>
	 * <tt>null</tt> if there are no more actions to undo
	 */
	public synchronized Action undo() {
		Action a = null;
		if (!doneStack.empty()) {
			a = doneStack.pop();
			undoneStack.add(a);
		}
		return a;
	}
	
	/**
	 * Takes the action that should be redone.
	 * 
	 * @return action that should be redone<br>
	 * <tt>null</tt> if there are no more actions to redo
	 */
	public synchronized Action redo() {
		Action a = null;
		if (!undoneStack.empty()) {
			a = undoneStack.pop();
			doneStack.add(a);
		}
		return a;
	}

	/**
	 * Adds the action to the actions that can be undone. None action can be
	 * redone at this moment.  An action can be redone only if an action was
	 * undone.
	 * 
	 * @param a action to be added
	 */
	public synchronized void addAction(Action a) {
		undoneStack.clear();
		doneStack.add(a);		
	}
}
