package nu.fw.jeti.plugins.drawing.shapes.actions;

import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

import nu.fw.jeti.plugins.drawing.shapes.PicturesContent;

/**
 * This class represents a priority stack containing {@link Action}s.  It
 * manages adding the actions in a good order (given by the action's counter).
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class ActionPriorityStack {
	private Stack<Action> stack;
	
	/**
	 * Creates a new priority stack.
	 */
	public ActionPriorityStack() {
		stack = new Stack<Action>();
	}

	/**
	 * Adds the action keeping the order given by action's counter.  It adds
	 * the action on a good position. All the actions that should be done after
	 * this action (that have bgger counters) and were on the stack before,
	 * should be undone. It creates complementary actions for them and executes
	 * them. After adding the action, it executes all the actions that should
	 * be executed after this action.
	 * 
	 * @param a action that will be added
	 * @param shapeBoards list of pictures that will be changed by
	 * the complemetary and regular actions
	 */
	public void add(Action a, List<PicturesContent> shapeBoards) {
		int position = 0;
		for (int i = stack.size() - 1; i >= 0; i--) {
			Action actionFromStack = stack.get(i);
			if (a.happenedBefore(actionFromStack)) {
				Action complementaryAction = actionFromStack.giveComplementaryAction(
						actionFromStack.getTime(), actionFromStack.getCounter());
				// it doesn't matter which counter we will use
				for (PicturesContent content : shapeBoards) {
					complementaryAction.execute(content);
				}				
			} else {
				position = i+1;
				break;
			}
		}
		stack.add(position, a);
		
		// we execute all the actions which were undone 
		for (int i = position+1; i < stack.size(); i++) {
			for (PicturesContent content : shapeBoards) {
				stack.get(i).execute(content);
			}			
		}
	}
	
	public void add(Action a,PicturesContent shapeBoard) {
		List<PicturesContent> list = new ArrayList<PicturesContent>();
		list.add(shapeBoard);
		add(a, list);
	}
	
	/**
	 * Returns stack's size.
	 * @return stack's size
	 */
	public int size() {
		return stack.size();
	}
	
	/**
	 * Returns the element at the specified position in this stack.
	 * 
	 * @param i index of element to return
	 * @return action at the specified index
	 */
	public Action elementAt(int i) {
		return stack.elementAt(i);
	}
}
