package nu.fw.jeti.plugins.drawing.shapes.actions;

import java.awt.Color;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.plugins.drawing.elements.ColorSettingMessage;
import nu.fw.jeti.plugins.drawing.elements.PictureMessage;
import nu.fw.jeti.plugins.drawing.shapes.PicturesContent;
import nu.fw.jeti.plugins.drawing.shapes.Shape;

/**
 * This class represents an Action that sets the given shapes' color.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class ColorSetting extends Action {
	private Color color;
	private Map<Long, Color> oldColorList;
	private Map<Long, Color> newColorList;
	
	/**
	 * Creates a new Action.  It sets all the shapes the same color.
	 * Only color is initialized, newColorList not.
	 * 
	 * @param sIdList list of shapes' ids that are changed by this action
	 * @param allS map of all shapes that have ever been created
	 * @param c new color for all the shapes
	 * @param t  action's time
	 * @param author Action's author
	 * @param nr action's counter
	 */
	public ColorSetting(List<Long> sIdList, Map<Long, Shape> allS,
			Color c, String t, String author, int nr) {
		super(sIdList, allS, t, author, nr);
		color = c;
		oldColorList = new HashMap<Long, Color>();
		
		setOldColorList();
	}
	
	/**
	 * Generated only by the getComplementaryAction method. All the shapes
	 * can have different colors. Only newColorList is initialized, color not.
	 * 
	 * @param sIdList list of shapes' ids that are changed by this action
	 * @param allS map of all shapes that have ever been created
	 * @param newColors list of colors that will be set to every shape; there
	 * is a color for every shape from the shapeIdList
	 * @param t action's time
	 * @param author Action's author
	 * @param nr action's counter
	 */
	public ColorSetting(List<Long> sIdList, Map<Long, Shape> allS,
			Map<Long, Color> newColors, String t, String author, int nr) {
		super(sIdList, allS, t, author, nr);
		newColorList = newColors;
		oldColorList = new HashMap<Long, Color>();
		
		setOldColorList();
	}

	public void execute(PicturesContent shapesBoard) {
//		setOldColorList();

		if (color != null) {
			// there is only one color we set
			for (long shapeId : shapeIdList) {
				Shape s = allShapes.get(shapeId);
				if (s != null) {
					s.setColor(color);
				}
			}
			return;
		}
		if (newColorList != null) {
			for (long shapeId : shapeIdList) {
				Shape s = allShapes.get(shapeId);
				if (s != null) {
					Color c = newColorList.get(shapeId);
					s.setColor(c);
				}
			}
		}
		
		shapesBoard.pictureChanged();
	}

	public Action giveComplementaryAction(String time, int counter) {
		return new ColorSetting(shapeIdList, allShapes, oldColorList,
				time, actionsAuthor, counter);
	}

	public List<PictureMessage> getPictureMessageList(List<JID> receivers, String thread) {
		List<PictureMessage> list = new ArrayList<PictureMessage>();
		
		for (JID jid : receivers) {		
			if (color != null) {
				list.add(new ColorSettingMessage(jid, shapeIdList, color, thread,
						time, actionsAuthor, counter++));
			} else if (newColorList != null) {
				for (long id : shapeIdList) {
					list.add(new ColorSettingMessage(jid, createShapeIdList(id),
							newColorList.get(id), thread, time, actionsAuthor, counter++));
				}
			} else {
				throw new IllegalArgumentException();
			}
		}
		return list;
	}
	
	private void setOldColorList() {
		for (Long id : shapeIdList) {
			Shape s = allShapes.get(id);
			if (s != null) {
				oldColorList.put(id, s.getColor());			
			}	
		}
	}

}
