package nu.fw.jeti.plugins.drawing.shapes.actions;

import java.awt.Color;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.plugins.drawing.elements.CreationMessage;
import nu.fw.jeti.plugins.drawing.elements.PictureMessage;
import nu.fw.jeti.plugins.drawing.shapes.PicturesContent;
import nu.fw.jeti.plugins.drawing.shapes.Shape;

/**
 * This class represents an Action that creates new shapes and put them
 * to the allShapes list.  It stores the lists of origin coordinates
 * and colors, because if we recreate the history, the shapes can be already
 * changed. That ensures that all the shapes will have the same parameters
 * after creation.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class Creation extends Action {
	private String coord;
	private Color color;

	/**
	 * It creates a new creation action.  It sets all the data it needs
	 * to recreate the shapes.
	 * 
	 * @param shape shape that is changed by this action
	 * @param allShapes map of all shapes that have ever been created
	 * @param t action's time
	 * @param author Action's author
	 * @param nr action's counter
	 */
	public Creation(Shape shape, Map<Long, Shape> allShapes,
			String t, String author, int nr) {
		super(createShapeIdList(shape.getShapeId()), allShapes, t, author, nr);
		
		coord = shape.getCoordinates();
		color = shape.getColor();
		if (allShapes.get(shape.getShapeId()) == null) {
			allShapes.put(shape.getShapeId(), shape);
		}
	}
	
	public void execute(PicturesContent shapesBoard) {
		long shapeId = shapeIdList.get(0);
		Shape s = allShapes.get(shapeId);
		s.setCoordinates(coord);
		s.setColor(color);
	}

	public Deletion giveComplementaryAction(String t, int c) {
		return new Deletion(shapeIdList, allShapes, t, actionsAuthor, c);
	}

	public List<PictureMessage> getPictureMessageList(List<JID> receivers,
			String thread) {
		List<PictureMessage> list = new ArrayList<PictureMessage>();

		long shapeId = shapeIdList.get(0);
		Shape s = allShapes.get(shapeId);
		for (JID jid : receivers) {
			list.add(new CreationMessage(jid, shapeId, s.getName(),
					coord, color, s.isFilled(), s.getAuthor(), thread,
					s.getDisplayHeight(), time, actionsAuthor, counter));
		}
		
		return list;
	}

}
