package nu.fw.jeti.plugins.drawing.shapes.actions;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.plugins.drawing.elements.DisplayMessage;
import nu.fw.jeti.plugins.drawing.elements.PictureMessage;
import nu.fw.jeti.plugins.drawing.shapes.PicturesContent;
import nu.fw.jeti.plugins.drawing.shapes.Shape;

/**
 * This class represents an Action that displays the given shapes on
 * the picture.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class Display extends Action {
	private boolean displayAsSelected = false;

	/**
	 * Creates a new display action.
	 * 
	 * @param sIdList list of shapes' ids that are changed by this action
	 * @param allS map of all shapes that have ever been created
	 * @param t action's time
	 * @param author Action's author
	 * @param nr action's counter
	 */
	public Display(List<Long> sIdList, Map<Long, Shape> allS, String t,
			String author, int nr) {
		super(sIdList, allS, t, author, nr);
	}

	public void execute(PicturesContent shapesBoard) {
		List<Shape> list = new ArrayList<Shape>();

		for (long shapeId : shapeIdList) {
			Shape s = allShapes.get(shapeId);
			if (s != null) {
				list.add(s);
			}
		}
		if (!list.isEmpty()) {
			// we had this shape already
			shapesBoard.addShapeList(list, displayAsSelected);
		}
		
	}

	public Action giveComplementaryAction(String time, int counter) {
		return new Deletion(shapeIdList, allShapes, time, actionsAuthor, counter);
	}

	public List<PictureMessage> getPictureMessageList(List<JID> receivers,
			String thread) {
		List<PictureMessage> list = new ArrayList<PictureMessage>();
		for (JID jid : receivers) {
			list.add(new DisplayMessage(jid, shapeIdList, thread, time,
					actionsAuthor, counter));
		}
		return list;
	}
	
	/**
	 * Sets <tt>true</tt> if the shapes should be displayed as selected
	 * (for example after pasting).
	 * 
	 * @param b <tt>true</tt> if the shapes should be displayed as selected
	 */
	public void setDisplayAsSelected(boolean b) {
		displayAsSelected = b;
	}

}
