package nu.fw.jeti.plugins.drawing.shapes.actions;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.plugins.drawing.Plugin;
import nu.fw.jeti.plugins.drawing.elements.PictureMessage;
import nu.fw.jeti.plugins.drawing.geometry.Vector;
import nu.fw.jeti.plugins.drawing.shapes.PicturesContent;
import nu.fw.jeti.plugins.drawing.shapes.Shape;

/**
 * This class represents an Action that pastes the given shapes.  It creates
 * new shapes, which have the same parameters as the copied shapes, then it
 * translates their coordinates and displays them.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class Pasting extends Action {
	private List<Creation> creationActionsList;

	private Display displayAction;

	/**
	 * Creates a new pasting action.  It creates new shapes and then displays
	 * them.
	 * 
	 * @param shapeList
	 *            shapes that will be copied
	 * @param allS
	 *            map of all shapes that have ever been created
	 * @param vector
	 *            how we traslate the pasted shapes
	 * @param t
	 *            action's time
	 * @param author 
	 * 			  action's author
	 * @param nr
	 *            action's counter
	 */
	public Pasting(List<Shape> shapeList, Map<Long, Shape> allS, Vector vector,
			String t, String author, int nr) {
		super(getShapeIdList(shapeList), allS, t, author, nr);
		creationActionsList = new ArrayList<Creation>();
		List<Long> newShapesIdList = new ArrayList<Long>();
		int displayCounter = counter + shapeList.size();

		for (int i = 0; i < shapeList.size(); i++) {
			// they should be in the good order: from the oldest to the newest
			Shape oldS = shapeList.get(i);
			Shape newShape = Plugin.createShape(oldS.getName(), Shape
					.generateId(), oldS.getAuthor(), oldS.getCoordinates(),
					oldS.getColor(), counter, oldS.isFilled());
			newShape.changeCoordinates(vector);
			Creation c = new Creation(newShape, allShapes, time,
					actionsAuthor, counter);
			creationActionsList.add(c);
			newShapesIdList.add(newShape.getShapeId());
			counter++;
		}
		
		displayAction = new Display(newShapesIdList, allShapes, time,
				actionsAuthor, displayCounter);
		displayAction.setDisplayAsSelected(true);
	}

	@Override
	public void execute(PicturesContent shapesBoard) {
		for (Creation creation : creationActionsList) {
			creation.execute(shapesBoard);
		}
		displayAction.execute(shapesBoard);
	}

	@Override
	public Action giveComplementaryAction(String time, int counter) {
		return displayAction.giveComplementaryAction(time, counter);
	}

	@Override
	public List<PictureMessage> getPictureMessageList(List<JID> receivers,
			String thread) {
		List<PictureMessage> list = new ArrayList<PictureMessage>();
		for (Creation creation : creationActionsList) {
			list.addAll(creation.getPictureMessageList(receivers, thread));
		}
		list.addAll(displayAction.getPictureMessageList(receivers, thread));
		return list;
	}

}
