package nu.fw.jeti.plugins.drawing.shapes.actions;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.plugins.drawing.elements.PictureMessage;
import nu.fw.jeti.plugins.drawing.elements.TranslationMessage;
import nu.fw.jeti.plugins.drawing.geometry.Vector;
import nu.fw.jeti.plugins.drawing.shapes.PicturesContent;
import nu.fw.jeti.plugins.drawing.shapes.Shape;

/**
 * This class represents an Action that translates (moves) the given shapes.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class Translation extends Action {
	private Vector transVector;

	/**
	 * Creates a new translation action.
	 * 
	 * @param sIdList list of shapes' ids that are changed by this action
	 * @param allS map of all shapes that have ever been created
	 * @param vector how we translate the shapes
	 * @param t action's time
	 * @param author Action's author
	 * @param nr action's counter
	 */
	public Translation(List<Long> sIdList, Map<Long, Shape> allS,
			Vector vector, String t, String author, int nr) {
		super(sIdList, allS, t, author, nr);
		transVector = vector;
	}

	public void execute(PicturesContent shapesBoard) {
		for (long shapeId : shapeIdList) {
			Shape s = allShapes.get(shapeId);
			if (s != null) {
				s.changeCoordinates(transVector);
			}
		}
		shapesBoard.pictureChanged();
	}

	public Action giveComplementaryAction(String t, int c) {
		return new Translation(shapeIdList, allShapes,
				transVector.getReverseVector(), t, actionsAuthor, c);
	}

	public List<PictureMessage> getPictureMessageList(List<JID> receivers,
			String thread) {
		List<PictureMessage> list = new ArrayList<PictureMessage>();
		for (JID jid : receivers) {
			list.add(new TranslationMessage(jid, shapeIdList, 
					transVector, thread, time, actionsAuthor, counter));
		}
		return list;
	}

}
