package nu.fw.jeti.plugins.drawing.ui;

import java.awt.Dimension;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import nu.fw.jeti.plugins.drawing.shapes.PictureChat;
import nu.fw.jeti.plugins.drawing.shapes.PicturesContent;
import nu.fw.jeti.plugins.drawing.shapes.actions.Action;
import nu.fw.jeti.plugins.drawing.shapes.actions.ActionPriorityStack;
import nu.fw.jeti.plugins.drawing.shapes.actions.Creation;
import nu.fw.jeti.plugins.drawing.shapes.actions.Translation;
import nu.fw.jeti.util.I18N;

public class HistoryPanel extends JPanel {
	private PictureChat chat;
	private PicturesContent currentPicture;
	private OneHistoryPicturePanel currentPanel;
	private JPanel mainPanel;
	private JScrollPane scrollPane;
	private List<OneHistoryPicturePanel> panelList;
	
	private ActionPriorityStack actions;
	private int mode = ACTIONS_NUMBER_MODE;
	private int actionsCounter;
	private Set<UserRepresentation> displayedUsers;

	private static final int ACTIONS_NUMBER_MODE = 1;
	private static final int TIME_MODE = 2;
	private int ACTIONS_NUMBER = 5;
	private int SECONDS = 60;
	
	private static final DateFormat DATE_FORMAT = 
		DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.MEDIUM);
	
	private static final long serialVersionUID = 4671029090794820300L;

	public HistoryPanel(PictureChat pictureChat) {
		chat = pictureChat;
		actions = new ActionPriorityStack();
		currentPicture = new PicturesContent();		
		panelList = new ArrayList<OneHistoryPicturePanel>();
		
		mainPanel = new JPanel();
		mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));

		createScrollPane();
		
		add(scrollPane);
		addSelectUsersToHistoryButton();
		
		hideHistory();
	}

	private void addSelectUsersToHistoryButton() {		
		SelectUsersToHistoryAction action = new SelectUsersToHistoryAction(
				"<html><center>" + I18N.getText("drawing","Display") + "<br>" + I18N.getText("drawing","selected_users"),
				null,
				I18N.getText("drawing","Display_in_history_only_selected_users"), null,
				chat, this);
		add(new JButton(action));
	}

	private void createScrollPane() {
		scrollPane = new JScrollPane(mainPanel) {
			
		    public Dimension getPreferredSize() {
		        return new Dimension(170, 425);
		    }
		};
	}

	public void showHistory() {
		setVisible(true);
	}
	
	public void hideHistory() {
		setVisible(false);
	}
	
	public void addAction(Action a) {
		actions.add(a, currentPicture);
		changePicture(a);
		increaseActionsCounter(a);
		mainPanel.repaint();
//		invalidate();
		repaint();
	}

	private void increaseActionsCounter(Action a) {
		if (isNotCreation(a)) {
			actionsCounter++;
		}
	}
	
	private boolean isNotCreation(Action a) {
		return !(a instanceof Creation);
	}
	
	private boolean isNotTranslation(Action a) {
		return !(a instanceof Translation);
	}
	
	private void changePicture(Action a) {
		switch (mode) {
		case ACTIONS_NUMBER_MODE:
			if (actionsCounter % ACTIONS_NUMBER == 0 && isNotCreation(a)) {
				// we add a new panel
				createNewPanel(a);
			}
			executeAction(a);
			break;

		case TIME_MODE:
			
			break;
			
		default:
			break;
		}
	}

	private String getTime(Action a) {
		String time = a.getTime();
//		Date date;
//		String result = "";
//		try {
//			date = DATE_FORMAT.parse(time);
//			result = DATE_FORMAT.format(date);
//		} catch (ParseException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		}
		return a.getTime();
			
	}
	
	private void executeAction(Action a) {
		if (isNotTranslation(a)) {
			a.execute(currentPicture);
		}
		if (isNotCreation(a)) {
			currentPanel.changeFrameTitle(getTime(a));
		}
	}

	private void createNewPanel(Action a) {
		String title = getTime(a);
		PicturesContent tmpPicture =  copyCurrentPicture();
		if (currentPanel != null) {
			currentPanel.setPicture(tmpPicture);
		}
		
		currentPanel = new OneHistoryPicturePanel(currentPicture, title, displayedUsers);
		panelList.add(currentPanel);
//		currentPanel.setVisible(true);
		mainPanel.add(currentPanel);
//		currentPanel.setVisible(true);
		scrollPane.revalidate();
	}
	
	private PicturesContent copyCurrentPicture() {
		return currentPicture.copy();
	}

	private void clearPanel() {
		scrollPane.removeAll();
	}
	
//	public Dimension getMinimumSize() {
//        return getPreferredSize();
//    }
//	
    public Dimension getPreferredSize() {
        return new Dimension(170, 480);
    }
//    
//    public Dimension getMaximumSize() {
//        return getPreferredSize();
//    }

	public void selectUsersToDisplay(Set<UserRepresentation> selectedUsers) {
		if (!selectedUsers.isEmpty()) {
			displayedUsers = selectedUsers;
			for (OneHistoryPicturePanel panel : panelList) {
				panel.setDisplayAuthors(selectedUsers);
				panel.repaint();
			}
			scrollPane.revalidate();
		}
	}
}
