package nu.fw.jeti.plugins.drawing.ui;

import java.awt.Color;
import java.awt.geom.Point2D.Double;

import javax.swing.ImageIcon;

import nu.fw.jeti.plugins.drawing.geometry.PointsOperations;
import nu.fw.jeti.plugins.drawing.shapes.Line;
import nu.fw.jeti.plugins.drawing.shapes.PictureChat;

/**
 * This class represents a mode that draws a
 * {@link nu.fw.jeti.jabber.elements.drawing.Line}.  A user presses
 * the mouse and this Point becomes the first point of the line. Then he drags
 * the mouse and draws the possible lines. When he releases
 * the mouse the line is drawn, memorized in the history and sent to the other
 * users.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class LineMode extends Mode {

	/**
	 * Creates a line mode.
	 * 
	 * @param c drawing chat
	 * @param b board where the shapes and the behaviour is displayed
	 * @param text text that is displayed on the button and menu item
	 * @param i icon that is displayed on the button and menu item
	 * @param desc description of the button shown on the tool tips
	 * @param m mnemonic for the mode
	 */
	public LineMode(PictureChat c, Board b, String text, ImageIcon i, String desc, Integer m) {
		super(c, b, text, i, desc, m);
	}
	
	/**
	 * Selects the first point of the line.
	 */
	public void mousePressed(Double point, Color c, int scaleX, int scaleY) {
		super.mousePressed(point, c, scaleX, scaleY);
		chat.unselect();
	}

	/**
	 * After releasing the mouse the ellipse is memorized in the history, drawn
	 * and sent to the other users.
	 */
	public void mouseReleased(Double point, int scaleX, int scaleY) {
		end = point;
		Line line = new Line(start, end, color, chat.getMyJID());
		chat.draw(line);
		board.setCurrentShape(null);
	}

	/**
	 * Selects the second point of the line and draws it. It does not draw
	 * the previous proposition of the line.
	 */
	public void mouseDragged(Double point) {
		end = point;
		if (board.getCurrentShape() != null) {
			(board.getCurrentShape()).setCoordinates(PointsOperations.pointsToString(start, end));
		} else {
			board.setCurrentShape(new Line(start, end, color, chat.getMyJID()));
		}
		board.repaint();
	}

}
