package nu.fw.jeti.plugins.drawing.ui;

import java.awt.Color;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.awt.geom.Point2D.Double;

import javax.swing.ImageIcon;

import nu.fw.jeti.plugins.drawing.shapes.PictureChat;

/**
 * This class represents an abstract drawing mode.  When a user chooses
 * a drawing mode, all the action he does after that will be handled by
 * this mode (except from choosing another mode).
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public abstract class Mode {
	Point2D.Double start, end;
	PictureChat chat;
	Board board;
	String buttonText;
	ImageIcon icon;
	String description;
	Integer mnemonic;
	Color color;
	
	/**
	 * Creates a new mode.
	 * 
	 * @param c drawing chat
	 * @param b board where the shapes and the behaviour is displayed
	 * @param text text that is displayed on the button and menu item
	 * @param i icon that is displayed on the button and menu item
	 * @param desc description of the button shown on the tool tips
	 * @param m mnemonic for the mode
	 */
	public Mode(PictureChat c, 
			Board b, 
    		String text, 
    		ImageIcon i,
    		String desc,
    		Integer m) {
		chat = c;
		board= b;
		buttonText = text;
		icon = i;
		description = desc;
		mnemonic = m;
	}
	
	/**
	 * Implements what should be done and displayed when the user pressed
	 * the mouse being in this mode.
	 * 
	 * @param point scaled point pointed by the mouse
	 * @param c current color
	 * @param scaleX TODO
	 * @param scaleY TODO
	 */
	public void mousePressed(Point2D.Double point, Color c, int scaleX, int scaleY) {
		start =point;
		color = c;
	}
	
	/**
	 * Implements what should be done and displayed when the user clicked
	 * the mouse being in this mode.
	 * 
	 * @param e event of the mouse click
	 * @param c current color
	 * @param point scaled point pointed by the mouse
	 */
	public void mouseClicked(MouseEvent e, Color c, Double point, int scaleX, int scaleY) {}

	/**
	 * Implements what should be done and displayed when the user released
	 * the mouse being in this mode.
	 * 
	 * @param point scaled point pointed by the mouse
	 * @param scaleX TODO
	 * @param scaleY TODO
	 */
	public void mouseReleased(Point2D.Double point, int scaleX, int scaleY) {}
	
	/**
	 * Implements what should be done and displayed when the user dragged
	 * the mouse being in this mode.
	 * 
	 * @param point scaled point pointed by the mouse
	 */
	public void mouseDragged(Point2D.Double point) {}
	
	/**
	 * Implements what should be done if the user chooses another mode.
	 */
	public void dispose() {
		board.setCurrentShape(null);
		board.repaint();
	}

	/**
	 * Gets the board.
	 * 
	 * @return picture's board
	 */
	public Board getBoard() {
		return board;
	}
	
	/**
	 * Get mode's text.
	 * 
	 * @return mode's text
	 */
	public String getButtonText() {
		return buttonText;
	}

	/**
	 * Gets mode's description.
	 * 
	 * @return mode's description
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * Gets mode's icon.
	 * 
	 * @return mode's icon
	 */
	public ImageIcon getIcon() {
		return icon;
	}

	/**
	 * Gets mode's mnemonic.
	 * 
	 * @return mode's mnemonic
	 */
	public Integer getMnemonic() {
		return mnemonic;
	}

	/**
	 * Implements what should be done and displayed when the user moved
	 * the mouse being in this mode.
	 * 
	 * @param point scaled point pointed by the mouse
	 */
	public void mouseMoved(Point2D.Double point) {}

}
