package nu.fw.jeti.plugins.drawing.ui;

import java.awt.Dimension;
import java.awt.Graphics;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import nu.fw.jeti.plugins.drawing.shapes.PicturesContent;

public class OneHistoryPicturePanel extends JPanel {
	private PicturesContent picture;
	private OneHistoryPicturePanelContent content;
	private TitledBorder titledBorder;
	private Set<UserRepresentation> displayedAuthors;
	private Dimension size = new Dimension(
			PICTURE_WIDTH + 12, PICTURE_HEIGHT + 32);
	
	private static final int PICTURE_WIDTH = 128;
	private static final int PICTURE_HEIGHT = 96;
	
	private static final long serialVersionUID = -711206238878186129L;

	public OneHistoryPicturePanel(PicturesContent picturesContent,
			String title, Set<UserRepresentation> usersList) {
		picture = picturesContent;
		displayedAuthors = usersList;
		content = new OneHistoryPicturePanelContent();
				
		content.setVisible(true);
		add(content);
		setFrame(this, title);
		setVisible(true);
	}
	
	public PicturesContent getPicture() {
		return picture;
	}
	
	public void changeFrameTitle(String newTitle) {
		titledBorder.setTitle(newTitle);
	}
	
	private void setFrame(JComponent comp, String title) {
		titledBorder = BorderFactory.createTitledBorder(title);
		comp.setBorder(titledBorder);
	}

	public void setPicture(PicturesContent picturesContent) {
		picture = picturesContent;
		
	}

	public void setDisplayAuthors(Set<UserRepresentation> selectedUsers) {
		displayedAuthors = selectedUsers;
	}
	
	public Dimension getMinimumSize() {
        return getPreferredSize();
    }
    public Dimension getPreferredSize() {
        return size;
    }
    public Dimension getMaximumSize() {
        return getPreferredSize();
    }


    
    private class OneHistoryPicturePanelContent extends JPanel {

		private static final long serialVersionUID = 5347205750717912204L;

		@Override
		protected void paintComponent(Graphics g) {
			super.paintComponent(g);
			if (displayedAuthors != null) {
				picture.drawPicture(g, PICTURE_WIDTH, PICTURE_HEIGHT, displayedAuthors);
			} else {
				picture.drawPicture(g, PICTURE_WIDTH, PICTURE_HEIGHT);
			}
		}

		public Dimension getMinimumSize() {
            return getPreferredSize();
        }
        public Dimension getPreferredSize() {
            return new Dimension(PICTURE_WIDTH, PICTURE_HEIGHT);
        }
        public Dimension getMaximumSize() {
            return getPreferredSize();
        }
	}
}
