/*
 * Created on 2006-03-22
 */
package nu.fw.jeti.plugins.drawing.ui;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.util.HashSet;
import java.util.List;
import java.util.Vector;

import javax.swing.*;
import javax.swing.text.AttributeSet;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.html.CSS;

import nu.fw.jeti.images.StatusIcons;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.plugins.drawing.shapes.PictureChat;
import nu.fw.jeti.plugins.drawing.shapes.PicturesContent;
import nu.fw.jeti.plugins.xhtml.fontchooser.FontDialog;
import nu.fw.jeti.util.I18N;
import nu.fw.jeti.util.Popups;

/**
 * This class implements the frame, where are all visible elements
 * of the drawing chat.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class PictureChatFrame extends JFrame {
	private Board board;
	private PictureChat chat;
	private Vector<Mode> drawingModes;
	
	private JFileChooser fileChooser;
	private File file;
	private ButtonGroup buttonGroup;
	private HistoryPanel historyPanel;
	
	JScrollPane pane;
	
	private static final long serialVersionUID = -8019354910612112821L;
	private static final int PICTURE_WIDTH = 640;
	private static final int PICTURE_HEIGHT = 480;
	
	/**
	 * Creates the drawing chat frame, creates the menu, toolbar, picture
	 * board and all the listeners. 
	 * 
	 * @param pictureChat
	 * @param picture
	 */
	public PictureChatFrame(PictureChat pictureChat, PicturesContent picture,
			HistoryPanel histPanel) {
		super();
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		setIconImage(StatusIcons.getImageIcon("jeti").getImage());
		chat = pictureChat;
		historyPanel = histPanel;
		
		buttonGroup = new ButtonGroup();
		JPanel panelForAll = new JPanel();
		panelForAll.setLayout(new BoxLayout(panelForAll, BoxLayout.Y_AXIS));
		JPanel boardPanel = createBoardAndHistory(picture);
		addMenuAndToolBar(panelForAll);
		
		panelForAll.add(boardPanel);
		getContentPane().add(panelForAll);
		pack();
		setLocationRelativeTo(null);
	}


	private void createDrawingModes() {
		drawingModes = new Vector<Mode>();
		
		drawingModes.add(new PolylineMode(chat, board,
				I18N.getText("drawing","Pen"),
				new ImageIcon(getClass().getResource("images/Pen24.png")),
				I18N.getText("drawing","Pen"), new Integer(KeyEvent.VK_P)));
		
		drawingModes.add(new LineMode(chat, board, I18N.getText("drawing","Line"),
				new ImageIcon(getClass().getResource("images/Line24.png")),
				I18N.getText("drawing","Line"), new Integer(KeyEvent.VK_L)));
		
		drawingModes.add(new FilledPolylineMode(chat, board, I18N.getText("drawing","Filled_Polyline"),
				new ImageIcon(getClass().getResource("images/FilledPolyline24.png")),
				I18N.getText("drawing","Filled_Polyline"), new Integer(KeyEvent.VK_I)));
		
		drawingModes.add(new RectangleMode(chat, board, I18N.getText("drawing","Rectangle"),
				new ImageIcon(getClass().getResource("images/Rectangle24.png")),
				I18N.getText("drawing","Rectangle"), new Integer(KeyEvent.VK_T), false));

		drawingModes.add(new RectangleMode(chat, board, I18N.getText("drawing","Filled_Rectangle"),
				new ImageIcon(getClass().getResource("images/FilledRectangle24.png")),
				I18N.getText("drawing","Filled_Rectangle"), new Integer(KeyEvent.VK_G), true));

		drawingModes.add(new EllipseMode(chat, board, I18N.getText("drawing","Ellipse"),
				new ImageIcon(getClass().getResource("images/Ellipse24.png")),
				I18N.getText("drawing","Ellipse"), new Integer(KeyEvent.VK_E), false));

		drawingModes.add(new EllipseMode(chat, board, I18N.getText("drawing","Filled_Ellipse"),
				new ImageIcon(getClass().getResource("images/FilledEllipse24.png")),
				I18N.getText("drawing","Filled_Ellipse"), new Integer(KeyEvent.VK_L), true));

		drawingModes.add(new PolygonMode(chat, board, I18N.getText("drawing","Polygon"),
				new ImageIcon(getClass().getResource("images/Polygon24.png")),
				I18N.getText("drawing","Polygon"), null, false));

		drawingModes.add(new PolygonMode(chat, board, I18N.getText("drawing","Filled_Polygon"),
				new ImageIcon(getClass().getResource("images/FilledPolygon24.png")),
				I18N.getText("drawing","Filled_Polygon"), null, true));
		
		drawingModes.add(new WritingMode(chat, board, I18N.getText("drawing","Write"),
				new ImageIcon(getClass().getResource("images/Font24.png")),
				I18N.getText("drawing","Write"), null));
		
		
	}

	private void addMenuAndToolBar(JPanel panelForAll) {
		JMenuBar menuBar = new JMenuBar();
		
		JToolBar  toolBar1 = new JToolBar();	
		JMenu fileMenu = createFileMenu(toolBar1);
		menuBar.add(fileMenu);
		JMenu editMenu = createEditMenu(toolBar1);
		menuBar.add(editMenu);
		
		JToolBar toolBar2 = new JToolBar();
		JMenu drawMenu = createDrawMenu(toolBar2);
		menuBar.add(drawMenu);
		
		JMenu fontAndColorMenu = createFontAndColorMenu(toolBar2);
		menuBar.add(fontAndColorMenu);
		JMenu chatMenu = createChatMenu(toolBar1);
		menuBar.add(chatMenu);
		
		panelForAll.add(toolBar1);
		panelForAll.add(toolBar2);
		
		setJMenuBar(menuBar);
	}

	private JMenuItem createMenuItem(Action a) {
		JMenuItem menuItem = new JMenuItem(a);
		menuItem.setIcon(null);
		return menuItem;
	}
	
	private JToggleButton createButtonToToolBar(Action a) {
		JToggleButton button = new JToggleButton(a);
		button.setText("");
		button.setMargin(new Insets(0,0,0,0));
		buttonGroup.add(button);
		return button;
	}
	
	private JMenu createChatMenu(JToolBar toolBar) {
		JMenu chatMenu = new JMenu(I18N.getText("drawing","Chat"));
		chatMenu.setMnemonic(KeyEvent.VK_A);

		Action addUsersSelection = new AddUsersSelectAction(
				I18N.getText("drawing","Add_users_from_roster"),
				new ImageIcon(getClass().getResource("images/AddUsers24.png")),
				I18N.getText("drawing","Add_users_to_the_chat"),
				new Integer(KeyEvent.VK_D),
				chat);
		chatMenu.add(createMenuItem(addUsersSelection));
		JMenuItem menuItem = new JMenuItem(I18N.getText("drawing","Add_user"));
		menuItem.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				String jidString =JOptionPane.showInputDialog(I18N.getText("drawing","Add_users_JID"));
				JID jid=null;
				try
				{
					jid = JID.checkedJIDFromString(jidString);
				} catch (InstantiationException e1)
				{
					Popups.errorPopup(e1.getMessage(), I18N.getText("drawing","Add_users_JID"));
				}
				HashSet<UserRepresentation> selectedUsers = new HashSet<UserRepresentation>();
				selectedUsers.add(new UserRepresentation(jid));
				if(jid!=null)chat.addUsers(selectedUsers);
			}
		});
		chatMenu.add(menuItem);
		toolBar.add(createButtonToToolBar(addUsersSelection));
		
		
		Action blockChatAction = new BlockChatAction(
				I18N.getText("drawing","Block_chat"),
				new ImageIcon(getClass().getResource("images/BlockChat24.png")),
				I18N.getText("drawing","Block_this_chat_(_you_will_receive_no_more_messages_)"),
				null,
				chat, this);
		chatMenu.add(createMenuItem(blockChatAction));
		toolBar.add(createButtonToToolBar(blockChatAction));
		
		
		Action showHideHistoryAction = new ShowHideHistorytAction(
				I18N.getText("drawing","Show_/_hide_history"),
				new ImageIcon(getClass().getResource("images/History24.png")),
				I18N.getText("drawing","Show_/_hide_history"),
				null,
				historyPanel, this);
		chatMenu.add(createMenuItem(showHideHistoryAction));
		toolBar.add(createButtonToToolBar(showHideHistoryAction));
		
		return chatMenu;
	}
	
	private void addModeToMenus(Mode mode, JMenu menu, JToolBar toolBar) {
		Action action = new ChangeModeAction(mode);
		JMenuItem modeItem = new JMenuItem(action);
		modeItem.setIcon(null);
		menu.add(modeItem);
		JToggleButton modeButton = new JToggleButton(action);
		modeButton.setText("");
		modeButton.setMargin(new Insets(0,0,0,0));
		buttonGroup.add(modeButton);
		toolBar.add(modeButton);
	}
	
	private JMenu createDrawMenu(JToolBar toolBar) {
		JMenu drawMenu = new JMenu(I18N.getText("drawing","Draw"));
		drawMenu.setMnemonic(KeyEvent.VK_D);
		
		for (Mode mode : drawingModes) {
			addModeToMenus(mode, drawMenu, toolBar);
		}
		
//		toolBar.addSeparator();
		
		return drawMenu;	
	}
	

	private JMenu createFontAndColorMenu(JToolBar toolBar) {
		JMenu fontAndColorMenu = new JMenu(I18N.getText("drawing","Font_and_color"));
		fontAndColorMenu.setMnemonic(KeyEvent.VK_C);

		Action colorSelection = new ColorSelectAction(
				I18N.getText("drawing","Color"),
				new ImageIcon(getClass().getResource("images/Colors24.png")),
				I18N.getText("drawing","Choose_a_color"),
				new Integer(KeyEvent.VK_K),
				this, board);
		fontAndColorMenu.add(createMenuItem(colorSelection));
		toolBar.add(createButtonToToolBar(colorSelection));
		
		Action shapeColorSetting = new ShapeColorSettingAction (
				I18N.getText("drawing","Set_shapes_color"),
				new ImageIcon(getClass().getResource("images/SetShapesColor24.png")),
				I18N.getText("drawing","Set_selected_shapes_color"),
				null,
				this,
				chat, board);
		JMenuItem shapeColorSettingItem = createMenuItem(shapeColorSetting);
		fontAndColorMenu.add(shapeColorSettingItem);
		toolBar.add(createButtonToToolBar(shapeColorSetting));
		
		Action backgroungColorSetting = new BackgroundColorSettingAction(
				I18N.getText("drawing","Background_color"),
				null,
				I18N.getText("drawing","Set_background_color"),
				null,
				this,
				chat, board);
		fontAndColorMenu.add(createMenuItem(backgroungColorSetting));
		
		Action transparencySetting = new TransparencySettingAction(
				I18N.getText("drawing","Set_colors_transparency"),
				null,
				I18N.getText("drawing","Set_colors_transparency"),
				null,
				this,
				board);
		fontAndColorMenu.add(createMenuItem(transparencySetting));

		fontAndColorMenu.addSeparator();
		
		Action fontSelection = new FontSelectAction(
				I18N.getText("drawing","Choose_font"),
				null,
				I18N.getText("drawing","Choose_font"),
				null,
				this,
				board);
		fontAndColorMenu.add(createMenuItem(fontSelection));
		
		return fontAndColorMenu;
	}

	private JMenu createEditMenu(JToolBar toolBar) {
		JMenu editMenu = new JMenu(I18N.getText("drawing","Edit"));
		editMenu.setMnemonic(KeyEvent.VK_E);

		Action undoSelection = new UndoSelectAction(
				I18N.getText("drawing","Undo"),
				new ImageIcon(getClass().getResource("images/Undo24.png")),
				I18N.getText("drawing","Undo"),
				new Integer(KeyEvent.VK_U),
				chat);
		JMenuItem undoItem = createMenuItem(undoSelection);
		undoItem.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_Z, ActionEvent.CTRL_MASK));
		editMenu.add(undoItem);
		toolBar.add(createButtonToToolBar(undoSelection));
		
		Action redoSelection = new RedoSelectAction(
				I18N.getText("drawing","Redo"),
				new ImageIcon(getClass().getResource("images/Redo24.png")),
				I18N.getText("drawing","Redo"),
				new Integer(KeyEvent.VK_R),
				chat);
		JMenuItem redoItem = createMenuItem(redoSelection);
		redoItem.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_Y, ActionEvent.CTRL_MASK));
		editMenu.add(redoItem);
		toolBar.add(createButtonToToolBar(redoSelection));
		
		editMenu.addSeparator();

		
		Action cutSelection = new CutSelectAction(
				I18N.getText("drawing","Cut"),
				new ImageIcon(getClass().getResource("images/Cut24.gif")),
				I18N.getText("drawing","Cut"),
				new Integer(KeyEvent.VK_X),
				chat);
		JMenuItem cutItem = createMenuItem(cutSelection);
		cutItem.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_X, ActionEvent.CTRL_MASK));
		editMenu.add(cutItem);
		toolBar.add(createButtonToToolBar(cutSelection));
		
		Action copySelection = new CopySelectAction(
				I18N.getText("drawing","Copy"),
				new ImageIcon(getClass().getResource("images/Copy24.gif")),
				I18N.getText("drawing","Copy"),
				new Integer(KeyEvent.VK_C),
				chat);
		JMenuItem copyItem = createMenuItem(copySelection);
		copyItem.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_C, ActionEvent.CTRL_MASK));
		editMenu.add(copyItem);
		toolBar.add(createButtonToToolBar(copySelection));
		
		Action pasteSelection = new PasteSelectAction(
				I18N.getText("drawing","Paste"),
				new ImageIcon(getClass().getResource("images/Paste24.gif")),
				I18N.getText("drawing","Paste"),
				new Integer(KeyEvent.VK_P),
				chat);
		JMenuItem pasteItem = createMenuItem(pasteSelection);
		pasteItem.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_V, ActionEvent.CTRL_MASK));
		editMenu.add(pasteItem);
		toolBar.add(createButtonToToolBar(pasteSelection));
		

		editMenu.addSeparator();

		Mode mode = new SelectionMode(chat, board, I18N.getText("drawing","Select"),
				new ImageIcon(getClass().getResource("images/Select24.png")),
				I18N.getText("drawing","Select"), new Integer(KeyEvent.VK_T));
		addModeToMenus(mode, editMenu, toolBar);
		
		Action selectionAll = new SelectAllAction(
				I18N.getText("drawing","Select_all"),
				null,
				I18N.getText("drawing","Select_all"),
				new Integer(KeyEvent.VK_A),
				chat);
		JMenuItem selectAllItem = createMenuItem(selectionAll);
		selectAllItem.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_A, ActionEvent.CTRL_MASK));
		editMenu.add(selectAllItem);
		
		Action deleteSelection = new DeleteSelectAction(
				I18N.getText("drawing","Delete"),
				new ImageIcon(getClass().getResource("images/Remove24.gif")),
				I18N.getText("drawing","Delete_selected_shapes"),
				new Integer(KeyEvent.VK_N),
				chat);
		JMenuItem deleteItem = createMenuItem(deleteSelection);
		deleteItem.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_DELETE, 0));
		editMenu.add(deleteItem);
		toolBar.add(createButtonToToolBar(deleteSelection));

		Action clearing = new ClearAction(
				I18N.getText("drawing","Clear_picture"),
				new ImageIcon(getClass().getResource("images/New24.gif")),
				I18N.getText("drawing","Clear_picture"),
				null,
				chat);
		editMenu.add(createMenuItem(clearing));
		toolBar.add(createButtonToToolBar(clearing));		
				
		return editMenu;
	}

	private JMenu createFileMenu(JToolBar toolBar) {
		JMenu fileMenu = new JMenu(I18N.getText("drawing","File"));
		fileMenu.setMnemonic(KeyEvent.VK_F);
		
		fileChooser = new JFileChooser();
		fileChooser.setFileFilter(new PictureFileFilter());
		
		Action saveSelection = new SaveSelectAction(
				I18N.getText("drawing","Save"),
				new ImageIcon(getClass().getResource("images/Save24.gif")),
				I18N.getText("drawing","Save_picture"),
				new Integer(KeyEvent.VK_S), chat);
		JMenuItem saveItem = createMenuItem(saveSelection);
		saveItem.setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_S, ActionEvent.CTRL_MASK));
		fileMenu.add(saveItem);
		toolBar.add(createButtonToToolBar(saveSelection));
		
		Action saveAsSelection = new SaveAsSelectAction(
				I18N.getText("drawing","Save_as"), null,
				I18N.getText("drawing","Save_picture_as"), null);
		fileMenu.add(createMenuItem(saveAsSelection));
		
		fileMenu.addSeparator();
		
		JMenuItem exitItem = new JMenuItem(I18N.getText("drawing","Exit"), KeyEvent.VK_X);
		exitItem.addActionListener(new ExitListener(this));
		fileMenu.add(exitItem);
		
		return fileMenu;
	}

	private JPanel createBoardAndHistory(PicturesContent picture) {
		JPanel mainPanel = new JPanel(new BorderLayout());
				
		board = new Board(chat, picture);
		createDrawingModes();
		board.setCurrentMode(drawingModes.firstElement());
		board.setPreferredSize(new Dimension(640, 480));
		
		mainPanel.add(board, BorderLayout.CENTER);
		mainPanel.add(historyPanel, BorderLayout.EAST);
		
		return mainPanel;
//		getContentPane().add(mainPanel);		
	}
	
	public void setColor(Color c) {
		board.setCurrentColor(c);
	}

	/**
	 * Sets current participants of the chat and displays their nick on
	 * the title bar.
	 * 
	 * @param receivers other participants of the chat
	 */
	public void setCurrentUsers(List<JID> receivers) {
		String s = "";
		for (JID jid : receivers) {
			s = s + (new UserRepresentation(jid)).toString() + "; ";
		}
		if (s.length() >= 2) {
			s = s.substring(0, s.length() - 2);
		}
		setTitle(s);
	}

	/**
	 * Gets current board's background color.
	 * 
	 * @return board's background color
	 */
	public Color getCurrentBackgroundColor() {
		return board.getCurrentBackgroundColor();
	}

    
	private void saveFromChosenFile() {
		int returnVal = fileChooser.showSaveDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
		   file = fileChooser.getSelectedFile();
		   if (!PictureFileFilter.hasGoodExtension(file)) {
			   File properFile = PictureFileFilter.getProperFile(file);
			   file = properFile;
		   }
		   saveFile(chat);
		}
	}

	private void saveFile(PictureChat c) {
		c.savePicture(file, PICTURE_WIDTH, PICTURE_HEIGHT);
	}


	class SaveSelectAction extends AbstractAction {
    	private static final long serialVersionUID = -904437773631661235L;
    	private PictureChat chat;

    	public SaveSelectAction(
        		String text, 
        		ImageIcon icon,
        		String desc,
        		Integer mnemonic,
        		PictureChat c) {
            super(text, icon);
            chat = c;
            putValue(SHORT_DESCRIPTION, desc);
            putValue(MNEMONIC_KEY, mnemonic);
        }
        
        public void actionPerformed(ActionEvent e) {
        	if (file == null) {
	            saveFromChosenFile();
        	} else {
        		saveFile(chat);
        	}
        }
    }
    
    class SaveAsSelectAction extends AbstractAction {
		private static final long serialVersionUID = 2866204072140697073L;

    	public SaveAsSelectAction(
        		String text, 
        		ImageIcon icon,
        		String desc,
        		Integer mnemonic) {
            super(text, icon);
            putValue(SHORT_DESCRIPTION, desc);
            putValue(MNEMONIC_KEY, mnemonic);
        }
        
        public void actionPerformed(ActionEvent e) {
            saveFromChosenFile();
        }
    }
}
    

class UndoSelectAction extends AbstractAction {
	private static final long serialVersionUID = 2018054779564654649L;
	private PictureChat chat;
	
    public UndoSelectAction(
    		String text, 
    		ImageIcon icon,
    		String desc,
    		Integer mnemonic,
    		PictureChat c) {
        super(text, icon);
        chat = c;
        putValue(SHORT_DESCRIPTION, desc);
        putValue(MNEMONIC_KEY, mnemonic);
    }
    
    public void actionPerformed(ActionEvent e) {
    	chat.undo();
    }
}

class RedoSelectAction extends AbstractAction {
	private static final long serialVersionUID = -2347628523315659788L;
	private PictureChat chat;
    public RedoSelectAction(
    		String text, 
    		ImageIcon icon,
    		String desc,
    		Integer mnemonic,
    		PictureChat c) {
        super(text, icon);
        chat = c;
        putValue(SHORT_DESCRIPTION, desc);
        putValue(MNEMONIC_KEY, mnemonic);
    }
    
    public void actionPerformed(ActionEvent e) {
    	chat.redo();
    }
}

class ChangeModeAction extends AbstractAction {
	private static final long serialVersionUID = -390685120849297955L;
	private Mode mode;
	
    public ChangeModeAction(Mode m) {
        super(m.getButtonText(), m.getIcon());
    	mode = m;
        putValue(SHORT_DESCRIPTION, mode.getDescription());
        putValue(MNEMONIC_KEY, mode.getMnemonic());
    }
    
    public void actionPerformed(ActionEvent e) {
    	(mode.getBoard()).setCurrentMode(mode);
    }
}


class ColorSelectAction extends AbstractAction {
	private static final long serialVersionUID = 4206735523233792824L;
	private PictureChatFrame frame;
	private Board board;
	
    public ColorSelectAction(
    		String text, 
    		ImageIcon icon,
    		String desc,
    		Integer mnemonic,
    		PictureChatFrame f, Board b) {
        super(text, icon);
        frame = f;
        board = b;
        putValue(SHORT_DESCRIPTION, desc);
        putValue(MNEMONIC_KEY, mnemonic);
    }
    
    public void actionPerformed(ActionEvent e) {
    	Color c = JColorChooser.showDialog(
    			frame,
    			I18N.getText("drawing","Please_choose_a_color"),
    			board.getCurrentColor());
    	if (c != null) {
	    	Color color = new Color(c.getRed(), c.getGreen(), c.getBlue(), board.getCurrentTransparency());
	    	frame.setColor(color);
    	}
    }
}

class FontSelectAction extends AbstractAction {
	private static final long serialVersionUID = -8637703530410277069L;
	private PictureChatFrame frame;
	private Board board;
	
    public FontSelectAction(
    		String text, 
    		ImageIcon icon,
    		String desc,
    		Integer mnemonic,
    		PictureChatFrame f,
    		Board b) {
        super(text, icon);
        frame = f;
        board = b;
        putValue(SHORT_DESCRIPTION, desc);
        putValue(MNEMONIC_KEY, mnemonic);
    }
    
    public void actionPerformed(ActionEvent e) {
		AttributeSet attr = new SimpleAttributeSet();
		FontDialog fontD = new FontDialog(frame, I18N.getText("drawing","Choose_font"), attr);
		fontD.setModal(true);
		fontD.setVisible(true);

		attr = fontD.getAttributes();
		String family = StyleConstants.getFontFamily(attr);
		if (family != null) {
			board.setFontFamily(family);
		}
		int size = StyleConstants.getFontSize(attr);
		if (size > 0) {
			board.setFontSize(size);
		}

		int fontStyle = Font.PLAIN;
		String value;
		if (attr.isDefined(CSS.Attribute.FONT_STYLE)) {
			value = attr.getAttribute(CSS.Attribute.FONT_STYLE).toString();
			if (value.equalsIgnoreCase(StyleConstants.Italic.toString())) {
				fontStyle = Font.ITALIC;
			}
		}
		if (attr.isDefined(CSS.Attribute.FONT_WEIGHT)) {
			value = attr.getAttribute(CSS.Attribute.FONT_WEIGHT).toString();
			if (value.equalsIgnoreCase(StyleConstants.Bold.toString())) {
				if (fontStyle == Font.ITALIC) {
					fontStyle = Font.ITALIC|Font.BOLD;
				} else {
					fontStyle = Font.BOLD;
				}
			}
		}
		board.setFontStyle(fontStyle);
		
		Color foreColor = StyleConstants.getForeground(attr);
		if (foreColor != null) {
			board.setFontForegroundColor(foreColor);
		}
		Color backColor = StyleConstants.getBackground(attr);
		if (backColor != null) {
			board.setFontBackgroundColor(backColor);
		}
    }
}

class DeleteSelectAction extends AbstractAction {
	private static final long serialVersionUID = -3244635742991905141L;
	private PictureChat chat;
	
    public DeleteSelectAction(
    		String text, 
    		ImageIcon icon,
    		String desc,
    		Integer mnemonic,
    		PictureChat c) {
        super(text, icon);
        chat = c;
        putValue(SHORT_DESCRIPTION, desc);
        putValue(MNEMONIC_KEY, mnemonic);
    }
    
    public void actionPerformed(ActionEvent e) {
    	chat.deleteSelectedShapes();
    }
}


class ExitListener implements ActionListener {
	JFrame frame;
	
	public ExitListener(JFrame f) {
		frame = f;
	}
	
	public void actionPerformed(ActionEvent e) {
		frame.dispose();
	}	
}

class ShapeColorSettingAction extends AbstractAction {
	private PictureChat chat;
	private static final long serialVersionUID = -2739235534252237769L;
	private PictureChatFrame frame;
	private Board board;
	
    public ShapeColorSettingAction(
    		String text, 
    		ImageIcon icon,
    		String desc,
    		Integer mnemonic,
    		PictureChatFrame f,
    		PictureChat pc, Board b) {
        super(text, icon);
        frame = f;
        chat = pc;
        board = b;
        putValue(SHORT_DESCRIPTION, desc);
        putValue(MNEMONIC_KEY, mnemonic);
    }
	public void actionPerformed(ActionEvent e) {
		Color c = JColorChooser.showDialog(
    			frame,
    			I18N.getText("drawing","Please_choose_a_color"),
    			board.getCurrentColor());
		if (c != null) {
	    	Color color = new Color(c.getRed(), c.getGreen(), c.getBlue(), board.getCurrentTransparency());
			chat.setShapeColor(color);
		}
	}	
}


class BackgroundColorSettingAction extends AbstractAction {
	private static final long serialVersionUID = 2892419585545062664L;
	private PictureChat chat;
	private PictureChatFrame frame;
	private Board board;
	
    public BackgroundColorSettingAction(
    		String text, 
    		ImageIcon icon,
    		String desc,
    		Integer mnemonic,
    		PictureChatFrame f,
    		PictureChat pc, Board b) {
        super(text, icon);
        frame = f;
        chat = pc;
        board = b;
        putValue(SHORT_DESCRIPTION, desc);
        putValue(MNEMONIC_KEY, mnemonic);
    }
	public void actionPerformed(ActionEvent e) {
		Color c = JColorChooser.showDialog(
    			frame,
    			I18N.getText("drawing","Please_choose_a_color"),
    			board.getCurrentColor());
		if (c != null) {
			chat.setBackgroundColor(c);
		}
	}	
}

class ClearAction extends AbstractAction {
	private static final long serialVersionUID = 2493153975625069208L;
	private PictureChat chat;
	
    public ClearAction(
    		String text, 
    		ImageIcon icon,
    		String desc,
    		Integer mnemonic,
    		PictureChat pc) {
        super(text, icon);
        chat = pc;
        putValue(SHORT_DESCRIPTION, desc);
        putValue(MNEMONIC_KEY, mnemonic);
    }
	public void actionPerformed(ActionEvent e) {
		chat.clearBoard();
	}	
}

class SelectAllAction extends AbstractAction {
	private static final long serialVersionUID = -5535852111726595034L;
	private PictureChat chat;
	
    public SelectAllAction(
    		String text, 
    		ImageIcon icon,
    		String desc,
    		Integer mnemonic,
    		PictureChat pc) {
        super(text, icon);
        chat = pc;
        putValue(SHORT_DESCRIPTION, desc);
        putValue(MNEMONIC_KEY, mnemonic);
    }
	public void actionPerformed(ActionEvent e) {
		chat.selectAll();
	}	
}

class PasteSelectAction extends AbstractAction {
	private static final long serialVersionUID = 6066986288652231573L;
	private PictureChat chat;
	
    public PasteSelectAction(
    		String text, 
    		ImageIcon icon,
    		String desc,
    		Integer mnemonic,
    		PictureChat pc) {
        super(text, icon);
        chat = pc;
        putValue(SHORT_DESCRIPTION, desc);
        putValue(MNEMONIC_KEY, mnemonic);
    }
	public void actionPerformed(ActionEvent e) {
		chat.paste();
	}	
}

class CopySelectAction extends AbstractAction {
	private static final long serialVersionUID = 698696669538423160L;
	private PictureChat chat;
	
    public CopySelectAction(
    		String text, 
    		ImageIcon icon,
    		String desc,
    		Integer mnemonic,
    		PictureChat pc) {
        super(text, icon);
        chat = pc;
        putValue(SHORT_DESCRIPTION, desc);
        putValue(MNEMONIC_KEY, mnemonic);
    }
	public void actionPerformed(ActionEvent e) {
		chat.copy();
	}	
}

class CutSelectAction extends AbstractAction {
	private static final long serialVersionUID = -7632636586918242904L;
	private PictureChat chat;
	
    public CutSelectAction(
    		String text, 
    		ImageIcon icon,
    		String desc,
    		Integer mnemonic,
    		PictureChat pc) {
        super(text, icon);
        chat = pc;
        putValue(SHORT_DESCRIPTION, desc);
        putValue(MNEMONIC_KEY, mnemonic);
    }
	public void actionPerformed(ActionEvent e) {
		chat.cut();
	}	
}

class BlockChatAction extends AbstractAction {
	private static final long serialVersionUID = 7163101549344944383L;
	private PictureChat chat;
	private PictureChatFrame frame;
	
    public BlockChatAction(
    		String text, 
    		ImageIcon icon,
    		String desc,
    		Integer mnemonic,
    		PictureChat pc,
    		PictureChatFrame f) {
        super(text, icon);
        chat = pc;
        frame = f;
        putValue(SHORT_DESCRIPTION, desc);
        putValue(MNEMONIC_KEY, mnemonic);
    }
    
	public void actionPerformed(ActionEvent e) {
		Object[] options = {I18N.getText("drawing","Yes"),
				I18N.getText("drawing","No")};
		
		int n = JOptionPane.showOptionDialog(frame,
				I18N.getText("drawing","If_you_block_this_chat_you_will_not_get_any_more_pictures_and_you_will_not_be_able_to_send_any"),
				I18N.getText("drawing","Do_you_want_to_block_this_chat?"),
			    JOptionPane.YES_NO_OPTION,
			    JOptionPane.QUESTION_MESSAGE,
			    null,
			    options,
			    options[0]);
		
		if (n == JOptionPane.YES_OPTION) {
			chat.blockChat();
		}
		
	}	
}


class ShowHideHistorytAction extends AbstractAction {
	private HistoryPanel panel;
	private PictureChatFrame chatFrame;
	private int counter = 0;
	private static final long serialVersionUID = 5792931602160558228L;
	
	public ShowHideHistorytAction(
    		String text, 
    		ImageIcon icon,
    		String desc,
    		Integer mnemonic,
    		HistoryPanel historyPanel,
    		PictureChatFrame cf) {
        super(text, icon);
        panel = historyPanel;
        chatFrame = cf;
        putValue(SHORT_DESCRIPTION, desc);
        putValue(MNEMONIC_KEY, mnemonic);
    }
	
	public void actionPerformed(ActionEvent e) {
		if (counter % 2 == 0) {
			panel.showHistory();
		} else {
			panel.hideHistory();
		}
		counter++;
		chatFrame.repaint();
	}	
}
