package nu.fw.jeti.plugins.drawing.ui;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.swing.filechooser.FileFilter;

/**
 * This class is a file filter. It determines which extensions are supported
 * by the picture chat.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class PictureFileFilter extends FileFilter {
	/**
	 * .png extension
	 */
	public static final String PNG_EXT = "png";
	/**
	 * .bmp extension
	 */
	public static final String BMP_EXT = "bmp";
	/**
	 * .jpg extension
	 */
	public static final String JPG_EXT = "jpg";
	/**
	 * .jpeg extension
	 */
	public static final String JPEG_EXT = "jpeg";
	
	private final static String DEFAULT_EXT = PNG_EXT;
	
	private static List<String> supportedExt;
	static {
		supportedExt = new ArrayList<String>();

		supportedExt.add(PNG_EXT);
		supportedExt.add(BMP_EXT);
		supportedExt.add(JPG_EXT);
		supportedExt.add(JPEG_EXT);
		
//		it doesn't save for some formats generated this way
//		String[] extArray = ImageIO.getWriterFormatNames();
//		for (int i = 0; i < extArray.length; i++) {
//			supportedExt.add(extArray[i]);
//		}
//		System.out.println(supportedExt);
	}
	
	/**
	 * Returns the file with a good extension.  If the extension was already
	 * good, it returns the same file. Otherwise it returns file with its
	 * name + default extension.
	 * 
	 * @param f file that may be changed to a proper one 
	 * @return if the extension was already good, it returns the same file;<br>
	 * otherwise it returns file with its name + default extension.
	 */
	public static File getProperFile(File f) {
		if (!hasGoodExtension(f)) {
			File file = new File(f + "." + DEFAULT_EXT);
			return file;
		} else {
			return f;
		}
	}
	
	/**
	 * Checks if the file is a directory or has a good extension.
	 * 
	 * @param f <tt>true</tt> if the file is a directory or has a good
	 * extension
	 */
	public boolean accept(File f) {
		if (f.isDirectory()) {
            return true;
        }
		
		return hasGoodExtension(f);
	}
	
	/**
	 * Checks if the file has a good extension.
	 * 
	 * @param f <tt>true</tt> if the file has a good extension
	 */
	public static boolean hasGoodExtension(File f) {
		String extension = getExtension(f);
		if (extension != null) {
			if (supportedExt.contains(extension)) {
				return true;
			} else {
				return false;
			}
		}
		return false;
	}
	
	/**
	 * Gets a string with all supported extensions.
	 * 
	 * @return string with all supported extensions
	 */
	public String getDescription() {
		String s = "";
		for (String ext : supportedExt) {
			s += "*." + ext + ", ";
		}
		s.substring(0, s.length() - 2);
		return s;
	}

	/**
	 * Gets the extension from the file.
	 * 
	 * @param f file it gets the extension from
	 * @return file's extension
	 */
	public static String getExtension(File f) {
        String ext = null;
        String s = f.getName();
        int i = s.lastIndexOf('.');

        if (i > 0 &&  i < s.length() - 1) {
            ext = s.substring(i+1).toLowerCase();
        }
        return ext;
    }
	
	/**
	 * Returns the list of supported extensions.
	 * 
	 * @return list of supported extensions
	 */
	public static List<String> getAcceptedExt() {
		return supportedExt;
	}

}
