package nu.fw.jeti.plugins.drawing.ui;

import java.awt.Color;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.awt.geom.Point2D.Double;
import java.util.Vector;

import javax.swing.ImageIcon;

import nu.fw.jeti.plugins.drawing.geometry.PointsOperations;
import nu.fw.jeti.plugins.drawing.shapes.PictureChat;
import nu.fw.jeti.plugins.drawing.shapes.Polygon;
import nu.fw.jeti.plugins.drawing.shapes.Shape;

/**
 * This class represents a mode that draws a
 * {@link nu.fw.jeti.jabber.elements.drawing.Polygon}.  A user clicks the mouse
 * and this Point becomes the first point of the polygon. Then he moves the
 * mouse and chooses where should be the next point of the polygon. The new
 * point is drawn as well as a line between it and the previous point and a line
 * between it and starting point. When a user double-clicks this point becomes
 * the last point in the polygon. If the user right-clicks, the last polygon's
 * point becomes the last left-clicked point.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class PolygonMode extends Mode {
	private Vector<Point2D.Double> points = new Vector<Point2D.Double>();

	private boolean isFilled;

	/**
	 * Creates a polygon mode.
	 * 
	 * @param c
	 *            drawing chat
	 * @param b
	 *            board where the shapes and the behaviour is displayed
	 * @param text
	 *            text that is displayed on the button and menu item
	 * @param i
	 *            icon that is displayed on the button and menu item
	 * @param desc
	 *            description of the button shown on the tool tips
	 * @param m
	 *            mnemonic for the mode
	 * @param filled
	 *            <tt>true</tt> if the ellipse is filled
	 */
	public PolygonMode(PictureChat c, Board b, String text, ImageIcon i,
			String desc, Integer m, boolean filled) {
		super(c, b, text, i, desc, m);
		isFilled = filled;
		points = new Vector<Point2D.Double>();
	}

	public void dispose() {
		super.dispose();
		points.clear();
	}

	/**
	 * The clicked point becomes the first point of the polygon. If the user
	 * double-clicks, this point becomes the last point in the polygon. If he
	 * right-clicks, the last polygon's point becomes the last left-clicked
	 * point.
	 */
	public void mouseClicked(MouseEvent e, Color c, Double point, int scaleX, int scaleY) {
		color = c;
		chat.unselect();
		if (e.getClickCount() == 2 || e.getButton() == MouseEvent.BUTTON3) {
			// we want to finish the poligon
			if (points.size() > 0) {
				if (points.size() - 1 > 0
						&& e.getButton() == MouseEvent.BUTTON3) {
					points.remove(points.size() - 1);
				}
				Shape shape = new Polygon(points, color, chat.getMyJID(),
						isFilled);
				chat.draw(shape);
				board.setCurrentShape(null);
				points.clear();
			}
		} else {
			if (points.isEmpty()) {
				end = point;
				points.add(end);
			}
			points.add(end);
		}
		board.repaint();
	}

	/**
	 * The new point is drawn as well as a line between it and the previous
	 * point and a line between it and starting point.
	 */
	public void mouseMoved(Double point) {
		if (!points.isEmpty()) {
			// the rectangle is already started
			end = point;
			points.setElementAt(end, points.size() - 1);
			if (board.getCurrentShape() != null) {
				(board.getCurrentShape()).setCoordinates(PointsOperations
						.pointsToString(points));
			} else {
				board.setCurrentShape(new Polygon(points, color, chat
						.getMyJID(), isFilled));
			}
			board.repaint();
		}
	}

}
