package nu.fw.jeti.plugins.drawing.ui;

import java.awt.Color;
import java.awt.geom.Point2D;
import java.awt.geom.Point2D.Double;
import java.util.Vector;

import javax.swing.ImageIcon;

import nu.fw.jeti.plugins.drawing.geometry.PointsOperations;
import nu.fw.jeti.plugins.drawing.shapes.PictureChat;
import nu.fw.jeti.plugins.drawing.shapes.Polyline;
import nu.fw.jeti.plugins.drawing.shapes.Shape;

/**
 * This class represents a mode that draws a
 * {@link nu.fw.jeti.jabber.elements.drawing.Polyline}.  A user presses
 * the mouse and this Point becomes the first point of the polyline. Then he
 * drags the mouse and draws smoothly all the other points. When he releases
 * the mouse the polyline is drawn, memorized in the history and sent to
 * the other users.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class PolylineMode extends Mode {

	private Vector<Point2D.Double> points = new Vector<Point2D.Double>();

	/**
	 * Creates a polyline mode.
	 * 
	 * @param c drawing chat
	 * @param b board where the shapes and the behaviour is displayed
	 * @param text text that is displayed on the button and menu item
	 * @param i icon that is displayed on the button and menu item
	 * @param desc description of the button shown on the tool tips
	 * @param m mnemonic for the mode
	 */
	public PolylineMode(PictureChat c, Board b, String text, ImageIcon i, String desc, Integer m) {
		super(c, b, text, i, desc, m);
		points = new Vector<Point2D.Double>();
	}
	
	public void dispose() {
		super.dispose();
		points.clear();
	}

	/**
	 * The clicked point becomes the first point of the polyline.
	 */
	public void mousePressed(Double point, Color c, int scaleX, int scaleY) {
		color = c;
		points.add(point);
		chat.unselect();
	}
	
	/**
	 * The polyline is drawn, memorized in the history and sent to
	 * the other users.
	 */
	public void mouseReleased(Double point, int scaleX, int scaleY) {	
		Shape shape = new Polyline(points, color, chat.getMyJID());
		chat.draw(shape);
		points.clear();
		board.setCurrentShape(null);
	}

	/**
	 * All the points are added to the polyline. The users draws smoothly
	 * the polyline.
	 */
	public void mouseDragged(Double point) {
		points.add(point);
		if (board.getCurrentShape() != null) {
			(board.getCurrentShape()).setCoordinates(PointsOperations.pointsToString(points));
		} else {
			board.setCurrentShape(new Polyline(points, color, chat.getMyJID()));
		}
		board.repaint();
	}

}
