package nu.fw.jeti.plugins.drawing.ui;

import java.awt.Color;
import java.awt.geom.Point2D.Double;

import javax.swing.ImageIcon;

import nu.fw.jeti.plugins.drawing.geometry.PointsOperations;
import nu.fw.jeti.plugins.drawing.shapes.PictureChat;
import nu.fw.jeti.plugins.drawing.shapes.Rectangle;
import nu.fw.jeti.plugins.drawing.shapes.Shape;

/**
 * This class represents a mode that draws a
 * {@link nu.fw.jeti.jabber.elements.drawing.Rectangle}.  A user presses
 * the mouse and this Point becomes a first corner of the rectangle.
 * Then he drags the mouse and chooses the dimension of the rectangle.
 * When he releases the mouse the rectangle is drawn, memorized in
 * the history and sent to the other users.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class RectangleMode extends Mode {
	private boolean isFilled;

	/**
	 * Creates a rectangle mode.
	 * 
	 * @param c drawing chat
	 * @param b board where the shapes and the behaviour is displayed
	 * @param text text that is displayed on the button and menu item
	 * @param i icon that is displayed on the button and menu item
	 * @param desc description of the button shown on the tool tips
	 * @param m mnemonic for the mode
	 * @param filled <tt>true</tt> if the ellipse is filled
	 */
	public RectangleMode(PictureChat c, Board b, String text, 
			ImageIcon i, String desc, Integer m, boolean filled) {
		super(c, b, text, i, desc, m);
		isFilled = filled;
	}

	/**
	 * The clicked point is the first corner of the rectangle.
	 */
	public void mousePressed(Double point, Color c, int scaleX, int scaleY) {
		super.mousePressed(point, c, scaleX, scaleY);
		chat.unselect();
	}

	/**
	 * The rectangle is drawn, memorized in the history and sent to the other
	 * users.
	 */
	public void mouseReleased(Double point, int scaleX, int scaleY) {
		end = point;
		Shape shape = new Rectangle(start, end, color, chat.getMyJID(), isFilled);
		chat.draw(shape);
		board.setCurrentShape(null);
	}

	/**
	 * The user chooses the dimension of the rectangle. The current rectangle
	 * is drawn.
	 */
	public void mouseDragged(Double point) {
		end = point;
		if (board.getCurrentShape() != null) {
			(board.getCurrentShape()).setCoordinates(
					PointsOperations.pointsToString(start, end));
		} else {
			board.setCurrentShape(new Rectangle(
					start, end, color, chat.getMyJID(), isFilled));
		}
		board.repaint();		
	}

}
