package nu.fw.jeti.plugins.drawing.ui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.util.Hashtable;

import javax.swing.*;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import nu.fw.jeti.util.I18N;

/**
 * This class implements the action of setting transparency of colors.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class TransparencySettingAction extends AbstractAction implements ChangeListener {
	private static final long serialVersionUID = 363618513140706658L;
	private Board board;
	private PictureChatFrame frame;
	/**
	 * Minimum transparency.
	 */
	public static final int TRANS_MIN = 0;
	/**
	 * Maximum transparency.
	 */
	public static final int TRANS_MAX = 255;
	
	/**
	 * Creates a transparency action.
	 * 
	 * @param text text that is displayed on the button and menu item
	 * @param icon icon that is displayed on the button and menu item
	 * @param desc description of the button shown on the tool tips
	 * @param mnemonic mnemonic for the mode
	 * @param f chat's frame
	 * @param b board where the shapes are displayed
	 */
    public TransparencySettingAction(
    		String text, 
    		ImageIcon icon,
    		String desc,
    		Integer mnemonic,
    		PictureChatFrame f,
    		Board b) {
        super(text, icon);
        frame = f;
        board = b;
        putValue(SHORT_DESCRIPTION, desc);
        putValue(MNEMONIC_KEY, mnemonic);
    }
    
    /**
     * Shows a dialog with a slider to choose the transparency level.
     */
	public void actionPerformed(ActionEvent e) {
		JDialog transparencyDialog = new JDialog(frame,
				I18N.getText("drawing","Choose_transparency"));
		JPanel panel = new JPanel();
		panel.setLayout(new BorderLayout());
		JLabel instruction = new JLabel(
				I18N.getText("drawing","Choose_the_transparency_of_the_future_colors"));
		JSlider slider = createSlider();
		slider.setPreferredSize(new Dimension(200, 80));
		slider.setVisible(true);
		panel.add(instruction, BorderLayout.NORTH);
		panel.add(slider, BorderLayout.CENTER);
		panel.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
		transparencyDialog.getContentPane().add(panel);
		transparencyDialog.pack();
		transparencyDialog.setModal(true);
		transparencyDialog.setVisible(true);
		transparencyDialog.setLocationRelativeTo(frame);
	}
	
	private JSlider createSlider() {
		JSlider slider = new JSlider(SwingConstants.HORIZONTAL,
				TRANS_MIN, TRANS_MAX, board.getCurrentTransparency());
		slider.addChangeListener(this);
		
		slider.setMajorTickSpacing(85);
		slider.setMinorTickSpacing(17);
		
		Hashtable<Integer, JLabel> labelTable = new Hashtable<Integer, JLabel>();
		labelTable.put(new Integer(TRANS_MIN), new JLabel(I18N.getText("drawing","Transparent")));
		labelTable.put(new Integer(TRANS_MAX), new JLabel(I18N.getText("drawing","Opaque")));
		slider.setLabelTable(labelTable);
		slider.setPaintTicks(true);

		slider.setPaintLabels(true);
		
		return slider;
	}
	
	/**
	 * Set current transparency level in the chat.
	 */
	public void stateChanged(ChangeEvent e) {
		JSlider source = (JSlider)e.getSource();
		if (!source.getValueIsAdjusting()) {
	    	int alpha = source.getValue();
	        board.setCurrentTransparency(alpha);
	    }
	}	
}
