package nu.fw.jeti.plugins.drawing.ui;

import javax.swing.ImageIcon;

import nu.fw.jeti.backend.roster.Roster;
import nu.fw.jeti.images.StatusIcons;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.jabber.JIDStatus;

/**
 * This class is a user's representation tool.
 * 
 * @author Elbieta Iwaszkiewicz (ei197866@students.mimuw.edu.pl)
 */
public class UserRepresentation {
	JIDStatus jidStatus;
	JID jid;

	/**
	 * Creates a user representation with the given JID status.
	 * 
	 * @param status user's JID status
	 */
	public UserRepresentation(JIDStatus status) {
		jidStatus = status;
		jid = jidStatus.getCompleteJID();
	}
	
	/**
	 * Creates a user representation with a JID status which id taken from
	 * Roster.
	 * 
	 * @param jid user's JID
	 */
	public UserRepresentation(JID jid) {
		this.jid = jid;
		jidStatus = Roster.getJIDStatus(jid);
	}

	/**
	 * Returns the actual status icon for this user.
	 * 
	 * @return actual status icon
	 */
	public ImageIcon getStatusIcon() {
		if (jidStatus != null) {
			return StatusIcons.getStatusIcon(jidStatus.getShow());
		} else {
			return null;
		}
	}
	
	/**
	 * Returns user's address.
	 * 
	 * @return user's address
	 */
	public String getJIDAddress() {
		return jid.toString();
	}
	
	/**
	 * Returns user's JID.
	 * 
	 * @return user's JID
	 */
	public JID getJID() {
		return jid;
	}
	
	/**
	 * Returns user's nick.  If the user is not in our contact list, it returns
	 * JID address.
	 * 
	 * @return user's nick
	 */
	public String toString() {
		if (jidStatus != null) {
			return jidStatus.getNick();
		} else {
			return getJIDAddress();
		}
	}
	
	/**
	 * Checks if the user is in the given group in our contact list.
	 * 
	 * @param group group we check if it contains the user
	 * @return <tt>true</tt> if the user belongs to the given group
	 */
	public boolean isInThisGroup(String group) {
		if (jidStatus != null) {
			return jidStatus.isGroupPresent(group);
		} else {
			return false;
		}
	}
}
