/* 
 *	Jeti, a Java Jabber client, Copyright (C) 2001 E.S. de Boer  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	For questions, comments etc, 
 *	use the website at http://jeti.jabberstudio.org
 *  or mail/IM me at jeti@jabber.org
 *  
 *  Created on 8-may-2005
 */

package nu.fw.jeti.plugins.groupchat;

import java.awt.Color;
import java.awt.Component;
import java.awt.Toolkit;
import java.awt.event.*;
import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import nu.fw.jeti.events.MessageListener;
import nu.fw.jeti.images.StatusIcons;
import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.jabber.elements.Message;
import nu.fw.jeti.util.I18N;
import nu.fw.jeti.util.Preferences;

/**
 * @author E.S. de Boer
 *
 */
public class GroupChatWindows implements MessageListener
{
	private Map groupchatWindows = new HashMap();
	private Backend backend;
	private JTabbedPane tabs;
	private JFrame window;
	private int chatDividerLocation;
	private ComponentListener chatDividerListener;
	private int dividerLocation;
	private ComponentListener dividerListener;
	
	public GroupChatWindows(Backend backend)
	{
		this.backend = backend;
		backend.addListener(MessageListener.class, this);
	}
	
	public Set getRooms()
	{
		synchronized (groupchatWindows)
		{
			return groupchatWindows.keySet();
		}
	}
	
	public  void exit()
	{
		synchronized (groupchatWindows)
		{
			for(Iterator i = groupchatWindows.values().iterator();i.hasNext();)
			{
				((GroupchatWindow)i.next()).exit();
			}
			if(window!=null)window.dispose();
		}
	}
	
	
	public GroupchatWindow createGroupChatWindow(JID jid) 
	{
		GroupchatWindow groupchatWindow = new GroupchatWindow(backend,jid,this,new JFrame());
		synchronized (groupchatWindows)
		{
			groupchatWindows.put(jid,groupchatWindow);
			return groupchatWindow;
		}
	}
	
	public JID isGroupchatStarted(JID jid)
	{
		synchronized (groupchatWindows)
		{
			if(groupchatWindows.containsKey(jid))
			{
				for(Iterator i = groupchatWindows.keySet().iterator();i.hasNext();)
				{
					JID j = (JID)i.next();
					if(jid.equals(j))return j;
				}
				return null;
			}
			return null;
		}
	}
		
	public GroupchatWindow getGroupchatWindow(final JID jid)
	{
		GroupchatWindow w;
		synchronized (groupchatWindows)
		{
			w = (GroupchatWindow) groupchatWindows.get(jid);
		}
		if(w==null) w = startGroupchat(jid);
		final GroupchatWindow w2=w;
		Runnable updateAComponent = new Runnable()
		{
			public void run()
			{
				if(tabs!=null)
				{
					
					if(tabs.indexOfComponent(w2)!=-1)
					{	
						w2.requestFocusInWindow();
						tabs.setSelectedComponent(w2);
					}
				}
				JOptionPane.getFrameForComponent(w2.getTopLevelAncestor()).toFront();
			}
		};
		SwingUtilities.invokeLater(updateAComponent);
		return w;
	}
	
	private GroupchatWindow startGroupchat(JID jid)
	{
		GroupchatWindow groupchatWindow;
		if(Preferences.getBoolean("jeti", "tabs", false))
		{
			if(window==null) makeNewTabWindow();
			groupchatWindow = new GroupchatWindow(backend,jid,this,window);
			if(!window.isDisplayable()) addMenu(groupchatWindow);
			addTab(groupchatWindow);
			groupchatWindow.setChatDividerLocation(chatDividerLocation);
			groupchatWindow.addChatSplitBarListener(chatDividerListener);
			groupchatWindow.addSplitBarListener(dividerListener);
			groupchatWindow.setDividerLocation(dividerLocation);
			
			window.setVisible(true);
		}
		else
		{
			groupchatWindow = new GroupchatWindow(backend,jid,this,null);
			setChatWindowPosition(groupchatWindow);
		}
		synchronized (groupchatWindow)
		{
			groupchatWindows.put(jid,groupchatWindow);
		}
		return groupchatWindow;
	}
	
	private void addTab(GroupchatWindow groupchatWindow)
	{
		/*
		String name = groupchatWindow.getJID().toStringNoResource();
		// = new FontMetrics((Font)UIManager.get(name));
		FontMetrics fontMetrics = tabs.getFontMetrics(tabs.getFont());
		name+="                                                         ";
		int fontLenght=0;
		for(int i=0;i<name.length();i++)
		{
			fontLenght+=fontMetrics.charWidth(name.charAt(i));
			if(fontLenght>=MAX_TABNAME_LENGTH)
			{
				name = name.substring(0,i);
				break;
			}
		}
		*/
		tabs.addTab(getTitle(groupchatWindow.getJID()),
				StatusIcons.getStatusIcon(groupchatWindow.getStatus()),groupchatWindow);
	}
	
	private void removeTab(GroupchatWindow groupchatWindow)
	{
		if(tabs.getTabCount()==1)
		{
			savePreferences();
			window.dispose();
			window=null;
		}
		tabs.remove(groupchatWindow);
	}

	private void makeNewTabWindow()
	{
		window = new JFrame();
		window.setIconImage(nu.fw.jeti.images.StatusIcons.getImageIcon("jeti").getImage());
		JMenuBar bar = new JMenuBar();
		tabs = new JTabbedPane()
		{//fix problem that change event is not called if there are tabs to the right
			public void remove(Component component)
			{
				super.remove( component );
		//		int after = getSelectedIndex();
		 
				//  The selected tab remained the same after this remove, consider
				//  it a state changed
		 
				//if (after == tab)
			//	{
					fireStateChanged();
			//	}
			}
		};
		
		tabs.addChangeListener(new ChangeListener()
		{
			public void stateChanged(ChangeEvent e)
			{
				Runnable updateAComponent = new Runnable()
				{
					public void run()
					{
						GroupchatWindow w = (GroupchatWindow) tabs.getSelectedComponent();
						if(w!=null)
						{
							window.setTitle(w.getJID().toStringNoResource());
							window.setIconImage(StatusIcons.getStatusIcon(w.getStatus()).getImage());
							tabs.setBackgroundAt(tabs.getSelectedIndex()
								,UIManager.getColor("TabbedPane.background"));
							tabs.setForegroundAt(tabs.getSelectedIndex()
								,UIManager.getColor("TabbedPane.foreground"));
							window.getJMenuBar().removeAll();
							addMenu(w);
							w.setChatDividerLocation(chatDividerLocation);
							w.setDividerLocation(dividerLocation);
						}
					}
				};
				SwingUtilities.invokeLater(updateAComponent);
			}
		});
		
		tabs.addMouseListener(new MouseAdapter()
		{
			public void mousePressed(MouseEvent e)
			{
				if(e.getButton() == MouseEvent.BUTTON3)
				{
					GroupchatWindow w =(GroupchatWindow)((JTabbedPane)
							e.getSource()).getSelectedComponent();
					if(w!=null)
					{
						JPopupMenu menu = tabMenu(w);
						menu.show(tabs, e.getX(),e.getY());
					}
				}
			}
		});
		window.setJMenuBar(bar);
		window.setSize(Preferences.getInteger("groupchat","chatTabWidth",700),Preferences.getInteger("groupchat","chatTabHeight",400));
		int posX = Preferences.getInteger("groupchat","chatTabPosX",100);
		int posY = Preferences.getInteger("groupchat","chatTabPosY",100);
		int screenX = (int)Toolkit.getDefaultToolkit().getScreenSize().getWidth();
		int screenY = (int)Toolkit.getDefaultToolkit().getScreenSize().getHeight();
		if(posX>screenX) posX=100;
		if(posY>screenY) posY=100;
		window.setLocation(posX,posY);
		window.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		window.addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e) 
			{
				int tabCount = tabs.getTabCount();
				if(tabCount>1)
				{
					int answer = JOptionPane.showConfirmDialog(window
						,MessageFormat.format(I18N.gettext(
						"main.chat.You_are_about_to_close_" +
						"{0}_tabs_Are_you_sure_you_want_to_close_them?")
						,new Object[]{new Integer(tabCount)}),""
						,JOptionPane.YES_NO_OPTION);
					if(answer==JOptionPane.NO_OPTION)return;
				}
				savePreferences();
				for(int i =0;i<tabCount;i++)
				{
					GroupchatWindow c =  (GroupchatWindow)tabs.getComponent(i);
					c.exit();
					removeGroupchat(c.getJID());
				}
				window.dispose();
				window=null;
			}
		});
		window.addWindowFocusListener(new WindowAdapter()
		{
			public void windowGainedFocus(WindowEvent e)
			{
				GroupchatWindow c= (GroupchatWindow)tabs.getSelectedComponent();
				if(c!=null)c.requestFocusInWindow();
			}
		});
		window.getContentPane().add(tabs);
		chatDividerLocation = Preferences.getInteger("groupchat","chatTabDivider",200);
		dividerLocation = Preferences.getInteger("groupchat","TabDivider",460);
		chatDividerListener = new ComponentAdapter(){
			public void componentMoved(ComponentEvent e){
				chatDividerLocation = ((JSplitPane)e.getComponent()
						.getParent()).getDividerLocation();
				//System.out.println(chatDividerLocation);
			}
		};
		dividerListener = new ComponentAdapter(){
			public void componentMoved(ComponentEvent e){
				dividerLocation = ((JSplitPane)e.getComponent()
						.getParent()).getDividerLocation();
				//System.out.println(dividerLocation);
			}
		};
	}
	
	private void addMenu(final GroupchatWindow groupchatWindow)
	{
		if(Preferences.getBoolean("groupchat", "hideGroupchatMenu", false))return;
		final JMenu[] menu = groupchatWindow.getMenu();
		if(menu[0].getItem(menu[0].getItemCount()-1).getName()==null)
		{
			final JMenuItem menuMoveToWindow = new JMenuItem();
			final JMenuItem menuClose = new JMenuItem();
			I18N.setTextAndMnemonic("main.chat.menu.Move_to_Window",menuMoveToWindow,true);
			menuMoveToWindow.addActionListener(new java.awt.event.ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					menu[0].remove(menuMoveToWindow);
					menu[0].remove(menuClose);
					removeTab(groupchatWindow);
					groupchatWindow.removeChatSplitBarListener(chatDividerListener);
					groupchatWindow.removeSplitBarListener(dividerListener);
					groupchatWindow.changeToWindow();
					setChatWindowPosition(groupchatWindow);
				}
			});
			menu[0].add(menuMoveToWindow);
			I18N.setTextAndMnemonic("main.chat.menu.Close",menuClose,true);
			menuClose.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					groupchatWindow.removeChatSplitBarListener(chatDividerListener);
					groupchatWindow.removeSplitBarListener(dividerListener);
					groupchatWindows.remove(groupchatWindow);
					removeTab(groupchatWindow);
					groupchatWindow.exit();
				}
			});
			menuClose.setName("close");
			menu[0].add(menuClose);
		}
		for(int i=0;i<menu.length;i++)
		{
			window.getJMenuBar().add(menu[i]);
		}
	}

	private JPopupMenu tabMenu(final GroupchatWindow groupchatWindow)
	{
		JPopupMenu groupPopupMenu = new JPopupMenu();
		JMenuItem menuItem = null;
		menuItem = new JMenuItem();
		I18N.setTextAndMnemonic("main.chat.menu.Close",menuItem,true);
		menuItem.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				groupchatWindow.removeChatSplitBarListener(chatDividerListener);
				groupchatWindow.removeSplitBarListener(dividerListener);
				groupchatWindow.exit();
				groupchatWindows.remove(groupchatWindow);
				removeTab(groupchatWindow);
			}
		});
		groupPopupMenu.add(menuItem);
		return groupPopupMenu;
	}
	
	public void convertToTab(GroupchatWindow gcw)
	{
		if(window==null)makeNewTabWindow();
		if(!window.isDisplayable()) addMenu(gcw);
		addTab(gcw);
		gcw.setChatDividerLocation(chatDividerLocation);
		gcw.addChatSplitBarListener(chatDividerListener);
		gcw.setDividerLocation(dividerLocation);
		gcw.addSplitBarListener(dividerListener);
		gcw.setParentFrame(window);
		window.setVisible(true);
	}
		
	public void setTitleAt(final GroupchatWindow cw,final JID jid)
	{
		if(tabs==null)return;//groupchat in applet main window
		SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				int index = tabs.indexOfComponent(cw);
				if(index==-1 || window==null)return;
				tabs.setTitleAt(index, getTitle(jid));
				if(tabs.getSelectedIndex()==index)
				{
					window.setTitle(jid.toStringNoResource());
				}
			}
		});
	}

	private String getTitle(JID jid)
	{
		String title = jid.getUser();
		if(title.indexOf('%')>0)
		{
			title = title.substring(0, title.indexOf('%'));
		}
		if(title.length()>20) title = title.substring(0, 20);
		else if(title.length()<5)
		{
			title = title + "     ".substring(0,5);
		}
		return title;
	}
	
	public void setIconAt(final GroupchatWindow cw, final ImageIcon icon)
	{
		if(tabs==null)return;//groupchat in applet main window
		SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				int index = tabs.indexOfComponent(cw);
				if(index==-1 || window==null)return;
				tabs.setIconAt(index, icon);
				if(tabs.getSelectedIndex()==index)
				{
					window.setIconImage(icon.getImage());
				}
			}
		});
	}
	
//	---------------------message event---------------------------
	public void message(final Message message)
	{
		//System.out.println("got message" + message.getBody());
		if(message.getType().equals("groupchat"))
		{
			SwingUtilities.invokeLater(new Runnable()
			{
				public void run()
				{
					GroupchatWindow gcw;
					synchronized (groupchatWindows)
					{
						gcw= (GroupchatWindow) groupchatWindows.get(message.getFrom());
					}
					if(gcw != null)
					{
						if(tabs!=null)
						{
							int index = tabs.indexOfComponent(gcw);
							if(index!=-1) 
							{
								if(tabs.getSelectedIndex()!=index)
								{
									tabs.setBackgroundAt(index,Color.BLUE);
									tabs.setForegroundAt(index,Color.RED);
								}
								tabs.setToolTipTextAt(index,message.getBody());
							}
						}
						gcw.appendMessage(message);
					}
					else System.out.println(message.getBody());
				}
			});
		}
	}
		
	private void setChatWindowPosition(GroupchatWindow chatWindow)
	{
		synchronized (groupchatWindows)
		{
			if(groupchatWindows.isEmpty())
			{
				int posX = Preferences.getInteger("groupchat","chatPosX",100);
				int posY = Preferences.getInteger("groupchat","chatPosY",100);
				int screenX = (int)Toolkit.getDefaultToolkit().getScreenSize().getWidth();
				int screenY = (int)Toolkit.getDefaultToolkit().getScreenSize().getHeight();
				if(posX>screenX) posX=100;
				if(posY>screenY) posY=100;
				chatWindow.setLocationOnScreen(posX,posY);
			}
			else
			{
				GroupchatWindow oldWindow = (GroupchatWindow)groupchatWindows.values().toArray()[groupchatWindows.size()-1];
				chatWindow.setLocationOnScreen(oldWindow);
			}
		}
	}
	
	public void removeGroupchat(JID jid) {
		synchronized (groupchatWindows)
		{
			groupchatWindows.remove(jid);
		}
	}

	private void savePreferences()
	{
		Preferences.putInteger("groupchat","chatTabPosX",window.getX());
		Preferences.putInteger("groupchat","chatTabPosY",window.getY());
		Preferences.putInteger("groupchat","chatTabHeight",window.getHeight());
		Preferences.putInteger("groupchat","chatTabWidth",window.getWidth());
		Preferences.putInteger("groupchat","chatTabDivider",chatDividerLocation);
		Preferences.putInteger("groupchat","TabDivider",dividerLocation);
	}

	/*TODO add is typing to list of participants
	public synchronized void onComposing(JID from, String thread, XMessageEvent messageEvent)
	{
		ChatWindow chatWindow = null;
		chatWindow = getChatwindow(from);
		if(chatWindow!=null)chatWindow.composing(messageEvent.getType());
	}
	
	public synchronized void requestComposing(JID from, String id, String thread)
	{
		ChatWindow chatWindow = null;
		chatWindow = getChatwindow(from);
		if(chatWindow!=null)chatWindow.composingID(id);
	}
	*/
}
