/* 
 *	Jeti, a Java Jabber client, Copyright (C) 2003 E.S. de Boer  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	For questions, comments etc, 
 *	use the website at http://jeti.jabberstudio.org
 *  or mail me at eric@jeti.tk
 *
 *	Created on 13-okt-2003
 */

package nu.fw.jeti.plugins.groupchat;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JMenu;
import javax.swing.JMenuItem;

import nu.fw.jeti.events.StatusChangeListener;
import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.jabber.JIDStatus;
import nu.fw.jeti.jabber.elements.Presence;
import nu.fw.jeti.plugins.Avatars;
import nu.fw.jeti.plugins.Plugins;
import nu.fw.jeti.plugins.PluginsInfo;
import nu.fw.jeti.plugins.RosterMenuListener;
import nu.fw.jeti.plugins.groupchat.elements.XMUC;
import nu.fw.jeti.plugins.groupchat.handlers.XConferenceHandler;
import nu.fw.jeti.plugins.groupchat.handlers.XMUCAdminHandler;
import nu.fw.jeti.plugins.groupchat.handlers.XMUCOwnerHandler;
import nu.fw.jeti.plugins.groupchat.handlers.XMUCUserHandler;
import nu.fw.jeti.ui.StatusButton;
import nu.fw.jeti.util.I18N;
import nu.fw.jeti.util.Preferences;

/**
 * @author E.S. de Boer
 *
 */
public class Plugin implements Plugins
{
	public final static String VERSION = "0.2";
	public final static String DESCRIPTION = "groupchat.groupchat";
	public final static String MIN_JETI_VERSION = "0.5.1";
	public final static String NAME = "groupchat";
	public final static String ABOUT = "by M. Forssen and E.S. de Boer";
	private GroupChatWindows groupchatWindows;
	private JMenu menu;//save menu to remove it
	private Bookmarks bookmarks;
	private static Plugin plugin;
		
	public static void init(Backend backend)
	{
		plugin = new Plugin(backend);
	}

	public Plugin(final Backend backend)
	{
		menu = new JMenu();
		//groupchat menu item
		if(!Preferences.getBoolean("groupchat","hideGroupchatMenuEntry", false))
		{
			I18N.setTextAndMnemonic("groupchat.groupchat",menu);
			JMenuItem menuItem = new JMenuItem();
			I18N.setTextAndMnemonic("groupchat.Join/Create", menuItem, true);
			menuItem.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					new GroupchatSignin(backend).setVisible(true);
				}
			});
			menu.add(menuItem);
			backend.getMain().addToMenu(menu);
		}
		
		backend.getMain().addToRosterMenu(I18N.gettext("groupchat.Invite_for_groupchat") + "...",new RosterMenuListener()
		{
			public void actionPerformed(JIDStatus jidStatus,nu.fw.jeti.backend.roster.JIDStatusGroup group)
			{
                GroupchatInvite.inviteUser(backend, jidStatus.getJID(),
                                           groupchatWindows.getRooms());
			}
		});
		//backend.addListener(MessageListener.class, this);
        backend.addExtensionHandler("http://jabber.org/protocol/muc#user",
                                    new XMUCUserHandler());
        backend.addExtensionHandler("http://jabber.org/protocol/muc#owner",
                                    new XMUCOwnerHandler());
        backend.addExtensionHandler("http://jabber.org/protocol/muc#admin",
                                    new XMUCAdminHandler());
        backend.addExtensionHandler("jabber:x:conference",
                                    new XConferenceHandler());
        groupchatWindows = new GroupChatWindows(backend);
		bookmarks = new Bookmarks(backend, menu);
	}
	
	/*
	public static synchronized void addGroupchat(JID jid, GroupchatWindow gcw)
	{
		setChatWindowPosition(gcw);
		groupchatWindows.put(jid,gcw); 
	}
	*/
	
	/**
	 * Creates a new groupchat window as Jpanel
	 * @param jid The groupchat JID
	 * @return The Groupchat window for jid
	 */
	public static GroupchatWindow createGroupChatWindow(JID jid) 
	{
		return plugin.groupchatWindows.createGroupChatWindow(jid);
	}
	
	/**
     * Checks if a groupchat with jid is started and return its JID
	 * @param jid The groupchat JID to check
	 * @return the JID of the loaded groupchat or null if not found
	 */
    public static JID getGroupchatJID(JID jid) 
    {
    	return plugin.groupchatWindows.isGroupchatStarted(jid);
    }
	
	/**
     * Gets the groupchat window for the groupchat JID jid, if there is no
     * window yet it will be created.
	 * @param jid The groupchat JID
	 * @return The Groupchat window for jid
	 */
    public static GroupchatWindow getGroupchat(JID jid) 
    {
    	return plugin.groupchatWindows.getGroupchatWindow(jid);
	}
    
    public static void startGroupchat(JID jid,Backend backend)
    {
    	if(PluginsInfo.isPluginLoaded("groupchatalerter"))
    	{
    		if(nu.fw.jeti.plugins.groupchatalerter.Plugin.disableAlerter(jid))return;
    		else getGroupchat(jid);
    	}
    	else getGroupchat(jid);
	    int status =Presence.AVAILABLE;
	    String message=null;
	    if (Preferences.getBoolean("jeti", "statusLinked", false)) {
	    	status = StatusButton.getStatus();
	    	message = StatusButton.getStatusMessage();
	    }
	    Avatars avatars = getAvatarInterface();
    	Presence presence=null;
		if(avatars!=null)
		{
			presence = avatars.addHash(jid,status,message,new XMUC());
		}
		else presence = new Presence(jid,status,message,new XMUC());
		backend.send(presence);
    }
   
    private static Avatars getAvatarInterface()
	{
		if(PluginsInfo.isPluginLoaded("ichatavatars"))
		{
			return  (Avatars)PluginsInfo.getPluginInstance("ichatavatars");
		}
		return null;
	}
			
	public static void addBookmark(JID jid, String nick)
	{
		plugin.bookmarks.addBookmark(jid,nick);
	}
	
	public void unload() {}
	
	public static void unload(Backend backend)
	{
		plugin.groupchatWindows.exit();
		backend.removeExtensionHandler("http://jabber.org/protocol/muc#user");
		backend.removeExtensionHandler("http://jabber.org/protocol/muc#owner");
		backend.removeExtensionHandler("jabber:x:conference");
//		backend.removeListener(MessageListener.class, plugin);
		backend.removeListener(StatusChangeListener.class,plugin.bookmarks);
		backend.getMain().removeFromMenu(plugin.menu);
		backend.getMain().removeFromRosterMenu(I18N.gettext("groupchat.Invite_for_groupchat"));
		plugin=null;
	}
}

/*
 * Overrides for emacs
 * Local variables:
 * tab-width: 4
 * End:
 */
