package nu.fw.jeti.plugins.groupchat.elements;

import java.text.MessageFormat;

import nu.fw.jeti.backend.XExecutableExtension;
import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.JID;

import nu.fw.jeti.jabber.elements.Extension;
import nu.fw.jeti.jabber.elements.Packet;
import nu.fw.jeti.jabber.elements.Presence;
import nu.fw.jeti.jabber.elements.XExtension;
import nu.fw.jeti.plugins.groupchat.Plugin;
import nu.fw.jeti.util.I18N;
import nu.fw.jeti.util.Popups;

/**
 * @author Martin Forssen
 *
 */
public class XMUCUserInvite extends Extension implements XExecutableExtension
{
    private JID user;
    private String reason;
    private boolean decline;

    public XMUCUserInvite(JID user, String reason) {
        this.user = user;
        this.reason = reason;
    }
    
    public XMUCUserInvite(JID user, String reason,boolean decline) {
        this.user = user;
        this.reason = reason;
        this.decline = decline;
    }
    
    public void execute(final Packet packet, final Backend backend)
    {
    	if(decline) {
    		String text;
    		if(reason == null) text = "{0}_has_declined_your_invitation";
    		else text = "{0}_has_declined_your_invitation because of {1}";
    		Popups.messagePopup(
    				MessageFormat.format(I18N.gettext(text),
    						new Object[]{user,reason}),
    				I18N.gettext("groupchat.Groupchat_Invitation"));
    	}
    	else answerInvite(packet, backend);
    }

	private void answerInvite(final Packet packet, final Backend backend)
	{
		Popups.OptionChoosed choose = new Popups.OptionChoosed()
		{
			public void optionChoosed(int option)
			{
				if (option == javax.swing.JOptionPane.NO_OPTION)
				{
					JID temp = packet.getFrom();  
					JID jid = new JID(temp.getUser(),temp.getDomain(),backend.getMyJID().getUser());
					Plugin.startGroupchat(jid, backend);
				}
				else 
				{
					
				}
			}
		};

		Object[] options = { I18N.gettext("groupchat.Decline") , I18N.gettext("groupchat.Accept") };
		Popups.showOptionDialog(
			MessageFormat.format(I18N.gettext("groupchat.{0}_invites_you_to_room_{1}_on_{2}"),
					new Object[]{user,packet.getFrom().getUser(),packet.getFrom().getDomain()}),
					I18N.gettext("groupchat.Groupchat_Invitation"),
			javax.swing.JOptionPane.YES_NO_OPTION,
			javax.swing.JOptionPane.QUESTION_MESSAGE,
			null,
			options,
			options[1],
			choose);
	}
    
    public boolean isDeclined()
    {
    	return decline;
    }
	
    public String getReason()
    {
    	return reason;
    }
    
    public void appendToXML(StringBuffer xml)
    {
        xml.append("<x xmlns='http://jabber.org/protocol/muc#user'>");
        xml.append("<invite");
        appendAttribute(xml, "to", user);
        xml.append(">");
        appendElement(xml, "reason", reason);
        xml.append("</invite>");
        xml.append("</x>");
    }
}

/*
 * Overrides for emacs
 * Local variables:
 * tab-width: 4
 * End:
 */
