/* 
 *	Jeti, a Java Jabber client, Copyright (C) 2001 E.S. de Boer  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	For questions, comments etc, 
 *	use the website at http://jeti.jabberstudio.org
 *  or mail/IM me at jeti@jabber.org
 *  
 *  Created on 7-aug-2005
 */

package nu.fw.jeti.plugins.groupchatalerter;

import java.awt.Container;
import java.awt.Frame;
import java.awt.Window;
import java.awt.event.WindowEvent;
import java.awt.event.WindowStateListener;
import java.util.HashMap;
import java.util.Iterator;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JFrame;
import javax.swing.SwingUtilities;

import nu.fw.jeti.events.ChatEndedListener;
import nu.fw.jeti.events.MessageListener;
import nu.fw.jeti.events.PresenceListener;
import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.jabber.elements.Message;
import nu.fw.jeti.jabber.elements.Presence;
import nu.fw.jeti.jabber.elements.XDelay;
import nu.fw.jeti.plugins.Avatars;
import nu.fw.jeti.plugins.PluginsInfo;
import nu.fw.jeti.plugins.groupchat.GroupchatWindow;
import nu.fw.jeti.plugins.groupchat.Plugin;
import nu.fw.jeti.util.Preferences;

/**
 * @author E.S. de Boer
 *
 */

//TODO login error
//<presence from="jdev@public.jabber.org/Jeti" to="esdeboer@l4l.be/JETI" type="error"><show>xa</show><status>alerter</status><x xmlns="http://jabber.org/protocol/muc"></x><error code="404" type="cancel"><remote-server-not-found xmlns="urn:ietf:params:xml:ns:xmpp-stanzas"></remote-server-not-found></error></presence>
//21:55:21 - p: <presence to="esdeboer@l4l.be/JETI" from="jdev@public.jabber.org/Jeti" type="error"><show>xa</show><status>alerter</status><error code="404" type="cancel"><remote-server-not-found xmlns='urn:ietf:params:xml:ns:xmpp-stanzas'/></error></presence>

public class Alerter implements MessageListener,PresenceListener, ChatEndedListener 
{
	private Matcher matcher;
	private Backend backend;
	private JID jid;
	private boolean monitoring;
	private boolean awake=false;
	private boolean iconofied;
	private MessagesStore messages;
	private HashMap presences;
	private Message subject;
	private GroupchatWindow groupchatWindow;
	private WindowStateListener windowStateListener;
	private boolean pause=false;
	
	public Alerter(Backend backend,JID jid, String rule)
	{
		this.backend = backend;
		messages = new MessagesStore();
		presences = new HashMap();
		backend.addListener(ChatEndedListener.class, this);
		if(jid.getResource()==null)jid = new JID(jid.getUser(),jid.getDomain()
				,backend.getMyJID().getUser());
		this.jid = jid;
		backend.addMessageListener(jid, this);
		backend.addPresenceListener(jid,this);
		sendPresence(jid,Presence.AWAY,Preferences.getString(
				"groupchatalerter","alerterStatusMessage","alerter"));
		if(Plugin.getGroupchatJID(jid)!=null)
		{
			groupchatWindow = Plugin.getGroupchat(jid);
			monitoring = false;
			addWindowStateListener(groupchatWindow);
		}
		else monitoring = true;
		matcher = Pattern.compile(rule,Pattern.CASE_INSENSITIVE).matcher("");
	}
	
	private Avatars getAvatarInterface()
	{
		if(PluginsInfo.isPluginLoaded("ichatavatars"))
		{
			return  (Avatars)PluginsInfo.getPluginInstance("ichatavatars");
		}
		return null;
	}
	
	private void sendPresence(JID jid,int status, String message)
	{
		Avatars avatars = getAvatarInterface();
		Presence presence=null;
		if(avatars!=null)
		{
			presence = avatars.addHash(jid,status,message,new nu.fw.jeti.plugins.groupchat.elements.XMUC());
		}
		else presence = new Presence(jid,status,message,new nu.fw.jeti.plugins.groupchat.elements.XMUC());
		backend.send(presence);
	}
	
	public JID getJID()
	{
		return jid;
	}
	
	public synchronized void message(final Message message)
	{
		if(!awake && !monitoring && message.getFrom().equals(jid) && jid.getResource().equals(message.getFrom().getResource()))
		{
			sendPresence(jid,Presence.AVAILABLE,null);
			awake=true;
		}
		
		if(message.getSubject()!=null && !"error".equals(message.getType()))
		{
			subject = message;
			if(iconofied || !monitoring)groupchatWindow.appendMessage(subject);
		}
		if(message.getBody()==null)	return;
		System.out.println("message " + message.getBody());
		if(!pause && !hasDelay(message) && matcher.reset(message.getBody()).find())
		{
			SwingUtilities.invokeLater(new Runnable()
			{
				public void run()
				{
					if(monitoring)
					{
						GroupchatWindow gcw = Plugin.getGroupchat(jid);
						monitoring = false;
						awake =false;
						if(gcw==null)return;
						for(Iterator i = presences.values().iterator();i.hasNext();)
						{
							gcw.presenceChanged((Presence)i.next());
						}
						if(subject!=null)gcw.appendMessage(subject);
						messages.releaseMessages(gcw);
						groupchatWindow = gcw;
						deIconify();
						addWindowStateListener(gcw);
					}
					else if (iconofied)
					{
						deIconify();
					}
					for(Iterator i = backend.getListeners(MessageListener.class);i.hasNext();)
					{
						((MessageListener)i.next()).message(message);
					}
				}
			});
		}
		else 
		{
			if(iconofied)groupchatWindow.appendMessage(message);
			else if(!monitoring)
			{
				for(Iterator i = backend.getListeners(MessageListener.class);i.hasNext();)
				{
					((MessageListener)i.next()).message(message);
				}
			}
			messages.addMessage(message);
		}
	}
	
	private void deIconify()
	{
		if(groupchatWindow.getTopLevelAncestor() instanceof JFrame)
		{
			JFrame frame = (JFrame)groupchatWindow.getTopLevelAncestor();
			if(frame.getExtendedState()==Frame.ICONIFIED)
			{
				frame.setState(Frame.NORMAL);
			}
			frame.toFront();
		}
	}
	
	private boolean hasDelay(Message message)
	{
		if(message.hasExtensions())
		{
			for(Iterator i =message.getExtensions();i.hasNext();)
			{
				if(i.next() instanceof XDelay)return true;
			}
		}
		return false;
	}
	
	public synchronized void updateRule(String rule)
	{
		matcher = Pattern.compile(rule,Pattern.CASE_INSENSITIVE).matcher("");
	}
	
	public  void disable()
	{
		groupchatWindow = Plugin.getGroupchat(jid);
		//backend.removeMessageListener(jid);
		//backend.removePresenceListener(jid);
		monitoring = false;
		for(Iterator i = presences.values().iterator();i.hasNext();)
		{
			groupchatWindow.presenceChanged((Presence)i.next());
		}
		if(subject!=null)groupchatWindow.appendMessage(subject);
		messages.releaseMessages(groupchatWindow);
		addWindowStateListener(groupchatWindow);
	}
	
	private void addWindowStateListener(GroupchatWindow gcw)
	{
		if(Preferences.getBoolean("groupchatalerter", "alerterOnMinimize", false))
		{
			Container c= gcw.getTopLevelAncestor();
			if(c instanceof Window)
			{
				if(windowStateListener==null)
				{
					windowStateListener= new WindowStateListener()
					{
						public void windowStateChanged(WindowEvent e)
						{
							if((e.getNewState() & Frame.ICONIFIED) != 0)
							{
								iconofied=true;
								awake =false;
								sendPresence(jid,Presence.AWAY,Preferences.getString(
										"groupchatalerter","alerterStatusMessage","alerter"));
								//backend.addMessageListener(jid,Alerter.this);
							}
							else
							{
								iconofied=false;
								//backend.removeMessageListener(jid);
							}
						}
					};
				}
				((Window)c).addWindowStateListener(windowStateListener);
			}
		}
	}
	
	public synchronized void pause(boolean pause) {this.pause = pause;}
	
	public void offline() {presences.clear();}
	
	public  void presenceChanged(Presence presence)
	{
		if(!monitoring)
		{
			groupchatWindow.presenceChanged(presence);
		}
		else if(presence.getType().equals("error") && !monitoring) 
		{
			GroupchatWindow gcw = Plugin.getGroupchat(jid);
			monitoring = false;
			gcw.presenceChanged(presence);
			return;
		}
		if(presence.getFrom().getResource()==null);//do nothing server presence
		else if(presence.getShow()==Presence.UNAVAILABLE)
		{
			if(presence.getFrom().equals(jid) && presence.getFrom().getResource().equals(presence))presences.clear();
			//if(presence.getFrom().equals(jid) && presence.getFrom().getResource().equals(presence.getFrom().getResource()))presences.clear();
			else presences.remove(presence.getFrom().toString());
		}
		else presences.put(presence.getFrom().toString(), presence);
	}
	
	
	public void chatEnded(JID jid)
	{
		if(monitoring == false && this.jid.equals(jid) && this.jid.getResource().equals(jid.getResource()))
		{
			sendPresence(jid,Presence.AWAY,Preferences.getString(
					"groupchatalerter","alerterStatusMessage","alerter"));
			monitoring = true;
			awake = false;
		}
		iconofied=false;
	}
	
	public void remove()
	{
		backend.removeMessageListener(jid);
		backend.removePresenceListener(jid);
		backend.removeListener(ChatEndedListener.class, this);
		if(windowStateListener!=null&&groupchatWindow!=null)
		{
			Window w = (Window)groupchatWindow.getTopLevelAncestor();
			if(w!=null)w.removeWindowStateListener(windowStateListener);
		}
		backend.send(new Presence(jid,"unavailable" ));
	}

}
