/* 
 *	Jeti, a Java Jabber client, Copyright (C) 2001 E.S. de Boer  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	For questions, comments etc, 
 *	use the website at http://jeti.jabberstudio.org
 *  or mail/IM me at jeti@jabber.org
 *  
 *  Created on 13-aug-2005
 */

package nu.fw.jeti.plugins.groupchatalerter;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.TimeZone;

import nu.fw.jeti.jabber.elements.Extension;
import nu.fw.jeti.jabber.elements.Message;
import nu.fw.jeti.jabber.elements.MessageBuilder;
import nu.fw.jeti.jabber.elements.XDelay;
import nu.fw.jeti.plugins.groupchat.GroupchatWindow;

/**
 * @author E.S. de Boer
 *
 */
public class MessagesStore
{
	private static final int SIZE = 10;
	private int pointer;
	private MessageAndDate[] messages= new MessageAndDate[SIZE];
		
	public void addMessage(Message message)
	{
		messages[pointer] =new MessageAndDate(message);
		pointer++;
		if(pointer>=SIZE)pointer=0;
	}
	
	public void releaseMessages(GroupchatWindow gcw)
	{
		for(int i=pointer;i<SIZE;i++)
		{
			MessageAndDate m = messages[i];
			if(m!=null)gcw.appendMessage(m.getMessage());
		}
		for(int i=0;i<pointer;i++)
		{
			MessageAndDate m = messages[i];
			if(m!=null)gcw.appendMessage(m.getMessage());
		}
		pointer=0;
		messages = new MessageAndDate[SIZE];
	}
}
	class MessageAndDate
	{
		private Message message;
		private long time;
		private static DateFormat dateFormat = new SimpleDateFormat("yyyyMMdd'T'hh:mm:ss");
				
		public MessageAndDate(Message message)
		{
			this.message = message;
			time = System.currentTimeMillis();
			dateFormat.setTimeZone(TimeZone.getTimeZone("UTC"));
		}
		
		/**
		 * returns the message and the time received
		 * @return
		 */
		public Message getMessage()
		{
			MessageBuilder builder = new MessageBuilder();
			builder.body = message.getBody();
			builder.from = message.getFrom();
			builder.to = message.getTo();
			builder.thread = message.getThread();
			builder.type = message.getType();
			boolean delayed=false;
			if(message.hasExtensions())
			{
				for (Iterator iter = message.getExtensions(); iter.hasNext();)
				{
					Extension extension = (Extension)iter.next();
					builder.addExtension(extension);
					if(extension instanceof XDelay)delayed=true;
				};
			}
			if(!delayed)
			{
				builder.addExtension(new XDelay( dateFormat.format(new Date(time)),message.getFrom()));
			}
			return (Message) builder.build();
		}
	}

