/* 
 *	Jeti, a Java Jabber client, Copyright (C) 2001 E.S. de Boer  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	For questions, comments etc, 
 *	use the website at http://jeti.jabberstudio.org
 *  or mail/IM me at jeti@jabber.org
 *  
 *  7-8-2005
 */

package nu.fw.jeti.plugins.groupchatalerter;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

import nu.fw.jeti.events.StatusChangeListener;
import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.jabber.elements.Extension;
import nu.fw.jeti.jabber.elements.Presence;
import nu.fw.jeti.jabber.elements.XExtension;
import nu.fw.jeti.plugins.Avatars;
import nu.fw.jeti.plugins.Plugins;
import nu.fw.jeti.plugins.PluginsInfo;
import nu.fw.jeti.util.Preferences;

/**
 * @author E.S. de Boer
 * @version 1.0
 */

public class Plugin implements Plugins,StatusChangeListener
{
	public final static String NAME = "groupchatalerter";
	public final static String VERSION = "1.0";
	public final static String DESCRIPTION = "groupchatalerter.Alerts_you_when_there_is_something_interesting";
	public final static String MIN_JETI_VERSION = "0.6.7";
	public final static String ABOUT = "by E.S. de Boer";
	public static final int ALERTERS_SIZE = 6;
	private static Plugin plugin;
	private Map alerters;
	private Backend backend;
	private boolean away=false;

	public static void init(Backend backend)
	{
		try {
		plugin = new Plugin(backend);
		}catch (Exception e) {
			e.printStackTrace();
		}
	}

	public Plugin(Backend backend)
	{
		alerters = new HashMap();
		this.backend = backend;
		backend.addListener(StatusChangeListener.class, this);
	}
	
	public static boolean isMonitoring(JID jid)
	{
		synchronized (plugin.alerters)
		{
			return plugin.alerters.containsKey(jid);
		}
	}
	
	public static boolean disableAlerter(JID jid)
	{
		Alerter a =null;
		synchronized (plugin.alerters)
		{
			a = (Alerter)plugin.alerters.get(jid);
		}
		if(a!=null)
		{
			a.disable();
			return true;
		}
		return false;
	}
	
	public static void updateRule(JID jid, String rule)
	{
		synchronized (plugin.alerters)
		{
			((Alerter)plugin.alerters.get(jid)).updateRule(rule);
		}
	}

	public static void addAlerter(Alerter alerter, JID jid)
	{
		synchronized (plugin.alerters)
		{
			if(plugin.alerters.containsKey(jid))
			{
				((Alerter)plugin.alerters.remove(jid)).remove();
			}
			plugin.alerters.put(jid, alerter);
		}
	}
	
	public static void removeAlerter(JID jid)
	{
		Alerter a =null;
		synchronized (plugin.alerters)
		{
			a = (Alerter)plugin.alerters.remove(jid);
		}
		if(a!=null)a.remove();
	}
				
	public void unload()
	{
		for(Iterator i = plugin.alerters.values().iterator();i.hasNext();)
		{
			((Alerter)i.next()).remove();
		}
		plugin.alerters.clear();
		backend.removeListener(StatusChangeListener.class, this);
	}

	public static void unload(Backend backend)
	{
		plugin=null;
	}
	
	private Avatars getAvatarInterface()
	{
		if(PluginsInfo.isPluginLoaded("ichatavatars"))
		{
			return  (Avatars)PluginsInfo.getPluginInstance("ichatavatars");
		}
		return null;
	}
	
	private void sendPresence(JID jid,int status, String message)
	{
		Avatars avatars = getAvatarInterface();
		Presence presence=null;
		if(avatars!=null)
		{
			XExtension e = new nu.fw.jeti.plugins.groupchat.elements.XMUC();
			presence = avatars.addHash(jid,status,message,e);
		}
		else presence = new Presence(jid,status,message,new nu.fw.jeti.plugins.groupchat.elements.XMUC());
		backend.send(presence);
	}
	
	public void connectionChanged(boolean online)
	{
		if(online)
		{
			for(int i=0;i<Plugin.ALERTERS_SIZE;i++)
	        {
				if(Preferences.getBoolean("groupchatalerter","ruleEnabled"+i, false))
				{
					JID jid = JID.jidFromString(Preferences.getString("groupchatalerter","room"+i,null));
					if(isMonitoring(jid))
					{
						synchronized (alerters)
						{
							jid =((Alerter)alerters.get(jid)).getJID();
							sendPresence(jid,Presence.AWAY,Preferences.getString(
									"groupchatalerter","alerterStatusMessage","alerter"));
						}
					}
					else
					{
						JID j2 = nu.fw.jeti.plugins.groupchat.Plugin.getGroupchatJID(jid);
						if(j2!=null)
						{//groupchat already loaded, init with that groupchat nickname
							jid =j2;
						}
						Plugin.addAlerter(new Alerter(backend,jid,Preferences.getString("groupchatalerter","rule"+i,null)),jid);
					}
					if(PluginsInfo.isPluginLoaded("ichatavatars"))
					{
						sendAvatarPresence(jid);
					}
				}
	        }
		}
		else
		{
			synchronized (plugin.alerters)
			{
				for(Iterator i =alerters.values().iterator();i.hasNext();)
				{
					((Alerter)i.next()).offline();
				}
			}
		}
	}
	
	public void sendAvatarPresence(final JID jid)
	{
		Timer t = new Timer();
		t.schedule(new TimerTask() {
			public void run() {
				sendPresence(jid,Presence.AWAY,Preferences.getString(
						"groupchatalerter","alerterStatusMessage","alerter"));

			}
		},60000);
	}
	
	public void exit() {}
	
	public void ownPresenceChanged(int show,String status) 
	{
		if(Preferences.getBoolean("groupchatalerter", "changeStatus", false))
		{
			if(show == Presence.DND || show == Presence.XA)
			{
				synchronized (plugin.alerters)
				{
					for(Iterator i =alerters.entrySet().iterator();i.hasNext();)
					{
						Map.Entry entry = (Map.Entry)i.next();
						sendPresence((JID)entry.getKey(),show,status);
						((Alerter)entry.getValue()).pause(true);
					}
				}
				away = true;
			}
			else if (away)
			{
				away=false;
				synchronized (plugin.alerters)
				{
					for(Iterator i =alerters.entrySet().iterator();i.hasNext();)
					{
						Map.Entry entry = (Map.Entry)i.next();
						sendPresence((JID)entry.getKey(),Presence.AWAY,Preferences.getString(
								"groupchatalerter","alerterStatusMessage","alerter"));
						((Alerter)entry.getValue()).pause(false);
					}
				}
			}
		}
	}

}
/*
 * Overrides for emacs
 * Local variables:
 * tab-width: 4
 * End:
 */
