/* 
 *	Jeti, a Java Jabber client, Copyright (C) 2002 E.S. de Boer  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	For questions, comments etc, 
 *	use the website at http://jeti.jabberstudio.org
 *  or mail/im me at jeti@jabber.org
 *	Created on 7-aug-2005
 */
package nu.fw.jeti.plugins.groupchatalerter;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import javax.swing.*;

import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.plugins.PreferencesPanel;
import nu.fw.jeti.util.I18N;
import nu.fw.jeti.util.Preferences;


public class PrefPanel extends PreferencesPanel {
   
	private Backend backend;
    private JTextField txtStatusMessage;
    private JCheckBox chkAlertOnMinimize;
    private JCheckBox chkChangeStatus;
       
	public PrefPanel(Backend backend)
	{
		this.backend = backend;
		setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        c.anchor = GridBagConstraints.LINE_START;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.insets = new Insets(3, 2, 0, 2);
        c.weighty = 0;
        JLabel label;
        for(int i=0;i<Plugin.ALERTERS_SIZE;i++)
        {
        	createRuleBlock(c,i);
        }
        
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.weighty = 1.0;
        c.weightx = 1.0;
        add(new JPanel(), c);
        
        c.weighty = 0;
        c.weightx = 0;
        
        c.gridwidth = 1;
         chkAlertOnMinimize = new JCheckBox(I18N.gettext("groupchatalerter.Enable_Alerter_on_minimize"));
         chkAlertOnMinimize.setSelected(Preferences.getBoolean("groupchatalerter", "alerterOnMinimize", false));
         add(chkAlertOnMinimize,c);
        
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.weightx = 1;
        chkChangeStatus = new JCheckBox(I18N.gettext("groupchatalerter.Change_alerter_status_when_extended_away"));
        chkChangeStatus.setSelected(Preferences.getBoolean("groupchatalerter", "changeStatus", false));
        add(chkChangeStatus,c);
        
        c.gridwidth = 2;
        c.weightx = 0;
        label = new JLabel(I18N.gettext("groupchatalerter.Alerter_Status_Message"));
        add(label, c);
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.weightx = 1;
        txtStatusMessage = new JTextField(Preferences.getString(
				"groupchatalerter","alerterStatusMessage","alerter"));
        add(txtStatusMessage, c);
        
        
        
        c.gridwidth = 2;
        c.weightx = 0;
        label = new JLabel(I18N.gettext("groupchatalerter.Test_Match_On"));
        label.setToolTipText(I18N.gettext("groupchatalerter.Test_rule_to_test_if_the_rule_works_on_test_text"));
        add(label, c);
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.weightx = 1;
        final JTextField txtRule = new JTextField();
        add(txtRule, c);
        
        c.gridwidth = 2;
        c.weightx = 0;
        label = new JLabel(I18N.gettext("groupchatalerter.Test_Text"));
        label.setToolTipText(I18N.gettext("groupchatalerter.Test_text_to_test_Test_Match_rule_on"));
        add(label, c);
        
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.weightx = 1;
        final JTextField txtTest = new JTextField();
        add(txtTest, c);
        
        JButton button = new JButton(I18N.gettext("groupchatalerter.Test"));
        button.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				try {
					Matcher m = Pattern.compile(txtRule.getText(),Pattern.CASE_INSENSITIVE)
							.matcher(txtTest.getText());
					txtTest.requestFocus();
					if(m.find())
					{
						txtTest.setCaretPosition(m.start()); 
						txtTest.moveCaretPosition(m.end());
					}
					else
					{
						txtTest.setCaretPosition(0); 
						txtTest.moveCaretPosition(0);
					}
					}catch(PatternSyntaxException ex) 
					{
						JOptionPane.showMessageDialog(PrefPanel.this
								, ex.getMessage());
						return;
					}
			}
		});
        add(button, c);
        
              
	}

	private void createRuleBlock(GridBagConstraints c, int ruleBlockNo)
	{
		RuleBlock ruleBlock = new RuleBlock(ruleBlockNo);
		c.gridwidth = 2;
		c.weightx = 0;
		add(ruleBlock.ruleEnabled, c);
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.weightx = 1;
		add(ruleBlock.room, c);
		c.gridwidth = 2;
		c.weightx = 0;
		JLabel label = new JLabel(I18N.gettext("groupchatalerter.Match_on"));
		label.setToolTipText(I18N.gettext("groupchatalerter.Rule_what_to_match,_use_keywords_separated_by_the_|_character,_regular_expressions_are_supported"));
		add(label, c);
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.weightx = 1;
		add(ruleBlock.rule, c);
	}

	public void savePreferences() 
	{
		Preferences.putString("groupchatalerter","alerterStatusMessage"
				,txtStatusMessage.getText());
		Preferences.putBoolean("groupchatalerter", "alerterOnMinimize", chkAlertOnMinimize.isSelected());
		Preferences.putBoolean("groupchatalerter", "changeStatus", chkChangeStatus.isSelected());
		
	}
		
	class RuleBlock
	{
		private JCheckBox ruleEnabled;
		private JTextField room;
		private JTextField rule;
		private int order;
		 
		RuleBlock(int ruleBlockNo)
		{
			order = ruleBlockNo;
			ruleEnabled = new JCheckBox(
	            I18N.gettext("groupchatalerter.Groupchat_room"));
		 	ruleEnabled.setToolTipText(I18N.gettext("groupchatalerter.Room_to_monitor_Add_/nickname_to_set_your_nickname"));
		 	boolean oldRuleEnabled = Preferences.getBoolean("groupchatalerter"
		 			,"ruleEnabled"+ruleBlockNo, false);
		 	ruleEnabled.setSelected(oldRuleEnabled);
		 	ruleEnabled.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					if(ruleEnabled.isSelected())
					{
						room.setEditable(false);
						try
						{
							if(room.getText().equals(""))return;
							JID jid = JID.checkedJIDFromString(room.getText());
							Preferences.putString("groupchatalerter","room"+order
									,room.getText());
							try {
								Pattern.compile(rule.getText());
							}catch(PatternSyntaxException e2) 
							{
								JOptionPane.showMessageDialog(PrefPanel.this, e2.getMessage());
								ruleEnabled.setSelected(false);
								return;
							}
							JID j2 = nu.fw.jeti.plugins.groupchat.Plugin.getGroupchatJID(jid);
							if(j2!=null)
							{//groupchat already loaded, init with that groupchat nickname
								jid =j2;
							}
							Plugin.addAlerter(new Alerter(backend,jid,rule.getText()),jid);
							Preferences.putBoolean("groupchatalerter","ruleEnabled"+order, ruleEnabled.isSelected());
						} catch (InstantiationException e2)
						{
							JOptionPane.showMessageDialog(PrefPanel.this, e2.getMessage());
							ruleEnabled.setSelected(false);
						}
					}
					else 
					{
						Preferences.putBoolean("groupchatalerter","ruleEnabled"+order,ruleEnabled.isSelected());
						Plugin.removeAlerter(JID.jidFromString(room.getText()));
						room.setEditable(true);
					}
				}
			});
		 	room = new JTextField(Preferences.getString(
					"groupchatalerter","room"+ruleBlockNo,null));
		 	rule = new JTextField(Preferences.getString(
					"groupchatalerter","rule"+ruleBlockNo,null));
		 	rule.addFocusListener(new FocusListener()
			{
		 		private String ruleText;
		 		
				public void focusLost(FocusEvent e)
				{
					Preferences.putString("groupchatalerter","rule"+order
							,rule.getText());
					Preferences.putString("groupchatalerter","room"+order
							,room.getText());
					if(ruleEnabled.isSelected() && !rule.getText().equals(ruleText))
					{
						try {
							Pattern.compile(rule.getText());
						}catch(PatternSyntaxException e2) 
						{
							JOptionPane.showMessageDialog(PrefPanel.this, e2.getDescription());
							return;
						}
					}
					JID jid = JID.jidFromString(room.getText());
					if(Plugin.isMonitoring(jid))
					{
						Plugin.updateRule(jid,rule.getText());
					}
				}
			
				public void focusGained(FocusEvent e)
				{
					ruleText = rule.getText();
				}
			});
		 	if(ruleEnabled.isSelected())
			{
		 		room.setEditable(false);
			}
		}
	}
}

/*
 * Overrides for emacs
 * Local variables:
 * tab-width: 4
 * End:
 */
