/* 
 *	Jeti, a Java Jabber client, Copyright (C) 2004 E.S. de Boer  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	For questions, comments etc, 
 *	use the website at http://jeti.jabberstudio.org
 *  or mail me at eric@jeti.tk
 */

// Created on 17-jun-2005
package nu.fw.jeti.plugins.ichatavatars;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.security.MessageDigest;
import java.util.HashMap;
import java.util.Iterator;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JToolTip;
import javax.swing.UIManager;

import nu.fw.jeti.backend.Connect;
import nu.fw.jeti.backend.Start;
import nu.fw.jeti.backend.roster.SecondaryJIDStatus;
import nu.fw.jeti.events.IQResultListener;
import nu.fw.jeti.events.PresenceListener;
import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.jabber.JIDStatus;
import nu.fw.jeti.jabber.elements.*;
import nu.fw.jeti.plugins.Avatars;
import nu.fw.jeti.plugins.Plugins;
import nu.fw.jeti.plugins.vcard.VCard;
import nu.fw.jeti.util.Base64;
import nu.fw.jeti.util.Log;
import nu.fw.jeti.util.Utils;


/**
 * @author E.S. de Boer
 *
 */
public class Plugin implements Plugins,Avatars,PresenceListener
{
	public final static String VERSION = "0.1";
	public final static String DESCRIPTION = "ichatavatars.shows_avatars";
	public final static String MIN_JETI_VERSION = "0.6.6";
	public final static String NAME = "ichatavatars";
	public final static String ABOUT = "by E.S. de Boer";
	private static Plugin plugin;
	private HashMap avatars = new HashMap();
	//private HashMap jidStatussen = new HashMap();
	private Backend backend;
	private JIDStatus toolTipJIDStatus;
	private ImageToolTip toolTip = new ImageToolTip();
	private String myAvatarHash;
	private Connect connect;
				
	public Plugin(Backend backend)
	{
		File file = new File(Start.path + "avatars"+ File.separator);
		if (!file.exists())
		{
			file.mkdir();
		}
		this.backend = backend;
		backend.addExtensionHandler("vcard-temp:x:update",new AvatarUpdateHandler());
		backend.addListener(PresenceListener.class,this);
	}
	
	public static void init(Backend backend)
	{
		plugin = new Plugin(backend);
	}
	
	public static Object getInstance()
	{
		return plugin;
	}
		
	public static void unload(Backend backend)
	{
		backend.removeExtensionHandler("vcard-temp:x:update");
		plugin =null;
	}
	
	public void update(Presence presence,SecondaryJIDStatus jidStatus)
	{
		for(Iterator i=presence.getExtensions();i.hasNext();)
		{
			Extension x =(Extension) i.next();
			if(x instanceof AvatarUpdate)
			{
				update((AvatarUpdate)x, jidStatus);
			}
		}
	}
	
	public void update(AvatarUpdate avatar,final SecondaryJIDStatus jidStatus)
	{
		if(avatar.getNick()!=null && !avatar.getNick().equals(""))jidStatus.setMSNNick(avatar.getNick());
		final String hash = avatar.getHash();
		if(hash==null)return;
		//System.out.println(hash);
//		if(hash.equals(jidStatussen.get(jidStatus)))return;
//		jidStatussen.put(jidStatus, hash);
		if(avatars.containsKey(hash))
		{
			jidStatus.setAvatar((ImageIcon)avatars.get(hash));
		}
		else
		{
			File file = new File(Start.path + "avatars"+ File.separator + hash);
			if (file.exists())
			{
				ImageIcon icon =  new ImageIcon(file.getAbsolutePath());
				avatars.put(hash, icon);
				jidStatus.setAvatar(icon);
			}
			else
			{
				backend.send(new InfoQuery(jidStatus.getJID(),"get", new VCard()),new IQResultListener()
				{
					public void iqResult(InfoQuery iq)
					{
						IQExtension iqe = iq.getIQExtension();
						if(iqe instanceof VCard)
						{
							decode(((VCard)iqe).getPhoto(),hash,jidStatus);
						}
					}
				},100);
			}
		}
	}
	
	public Extension addHash()
	{
		return new AvatarUpdate(myAvatarHash);
	}
	
	public Presence addHash(JID jid,int show, String status)
	{
		return new Presence(jid,show,status,new AvatarUpdate(myAvatarHash));
	}
	
	public Presence addHash(JID jid, int show, String status,XExtension iqX)
	{
		PresenceBuilder pb = new PresenceBuilder();
		pb.to= jid;
		pb.show=show;
		pb.status=status;
		pb.addExtension((Extension)iqX);
		pb.addExtension(new AvatarUpdate(myAvatarHash));
		Presence presence=null;
		try {
			presence=(Presence)pb.build();
		}catch (InstantiationException e){}
		return presence;
	}
	
	private void decode(String avatarData ,String hash, SecondaryJIDStatus jidStatus)
	{
		if (avatarData==null || avatarData.equals(""))return;
		byte[] data = Base64.decode(avatarData);
		ImageIcon icon=null;
		try {
			icon = new ImageIcon(data);
		}catch (Exception e)
		{//wrong image
			System.err.println(e.getMessage());
			return;
		}
		//TODO crop icon to 96*96?
		jidStatus.setAvatar(icon);
		
		String newHash = hash(data);
		if(!hash.equals(newHash))
		{
			System.out.println(newHash);
			System.out.println(hash);
			Log.notParsedXML("Wrong hash of avatar image from" + jidStatus.getNick());
		}
		
//		
//		JFrame f = new JFrame();
//		f.getContentPane().add(new JLabel(icon));
//		f.pack();
//		f.setVisible(true);
//		
		avatars.put(hash, icon);
		save(hash,data);
	}
	
	private void save(String hash, byte[] data)
	{
		try
		{
			OutputStream os = new FileOutputStream(Start.path + "avatars"+ File.separator + hash);
			os.write(data);
			os.flush();
			os.close();
		} catch (IOException e)
		{
			e.printStackTrace();
		}
	}
	
	public void updateAvatar(byte[] bytes)
	{
		myAvatarHash = hash(bytes);
		//System.out.println(myAvatarHash);
	}
	
	public String hash(byte[] bytes)
	{
		MessageDigest sha;
		try {
		  sha = MessageDigest.getInstance("SHA");
		} catch (Exception ex){
		  System.err.println("no SHA");
		  return null;
		}
		return Utils.toString(sha.digest(bytes));
	}
	
	public void sendPresence()
	{
		if(connect!=null)connect.sendStatus();
	}
	
	public void getVCard(Connect connect)
	{
		this.connect = connect;
		backend.send(new InfoQuery("get", new VCard()),new IQResultListener()
		{
			public void iqResult(InfoQuery iq)
			{
				IQExtension iqe = iq.getIQExtension();
				if(iqe instanceof VCard)
				{
					String avatarData = ((VCard)iqe).getPhoto();
					if (avatarData==null || avatarData.equals(""))return;
					byte[] data = Base64.decode(avatarData);
					updateAvatar(data);
					sendPresence();
				}
			}
		},100);
	}
	
	public void unload()
	{
		backend.removeListener(PresenceListener.class,this);
	}
	
	public void presenceChanged(Presence presence)
	{//detect own presence change to determine hash sending
		if(!backend.getMyJID().equals(presence.getFrom()))return;
		//update from other resource
		AvatarUpdate au=null;
		if(presence.hasExtensions())
		{
			for(Iterator i=presence.getExtensions();i.hasNext();)
			{
				Extension x =(Extension) i.next();
				if(x instanceof AvatarUpdate)
				{
					au = (AvatarUpdate)x;
				}
			}
		}
		if(au==null)
		{//other resource does not support avatars, clear hash
			myAvatarHash=null;
		}
		else if(au.getHash() == null)
		{//other resource has no avatar
		}
		else if(au.getHash().equals(myAvatarHash))
		{//other resource has the same avatar
		}	
		else
		{//other resource has a different avatar, update ours
			myAvatarHash=null;
			sendPresence();
			getVCard(connect);
		}
	}
				
	public void setToolTipJIDStatus(JIDStatus jidStatus)
	{
		toolTipJIDStatus = jidStatus;
	}
	
	public JToolTip createToolTip()
	{
		return toolTip;
	}
	
	public class ImageToolTip extends JToolTip
	{
		private ImageIcon icon;
		private JLabel label;
		private JLabel lblIcon;
	 
		public ImageToolTip()
		{
			setLayout(new BorderLayout());
			label = new JLabel();
			label.setOpaque(true);
			label.setBackground(UIManager
					.getColor("ToolTip.background"));
			add(label,BorderLayout.CENTER );
			lblIcon = new JLabel();
			lblIcon.setOpaque(true);
			lblIcon.setBackground(UIManager
					.getColor("ToolTip.background"));
			add(lblIcon,BorderLayout.WEST);
			label.setFont(super.getFont());
		}
		
		public void setTipText(String text)
		{
			label.setText(text);
			if(toolTipJIDStatus!=null)
			{
				icon = toolTipJIDStatus.getAvatar();
				lblIcon.setIcon(icon);
			}
		}
		
		
		public Dimension getPreferredSize()
		{
			Dimension labelSize = label.getPreferredSize();
			if (icon==null)
			{
				return  new Dimension(labelSize.width+2,labelSize.height);
			}
			int y = icon.getIconHeight();
			if (labelSize.getHeight()> y) y= (int)labelSize.getHeight();
			return new Dimension(icon.getIconWidth()+(int)labelSize.getWidth()+2, y);		
		}
	}
	
}
/*
 * Overrides for emacs
 * Local variables:
 * tab-width: 4
 * End:
 */
