/* 
 *	Jeti, a Java Jabber client, Copyright (C) 2007 E.S. de Boer  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	For questions, comments etc, 
 *	use the website at http://jeti.jabberstudio.org
 *
 *	Created on 3-april-2007
 */

package nu.fw.jeti.plugins.jingle;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JMenuItem;
import javax.swing.JOptionPane;

import nu.fw.jeti.backend.roster.Roster;
import nu.fw.jeti.events.IQResultListener;
import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.jabber.JIDStatus;
import nu.fw.jeti.jabber.elements.InfoQuery;
import nu.fw.jeti.plugins.Plugins;
import nu.fw.jeti.plugins.PluginsInfo;
import nu.fw.jeti.plugins.RosterMenuListener;
import nu.fw.jeti.plugins.jingle.jingle.IncomingJingleSession;
import nu.fw.jeti.plugins.jingle.jingle.JingleManager;
import nu.fw.jeti.plugins.jingle.jingle.JingleSession;
import nu.fw.jeti.plugins.jingle.jingle.JingleSessionRequest;
import nu.fw.jeti.plugins.jingle.jingle.OutgoingJingleSession;
import nu.fw.jeti.plugins.jingle.jingle.listeners.JingleSessionListener;
import nu.fw.jeti.plugins.jingle.jingle.listeners.JingleSessionRequestListener;
import nu.fw.jeti.plugins.jingle.jingle.media.PayloadType;
import nu.fw.jeti.plugins.jingle.jingle.mediaimpl.jmf.JmfMediaManager;
import nu.fw.jeti.plugins.jingle.jingle.mediaimpl.jspeex.SpeexMediaManager;
import nu.fw.jeti.plugins.jingle.jingle.mediaimpl.multi.MultiMediaManager;
import nu.fw.jeti.plugins.jingle.jingle.nat.ICETransportManager;
import nu.fw.jeti.plugins.jingle.jingle.nat.JingleTransportManager;
import nu.fw.jeti.plugins.jingle.jingle.nat.STUN;
import nu.fw.jeti.plugins.jingle.jingle.nat.TransportCandidate;
import nu.fw.jeti.plugins.jingle.jingle.nat.RTPBridge.Provider;
import nu.fw.jeti.plugins.jingle.packet.Jingle;
import nu.fw.jeti.plugins.jingle.provider.JingleContentDescriptionProvider;
import nu.fw.jeti.plugins.jingle.provider.JingleContentInfoProvider;
import nu.fw.jeti.plugins.jingle.provider.JingleProvider;
import nu.fw.jeti.plugins.jingle.provider.JingleTransportProvider;
import nu.fw.jeti.util.I18N;
import nu.fw.jeti.util.Popups;

/**
 * @author E.S. de Boer
 *
 */
public class Plugin implements Plugins
{
	public final static String VERSION = "0.2";
	public final static String DESCRIPTION = "jingle.VOIP";
	public final static String MIN_JETI_VERSION = "0.8";
	public final static String NAME = "jingle";
	public final static String ABOUT = "Based on the Jingle implementation of Smack by Jive software";
	private static Plugin plugin;
	private JingleManager manager;
	private Map<JID, JingleSession> sessions;
	private JMenuItem endVoiceMenu;
	
    		
	public static void init(Backend backend)
	{
		PluginsInfo.loadLibrary("jmf");
		PluginsInfo.loadLibrary("jstun");
		PluginsInfo.loadLibrary("Speex");
		PluginsInfo.loadLibrary("jspeex");
		//1.4 compatibility
		PluginsInfo.loadLibrary("retroweaver");
		plugin = new Plugin(backend);
	}

	public Plugin(final Backend backend)
	{
		sessions = new HashMap<JID, JingleSession>(0);
		endVoiceMenu = new JMenuItem("End voice calls");
		endVoiceMenu.setEnabled(false);
		endVoiceMenu.addActionListener(new ActionListener() {
		
			@Override
			public void actionPerformed(ActionEvent e) {
				for(JingleSession session : sessions.values()){
					try {
						session.terminate();
					} catch (XMPPException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					}
				}
				sessions.clear();
				endVoiceMenu.setEnabled(false);
			}
		});
		backend.getMain().addToMenu(endVoiceMenu);
		
		backend.getMain().addToRosterMenu(I18N.gettext("jingle.voice"),new RosterMenuListener()
		{
			public void actionPerformed(JIDStatus jidStatus,nu.fw.jeti.backend.roster.JIDStatusGroup group)
			{
				JingleSession session = sessions.get(jidStatus.getCompleteJID());
				 if (session != null) {
					try
					{
						session.terminate();
						System.out.println("session removing because not yet removed");
						
					} catch (XMPPException e)
					{
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				 }
			      try {
		             OutgoingJingleSession jos = manager.createOutgoingJingleSession(jidStatus.getCompleteJID());
		             jos.addListener(createSessionListener(jidStatus.getCompleteJID()));
		             jos.start();
		             sessions.put(jidStatus.getCompleteJID(),jos);
		             endVoiceMenu.setEnabled(true);
		            } catch (XMPPException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				 
			}
		});
		
		
		String stunServer = "stun.xten.net";
		int stunPort = 3478;

//		if (STUN.serviceAvailable(backend)) {
//			STUN stun = STUN.getSTUNServer(backend);
//			if (stun != null) {
//				List<STUN.StunServerAddress> servers = stun.getServers();
//
//				if (servers.size() > 0) {
//					stunServer = servers.get(0).getServer();
//					stunPort = Integer.parseInt(servers.get(0).getPort());
//				}
//			}
//		}

		JingleTransportManager transportManager = new ICETransportManager(
				backend, stunServer, stunPort);
		// if running on Windows use Direct Sound for better performance
		//String locator = Spark.isWindows() && !Spark.isVista() ? "dsound://" : "javasound://";
		String locator =  "javasound://8000"; //48000 to fix os2 and mac bug
		MultiMediaManager jingleMediaManager = new MultiMediaManager();
		jingleMediaManager.addMediaManager(new JmfMediaManager(locator));
		jingleMediaManager.addMediaManager(new SpeexMediaManager());
		if (System.getProperty("codec") != null) {
			try {
				int codec = Integer.parseInt(System.getProperty("codec"));
				jingleMediaManager.setPreferredPayloadType(jingleMediaManager
						.getPayloads().get(codec));

			} catch (NumberFormatException e) {
				// do nothing
			}
		}
		
	    manager = new JingleManager(backend, transportManager, jingleMediaManager);
	    manager.addCreationListener((ICETransportManager)transportManager);
		
	    	    
	    manager.addJingleSessionRequestListener(new JingleSessionRequestListener() {
            public void sessionRequested(final JingleSessionRequest request) {

                
                JID from = request.getFrom();
                JingleSession session = sessions.get(from);
                if (session != null) {
                	try {
						session.terminate();
						System.out.println("incom session removing because not yet removed");
					} catch (XMPPException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
                	return;
                }
                
                String name = from.getUser();
                JIDStatus jidStatus = Roster.getJIDStatus(from);
                if(jidStatus!=null) name = jidStatus.getNick();
                
                 
                Object[] options = { I18N.gettext("jeti.Decline") , I18N.gettext("jeti.Answer") };                
                Popups.showOptionDialog(MessageFormat.format(I18N.gettext("jingle.{0}_calls,_answer_call?"),name), 
                		I18N.gettext("jingle.New Call"), JOptionPane.YES_NO_OPTION,
                		JOptionPane.QUESTION_MESSAGE, null, 
                		options,options[1], new Popups.OptionChoosed() {
				
					public void optionChoosed(int option) {
						if(option == JOptionPane.NO_OPTION){
			                try {
			                    // Accept the call
			                    IncomingJingleSession incoming = request.accept();
			                    incoming.start();
		                        incoming.addListener(createSessionListener(request.getFrom()));
			                }
			                catch (Exception e) {
			                    e.printStackTrace();
			                }
						}
						else{
							request.reject();
						}
					}
				});
            }
        });
	    
		backend.addExtensionHandler("http://jabber.org/protocol/jingle",new JingleProvider(manager));
		backend.addExtensionHandler("http://jabber.org/protocol/jingle/description/audio",
		                new JingleContentDescriptionProvider.Audio());
		backend.addExtensionHandler("http://jabber.org/protocol/jingle/transport/ice",
		                new JingleTransportProvider.Ice());
		backend.addExtensionHandler("http://jabber.org/protocol/jingle/transport/raw-udp",
		                new JingleTransportProvider.RawUdp());
		backend.addExtensionHandler("http://jabber.org/protocol/jingle/info/audio",
		                new JingleContentInfoProvider.Audio());
		backend.addExtensionHandler("http://www.jivesoftware.com/protocol/rtpbridge", new Provider());
		backend.addExtensionHandler("google:jingleinfo", new STUN.Provider());
		
		backend.addCapability("jingle","http://www.xmpp.org/extensions/xep-0166.html#ns");

		backend.addListener(IQResultListener.class,new IQResultListener(){
			public void iqResult(InfoQuery iq) {
				for (JingleSession session : manager.getAllSessions()){
					try {
						session.respond(new Jingle(iq));
					} catch (XMPPException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
			}
		});
	}
	
	
	private JingleSessionListener createSessionListener(final JID jid) {
		return new JingleSessionListener() {
			
			@Override
			public void sessionRedirected(String redirection,
					JingleSession jingleSession) {
				// TODO Auto-generated method stub
		
			}
		
			@Override
			public void sessionMediaReceived(JingleSession jingleSession,
					String participant) {
				// TODO Auto-generated method stub
		
			}
		
			@Override
			public void sessionEstablished(PayloadType pt,
					TransportCandidate remoteCandidate,
					TransportCandidate localCandidate, JingleSession jingleSession) {
	            sessions.put(jid, jingleSession);
	            endVoiceMenu.setEnabled(true);
			}
		
			@Override
			public void sessionDeclined(String reason, JingleSession jingleSession) {
				// TODO Auto-generated method stub
		
			}
		
			@Override
			public void sessionClosedOnError(XMPPException e,
					JingleSession jingleSession) {
				sessions.remove(jid);
	            endVoiceMenu.setEnabled(false);
			}
		
			@Override
			public void sessionClosed(String reason, JingleSession jingleSession) {
				sessions.remove(jid);
	            endVoiceMenu.setEnabled(false);
		
			}
		
		};
	}
	
	public void unload() {}
	
	public static void unload(Backend backend)
	{
		for(JingleSession session : plugin.sessions.values()){
			try {
				session.terminate();
			} catch (XMPPException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		backend.removeExtensionHandler("http://jabber.org/protocol/jingle");
		backend.removeExtensionHandler("http://jabber.org/protocol/jingle/description/audio");
		backend.removeExtensionHandler("http://jabber.org/protocol/jingle/transport/ice");
		backend.removeExtensionHandler("http://jabber.org/protocol/jingle/transport/raw-udp");
		backend.removeExtensionHandler("http://jabber.org/protocol/jingle/info/audio");
		backend.removeExtensionHandler("http://www.jivesoftware.com/protocol/rtpbridge");
	//		backend.removeListener(MessageListener.class, plugin);
	//	backend.removeListener(StatusChangeListener.class,plugin.bookmarks);
		
		backend.getMain().removeFromMenu(plugin.endVoiceMenu);
		backend.getMain().removeFromRosterMenu(I18N.gettext(I18N.gettext("jingle.voice")));
		plugin=null;
	}

	
}