/**
 * $RCSfile$
 * $Revision: $
 * $Date: 28/12/2006
 * <p/>
 * Copyright 2003-2006 Jive Software.
 * <p/>
 * All rights reserved. Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * <p/>
 * http://www.apache.org/licenses/LICENSE-2.0
 * <p/>
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nu.fw.jeti.plugins.jingle.jingle.mediaimpl.demo;

import java.awt.event.ActionEvent;
import java.net.InetAddress;

import javax.media.rtp.SessionAddress;
import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextField;

import nu.fw.jeti.backend.LoginInfo;
import nu.fw.jeti.backend.Start;
import nu.fw.jeti.events.IQResultListener;
import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.jabber.elements.InfoQuery;
import nu.fw.jeti.plugins.jingle.XMPPException;
import nu.fw.jeti.plugins.jingle.jingle.IncomingJingleSession;
import nu.fw.jeti.plugins.jingle.jingle.JingleManager;
import nu.fw.jeti.plugins.jingle.jingle.JingleSession;
import nu.fw.jeti.plugins.jingle.jingle.JingleSessionRequest;
import nu.fw.jeti.plugins.jingle.jingle.OutgoingJingleSession;
import nu.fw.jeti.plugins.jingle.jingle.listeners.JingleSessionRequestListener;
import nu.fw.jeti.plugins.jingle.jingle.mediaimpl.jmf.JmfMediaManager;
import nu.fw.jeti.plugins.jingle.jingle.nat.ICETransportManager;
import nu.fw.jeti.plugins.jingle.jingle.nat.JingleTransportManager;
import nu.fw.jeti.plugins.jingle.jingle.nat.RTPBridge.Provider;
import nu.fw.jeti.plugins.jingle.packet.Jingle;
import nu.fw.jeti.plugins.jingle.provider.JingleContentDescriptionProvider;
import nu.fw.jeti.plugins.jingle.provider.JingleContentInfoProvider;
import nu.fw.jeti.plugins.jingle.provider.JingleProvider;
import nu.fw.jeti.plugins.jingle.provider.JingleTransportProvider;
import nu.fw.jeti.ui.LoginStatusWindow;

/**
 * Jingle Demo Application. It register in a XMPP Server and let users place calls using a full JID and auto-receive calls.
 * Parameters: Server User Pass.
 */
public class Demo extends JFrame {

    private JingleTransportManager transportManager = null;
    private Backend xmppConnection = null;

    private String server = null;
    private String user = null;
    private String pass = null;

    private static JingleManager jm = null;
    private IncomingJingleSession incoming = null;
    private OutgoingJingleSession outgoing = null;

    private JTextField jid = new JTextField(30);
    
    public Demo(String server, String user, String pass) {

        this.server = server;
        this.user = user;
        this.pass = pass;

        Start start = new Start(null,null);
        xmppConnection =start.getBackend();
        LoginInfo info = new LoginInfo(server,null,user,pass,"dd",5222,false,0);
        new LoginStatusWindow(info,xmppConnection,1);
        
       // xmppConnection = new XMPPConnection(server);
//        try {
         //   xmppConnection.connect();
           // xmppConnection.login(user, pass);
            initialize();
//        }
//        catch (XMPPException e) {
//            e.printStackTrace();
//        }
    }

    public void initialize() {
        ICETransportManager icetm0 = new ICETransportManager(xmppConnection, "jivesoftware.com", 3478);
        jm = new JingleManager(xmppConnection, icetm0, new JmfMediaManager());
        jm.addCreationListener(icetm0);

        jm.addJingleSessionRequestListener(new JingleSessionRequestListener() {
            public void sessionRequested(JingleSessionRequest request) {

                if (incoming != null)
                    return;

                try {
                    // Accept the call
                    incoming = request.accept();

                    // Start the call
                    incoming.start();
                }
                catch (Exception e) {
                    e.printStackTrace();
                }

            }
        });
        
xmppConnection.addExtensionHandler("http://jabber.org/protocol/jingle",new JingleProvider(jm));
    	
xmppConnection.addExtensionHandler("http://jabber.org/protocol/jingle/description/audio",
                new JingleContentDescriptionProvider.Audio());

        
xmppConnection.addExtensionHandler("http://jabber.org/protocol/jingle/transport/ice",
                new JingleTransportProvider.Ice());
xmppConnection.addExtensionHandler("http://jabber.org/protocol/jingle/transport/raw-udp",
                new JingleTransportProvider.RawUdp());

xmppConnection.addExtensionHandler("http://jabber.org/protocol/jingle/info/audio",
                new JingleContentInfoProvider.Audio());
		
xmppConnection.addExtensionHandler("http://www.jivesoftware.com/protocol/rtpbridge",	 
		  new Provider());
		

			xmppConnection.addListener(IQResultListener.class,new IQResultListener(){
				public void iqResult(InfoQuery iq) {
					for (JingleSession session : jm.getAllSessions()){
						try {
							session.respond(new Jingle(iq));
						} catch (XMPPException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
					}
				}
			});
        createGUI();
    }

    public void createGUI() {

        JPanel jPanel = new JPanel();

        jPanel.add(jid);

        jPanel.add(new JButton(new AbstractAction("Call") {
            public void actionPerformed(ActionEvent e) {
                if (outgoing != null) return;
                try {
                    outgoing = jm.createOutgoingJingleSession(JID.jidFromString(jid.getText()));
                    outgoing.start();
                }
                catch (Exception e1) {
                    e1.printStackTrace();
                }
            }
        }));

        jPanel.add(new JButton(new AbstractAction("Hangup") {
            public void actionPerformed(ActionEvent e) {
                if (outgoing != null)
                    try {
                        outgoing.terminate();
                    }
                    catch (Exception e1) {
                        e1.printStackTrace();
                    }
                    finally {
                        outgoing = null;
                    }
                if (incoming != null)
                    try {
                        incoming.terminate();
                    }
                    catch (Exception e1) {
                        e1.printStackTrace();
                    }
                    finally {
                        incoming = null;
                    }
            }
        }));

        this.add(jPanel);

    }

    
    
    
    
    
    
    public static void main(String args[]) throws Exception{

    	InetAddress host = InetAddress.getLocalHost();
          String hostname = host.getHostName();
          InetAddress[] addrlist = InetAddress.getAllByName(hostname);
          for(InetAddress iaddress : addrlist){
          System.out.println(iaddress);
          }
          
          SessionAddress localAddr = new SessionAddress(InetAddress.getByName(InetAddress.getByName("192.168.1.108").getHostName()),
                  34244);
          System.out.println(localAddr);
          
        Demo demo = null;
//!fill in demo to use the test
      //  if (args.length > 2) {
     //   demo = new Demo("server" , "user", "password");
  //demo = new Demo("192.168.1.108" , "jingle01", "jingle");
    //demo = new Demo("192.168.1.108" , "jingle02", "jingle");
       demo = new Demo("jabber.org" , "jeti", "10this");
      //  demo = new Demo("jabber.jeti.fw.nu" , "test", "test");
           demo.pack();
            demo.setVisible(true);
            demo.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
       // }

    }

}
