/**
 * $RCSfile: BridgedResolver.java,v $
 * $Revision: 1.1 $
 * $Date: 2007/07/15 15:20:42 $
 *
 * Copyright 2003-2005 Jive Software.
 *
 * All rights reserved. Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nu.fw.jeti.plugins.jingle.jingle.nat;

import java.net.*;
import java.util.Enumeration;
import java.util.Random;

import nu.fw.jeti.events.IQResultListener;
import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.elements.InfoQuery;
import nu.fw.jeti.plugins.jingle.jingle.JingleSession;

/**
 * Bridged Resolver use a RTPBridge Service to add a relayed candidate.
 * A very reliable solution for NAT Traversal.
 * <p/>
 * The resolver verify is the XMPP Server that the client is connected offer this service.
 * If the server supports, a candidate is requested from the service.
 * The resolver adds this candidate
 */
public class BridgedResolver extends TransportResolver{

    Backend connection;

    Random random = new Random();

    long sid;

    /**
     * Constructor.
     * A Bridged Resolver need a XMPPConnection to connect to a RTP Bridge.
     */
    public BridgedResolver(Backend connection) {
        super();
        this.connection = connection;
    }

    /**
     * Resolve Bridged Candidate.
     * <p/>
     * The BridgedResolver takes the IP addresse and ports of a jmf proxy service.
     */
    public synchronized void resolve(JingleSession session) {
    	System.out.println("bridged resolver");
        setResolveInit();

        clearCandidates();

        sid = Math.abs(random.nextLong());

        
        InfoQuery query = RTPBridge.getRTPBridge(connection, String.valueOf(sid));

        connection.send(query, new IQResultListener()
        		{
        		
        			public void iqResult(InfoQuery iq)
        			{
	        RTPBridge rtpBridge = (RTPBridge)iq.getIQExtension();
	
	        
        	String localIp = getLocalHost();
	
	        TransportCandidate localCandidate = new TransportCandidate.Fixed(
	                rtpBridge.getIp(), rtpBridge.getPortA());
	        localCandidate.setLocalIp(localIp);
	
	        TransportCandidate remoteCandidate = new TransportCandidate.Fixed(
	                rtpBridge.getIp(), rtpBridge.getPortB());
	        remoteCandidate.setLocalIp(localIp);
	
	        localCandidate.setSymmetric(remoteCandidate);
	        remoteCandidate.setSymmetric(localCandidate);
	
	        localCandidate.setPassword(rtpBridge.getPass());
	        remoteCandidate.setPassword(rtpBridge.getPass());
	
	        localCandidate.setSessionId(rtpBridge.getSid());
	        remoteCandidate.setSessionId(rtpBridge.getSid());
	
	        localCandidate.setConnection(connection);
	        remoteCandidate.setConnection(connection);
	
	        addCandidate(localCandidate);
	
	        setResolveEnd();
        			}
        },10);
    }

    public void initialize()  throws Exception{

        clearCandidates();

        RTPBridge.serviceAvailable(connection,null); 
        //{
            //setInitialized();
            //System.err.println("No RTP Bridge service available");
        //}
        setInitialized();

    }

    public void cancel()  {
        // Nothing to do here
    }

    public static String getLocalHost() {
        Enumeration ifaces = null;

        try {
            ifaces = NetworkInterface.getNetworkInterfaces();
        }
        catch (SocketException e) {
            e.printStackTrace();
        }

        while (ifaces.hasMoreElements()) {

            NetworkInterface iface = (NetworkInterface) ifaces.nextElement();
            Enumeration iaddresses = iface.getInetAddresses();

            while (iaddresses.hasMoreElements()) {
                InetAddress iaddress = (InetAddress) iaddresses.nextElement();
                if (!iaddress.isLoopbackAddress() && !iaddress.isLinkLocalAddress() && !iaddress.isSiteLocalAddress() && !(iaddress instanceof Inet6Address)) {
                    return iaddress.getHostAddress();
                }
            }
        }

        try {
            return InetAddress.getLocalHost().getHostAddress();
        }
        catch (Exception e) {
            e.printStackTrace();
        }

        return "127.0.0.1";

    }

}
