/**
 * $RCSfile: ICEResolver.java,v $
 * $Revision: 1.2 $
 * $Date: 2007/11/24 16:50:43 $
 *
 * Copyright 2003-2005 Jive Software.
 *
 * All rights reserved. Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package nu.fw.jeti.plugins.jingle.jingle.nat;

import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.util.Enumeration;
import java.util.Observable;
import java.util.Observer;
import java.util.Random;

import nu.fw.jeti.events.IQResultListener;
import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.elements.InfoQuery;
import nu.fw.jeti.plugins.jingle.jingle.JingleSession;
import de.javawi.jstun.test.demo.ice.Candidate;
import de.javawi.jstun.test.demo.ice.ICENegociator;
import de.javawi.jstun.util.UtilityException;

/**
 * ICE Resolver for Jingle transport method that results in sending data between two entities using the Interactive Connectivity Establishment (ICE) methodology. (XEP-0176)
 * The goal of this resolver is to make possible to establish and manage out-of-band connections between two XMPP entities, even if they are behind Network Address Translators (NATs) or firewalls.
 * To use this resolver you must have a STUN Server and be in a non STUN blocked network. Or use a XMPP server with public IP detection Service.
 *
 * @author Thiago Camargo
 */
public class ICEResolver extends TransportResolver {

    Backend connection;
    Random random = new Random();
    long sid;
    String server = "stun.xten.net";
    int port = 3478;
    ICENegociator iceNegociator = null;

    public ICEResolver(Backend connection, String server, int port) {
        super();
        this.connection = connection;
        this.server = server;
        this.port = port;
        this.setType(Type.ice);
    }

    public void initialize() {
        if (!isResolving() && !isResolved()) {
            System.out.println("Initialized");

            iceNegociator = new ICENegociator((short) 1, server, port);
            // gather candidates
            iceNegociator.gatherCandidateAddresses();
            // priorize candidates
            iceNegociator.prioritizeCandidates();

        }
        this.setInitialized();
    }

    public void cancel()  {

    }

    /**
     * Resolve the IP and obtain a valid transport method.
     */
    public synchronized void resolve(final JingleSession session)  {
    	System.out.println("ice resolver");
        this.setResolveInit();

        for (TransportCandidate candidate : this.getCandidatesList()) {
            if (candidate instanceof ICECandidate) {
                ICECandidate iceCandidate = (ICECandidate) candidate;
                iceCandidate.removeCandidateEcho();
            }
        }

        this.clear();

        for (Candidate candidate : iceNegociator.getSortedCandidates())
            try {
                Candidate.CandidateType type = candidate.getCandidateType();
                ICECandidate.Type iceType = ICECandidate.Type.local;
                if (type.equals(Candidate.CandidateType.ServerReflexive))
                    iceType = ICECandidate.Type.srflx;
                else if (type.equals(Candidate.CandidateType.PeerReflexive))
                    iceType = ICECandidate.Type.prflx;
                else if (type.equals(Candidate.CandidateType.Relayed))
                    iceType = ICECandidate.Type.relay;
                else
                    iceType = ICECandidate.Type.host;

                TransportCandidate transportCandidate = new ICECandidate(candidate.getAddress().getInetAddress().getHostAddress(), 1, candidate.getNetwork(), String.valueOf(Math.abs(random.nextLong())), candidate.getPort(), "1", candidate.getPriority(), iceType);
                transportCandidate.setLocalIp(candidate.getBase().getAddress().getInetAddress().getHostAddress());
                transportCandidate.setPort(getFreePort());
                try {
                    transportCandidate.addCandidateEcho(session);
                }
                catch (SocketException e) {
                    e.printStackTrace();
                }
                
                if(candidate.getAddress().getInetAddress().getHostAddress().equals("172.16.192.1"))continue;
                else  if(candidate.getAddress().getInetAddress().getHostAddress().equals("192.168.0.1"))continue;
                
                this.addCandidate(transportCandidate);

                System.out.println("C: " + candidate.getAddress().getInetAddress() + "|" + candidate.getBase().getAddress().getInetAddress() + " p:" + candidate.getPriority());

            }
            catch (UtilityException e) {
                e.printStackTrace();
            }
            catch (UnknownHostException e) {
                e.printStackTrace();
            }

        // Get a Relay Candidate from XMPP Server

        RTPBridge.serviceAvailable(connection, new Observer()
		{
		
			public void update(Observable o, Object arg)
			{
				sid = Math.abs(random.nextLong());

                InfoQuery query = RTPBridge.getRTPBridge(connection, String.valueOf(sid));

                connection.send(query, new IQResultListener()
        		{
        		
        			public void iqResult(InfoQuery iq)
        			{
        				try {
        				String localIp;
                        int network;

                        if (iceNegociator.getPublicCandidate() != null) {
                            localIp = iceNegociator.getPublicCandidate().getBase().getAddress().getInetAddress().getHostAddress();
                            network = iceNegociator.getPublicCandidate().getNetwork();
                        }
                        else {
                    localIp = BridgedResolver.getLocalHost();
                            network = 0;
                        }

                        		
        				RTPBridge rtpBridge = (RTPBridge) iq.getIQExtension();
        				  TransportCandidate localCandidate = new ICECandidate(
                        rtpBridge.getIp(), 1, network, String.valueOf(Math.abs(random.nextLong())), rtpBridge.getPortA(), "1", 0, ICECandidate.Type.relay);
        	                localCandidate.setLocalIp(localIp);

        	                TransportCandidate remoteCandidate = new ICECandidate(
                        rtpBridge.getIp(), 1, network, String.valueOf(Math.abs(random.nextLong())), rtpBridge.getPortB(), "1", 0, ICECandidate.Type.relay);
        	                remoteCandidate.setLocalIp(localIp);

        	                localCandidate.setSymmetric(remoteCandidate);
        	                remoteCandidate.setSymmetric(localCandidate);

        	                localCandidate.setPassword(rtpBridge.getPass());
        	                remoteCandidate.setPassword(rtpBridge.getPass());

        	                localCandidate.setSessionId(rtpBridge.getSid());
        	                remoteCandidate.setSessionId(rtpBridge.getSid());

                localCandidate.setConnection(connection);
                remoteCandidate.setConnection(connection);

        	                addCandidate(localCandidate);
        			}
                    catch (UtilityException e) {
                        e.printStackTrace();
                    }
                    catch (UnknownHostException e) {
                        e.printStackTrace();
                    }
        		
        			}
        		
        		},10);
                
//              Get Public Candidate From XMPP Server

                if (iceNegociator.getPublicCandidate() == null) {

                	connection.send(RTPBridge.getPublicIP(connection),new IQResultListener()
            		{
            		
            			public void iqResult(InfoQuery iq)
            			{
            				RTPBridge response = (RTPBridge) iq.getIQExtension();
            				if (response.getIp() == null || response.getIp().equals("")) return;

            		        Enumeration ifaces = null;
            		        try {
            		            ifaces = NetworkInterface.getNetworkInterfaces();
            		        }
            		        catch (SocketException e) {
            		            e.printStackTrace();
            		        }
            		        while (ifaces!=null&&ifaces.hasMoreElements()) {

            		            NetworkInterface iface = (NetworkInterface) ifaces.nextElement();
            		            Enumeration iaddresses = iface.getInetAddresses();

            		            while (iaddresses.hasMoreElements()) {
            		                InetAddress iaddress = (InetAddress) iaddresses.nextElement();
            		                if (!iaddress.isLoopbackAddress())
            		                    if (iaddress.getHostAddress().indexOf(response.getIp()) >= 0)
            		                        return;

            		            }
            		        }
            		        String publicIp = response.getIp();

                            if (publicIp != null && !publicIp.equals("")) {

                                //Enumeration ifaces = null;

                                try {
                                    ifaces = NetworkInterface.getNetworkInterfaces();
                                }
                                catch (SocketException e) {
                                    e.printStackTrace();
                                }

                                // If detect this address in local machine, don't use it.

                                boolean found = false;

                                while (ifaces.hasMoreElements() && !false) {

                                    NetworkInterface iface = (NetworkInterface) ifaces.nextElement();
                                    Enumeration iaddresses = iface.getInetAddresses();

                                    while (iaddresses.hasMoreElements()) {
                                        InetAddress iaddress = (InetAddress) iaddresses.nextElement();
                                        if (iaddress.getHostAddress().indexOf(publicIp) > -1) {
                                            found = true;
                                            break;
                                        }
                                    }
                                }

                                if (!found) {
                                    try {
                                        TransportCandidate publicCandidate = new ICECandidate(
                                    publicIp, 1, 0, String.valueOf(Math.abs(random.nextLong())), getFreePort(), "1", 0, ICECandidate.Type.srflx);
                                        publicCandidate.setLocalIp(InetAddress.getLocalHost().getHostAddress());

                                        try {
                                            publicCandidate.addCandidateEcho(session);
                                        }
                                        catch (SocketException e) {
                                            e.printStackTrace();
                                        }

                                        addCandidate(publicCandidate);
                                    }
                                    catch (UnknownHostException e) {
                                        e.printStackTrace();
                                    }
                                }
                            }
            			}
            		
            		}, 10);
                	
                    
                }
		
			}
		
		});
		
		
            	                
              

            

            

        

        this.setResolveEnd();
    }

}
