/**
 * $RCSfile: STUN.java,v $
 * $Revision: 1.2 $
 * $Date: 15/11/2006
 *
 * Copyright 2003-2006 Jive Software.
 *
 * All rights reserved. Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nu.fw.jeti.plugins.jingle.jingle.nat;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.xml.sax.Attributes;

import nu.fw.jeti.events.DiscoveryListener;
import nu.fw.jeti.events.IQResultListener;
import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.jabber.elements.*;
import nu.fw.jeti.jabber.elements.Extension;
import nu.fw.jeti.jabber.elements.IQExtension;
import nu.fw.jeti.jabber.elements.InfoQuery;
import nu.fw.jeti.jabber.handlers.ExtensionHandler;
import nu.fw.jeti.plugins.jingle.XMPPException;
import nu.fw.jeti.plugins.jingle.packet.JingleTransport.JingleTransportCandidate;
import nu.fw.jeti.util.Log;

/**
 * STUN IQ Packet used to request and retrieve a STUN server and port to make p2p connections easier. STUN is usually used by Jingle Media Transmission between two parties that are behind NAT.
 * <p/>
 * High Level Usage Example:
 * <p/>
 * STUN stun = STUN.getSTUNServer(xmppConnection);
 *
 * @author Thiago Camargo
 */
public class STUN extends Extension implements IQExtension {

    private List<StunServerAddress> servers = new ArrayList<StunServerAddress>();

    private String publicIp = null;

    /**
     * Element name of the packet extension.
     */
    public static final String NAME = "stun";

    /**
     * Element name of the packet extension.
     */
    public static final String ELEMENT_NAME = "query";

//    static {
//        ProviderManager.getInstance().addIQProvider(NAME, NAMESPACE, new STUN.Provider());
//    }

    /**
     * Creates a STUN IQ
     */
    public STUN() {
    }

    /**
     * Get a list of STUN Servers recommended by the Server
     *
     * @return
     */
    public List<StunServerAddress> getServers() {
        return servers;
    }

    /**
     * Get Public Ip returned from the XMPP server
     *
     * @return
     */
    public String getPublicIp() {
        return publicIp;
    }

    /**
     * Set Public Ip returned from the XMPP server
     *
     * @param publicIp
     */
    private void setPublicIp(String publicIp) {
        this.publicIp = publicIp;
    }

    /**
     * Get the Child Element XML of the Packet
     *
     * @return
     */
    public void appendToXML(StringBuffer str) {
        str.append("<" + ELEMENT_NAME + " xmlns='google:jingleinfo'/>");
    }

    /**
     * IQProvider for RTP Bridge packets.
     * Parse receive RTPBridge packet to a RTPBridge instance
     *
     * @author Thiago Rocha
     */
    public static class Provider extends ExtensionHandler {
    	private STUN iq;
        
    	public Provider() {
            super();
        }
        
        public void startHandling(Attributes attr){
        	iq = new STUN();
        }

        public void startElement(String name, Attributes attr) {
    	    if (name.equals("server")) {
    	    	  String host = attr.getValue("host");
    	    	  String port = attr.getValue("udp");
                  iq.servers.add(new StunServerAddress(host, port));
            }
    	    else if (name.equals("publicip")) {
                	String host = attr.getValue("ip");
                	if (host != null && !host.equals("")) {
                        iq.setPublicIp(host);
                	}
            }
            else {
            	Log.notParsedXML("Unknown tag \"" + name + "\" in STUN.");
            }
        }
        
        public Extension build() {
        	return iq;
        }

//        public IQ parseIQ(XmlPullParser parser) throws Exception {
//
//            boolean done = false;
//
//            int eventType;
//            String elementName;
//            String namespace;
//
//            if (!parser.getNamespace().equals(NAMESPACE))
//                throw new Exception("Not a STUN packet");
//
//            STUN iq = new STUN();
//
//            // Start processing sub-elements
//            while (!done) {
//                eventType = parser.next();
//                elementName = parser.getName();
//                namespace = parser.getNamespace();
//
//                if (eventType == XmlPullParser.START_TAG) {
//                    if (elementName.equals("server")) {
//                        String host = null;
//                        String port = null;
//                        for (int i = 0; i < parser.getAttributeCount(); i++) {
//                            if (parser.getAttributeName(i).equals("host"))
//                                host = parser.getAttributeValue(i);
//                            else if (parser.getAttributeName(i).equals("udp"))
//                                port = parser.getAttributeValue(i);
//                        }
//                        if (host != null && port != null)
//                            iq.servers.add(new StunServerAddress(host, port));
//                    }
//                    else if (elementName.equals("publicip")) {
//                        String host = null;
//                        for (int i = 0; i < parser.getAttributeCount(); i++) {
//                            if (parser.getAttributeName(i).equals("ip"))
//                                host = parser.getAttributeValue(i);
//                        }
//                        if (host != null && !host.equals(""))
//                            iq.setPublicIp(host);
//                    }
//                }
//                else if (eventType == XmlPullParser.END_TAG) {
//                    if (parser.getName().equals(ELEMENT_NAME)) {
//                        done = true;
//                    }
//                }
//            }
//            return iq;
//        }
    }

    /**
     * Get a new STUN Server Address and port from the server.
     * If a error occurs or the server don't support STUN Service, null is returned.
     *
     * @param xmppConnection
     * @return
     */
    public static void getSTUNServer(Backend xmppConnection,IQResultListener listener) {

        if (!xmppConnection.isLoggedIn()) {
            return;
        }

        STUN stunPacket = new STUN();
       // stunPacket.setTo(NAME + "." + xmppConnection.getServiceName());

//        PacketCollector collector = xmppConnection
//                .createPacketCollector(new PacketIDFilter(stunPacket.getPacketID()));

        xmppConnection.send(new InfoQuery("set",stunPacket), listener,10);

//        STUN response = (STUN) collector
//                .nextResult(SmackConfiguration.getPacketReplyTimeout());

        // Cancel the collector.
       // collector.cancel();

        //return response;
    }
    
    public void execute(InfoQuery iq, Backend backend){
    	
    }
    

    /**
     * Check if the server support STUN Service.
     *
     * @param xmppConnection
     * @return
     */
   
    public static boolean serviceAvailable(Backend xmppConnection) {

        if (!xmppConnection.isLoggedIn()) {
            return false;
        }

        System.out.println("Service listing");

        //TODO fix
        return false;
    }

        /*
        
        
        xmppConnection.getInfo(new JID(xmppConnection.getMyJID().getDomain()), new DiscoveryListener(){
			public void discoveryInfoResult(JID jid, DiscoveryInfo info){}
		
			public void discoveryItemResult(JID jid, DiscoveryItem item){
				boolean available = false;
				if (item.hasItems()){
					for(Iterator i = item.getItems();i.hasNext();){
						DiscoveryItem di = (DiscoveryItem)i.next();
						if (di.getJID().getDomain().startsWith("rtpbridge.")) {
		                    available = true;
		                    break;
		                }
					}
				}
				if(!available) System.err.println("No RTP Bridge service available");
				else if(observer!=null)observer.update(null,null);
			}
		});
        
//        ServiceDiscoveryManager disco = ServiceDiscoveryManager
//                .getInstanceFor(xmppConnection);
//        try {
//            DiscoverItems items = disco.discoverItems(xmppConnection.getServiceName());
//
//            Iterator iter = items.getItems();
//            while (iter.hasNext()) {
//                DiscoverItems.Item item = (DiscoverItems.Item) iter.next();
//                DiscoverInfo info = disco.discoverInfo(item.getEntityID());
//
//                Iterator<DiscoverInfo.Identity> iter2 = info.getIdentities();
//                while (iter2.hasNext()) {
//                    DiscoverInfo.Identity identity = iter2.next();
//                    if (identity.getCategory().equals("proxy") && identity.getType().equals("stun"))
//                        if (info.containsFeature(NAMESPACE))
//                            return true;
//                }
//
//                System.out.println(item.getName()+"-"+info.getType());
//
//            }
//        }
//        catch (XMPPException e) {
//            e.printStackTrace();
//        }
        return false;
    }
*/
    
    /**
     * Provides easy abstract to store STUN Server Addresses and Ports
     */
    public static class StunServerAddress {

        private String server;
        private String port;

        public StunServerAddress(String server, String port) {
            this.server = server;
            this.port = port;
        }

        /**
         * Get the Host Address
         *
         * @return
         */
        public String getServer() {
            return server;
        }

        /**
         * Get the Server Port
         *
         * @return
         */
        public String getPort() {
            return port;
        }
    }
}
