/**
 * $RCSfile: JingleContentDescriptionProvider.java,v $
 * $Revision: 1.1 $
 * $Date: 2007/07/15 15:18:39 $
 *
 * Copyright 2003-2005 Jive Software.
 *
 * All rights reserved. Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nu.fw.jeti.plugins.jingle.provider;

import org.xml.sax.Attributes;

import nu.fw.jeti.jabber.elements.Extension;
import nu.fw.jeti.jabber.handlers.ExtensionHandler;
import nu.fw.jeti.plugins.jingle.jingle.media.PayloadType;
import nu.fw.jeti.plugins.jingle.packet.JingleContentDescription;
import nu.fw.jeti.plugins.jingle.packet.JingleContentDescription.JinglePayloadType;

/**
 * Parser for a Jingle description
 *
 * @author Alvaro Saurin <alvaro.saurin@gmail.com>
 */
public abstract class JingleContentDescriptionProvider extends ExtensionHandler {

	private JingleContentDescription desc;
	
    /**
     * Default constructor
     */
    public JingleContentDescriptionProvider() {
        super();
    }
    
    
    public void startHandling(Attributes attr){
    	desc = getInstance();
    }
    
    public void startElement(String name, Attributes attr) {
    	if (name.equals(JingleContentDescription.JinglePayloadType.NODENAME)) {
    		desc.addJinglePayloadType(parsePayload(attr));
        }
    }
    
    /**
     * Parse a iq/jingle/description/payload-type element.
     *
     * @param parser the input to parse
     * @return a payload type element
     */
    protected JinglePayloadType parsePayload(Attributes attr){
        int ptId = 0;
        String ptName;
        int ptChannels = 0;

        try {
            ptId = Integer.parseInt(attr.getValue("id"));
        } catch (NumberFormatException e) {
        }

        ptName = attr.getValue("name");

        try {
            ptChannels = Integer.parseInt(attr.getValue("channels"));
        } catch (NumberFormatException e) {
        }

        return new JinglePayloadType(new PayloadType(ptId, ptName, ptChannels));
    }

//    /**
//     * Parse a iq/jingle/description element.
//     *
//     * @param parser the input to parse
//     * @return a description element
//     * @throws Exception
//     */
//    public PacketExtension parseExtension(final XmlPullParser parser) throws Exception {
//        boolean done = false;
//        JingleContentDescription desc = getInstance();
//
//        while (!done) {
//            int eventType = parser.next();
//            String name = parser.getName();
//
//            if (eventType == XmlPullParser.START_TAG) {
//                if (name.equals(JingleContentDescription.JinglePayloadType.NODENAME)) {
//                    desc.addJinglePayloadType(parsePayload(parser));
//                } else {
//                    throw new Exception("Unknow element \"" + name + "\" in content.");
//                }
//            } else if (eventType == XmlPullParser.END_TAG) {
//                if (name.equals(JingleContentDescription.NODENAME)) {
//                    done = true;
//                }
//            }
//        }
//        return desc;
//    }

    public Extension build() {
    	return desc;
    }
    
    /**
     * Return a new instance of this class. Subclasses must overwrite this
     * method.
     */
    protected abstract JingleContentDescription getInstance();

    /**
     * Jingle audio
     */
    public static class Audio extends JingleContentDescriptionProvider {

        /**
         * Default constructor
         */
        public Audio() {
            super();
        }

        /**
         * Parse an audio payload type.
         */
        public JinglePayloadType parsePayload(Attributes attr){
            JinglePayloadType pte = super.parsePayload(attr);
            PayloadType.Audio pt = new PayloadType.Audio(pte.getPayloadType());
            int ptClockRate = 0;

            try {
                ptClockRate = Integer.parseInt(attr.getValue("clockrate"));
            } catch (Exception e) {
            }
            pt.setClockRate(ptClockRate);

            return new JinglePayloadType.Audio(pt);
        }

        /**
         * Get a new instance of this object.
         */
        protected JingleContentDescription getInstance() {
            return new JingleContentDescription.Audio();
		}
	}
}
