/**
 * $RCSfile: JingleProvider.java,v $
 * $Revision: 1.1 $
 * $Date: 2007/07/15 15:18:39 $
 *
 * Copyright 2003-2005 Jive Software.
 *
 * All rights reserved. Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package nu.fw.jeti.plugins.jingle.provider;

import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.jabber.elements.Extension;
import nu.fw.jeti.jabber.handlers.ExtensionHandler;
import nu.fw.jeti.plugins.jingle.jingle.JingleManager;
import nu.fw.jeti.plugins.jingle.packet.Jingle;
import nu.fw.jeti.plugins.jingle.packet.JingleContentDescription;
import nu.fw.jeti.plugins.jingle.packet.JingleContentInfo;
import nu.fw.jeti.plugins.jingle.packet.JingleTransport;
import nu.fw.jeti.util.Log;

import org.xml.sax.Attributes;

/**
 * The JingleProvider parses Jingle packets.
 *
 * @author Alvaro Saurin
 */
public class JingleProvider extends ExtensionHandler {

	private JingleManager manager;
	private Jingle jingle;// = new Jingle(manager);
	
    public JingleProvider(JingleManager manager) {
    	this.manager = manager;
    }

    public void startHandling(Attributes attr){
    	jingle = new Jingle(manager);
    	  // Get some attributes for the <jingle> element
    	jingle.setSid(attr.getValue("sid"));
    	jingle.setAction(Jingle.Action.getAction(attr.getValue("action")));
        try	{
        	jingle.setInitiator(JID.checkedJIDFromString(attr.getValue("initiator")));
        	jingle.setResponder(JID.checkedJIDFromString(attr.getValue("responder")));
		} catch (InstantiationException e){
			Log.notParsedXML(e.getMessage());
		}
	}
     
    public void addExtension(Extension extension)
	{//overide for extension embedded in extension
		if(extension instanceof JingleContentDescription ) jingle.addDescription((JingleContentDescription) extension);
		else if(extension instanceof JingleTransport) jingle.addTransport((JingleTransport) extension);
		else if(extension instanceof JingleContentInfo) jingle.setContentInfo((JingleContentInfo) extension);
		else Log.notParsedXML("Extension not supported by Jingle");
	}
    
    public Extension build() {
    	return jingle;
    }
    
    
//    /**
//     * Parse a iq/jingle element.
//     */
//    public IQ parseIQ(final XmlPullParser parser) throws Exception {
//
//        Jingle jingle = new Jingle();
//        String sid = "";
//        Jingle.Action action;
//        String initiator = "";
//        String responder = "";
//        boolean done = false;
//
//        
//        int eventType;
//        String elementName;
//        String namespace;
//
//      
//
//        // Start processing sub-elements
//        while (!done) {
//            eventType = parser.next();
//            elementName = parser.getName();
//            namespace = parser.getNamespace();
//
//            if (eventType == XmlPullParser.START_TAG) {
//
//                // Parse some well know subelements, depending on the namespaces
//                // and element names...
//
//                if (elementName.equals(JingleContentDescription.NODENAME)
//                        && namespace.equals(JingleContentDescription.Audio.NAMESPACE)) {
//                    jingle.addDescription((JingleContentDescription) jdpAudio
//                            .parseExtension(parser));
//                } else if (elementName.equals(JingleTransport.NODENAME)) {
//
//                    // Parse the possible transport namespaces
//                    if (namespace.equals(JingleTransport.RawUdp.NAMESPACE)) {
//                        jingle.addTransport((JingleTransport) jtpRawUdp
//                                .parseExtension(parser));
//                    } else if (namespace.equals(JingleTransport.Ice.NAMESPACE)) {
//                        jingle.addTransport((JingleTransport) jtpIce
//                                .parseExtension(parser));
//                    } else {
//                        throw new XMPPException("Unknown transport namespace \""
//                                + namespace + "\" in Jingle packet.");
//                    }
//                } else if (namespace.equals(JingleContentInfo.Audio.NAMESPACE)) {
//                    jingle.setContentInfo((JingleContentInfo) jmipAudio
//                            .parseExtension(parser));
//                } else if (elementName.equals("content")) {
//                    //TODO Separate Contents (XEP-0166)
//                } else {
//                    throw new XMPPException("Unknown combination of namespace \""
//                            + namespace + "\" and element name \"" + elementName
//                            + "\" in Jingle packet.");
//                }
//
//            } else if (eventType == XmlPullParser.END_TAG) {
//                if (parser.getName().equals(Jingle.getElementName())) {
//                    done = true;
//                }
//            }
//        }
//
//        return jingle;
//    }
}
