/**
 * $RCSfile: JingleTransportProvider.java,v $
 * $Revision: 1.1 $
 * $Date: 2007/07/15 15:18:39 $
 *
 * Copyright 2003-2005 Jive Software.
 *
 * All rights reserved. Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nu.fw.jeti.plugins.jingle.provider;

import org.xml.sax.Attributes;

import nu.fw.jeti.jabber.elements.Extension;
import nu.fw.jeti.jabber.handlers.ExtensionHandler;
import nu.fw.jeti.plugins.jingle.jingle.nat.ICECandidate;
import nu.fw.jeti.plugins.jingle.jingle.nat.TransportCandidate;
import nu.fw.jeti.plugins.jingle.packet.JingleTransport;
import nu.fw.jeti.plugins.jingle.packet.JingleTransport.JingleTransportCandidate;
import nu.fw.jeti.util.Log;

/**
 * Provider for a Jingle transport element
 *
 * @author Alvaro Saurin <alvaro.saurin@gmail.com>
 */
public abstract class JingleTransportProvider extends ExtensionHandler {
    private JingleTransport trans;
	
    /**
     * Creates a new provider. ProviderManager requires that every
     * PacketExtensionProvider has a public, no-argument constructor
     */
    public JingleTransportProvider() {
        super();
    }

    /**
     * Obtain the corresponding TransportNegotiator instance.
     *
     * @return a new TransportNegotiator instance
     */
    protected JingleTransport getInstance() {
        return new JingleTransport();
    }
    
    public void startHandling(Attributes attr){
        trans = getInstance();
    }

    public void startElement(String name, Attributes attr) {
	    if (name.equals(JingleTransportCandidate.NODENAME)) {
            JingleTransportCandidate jtc = parseCandidate(attr);
            if (jtc != null) trans.addCandidate(jtc);
        }
        else {
        	Log.notParsedXML("Unknown tag \"" + name + "\" in transport element.");
        }
    }
    
    public Extension build() {
    	return trans;
    }
    
    protected abstract JingleTransportCandidate parseCandidate(final Attributes attr);
           

    /**
     * RTP-ICE profile
     */
    public static class Ice extends JingleTransportProvider {

        /**
         * Defauls constructor.
         */
        public Ice() {
            super();
        }

        /**
         * Obtain the corresponding TransportNegotiator.Ice instance.
         *
         * @return a new TransportNegotiator.Ice instance
         */
        protected JingleTransport getInstance() {
            return new JingleTransport.Ice();
        }

        /**
         * Parse a iq/jingle/transport/candidate element.
         *
         * @param parser the structure to parse
         * @return a candidate element
         * @throws Exception
         */
        protected JingleTransportCandidate parseCandidate(Attributes attr){
            ICECandidate mt = new ICECandidate();

            String channel = attr.getValue("channel");
            String generation = attr.getValue("generation");
            String ip = attr.getValue("ip");
            String name = attr.getValue("name");
            String network = attr.getValue("network");
            String username = attr.getValue("username");
            String password = attr.getValue("password");
            String port = attr.getValue("port");
            String preference = attr.getValue("preference");
            String proto = attr.getValue("proto");
            String type = attr.getValue("type");

            if (channel != null) {
                mt.setChannel(new TransportCandidate.Channel(channel));
            }

            if (generation != null) {
                try {
                    mt.setGeneration(Integer.parseInt(generation));
                }
                catch (Exception e) {
                }
            }

            if (ip != null) {
                mt.setIp(ip);
            }
            else {
                return null;
            }

            if (name != null) {
                mt.setName(name);
            }

            if (network != null) {
                try {
                    mt.setNetwork(Integer.parseInt(network));
                }
                catch (Exception e) {
                }
            }

            if (username != null) {
                mt.setUsername(username);
            }

            if (password != null) {
                mt.setPassword(password);
            }

            if (port != null) {
                try {
                    mt.setPort(Integer.parseInt(port));
                }
                catch (Exception e) {
                }
            }

            if (preference != null) {
                try {
                    mt.setPreference(Integer.parseInt(preference));
                }
                catch (Exception e) {
                }
            }

            if (proto != null) {
                mt.setProto(new TransportCandidate.Protocol(proto));
            }

            if (type != null) {
                mt.setType(ICECandidate.Type.valueOf(type));
            }

            return new JingleTransport.Ice.Candidate(mt);
        }
    }

    /**
     * Raw UDP profile
     */
    public static class RawUdp extends JingleTransportProvider {

        /**
         * Defauls constructor.
         */
        public RawUdp() {
            super();
        }

        /**
         * Obtain the corresponding TransportNegotiator.RawUdp instance.
         *
         * @return a new TransportNegotiator.RawUdp instance
         */
        protected JingleTransport getInstance() {
            return new JingleTransport.RawUdp();
        }

        /**
         * Parse a iq/jingle/transport/candidate element.
         *
         * @param parser the structure to parse
         * @return a candidate element
         * @throws Exception
         */
        protected JingleTransportCandidate parseCandidate(Attributes attr) {
            TransportCandidate.Fixed mt = new TransportCandidate.Fixed();

            String generation = attr.getValue("generation");
            String ip = attr.getValue("ip");
            String name = attr.getValue("name");
            String port = attr.getValue("port");

            //System.out.println();

            if (generation != null) {
                try {
                    mt.setGeneration(Integer.parseInt(generation));
                }
                catch (Exception e) {
                }
            }

            if (ip != null) {
                mt.setIp(ip);
            }

            if (name != null) {
                mt.setName(name);
            }

            if (port != null) {
                try {
                    mt.setPort(Integer.parseInt(port));
                }
                catch (Exception e) {
                }
            }
            return new JingleTransport.RawUdp.Candidate(mt);
        }
    }
}
