/* 
 *	Jeti, a Java Jabber client, Copyright (C) 2001 E.S. de Boer  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	For questions, comments etc, 
 *	use the website at http://jeti.jabberstudio.org
 *  or mail me at jeti@jabber.org
 *  
 *  Created on 10-jul-2005
 */

package nu.fw.jeti.plugins.metaltheme;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.*;
import javax.swing.plaf.ColorUIResource;
import javax.swing.plaf.metal.MetalLookAndFeel;

import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.plugins.PreferencesPanel;
import nu.fw.jeti.util.Preferences;

/**
 * @author E.S. de Boer
 * created
 */

public class PrefPanel extends PreferencesPanel
{
	private Backend backend;
	private MyTheme theme = new MyTheme();
	private JButton[] buttons = new JButton[8]; 

	public PrefPanel(Backend backend)
	{
		this.backend = backend;
		theme.load();
		try
		{
			jbInit();
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
	}

	void jbInit() throws Exception
	{
		ColorUIResource[] colors = theme.getColors();
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
        c.gridwidth = 1;
        c.anchor = GridBagConstraints.LINE_START;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.insets = new Insets(3, 5, 0, 3);
        
		addColorButton("Primary Color 1",colors,0,c);
		addColorButton("Primary Color 2",colors,1,c);
		c.gridwidth = GridBagConstraints.REMAINDER;
		addColorButton("Primary Color 3",colors,2,c);
		
		c.gridwidth = 1;
		addColorButton("Secondary Color 1",colors,3,c);
		addColorButton("Secondary Color 2",colors,4,c);
		c.gridwidth = GridBagConstraints.REMAINDER;
		addColorButton("Secondary Color 3",colors,5,c);
		
		c.gridwidth = 1;
		addColorButton("Background color",colors,6,c);
		addColorButton("Foreground color",colors,7,c);
		
		update();
	}
	
	private void addColorButton(String name, final ColorUIResource[] resource,final int num,GridBagConstraints c)
	{
		buttons[num] = new JButton(name);
		buttons[num].setBackground(resource[num]);
		buttons[num].addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent arg0)
			{
				Color c =JColorChooser.showDialog(PrefPanel.this,"primary1", null);
				resource[num] = new ColorUIResource(c);
				update();
			}
		});
		add(buttons[num],c);
	}

	private void update()
	{
		MetalLookAndFeel.setCurrentTheme(theme);
		try
		{
			UIManager.setLookAndFeel( new MetalLookAndFeel() );
		} catch (UnsupportedLookAndFeelException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		SwingUtilities.updateComponentTreeUI(backend.getMainWindow());
		if(getTopLevelAncestor()!=null)	SwingUtilities.updateComponentTreeUI(getTopLevelAncestor());
		ColorUIResource[] colors = theme.getColors();
		for(int i=0;i<buttons.length;i++)buttons[i].setBackground(colors[i]);
	}
	
	
	public void savePreferences()
	{
		ColorUIResource[] colors = theme.getColors(); 
		for(int i=0;i<colors.length;i++)
		{
			Preferences.putInteger("metaltheme","color"+i, colors[i].getRGB());
		}
	}
}
/*
 * Overrides for emacs
 * Local variables:
 * tab-width: 4
 * End:
 */
