/* 
 *	Jeti, a Java Jabber client, Copyright (C) 2004 E.S. de Boer  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	For questions, comments etc, 
 *	use the website at http://jeti.jabberstudio.org
 *  or mail me at eric@jeti.tk or Jabber at jeti@jabber.org
 *
 *	Created on 10-okt-2004
 */
 
package nu.fw.jeti.plugins.openpgp;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.util.*;

import javax.swing.*;

import nu.fw.jeti.backend.Start;
import nu.fw.jeti.events.PresenceListener;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.jabber.elements.Extension;
import nu.fw.jeti.jabber.elements.Presence;
import nu.fw.jeti.jabber.elements.XExtension;
import nu.fw.jeti.plugins.OpenPGP;
import nu.fw.jeti.util.Popups;
import cryptix.message.*;
import cryptix.openpgp.PGPDetachedSignatureMessage;
import cryptix.openpgp.PGPKeyBundle;
import cryptix.openpgp.PGPSignedMessage;
import cryptix.openpgp.util.PGPArmoury;
import cryptix.pki.KeyBundle;

/**
 * @author E.S. de Boer
 *
 */
public class Encrypter implements OpenPGP, PresenceListener
{
	private static Encrypter encrypter;
	private char[] passpharse;
	private Map keys= new HashMap();
	private List usesPGP = new ArrayList();
	private KeyBundle secretKey;
	
	public Encrypter()
	{
		encrypter = this;
		
//		**********************************************************************
        // Dynamically register both the Cryptix JCE and Cryptix OpenPGP 
        // providers. 
        //**********************************************************************
        java.security.Security.addProvider(
            new cryptix.jce.provider.CryptixCrypto() );
        java.security.Security.addProvider(
          new cryptix.openpgp.provider.CryptixOpenPGP() );
     //  secretKey = readKey("alice.sec");
       // secretKey = readKey("bob-secret.pgp.asc");
        secretKey = readKey("jeti.sec");
                
       importKeys();
		
		
	}
	
	public static Encrypter getInstance()
	{
		return encrypter;
	}
	
	private void importKeys()
	{
		File dir = new File(Start.path + "keys");
		if(!dir.exists())
		{
			Popups.messagePopup("No jeti keys directory found", "Keys not found");
			return;
		}
		File files[]  = dir.listFiles();
		for(int i = 0;i<files.length;i++)
		{
			String name = files[i].getName();
			name = name.substring(0,name.length()-4);
			JID jid=null;
			try
			{
				jid = JID.checkedJIDFromString(name);
			} catch (InstantiationException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			System.out.println(name);
			keys.put(jid,readKey(files[i].getPath()));
			//keys.put(new JID("es","buro"),readKey("alice.asc"));
			//keys.put(new JID("eric","buro"),readKey("bob-public.pgp.asc"));
		}
	}
	
	
	private KeyBundle readKey(String file)
	{
	   KeyBundle bundle = null;

        try {
            
            FileInputStream in = new FileInputStream(file);

            MessageFactory mf = MessageFactory.getInstance("OpenPGP");
            Collection msgs = mf.generateMessages(in);
            
            in.close();
            
            KeyBundleMessage kbm = (KeyBundleMessage)msgs.iterator().next();
            bundle = (PGPKeyBundle)kbm.getKeyBundle();
            
        
        } catch (IOException ioe) {
            System.err.println("IOException... You did remember to run the "+
                "GenerateAndWriteKey example first, right?");
            ioe.printStackTrace();
      
        } catch (NoSuchAlgorithmException nsae) {
            System.err.println("Cannot find the OpenPGP KeyBundleFactory. "+
                "This usually means that the Cryptix OpenPGP provider is not "+
                "installed correctly.");
            nsae.printStackTrace();
      
        } catch (MessageException me) {
            System.err.println("Reading keybundle failed.");
            me.printStackTrace();
        }

        return bundle;
	}
	
	
		
	public Extension signPresence(String status)
	{
		if(passpharse==null)askPassphrase();
		Extension signed =null;
		if(status!=null)
		{
			signed = new XSigned(sign(status));
		}
		return signed;
	}
	
	private void askPassphrase()
	{
		final JDialog dialog = new JDialog((JFrame)null,"Please give your encryption passpharse",true);
		final JPasswordField password = new JPasswordField(); 
		dialog.getContentPane().setLayout(new BorderLayout());
		dialog.getContentPane().add(password,BorderLayout.CENTER);
		JButton ok = new JButton("OK");
		ok.addActionListener(new ActionListener()
		{
		
			public void actionPerformed(ActionEvent e)
			{
				passpharse = password.getPassword();
				dialog.setVisible(false);
			}
		});
		dialog.getContentPane().add(ok,BorderLayout.SOUTH);
		dialog.pack();
		dialog.setLocationRelativeTo(null);
		dialog.setVisible(true);
	}
	
	public String sign(String plainText)
	{
		if(passpharse==null)return null;
	        //**********************************************************************
	        // Create the message.
	        //**********************************************************************
	        Message msg = null;
	        
	        try {
	        
	            LiteralMessageBuilder lmb = 
	                LiteralMessageBuilder.getInstance("OpenPGP");

	            lmb.init(plainText.getBytes());

	            msg = lmb.build();
	            
	        } catch (NoSuchAlgorithmException nsae) {
	            System.err.println("Cannot find the OpenPGP LiteralMessageBuilder. "+
	                "This usually means that the Cryptix OpenPGP provider is not "+
	                "installed correctly.");
	            nsae.printStackTrace();
	            return null;
	        } catch (MessageException me) {
	            System.err.println("Generating the message failed.");
	            me.printStackTrace();
	            return null;
	        }
	        
	        PGPSignedMessage message=null;
	        //**********************************************************************
	        // Sign the message.
	        //**********************************************************************
	        try {
	        
	            SignedMessageBuilder smb = 
	                SignedMessageBuilder.getInstance("OpenPGP");

	            smb.init(msg);
	            smb.addSigner(secretKey, passpharse);

	            message = (PGPSignedMessage)smb.build();
	        
	        } catch (NoSuchAlgorithmException nsae) {
	            System.err.println("Cannot find the OpenPGP SignedMessageBuilder. "+
	                "This usually means that the Cryptix OpenPGP provider is not "+
	                "installed correctly.");
	            nsae.printStackTrace();
	            return null;
	        } catch (UnrecoverableKeyException uke) {
	            System.err.println("Incorrect passphrase.");
	            uke.printStackTrace();
	            return null;
	        } catch (MessageException me) {
	            System.err.println("Generating the message failed.");
	            me.printStackTrace();
	            return null;
	        }

	        //**********************************************************************
	        // Armour the message.
	        //**********************************************************************
	        try{
	        	return  PGPArmoury.armour(message.getDetachedSignature().getEncoded());
	        } catch (MessageException me) {
	        	System.err.println("Writing the message failed.");
        		me.printStackTrace();
        		return null;
	        }
  
	    }
	
	public void presenceChanged(Presence presence)
	{
		if(presence.hasExtensions())
		{
			for(Iterator i = presence.getExtensions();i.hasNext();)
			{
				Object o = i.next();
				if(o instanceof XSigned)
				{
					verify(presence.getFrom(),((XSigned)o).getSigned(),presence.getStatus());
					break;
				}
			}
		}
		else usesPGP.remove(presence.getFrom());
	}
	
	private void verify(JID jid,String signed,String plain)
	{
		System.out.println("verifying");
		signed = signed.replaceAll("\n","\r\n");
		
		
		//**********************************************************************
        // First get the key. 
        //**********************************************************************
        KeyBundle bundle = (KeyBundle)keys.get(jid);
        if(bundle==null)return;
       
    
        //**********************************************************************
        // Read the signed file
        //**********************************************************************
        Message originalMsg = null;
        
        try {
        	
          
            LiteralMessageBuilder lmb = 
                LiteralMessageBuilder.getInstance("OpenPGP");
            lmb.init(plain.getBytes());
            originalMsg = lmb.build();

        } catch (NoSuchAlgorithmException nsae) {
            System.err.println("Cannot find the OpenPGP LiteralMessageBuilder."+
                " This usually means that the Cryptix OpenPGP provider is not "+
                "installed correctly.");
            nsae.printStackTrace();
            return;
         
        } catch (MessageException me) {
            System.err.println("Reading the message failed.");
            me.printStackTrace();
            return;
        }
        

        
        
        
        //**********************************************************************
        // Read the detached signature
        //**********************************************************************
        
             
        
        PGPDetachedSignatureMessage detachedSig = null;
                
        
        
        try {
        
            MessageFactory mf = MessageFactory.getInstance("OpenPGP");
            
            //System.out.println(signed);
                      
            byte[] bm = PGPArmoury.unarmour(signed);
						
            detachedSig = (PGPDetachedSignatureMessage)
                           mf.generateMessages(new ByteArrayInputStream(bm)).iterator().next();
            
        } catch (NoSuchAlgorithmException nsae) {
            System.err.println("Cannot find the OpenPGP MessageFactory. "+
                "This usually means that the Cryptix OpenPGP provider is not "+
                "installed correctly.");
            nsae.printStackTrace();
            return;
        } catch (MessageException me) {
            System.err.println("Reading the message failed.");
            me.printStackTrace();
            return;
        } catch (IOException ioe) {
            System.err.println("Reading the message failed.");
            ioe.printStackTrace();
            return;
        }
        

        //**********************************************************************
        // Verify the message.
        //**********************************************************************
        try {
        
            if (detachedSig.verify(originalMsg, bundle)) {
                System.out.println("Signature OK");
                
            } else {
                System.out.println("Signature BAD");
            }
        
        } catch (MessageException me) {
            System.err.println("Verifying the signature failed.");
            me.printStackTrace();
            return;
        }
        usesPGP.add(jid);
	}
	
	public boolean canEncrypt(JID jid)
	{
		return keys.containsKey(jid) && usesPGP.contains(jid);
	}
	
	public XExtension encrypt(String text,JID jid)
	{
		
		 KeyBundle bundle = (KeyBundle)keys.get(jid);
	     if(bundle==null)
	     {
	    	 Popups.messagePopup("Encryption not possible, key not found", "Key not found");
	    	 return null;
	     }
		
		//**********************************************************************
        //  Build the literal message.
        //**********************************************************************
        LiteralMessage litmsg = null;
        try {
            
            LiteralMessageBuilder lmb = 
                LiteralMessageBuilder.getInstance("OpenPGP");
            lmb.init(text.getBytes());
            litmsg = (LiteralMessage)lmb.build();
        } catch (NoSuchAlgorithmException nsae) {
            System.err.println("Cannot find the OpenPGP LiteralMessageBuilder."+
                " This usually means that the Cryptix OpenPGP provider is not "+
                "installed correctly.");
            nsae.printStackTrace();
            return null;
            
        } catch (MessageException me) {
            System.err.println("Creating the literal message failed.");
            me.printStackTrace();
            return null;
        }
        

        //**********************************************************************
        // Encrypt the message.
        //**********************************************************************
        Message msg = null;
        try {
            
            EncryptedMessageBuilder emb = 
                EncryptedMessageBuilder.getInstance("OpenPGP");
            emb.init(litmsg);
            emb.addRecipient(bundle);
            emb.setAttribute("compressed", "true");
            msg = emb.build();
        
        } catch (NoSuchAlgorithmException nsae) {
            System.err.println("Cannot find the OpenPGP "+
                "EncryptedMessageBuilder. "+
                "This usually means that the Cryptix OpenPGP provider is not "+
                "installed correctly.");
            nsae.printStackTrace();
            return null;
         
        } catch (MessageException me) {
            System.err.println("Creating the encrypted message failed.");
            me.printStackTrace();
            return null;
        }
        

        //**********************************************************************
        // Armour the message and write it to disk
        //**********************************************************************
        try {
            
            //PGPArmouredMessage armoured;
            //PGPArmoury.armour(msg.getEncoded());
                        
            return new XEncrypted(PGPArmoury.armour(msg.getEncoded()));
        
        } catch (MessageException me) {
            System.err.println("Writing the encrypted message failed.");
            me.printStackTrace();
            return null;
        }
	}
	
	public String decrypt(Extension extension)
	{
		if(passpharse==null)return null;
		if(!(extension instanceof XEncrypted)) return null;
		System.out.println("decrypting....");
		String ciphertext = ((XEncrypted)extension).getCrypted();
		ciphertext = ciphertext.replaceAll("\n","\r\n");
		
        //**********************************************************************
        // First read the key. 
        //**********************************************************************
    
        MessageFactory mf = null;
        
        try {
        	mf = MessageFactory.getInstance("OpenPGP");
        }            
        catch (NoSuchAlgorithmException nsae) {
            System.err.println("Cannot find the OpenPGP MessageFactory. "+
                "This usually means that the Cryptix OpenPGP provider is not "+
                "installed correctly.");
            nsae.printStackTrace();
            return null;
       }

        //**********************************************************************
        //  Read the message.
        //**********************************************************************
        EncryptedMessage em = null;
        
        try {

           
		   

            Collection msgs = mf.generateMessages(new ByteArrayInputStream(PGPArmoury.unarmour(ciphertext)));
            em = (EncryptedMessage)msgs.iterator().next();

            
        
        } catch (IOException ioe) {
            System.err.println("IOException... You did remember to run the "+
                "Encrypt example first, right?");
            ioe.printStackTrace();
            return null;
        } catch (MessageException me) {
            System.err.println("Reading message failed.");
            me.printStackTrace();
            return null;
        }
        

        //**********************************************************************
        // Decrypt the message.
        //**********************************************************************
        try {

            Message msg = em.decrypt(secretKey, passpharse);
            System.out.println(((LiteralMessage)msg).getTextData());
            return ((LiteralMessage)msg).getTextData();
        
        } catch (NotEncryptedToParameterException netpe) {
            System.err.println("Not encrypted to this key.");
            netpe.printStackTrace();
            return null;
        } catch (UnrecoverableKeyException uke) {
            System.err.println("Invalid passphrase.");
            uke.printStackTrace();
            return null;
        } catch (MessageException me) {
            System.err.println("Decrypting message failed.");
            me.printStackTrace();
            return null;
        }
	}
}

/*
 * Overrides for emacs
 * Local variables:
 * tab-width: 4
 * End:
 */
