/* 
 *	Jeti, a Java Jabber client, Copyright (C) 2007 E.S. de Boer  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	For questions, comments etc, 
 *	use the website at http://jeti.jabberstudio.org
 */

// Created on 21-jan-2007

package nu.fw.jeti.plugins.pubsub;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import javax.swing.JMenuItem;
import javax.swing.JOptionPane;

import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.jabber.elements.IQAuth;
import nu.fw.jeti.jabber.elements.InfoQuery;
import nu.fw.jeti.jabber.elements.XDataBuilder;
import nu.fw.jeti.jabber.elements.XDataField;
import nu.fw.jeti.plugins.Plugins;
import nu.fw.jeti.util.I18N;
import nu.fw.jeti.util.Log;
import nu.fw.jeti.util.Preferences;
import nu.fw.jeti.util.Utils;

/**
 * @author E.S. de Boer
 *
 */
public class Plugin implements Plugins
{
	public final static String VERSION = "0.1";
	public final static String DESCRIPTION = "pubsub.publish_and_subscribe";
	public final static String MIN_JETI_VERSION = "0.5.4";
	public final static String NAME = "pubsub";
	public final static String ABOUT = "by E.S. de Boer";
	private JMenuItem menuItem;//save menu to remove it
	private static Plugin plugin;
	private JID pepServer;
	private String pepBaseNode;
	private Backend backend;
	
	public static void init(Backend backend)
	{
		plugin = new Plugin(backend);
	}
	
	public static Plugin getPluginInstance(){
		return plugin;
	}
	
	private void initPep()
	{
//		check if pep hack node exist todo use private datastorage
		String pepServer = Preferences.getString("pubsub", "pepServer",null);
		if(pepServer!=null) {
			this.pepServer = JID.jidFromString(pepServer);
			createBaseNode();
		}
		else createPep();
	}
		
	private void createBaseNode(){
		MessageDigest sha = null;
		try {
		  sha = MessageDigest.getInstance("SHA");
		} catch (NoSuchAlgorithmException ex){
			ex.printStackTrace();
		  return;
		}
		pepBaseNode = Utils.toString(sha.digest(backend.getMyJID().toStringNoResource().getBytes()));
		System.out.println( pepBaseNode);
	}
	
	private void createPep()
	{
		String pepserver = JOptionPane.showInputDialog("Pep Server");
		try {
			pepServer = JID.checkedJIDFromString(pepserver);
		} catch (InstantiationException e) {
			JOptionPane.showMessageDialog(null, e.getMessage());
			return;
		}
			
		XDataBuilder builder = new XDataBuilder();
		builder.type = "submit";
		builder.addField(new XDataField("FORM_TYPE",
				"http://jabber.org/protocol/pubsub#node_config","hidden"));
		builder.addField(new XDataField("pubsub#node_type","collection"));
		 		
		createBaseNode();
			
		try {
			backend.send(new InfoQuery(pepServer,"set",new PubsubCreate(pepBaseNode,builder.build())));
		} catch (InstantiationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		Preferences.putString("pubsub","pepServer",pepserver);
	}

	private void createPepNode(String nodeID)
	{
		XDataBuilder builder = new XDataBuilder();
		builder.type = "submit";
		builder.addField(new XDataField("pubsub#collection","/" +pepBaseNode));
		backend.send(new InfoQuery(pepServer,"set",new PubsubCreate(pepBaseNode +"/" + nodeID)));
	}
	
	public void publish(String node,PubSubExtension extension){
		initPep();
		createPepNode(node);
		backend.send(new InfoQuery(pepServer,"set", new Pubsub("/" + pepBaseNode +"/" + node,extension)));
	}
	

	public Plugin(final Backend backend)
	{
		this.backend = backend;
		//groupchat menu item
		menuItem = new JMenuItem(I18N.gettext("pubsub.Pubsub_(beta)"));
		menuItem.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				initPep();
				//createPepNode("http://jabber.org/protocol/activity");
				//new PubSubBrowse(backend).show();
			}
		});
		backend.getMain().addToMenu(menuItem);
		
		backend.addExtensionHandler("http://jabber.org/protocol/pubsub#event",new PubsubEventHandler());
		//backend.addExtensionHandler("http://jabber.org/protocol/muc#owner",new XMUCOwnerHandler());
		//backend.addExtensionHandler("jabber:x:conference",new XConferenceHandler());
	}
	
	public void unload() {}
	
	public static void unload(Backend backend)
	{
		backend.removeExtensionHandler("http://jabber.org/protocol/pubsub#event");
		//backend.removeExtensionHandler("http://jabber.org/protocol/muc#owner");
		//backend.removeExtensionHandler("jabber:x:conference");
		backend.getMain().removeFromMenu(plugin.menuItem);
	}

}

/*
 * Overrides for emacs
 * Local variables:
 * tab-width: 4
 * End:
 */
