/* 
 *	Jeti, a Java Jabber client, Copyright (C) 2001 E.S. de Boer  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	For questions, comments etc, 
 *	use the website at http://jeti.jabberstudio.org
 *  or mail/IM me at jeti@jabber.org
 *  
 *  Created on 6-mrt-2005
 */

package nu.fw.jeti.plugins.search;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.swing.*;

import nu.fw.jeti.events.DiscoveryListener;
import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.jabber.elements.DiscoveryInfo;
import nu.fw.jeti.jabber.elements.DiscoveryItem;
import nu.fw.jeti.jabber.elements.InfoQuery;
import nu.fw.jeti.util.I18N;
import nu.fw.jeti.util.Utils;

/**
 * @author E.S. de Boer
 *
 */
public class JUDChooser extends JFrame implements DiscoveryListener
{
	private Backend backend;
	private JID server;
	private int numberOfServices;
	private List services = new LinkedList();
	
	public JUDChooser(Backend backend)
	{
		this.backend = backend;
		server = new JID(backend.getMyJID().getDomain());
		backend.getItems(server,this);
	}
	
	private void showFrame()
	{
		setIconImage(nu.fw.jeti.images.StatusIcons.getImageIcon("jeti").getImage());
		setTitle(I18N.gettext("search.User_Search"));
		this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		

		final JComboBox cmbJuds = new JComboBox(services.toArray());
		cmbJuds.setAlignmentX((float) 0.0);
		cmbJuds.setEditable(true);

		JButton btnCancel = new JButton();
		Action cancelAction = new AbstractAction(I18N.gettext("Cancel"))
		{
			public void actionPerformed(ActionEvent e)
			{
				dispose();
			}
		};
		Utils.addCancelButton(this, btnCancel, cancelAction);
		
		JButton btnOK = new JButton();
		btnOK.setText(I18N.gettext("OK"));
		btnOK.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				Object o =cmbJuds.getSelectedItem();
				if(o==null)return;
				JID server = null;
				if(o instanceof Item )
				{
					server = ((Item)o).server;
				}
				else
				{
					try
					{
						server = JID.checkedJIDFromString((String)o);
					} catch (InstantiationException e1)
					{
						JOptionPane.showMessageDialog(JUDChooser.this, e1.getMessage());
						return;
					}
				}
				backend.send(new InfoQuery(server,"get",new IQSearch()));
				dispose();
			}
		});
		getRootPane().setDefaultButton(btnOK);
        getContentPane().setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.anchor = GridBagConstraints.LINE_START;
        c.fill = GridBagConstraints.BOTH;
        c.insets = new Insets(3, 3, 0, 3);
        c.weightx = 1.0;

        getContentPane().add(new JLabel(I18N.gettext("search.Select_the_user_directory_to_search")), c);
        getContentPane().add(cmbJuds, c);
        JPanel btnPanel =  new JPanel();
        btnPanel.add(btnOK, null);
        btnPanel.add(btnCancel, null);
        getContentPane().add(btnPanel, c);
        pack();
        setLocationRelativeTo(null);
        setVisible(true);
	}
		
	public void discoveryItemResult(JID jid,DiscoveryItem item)
	{
		if (!item.hasItems()) nu.fw.jeti.util.Popups.errorPopup(I18N.gettext("main.error.No_services"), I18N.gettext("main.error.Register_Services"));
		else
		{
			numberOfServices=item.getSize();
			services.clear();
			for(Iterator i = item.getItems();i.hasNext();)
			{
				DiscoveryItem di = (DiscoveryItem)i.next();
				String name = di.getName();
				if(name==null) name= di.getJID().toStringNoResource();
				//numberOfServices++;
				backend.getInfo(di.getJID(),this);
			}
		}
	}
	  
	public void discoveryInfoResult(JID jid, DiscoveryInfo item)
	{
		numberOfServices--;
		//System.out.println(numberOfServices);
		String name = item.getName();
		if (item.getFeatures().contains("jabber:iq:search"))
		{
			if(name!=null) services.add(new Item(item.getName(),jid));
			else services.add(new Item(jid.toStringNoResource(),jid));
		}
		if(numberOfServices==0)
		{//finished with searching
			//System.out.println("finished");
//			if(services.size()==1)
//			{
//				//System.out.println("found 1");
//				JID server = ((Item)services.get(0)).server;
//				backend.send(new InfoQuery(server,"get",new IQSearch()));
//			}
			//else
				showFrame();
		}
	}
	
	private static class Item
	{
		String name;
		JID server;
		
		public Item(String string, JID jid)
		{
			name = string;
			server = jid;
		}
		
		public String toString()
		{
			return name;
		}
	}
	
}
