/* 
 *	Jeti, a Java Jabber client, Copyright (C) 2007 E.S. de Boer  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	For questions, comments etc, 
 *	use the website at http://jeti.jabberstudio.org
 */

//created on Sep 15, 2007
package nu.fw.jeti.plugins.systemtray;

import java.awt.AWTException;
import java.awt.Menu;
import java.awt.MenuItem;
import java.awt.PopupMenu;
import java.awt.SystemTray;
import java.awt.TrayIcon;
import java.awt.event.ActionEvent;
import java.util.Iterator;
import java.util.List;

import javax.swing.ImageIcon;
import javax.swing.JOptionPane;

import nu.fw.jeti.backend.Start;
import nu.fw.jeti.images.StatusIcons;
import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.elements.Presence;
import nu.fw.jeti.ui.RegisterServices;
import nu.fw.jeti.ui.StatusButton;
import nu.fw.jeti.ui.login.LoginWindow;
import nu.fw.jeti.util.I18N;
import nu.fw.jeti.util.Preferences;



public class JavaTray implements Tray {
	private TrayIcon trayIcon;
	private Backend backend;
	private int status;
	
	
	public JavaTray(Backend backend, Plugin plugin) throws AWTException{
		this.backend = backend;
		if(SystemTray.isSupported()){
			trayIcon = new TrayIcon(StatusIcons.getOfflineIcon().getImage(), "Jeti" + Start.OS2,makeMenu());
			trayIcon.addActionListener(plugin);
			//trayIcon.setImageAutoSize(true);
			SystemTray.getSystemTray().add(trayIcon);
		}
	}
	
	// Create the popup menu for each Tray Icon (on right mouse click)
	public PopupMenu makeMenu()
	{
		// Make new popup menu
		PopupMenu menu = new PopupMenu();
		MenuItem item;
		Menu subMenu = new Menu(I18N.gettext("main.main.jetimenu.Account"));
		item = new MenuItem(I18N.gettext("main.main.jetimenu.Login")+"...");
		item.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				LoginWindow.createLoginWindow(backend);
			}
		});
		subMenu.add(item);
		item = new MenuItem(I18N.gettext("main.main.jetimenu.Manage_Services")+"...");
		item.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				//new RegisterServices(backend.getRegisterServices(),backend).show();
				new RegisterServices(backend).setVisible(true);
			}
		});
		subMenu.add(item);


		menu.add(subMenu);
		subMenu = new Menu(I18N.gettext("systemtray.Status"));
		item = new MenuItem(I18N.gettext("main.main.statusmenu.Change_message"));
		item.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				String message = JOptionPane.showInputDialog(backend.getMainWindow(),I18N.gettext("main.main.statusmenu.Status_message"));
				if (message == null) return;
				StatusButton.changeStatus(status,message);
			}
		});
		subMenu.add(item);
	
		subMenu.add(statusMenu(Presence.FREE_FOR_CHAT));
		subMenu.add(statusMenu(Presence.AVAILABLE));
		subMenu.add(statusMenu(Presence.DND));
		subMenu.add(statusMenu(Presence.AWAY));
		subMenu.add(statusMenu(Presence.XA));

		
		item = new MenuItem(I18N.gettext("main.main.presencebutton.Invisible"));
		item.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				StatusButton.changeStatus(Presence.INVISIBLE,null);
			}
		});
		subMenu.add(item);
		menu.add(subMenu);

		// Add exit item
		item = new MenuItem(I18N.gettext("main.main.jetimenu.Exit"));
		item.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				backend.getMain().close();
			}
		});
		menu.add(item);
		return menu;
	}
	
	private Menu statusMenu(final int status)
	{
		Menu subMenu = new Menu (Presence.toLongShow(status));
		List messages = Preferences.getStatusMessages(status);
		for (Iterator it = messages.iterator(); it.hasNext();)
		{
			final String messageText = (String)it.next();
			MenuItem menuItem = new MenuItem(messageText);
			menuItem.addActionListener(new java.awt.event.ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					StatusButton.changeStatus(status, messageText);
				}
			});
			subMenu.add(menuItem);
		}
		return subMenu;
	}
	
	public void setStatus(int status){
		this.status = status;
	}
		
	public void setIcon(ImageIcon icon){
		trayIcon.setImage(icon.getImage());
	}
	
	
	public void unload(){
		if(trayIcon!=null)SystemTray.getSystemTray().remove(trayIcon);
	}
	
}
