/* 
 *	Jeti, a Java Jabber client, Copyright (C) 2007 E.S. de Boer  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	For questions, comments etc, 
 *	use the website at http://jeti.jabberstudio.org
 */

//created on Sep 15, 2007
package nu.fw.jeti.plugins.systemtray;

import java.awt.event.ActionEvent;
import java.io.File;
import java.lang.reflect.Field;
import java.util.Iterator;
import java.util.List;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;

import nu.fw.jeti.backend.Start;
import nu.fw.jeti.images.StatusIcons;
import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.elements.Presence;
import nu.fw.jeti.plugins.PluginsInfo;
import nu.fw.jeti.ui.RegisterServices;
import nu.fw.jeti.ui.StatusButton;
import nu.fw.jeti.ui.login.LoginWindow;
import nu.fw.jeti.util.I18N;
import nu.fw.jeti.util.Preferences;

import org.jdesktop.jdic.tray.SystemTray;
import org.jdesktop.jdic.tray.TrayIcon;

public class JdicTray implements Tray{

	private TrayIcon trayIcon;
	private Backend backend;
	private int status;
	
	public JdicTray(Backend backend,Plugin plugin) {
		this.backend = backend;
		addDir(Start.path);
		PluginsInfo.loadLibrary("jdic");
		if(System.getProperty("os.name").startsWith("Windows"))PluginsInfo.loadLibrary("jdic_win");
		else PluginsInfo.loadLibrary("jdic_stub");
		
		trayIcon = new TrayIcon(StatusIcons.getOfflineIcon(), "Jeti" + Start.OS2,makeMenu());
		trayIcon.setIconAutoSize(true);
		trayIcon.addActionListener(plugin);
		SystemTray.getDefaultSystemTray().addTrayIcon(trayIcon);
		
	}
	
	public static void addDir(String binaryPath) 
	{
		try {
		
			// Add the binary path (including jdic.dll or libjdic.so) to
			// "java.library.path", since we need to use the native methods in
			// class InitUtility.
			String newLibPath = binaryPath + File.pathSeparator
					+ System.getProperty("java.library.path");
			System.out.println(newLibPath);
			System.setProperty("java.library.path", newLibPath);
			Field fieldSysPath = ClassLoader.class
					.getDeclaredField("sys_paths");
			if (fieldSysPath != null) {
				fieldSysPath.setAccessible(true);
				fieldSysPath.set(System.class.getClassLoader(), null);
			}

		} catch (Throwable e) {
			e.printStackTrace();
		}
	}
	
	// Create the popup menu for each Tray Icon (on right mouse click)
	public JPopupMenu makeMenu()
	{
		// Make new popup menu
		JPopupMenu menu = new JPopupMenu();
		JMenuItem item;
		JMenu subMenu = new JMenu(I18N.gettext("main.main.jetimenu.Account"));
		item = new JMenuItem(I18N.gettext("main.main.jetimenu.Login")+"...");
		item.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				LoginWindow.createLoginWindow(backend);
			}
		});
		subMenu.add(item);
		item = new JMenuItem(I18N.gettext("main.main.jetimenu.Manage_Services")+"...");
		item.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				//new RegisterServices(backend.getRegisterServices(),backend).show();
				new RegisterServices(backend).setVisible(true);
			}
		});
		subMenu.add(item);


		menu.add(subMenu);
		subMenu = new JMenu(I18N.gettext("systemtray.Status"));
		item = new JMenuItem(I18N.gettext("main.main.statusmenu.Change_message"));
		item.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				String message = JOptionPane.showInputDialog(backend.getMainWindow(),I18N.gettext("main.main.statusmenu.Status_message"));
				if (message == null) return;
				StatusButton.changeStatus(status,message);
			}
		});
		subMenu.add(item);
	
		subMenu.add(statusMenu(Presence.FREE_FOR_CHAT));
		subMenu.add(statusMenu(Presence.AVAILABLE));
		subMenu.add(statusMenu(Presence.DND));
		subMenu.add(statusMenu(Presence.AWAY));
		subMenu.add(statusMenu(Presence.XA));

		
		item = new JMenuItem(I18N.gettext("main.main.presencebutton.Invisible"));
		item.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				StatusButton.changeStatus(Presence.INVISIBLE,null);
			}
		});
		subMenu.add(item);
		menu.add(subMenu);

		// Add exit item
		item = new JMenuItem(I18N.gettext("main.main.jetimenu.Exit"));
		item.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				backend.getMain().close();
			}
		});
		menu.add(item);
		return menu;
	}
	
	private JMenu statusMenu(final int status)
	{
		JMenu subMenu = new JMenu (Presence.toLongShow(status));
		List messages = Preferences.getStatusMessages(status);
		for (Iterator it = messages.iterator(); it.hasNext();)
		{
			final String messageText = (String) it.next();
			JMenuItem menuItem = new JMenuItem(messageText);
			menuItem.addActionListener(new java.awt.event.ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					StatusButton.changeStatus(status, messageText);
				}
			});
			subMenu.add(menuItem);
		}
		return subMenu;
	}
	
	public void setStatus(int status){
		this.status = status;
	}
	
	public void setIcon(ImageIcon icon){
		trayIcon.setIcon(icon);
	}
	
	
	public void unload(){
		if(trayIcon!=null)org.jdesktop.jdic.tray.SystemTray.getDefaultSystemTray().removeTrayIcon(trayIcon);
	}
	
	
}
