package nu.fw.jeti.plugins.systemtray;
import java.awt.AWTException;
import java.awt.Container;
import java.awt.Frame;
import java.awt.SystemTray;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.lang.reflect.Field;
import java.util.Iterator;
import java.util.List;

import javax.swing.*;

import nu.fw.jeti.backend.Start;
import nu.fw.jeti.events.StatusChangeListener;
import nu.fw.jeti.images.StatusIcons;
import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.elements.Presence;
import nu.fw.jeti.plugins.Plugins;
import nu.fw.jeti.plugins.PluginsInfo;
import nu.fw.jeti.ui.Jeti;
import nu.fw.jeti.ui.RegisterServices;
import nu.fw.jeti.ui.StatusButton;
import nu.fw.jeti.ui.login.LoginWindow;
import nu.fw.jeti.util.I18N;
import nu.fw.jeti.util.Preferences;
import nu.fw.jeti.util.Utils;


import org.jdesktop.jdic.tray.TrayIcon;

public class Plugin implements ActionListener, StatusChangeListener,Plugins
{
	private Jeti main;
	private Tray trayIcon;
	private Backend backend;
	public final static String VERSION = "2.0";
	public final static String DESCRIPTION = "systemtray.shows_system_tray_icon";
	public final static String MIN_JETI_VERSION = "0.5";
	public final static String NAME = "systemtray";
	public final static String ABOUT = "by E.S. de boer, uses TrayIcon from JDIC";
	private static Plugin plugin;
	
	public static void init(Backend backend) throws AWTException
	{
		new Plugin(backend);
	}
		
	public Plugin(Backend backend) throws AWTException
	{
		if(Utils.isAtleastJava(6)){
			trayIcon = new JavaTray(backend,this);
		}
		else {
			trayIcon = new JdicTray(backend,this);
			plugin =  new Plugin(backend);
		}
		
		this.backend = backend;
		backend.addListener(StatusChangeListener.class,this);
	}

	public void unload() {}
	
	public static void unload(Backend backend)
	{
		backend.removeListener(StatusChangeListener.class,plugin);
		if(plugin!=null)plugin.trayIcon.unload();
		plugin = null;
	}

	public void actionPerformed(ActionEvent e)
	{
		Container w = backend.getMainWindow();
		w.setVisible(true);
		JFrame frame = backend.getMainFrame();
		if(frame!=null)
		{
			if(frame.getExtendedState()==Frame.ICONIFIED)frame.setExtendedState(Frame.NORMAL);
			frame.toFront();
		}
	}

	/*-----------------StatusChangeEvents---------------------------*/
	public void connectionChanged(boolean online)
	{
		try{
		if(!online)trayIcon.setIcon(StatusIcons.getOfflineIcon());
		}catch(Exception e){e.printStackTrace();}
	}

	public void exit()
	{
		unload(backend);
	}

	public void ownPresenceChanged(int status, String message)
	{
		try{
			trayIcon.setStatus(status);
		    trayIcon.setIcon(StatusIcons.getStatusIcon(status));
		}catch(Exception e){e.printStackTrace();}
	}
}
/*
 * Overrides for emacs
 * Local variables:
 * tab-width: 4
 * End:
 */
