/* 
 *	Jeti, a Java Jabber client, Copyright (C) 2003 E.S. de Boer  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	For questions, comments etc, 
 *	use the website at http://jeti.jabberstudio.org
 *  or mail/IM me at jeti@jabber.org
 *
 *	Created on 28-nov-2004
 */
package nu.fw.jeti.plugins.vcard;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.util.*;
import java.util.List;

import javax.swing.*;

import nu.fw.jeti.images.StatusIcons;
import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.elements.InfoQuery;
import nu.fw.jeti.plugins.Avatars;
import nu.fw.jeti.plugins.PluginsInfo;
import nu.fw.jeti.util.*;

/**
 * @author E.S. de Boer
 *
 */
public class VCardEdit extends JFrame
{
	private Map personal;
	private Map business;
	private LinkedList personalFields;
	private LinkedList businessFields;
	private List homeTels;
	private List workTels;
	private Backend backend;
	private static int HOME=1;
	private static int WORK=2;
	private File photo=null;
	private File logo=null;
		
	public VCardEdit(Map personal, Map business, List homeTels, List workTels
					,Backend backend) 
	{
		this.personal = personal;
		this.business = business;
		this.homeTels = homeTels;
		this.workTels = workTels;
		this.backend = backend;
		businessFields = new LinkedList();
		personalFields = new LinkedList();
		init();
	}
	
	private void init()
	{
		setTitle(I18N.gettext("vcard.Edit_Details"));
		setIconImage(StatusIcons.getImageIcon("jeti").getImage());

		JTabbedPane tabs = new JTabbedPane();
		tabs.addTab(I18N.gettext("vcard.Personal"),createPersonalPanel());
		tabs.addTab(I18N.gettext("vcard.Business"),createBusinessPanel());
		getContentPane().setLayout(new BorderLayout());
	    getContentPane().add(tabs,BorderLayout.CENTER);
	    JPanel panel = new JPanel();
	    JButton button = new JButton();
	    I18N.setTextAndMnemonic("vcard.Submit",button);
	    button.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent arg0)
			{
				submitVCard();
			}
		});
	    panel.add(button);
	    button = new JButton();
	    Action cancelAction = new AbstractAction(I18N.gettext("Cancel"))
		{
			public void actionPerformed(ActionEvent e)
			{
				dispose();
			}
		};
		Utils.addCancelButton(this, button, cancelAction);
		
		panel.add(button);
		getContentPane().add(panel,BorderLayout.SOUTH);
		
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
	    pack();
	    int screenX = (int)Toolkit.getDefaultToolkit().getScreenSize().getWidth();
		int screenY = (int)Toolkit.getDefaultToolkit().getScreenSize().getHeight();
	    if(getHeight()>screenY)setSize(screenX,getWidth());
	    if(getWidth()>screenX)setSize(getHeight(),screenY);
	    setLocationRelativeTo(null);
	    setVisible(true);
	}
	
	private JScrollPane createPersonalPanel()
	{
		JPanel pnlPersonal = new JPanel(new GridBagLayout());
	    GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(3, 5, 0, 3);
		// photo
		String[] data = (String[])personal.get("PHOTO");
		final JLabel lblPhoto =new JLabel();
		if (data!=null)
		{
			try{
				byte[] bytes = Base64.decode(data[1]);
				lblPhoto.setIcon(new ImageIcon(bytes));
			}catch (Exception e)
			{
				e.printStackTrace();
			}
		}
		c.gridwidth = GridBagConstraints.REMAINDER;
		pnlPersonal.add(lblPhoto,c);
		JButton button = new JButton();
		I18N.setTextAndMnemonic("vcard.change_photo",button);
		button.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent arg0)
			{
				photo = getImage();
				if(photo!=null){
					try{
						lblPhoto.setIcon(new ImageIcon(photo.toURL()));
					} catch (MalformedURLException e){
						e.printStackTrace();
					}
					pack();
		        }
			}
		});
		c.gridwidth = GridBagConstraints.REMAINDER;
		pnlPersonal.add(button,c);
		/* telephone
		button = new JButton();
		I18N.setTextAndMnemonic("vcard.add_Telephone_Number",button);
		button.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent arg0)
			{
				
			}
		});
		c.gridwidth = GridBagConstraints.REMAINDER;
		pnlPersonal.add(button,c);
*/		

		addLine("FN", HOME, pnlPersonal, c);
		addLine("GIVEN", HOME, pnlPersonal, c);
		addLine("MIDDLE", HOME, pnlPersonal, c);
		addLine("FAMILY", HOME, pnlPersonal, c);
		addLine("NICKNAME", HOME, pnlPersonal, c);
		// bday
		// TODO format, be aware of errors, add age
		addLine("BDAY", HOME, pnlPersonal, c);
		addAdress(HOME,pnlPersonal, c);

		// tel
		if(homeTels!=null)addTel(HOME,pnlPersonal, c);
		addLine("JABBERID", HOME, pnlPersonal, c);
		addLine("EMAIL", HOME, pnlPersonal, c);
		addLine("URL", HOME, pnlPersonal, c);
		//TODO add linebreaks if too long
		addLine("DESC", HOME, pnlPersonal, c);

		c.gridwidth = 2;
	    c.weighty = 1.0;
	    c.weightx = 1.0;
	    pnlPersonal.add(Box.createVerticalGlue(), c);
	    return new JScrollPane(pnlPersonal);
	}
	
	private JScrollPane createBusinessPanel()
	{
		JPanel panel = new JPanel(new GridBagLayout());
	    GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(3, 5, 0, 3);
		// logo
		String[] data = (String[])business.get("LOGO");
		final JLabel lblPhoto =new JLabel();
		if (data!=null)
		{
			try{
				byte[] bytes = Base64.decode(data[1]);
				lblPhoto.setIcon(new ImageIcon(bytes));
			}catch (Exception e)
			{
				e.printStackTrace();
			}
		}
		c.gridwidth = GridBagConstraints.REMAINDER;
		panel.add(lblPhoto,c);
		JButton button = new JButton();
		I18N.setTextAndMnemonic("vcard.change_photo",button);
		button.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent arg0)
			{
				logo = getImage();
				if(logo!=null){
					try{
						lblPhoto.setIcon(new ImageIcon(logo.toURL()));
					} catch (MalformedURLException e){
						e.printStackTrace();
					}
					pack();
		        }
			}
		});
		c.gridwidth = GridBagConstraints.REMAINDER;
		panel.add(button,c);
		
		if(workTels!=null)addTel(WORK,panel, c);
		addLine("TITLE", WORK, panel, c);
		addLine("ROLE", WORK, panel, c);
		addLine("ORGNAME", WORK, panel, c);
		addLine("ORGUNIT", WORK, panel, c);
		addLine("EMAIL", WORK, panel, c);
		addAdress(WORK,panel, c);
		
		c.gridwidth = 2;
	    c.weighty = 1.0;
	    c.weightx = 1.0;
	    panel.add(Box.createVerticalGlue(), c);
	    return new JScrollPane(panel);
	}
	
	private void addAdress(int type,JPanel panel, GridBagConstraints c)
	{
		if(type==WORK) addLine("POBOX", type, panel, c);
		addLine("STREET", type, panel, c);
		addLine("EXTADR", type, panel, c);
		addLine("LOCALITY", type, panel, c);
		addLine("REGION", type, panel, c);
		addLine("PCODE", type, panel, c);
		addLine("CTRY", type, panel, c);
	}
	
	private void addTel(int type,JPanel panel, GridBagConstraints c)
	{
		List tels;
		List fields;
		if(type==HOME)
		{
			tels = homeTels;
			fields = personalFields;
		}
		else
		{
			tels = workTels;
			fields = businessFields;
		}
		for(Iterator i = tels.iterator();i.hasNext();)
		{
			String[] t = (String[])i.next();
			if(t[0]!=null)addLine(t[1],t[0], panel, c,fields);
		}
	}
	
	private void addLine(String name, String value, JPanel pnl, GridBagConstraints c, List fields)
	{
		c.gridwidth = 1;
		JTextField text = new JTextField(value);
		fields.add(new Object[]{name,text});
		name = I18N.gettext("vcard."+name);
		pnl.add(new JLabel(name),c);
		c.gridwidth = GridBagConstraints.REMAINDER;
		pnl.add(text,c);
	}
		    
	private void addLine(String name,int type,JPanel pnl,GridBagConstraints c)
	{
		Map values;
		List fields;
		if(type==HOME)
		{
			values = personal;
			fields = personalFields;
		}
		else
		{
			values=business;
			fields = businessFields;
		}
		String value = (String)values.get(name);
		addLine(name, value, pnl, c, fields);
	}

	
	
	
	private File getImage()
	{
		JFileChooser fileChooser = new JFileChooser();
		fileChooser.setFileFilter(new ImageFileFilter());
		int s = fileChooser.showOpenDialog(VCardEdit.this);
		if(s == JFileChooser.APPROVE_OPTION){
			if(fileChooser.getSelectedFile().length()>51200){
				Popups.errorPopup(
						I18N.gettext("vcard.Image_may_not_be_larger_then_50kB")
						,I18N.gettext("vcard.Image_to_large"));
			}
			else return fileChooser.getSelectedFile();
		}
		return null;
	}
	
	private void submitVCard()
	{
		Map tempPersonal = new HashMap(20);
		Map tempBusiness = new HashMap(20);
		for(Iterator i=personalFields.iterator();i.hasNext();)
		{
			Object[] temp = (Object[])i.next();
			String value =((JTextField)temp[1]).getText();
			if(!value.equals("")) tempPersonal.put(temp[0],value);
		}
		for(Iterator i=businessFields.iterator();i.hasNext();)
		{
			Object[] temp = (Object[])i.next();
			String value =((JTextField)temp[1]).getText();
			if(!value.equals("")) tempBusiness.put(temp[0],value);
		}
		if(photo!=null)
		{
			String[] p = new String[2];
			p[1] = encode(photo);
			tempPersonal.put("PHOTO",p);
		}
		else tempPersonal.put("PHOTO",personal.get("PHOTO"));
		
		if(logo!=null)
		{
			String[] p = new String[2];
			p[1] = encode(logo);
			tempBusiness.put("LOGO",p);
		}
		else tempBusiness.put("LOGO",business.get("LOGO"));
		
		backend.send(new InfoQuery("set",new VCard(tempPersonal,tempBusiness,null,null)));
		if(PluginsInfo.isPluginLoaded("ichatavatars"))
		{
			Avatars a = (Avatars)PluginsInfo.getPluginInstance("ichatavatars");
			a.sendPresence();
		}
		dispose();
	}
	
	private String encode(File file)
	{
       	InputStream is = null;
       	String encoded=null;
    	try {
    		is = new FileInputStream(file.getAbsolutePath());
            int len = (int)file.length();
            byte[] buf = new byte[len];
            is.read(buf);
			if(PluginsInfo.isPluginLoaded("ichatavatars"))
			{
				Avatars a = (Avatars)PluginsInfo.getPluginInstance("ichatavatars");
				a.updateAvatar(buf);
			}
	        encoded = Base64.encodeBytes(buf,Base64.DONT_BREAK_LINES );
	        is.close();
    	}catch(IOException e)
    	{
    		e.printStackTrace();
    	}
    	return encoded;
	}
}
