package nu.fw.jeti.plugins.windowsutils;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.plugins.NativeUtils;
import nu.fw.jeti.plugins.Plugins;
import nu.fw.jeti.ui.ChatSplitPane;
import nu.fw.jeti.util.I18N;
import nu.fw.jeti.util.Preferences;


/**
 * @author E.S. de Boer
 * @version 1.0
 */

public class Plugin implements Plugins, NativeUtils
{
	public final static String VERSION = "0.2";
	public final static String DESCRIPTION = "windowsutils.Provides_always_on_top,_transparency_and_flashing_windows";
	public final static String MIN_JETI_VERSION = "0.5.3";
	public final static String NAME = "windowsutils";
	public final static String ABOUT = "by E.S. de Boer";
	private static WindowUtil util;
	private static JMenuItem menuAlwaysOnTop;
	private static JMenuItem menuTransparent;
      
	public static void init(final Backend backend)
	{
	   	//does this work??
	   	System.setProperty("sun.java2d.noddraw",String.valueOf(Preferences.getBoolean("windowsutils","ddraw",true)));
		//-Dsun.java2d.noddraw=true
	   	util = new WindowUtil();	
		menuAlwaysOnTop = new JMenuItem(I18N.gettext("windowsutils.Set_Always_On_Top"));
		menuAlwaysOnTop.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent arg0)
			{
				if(menuAlwaysOnTop.getText().equals(I18N.gettext("windowsutils.Set_Always_On_Top")))
				{	
					menuAlwaysOnTop.setText(I18N.gettext("windowsutils.Not_Always_On_Top"));
					util.windowAlwaysOnTop(backend.getMainWindow(), true);
				}
				else
				{	
					menuAlwaysOnTop.setText(I18N.gettext("windowsutils.Set_Always_On_Top"));
					util.windowAlwaysOnTop(backend.getMainWindow(), false);
				}
			}
		});
		backend.getMain().addToMenu(menuAlwaysOnTop);
		if(is2000orGreater())
		{	
			menuTransparent = new JMenuItem(I18N.gettext("windowsutils.Make_transparent"));
			menuTransparent.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent arg0)
				{
					if(menuTransparent.getText().equals(I18N.gettext("windowsutils.Make_transparent")))
					{	
						menuTransparent.setText(I18N.gettext("windowsutils.Make_opaque"));
						util.setWindowAlpha(backend.getMainWindow(),Preferences.getInteger("windowsutils","alpha", 70));
					}
					else
					{	
						menuTransparent.setText(I18N.gettext("windowsutils.Make_transparent"));
						util.setWindowAlpha(backend.getMainWindow(), 100);
					}
				}
			});
			backend.getMain().addToMenu(menuTransparent);
		}
	}
	    
    private static boolean is2000orGreater()
    {
    	String os = System.getProperty("os.name");
    	return (os.equals("Windows 2000") || os.equals("Windows 2000") || os.equals("Windows XP") || os.equals("Windows 2003"));
    }
    
    public boolean supportsAlpha()
    {
    	return is2000orGreater();
    }
    
    public void addChatMenus(JMenu menu ,final ChatSplitPane chatPane)
    {
    	JCheckBoxMenuItem item = new JCheckBoxMenuItem(
				I18N.gettext("windowsutils.Set_Always_On_Top"));
		item.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				if(((JCheckBoxMenuItem)e.getSource()).isSelected())
				{	
					util.windowAlwaysOnTop(chatPane.getTopLevelAncestor(), true);
				}
				else
				{	
					util.windowAlwaysOnTop(chatPane.getTopLevelAncestor(), false);
				}
			}
		});
		menu.add(item);
		if(supportsAlpha())
		{
			item = new JCheckBoxMenuItem(
					I18N.gettext("windowsutils.Make_transparent"));
			item.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					if(((JCheckBoxMenuItem)e.getSource()).isSelected())
					{	
						util.setWindowAlpha(chatPane.getTopLevelAncestor(),Preferences.getInteger("windowsutils","alpha", 70));
					}
					else
					{	
						util.setWindowAlpha(chatPane.getTopLevelAncestor(), 100);
					}
				}
			});
			menu.add(item);
		}
    }
  
	public void flash(Component c, boolean flash)
	{
		util.flash(c,flash);
	}
	
	public void windowAlwaysOnTop(Component c, boolean flag)
	{
		util.windowAlwaysOnTop(c,flag);
	}
	
	public void setWindowAlpha(Component c, int alpha)
	{
		util.setWindowAlpha(c,alpha);	
	}

	public void unload() {}
	        
	public static void unload(Backend backend)
	{
		if(util!=null)
		{
			backend.getMain().removeFromMenu(menuTransparent);
			backend.getMain().removeFromMenu(menuAlwaysOnTop);
		}
		util = null;
	}
}
/*
 * Overrides for emacs
 * Local variables:
 * tab-width: 4
 * End:
 */
