#include <assert.h>
#include "jawt_md.h"
#include "nu_fw_jeti_plugins_windowsutils_WindowUtil.h"
#define WS_EX_LAYERED 0x00080000
#define LWA_COLORKEY 0x00000001
#define LWA_ALPHA 0x00000002

// Preparation for the function we want to import from USER32.DLL
typedef BOOL (WINAPI *lpfnSetLayeredWindowAttributes)(HWND hWnd, COLORREF crKey$lpfnSetLayeredWindowAttributes SetLayeredWindowAttributes;
typedef BOOL (WINAPI *lpfnGetLayeredWindowAttributes)(HWND hWnd, COLORREF* crKe$lpfnGetLayeredWindowAttributes GetLayeredWindowAttributes;

HMODULE hUser32 = NULL;

BOOL WINAPI DllMain(HANDLE hInstDLL, DWORD dwReason, LPVOID lpvReserved)
{
	if (dwReason == DLL_PROCESS_ATTACH) {
		// Here we import the function from USER32.DLL
		hUser32 = GetModuleHandle("USER32.DLL");
		if (hUser32 != NULL)
		{
			SetLayeredWindowAttributes = (lpfnSetLayeredWindowAttributes)GetProcAddress(hUser32, "SetLayeredWindowAttributes");
			GetLayeredWindowAttributes = (lpfnGetLayeredWindowAttributes)GetProcAddress(hUser32, "GetLayeredWindowAttributes");
		}
		else return FALSE;
	}
	return TRUE;
}

JNIEXPORT void JNICALL
Java_nu_fw_jeti_plugins_windowsutils_WindowUtil_flash(
JNIEnv * env, jobject canvas, jobject component, jboolean bool)
{
    JAWT awt;
        JAWT_DrawingSurface* ds;
        JAWT_DrawingSurfaceInfo* dsi;
        JAWT_Win32DrawingSurfaceInfo* dsi_win;
        jboolean result;

        jint lock;

        // Get the AWT
        awt.version = JAWT_VERSION_1_3;
        result = JAWT_GetAWT(env, &awt);
        assert(result != JNI_FALSE);
        // Get the drawing surface
        ds = awt.GetDrawingSurface(env, component);
        if(ds == NULL)
                return;
        // Lock the drawing surface
        lock = ds->Lock(ds);
        assert((lock & JAWT_LOCK_ERROR) == 0);

        // Get the drawing surface info
        dsi = ds->GetDrawingSurfaceInfo(ds);

        // Get the platform-specific drawing info
	     dsi_win =
                (JAWT_Win32DrawingSurfaceInfo*)dsi->platformInfo;

        FlashWindow(dsi_win->hwnd,bool);

        // Free the drawing surface info
        ds->FreeDrawingSurfaceInfo(dsi);
        // Unlock the drawing surface
        ds->Unlock(ds);
        // Free the drawing surface
        awt.FreeDrawingSurface(ds);
}

JNIEXPORT void JNICALL Java_nu_fw_jeti_plugins_windowsutils_WindowUtil_windowAlwaysOnTop(JNIEnv *env, jobject canvas, jobject component, jboolean flag)
{
        JAWT awt;
        JAWT_DrawingSurface* ds;
        JAWT_DrawingSurfaceInfo* dsi;
        JAWT_Win32DrawingSurfaceInfo* dsi_win;
        jboolean result;

        jint lock;

        // Get the AWT
        awt.version = JAWT_VERSION_1_3;
        result = JAWT_GetAWT(env, &awt);
        assert(result != JNI_FALSE);
        // Get the drawing surface
        ds = awt.GetDrawingSurface(env, component);
        if(ds == NULL)
                return;
        // Lock the drawing surface
        lock = ds->Lock(ds);
	  assert((lock & JAWT_LOCK_ERROR) == 0);

        // Get the drawing surface info
        dsi = ds->GetDrawingSurfaceInfo(ds);

        // Get the platform-specific drawing info
        dsi_win =
                (JAWT_Win32DrawingSurfaceInfo*)dsi->platformInfo;


         if (flag)
	         SetWindowPos(dsi_win->hwnd,HWND_TOPMOST,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE);
         else
    	      SetWindowPos(dsi_win->hwnd,HWND_NOTOPMOST,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE);
         
        // Free the drawing surface info
        ds->FreeDrawingSurfaceInfo(dsi);
        // Unlock the drawing surface
        ds->Unlock(ds);
        // Free the drawing surface
        awt.FreeDrawingSurface(ds);
}

JNIEXPORT void JNICALL Java_nu_fw_jeti_plugins_windowsutils_WindowUtil_setWindowAlpha(JNIEnv *env, jobject canvas, jobject component, jint alpha)
{
        JAWT awt;
        JAWT_DrawingSurface* ds;
        JAWT_DrawingSurfaceInfo* dsi;
        JAWT_Win32DrawingSurfaceInfo* dsi_win;
        jboolean result;
        HWND hWnd;

        jint lock;
	
        // Get the AWT
        awt.version = JAWT_VERSION_1_3;
        result = JAWT_GetAWT(env, &awt);
        assert(result != JNI_FALSE);
        // Get the drawing surface
        ds = awt.GetDrawingSurface(env, component);
        if(ds == NULL)
                return;
        // Lock the drawing surface
        lock = ds->Lock(ds);
        assert((lock & JAWT_LOCK_ERROR) == 0);

        // Get the drawing surface info
        dsi = ds->GetDrawingSurfaceInfo(ds);

        // Get the platform-specific drawing info
        dsi_win =
                (JAWT_Win32DrawingSurfaceInfo*)dsi->platformInfo;

         hWnd = dsi_win->hwnd;
        if (hWnd == NULL) return;

	   if ( (alpha >= 100) || (alpha < 0) ) {
                // Remove WS_EX_LAYERED from this window styles
                SetWindowLong(hWnd, GWL_EXSTYLE, GetWindowLong(hWnd, GWL_EXSTYLE) & ~WS_EX_LAYERED);
                // Ask the window and its children to repaint
                RedrawWindow(hWnd, NULL, NULL, RDW_ERASE | RDW_INVALIDATE | RDW_FRAME | RDW_ALLCHILDREN);
        } else {
           // Set WS_EX_LAYERED on this window
                SetWindowLong(hWnd, GWL_EXSTYLE,
                        GetWindowLong(hWnd, GWL_EXSTYLE) | WS_EX_LAYERED);
                // Make this window 70% alpha
                SetLayeredWindowAttributes(hWnd, 0, (255 * alpha) / 100, LWA_ALPHA);
                //SetLayeredWindowAttributes(hWnd, 0, (BYTE) (255 * alpha), LWA_ALPHA);
                 //RedrawWindow(hWnd, NULL, NULL, RDW_ERASE | RDW_INVALIDATE | RDW_FRAME | RDW_ALLCHILDREN);
        }
     
        // Free the drawing surface info
        ds->FreeDrawingSurfaceInfo(dsi);
        // Unlock the drawing surface
        ds->Unlock(ds);
        // Free the drawing surface
        awt.FreeDrawingSurface(ds);
}



	
