package nu.fw.jeti.plugins.xmpp;
import java.util.Hashtable;

import javax.naming.directory.Attributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;



/**
 * Utilty class to perform DNS lookups.
 *
 * @author Matt Tucker
 */
public class DnsSrv {
	
	private static DirContext context;
	
	static {
		try {
			Hashtable<String,String> env = new Hashtable<String,String>();
			env.put("java.naming.factory.initial",
			"com.sun.jndi.dns.DnsContextFactory");
			env.put("com.sun.jndi.dns.timeout.initial", "1000");
			env.put("com.sun.jndi.dns.timeout.retries", "1");
			context = new InitialDirContext(env);
		}
		catch (Exception e) {
			System.out.println(e.getMessage());
		}
		catch (Error e)
		{
			System.out.println(e.getMessage());
		}
	}
	
	/**
	 * Returns the host name and port that the specified XMPP server can be
	 * reached at for server-to-server communication. A DNS lookup for a SRV
	 * record in the form "_xmpp-server._tcp.example.com" is attempted, according
	 * to section 14.4 of RFC 3920. If that lookup fails, a lookup in the older form
	 * of "_jabber._tcp.example.com" is attempted since servers that implement an
	 * older version of the protocol may be listed using that notation. If that
	 * lookup fails as well, it's assumed that the XMPP server lives at the
	 * host resolved by a DNS lookup at the specified domain on the default port
	 * of 5269.<p>
	 *
	 * As an example, a lookup for "example.com" may return	 "im.example.com:5269".
	 *
	 * @param domain the domain.
	 * @return a HostAddress, which encompasses the hostname and port that the XMPP
	 *      server can be reached at for the specified domain.
	 */
	public static String resolveXMPPDomain(String domain) {
		if (context == null) {
			//return new HostAddress(domain, 5222);
			return domain;
		}
		String host = domain;
		int port = 5222;
		try {
			Attributes dnsLookup =
				context.getAttributes("_xmpp-client._tcp." + domain);
			String srvRecord = (String)dnsLookup.get("SRV").get();
			String [] srvRecordEntries = srvRecord.split(" ");
			port = Integer.parseInt(srvRecordEntries[srvRecordEntries.length-2]);
			host = srvRecordEntries[srvRecordEntries.length-1];
		}
		catch (Exception e) {
			System.out.println(e.getMessage());
			return domain;
			}
		// Host entries in DNS should end with a ".".
		if (host.endsWith(".")) {
			host = host.substring(0, host.length()-1);
		}
		System.out.println(host);
		return host;//new HostAddress(host, port);
	}

} 