/* 
 *	Jeti, a Java Jabber client, Copyright (C) 2005 E.S. de Boer  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	For questions, comments etc, 
 *	use the website at http://jeti.jabberstudio.org
 *  or mail/im me at jeti@jabber.org
 *  
 *  created 9-10-2005
 */

package nu.fw.jeti.plugins.xmpp;

import java.util.HashMap;
import java.util.Map;

import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.sasl.RealmCallback;
import javax.swing.JOptionPane;

import nu.fw.jeti.backend.Connect;
import nu.fw.jeti.backend.ConnectionPacketReceiver;
import nu.fw.jeti.backend.LoginInfo;
import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.jabber.elements.IQExtension;
import nu.fw.jeti.jabber.elements.InfoQuery;
import nu.fw.jeti.jabber.elements.Packet;
import nu.fw.jeti.jabber.elements.StreamError;
import nu.fw.jeti.jabber.handlers.PacketHandler;
import nu.fw.jeti.plugins.Plugins;
import nu.fw.jeti.plugins.PluginsInfo;
import nu.fw.jeti.plugins.XMPP;
import nu.fw.jeti.plugins.xmpp.sasl.ChallengeHandler;
import nu.fw.jeti.plugins.xmpp.sasl.ChallengePacket;
import nu.fw.jeti.plugins.xmpp.sasl.SaslClient;
import nu.fw.jeti.plugins.xmpp.sasl.SaslHandler;
import nu.fw.jeti.plugins.xmpp.sasl.SaslMechanismHandler;
import nu.fw.jeti.plugins.xmpp.sasl.SaslMechanisms;
import nu.fw.jeti.plugins.xmpp.sasl.SaslPacket;
import nu.fw.jeti.plugins.xmpp.tls.StartTLSExtensionHandler;
import nu.fw.jeti.plugins.xmpp.tls.StartTls;
import nu.fw.jeti.plugins.xmpp.tls.StartTlsPacket;
import nu.fw.jeti.plugins.xmpp.tls.StartTlsPacketHandler;
import nu.fw.jeti.util.Base64;
import nu.fw.jeti.util.I18N;
import nu.fw.jeti.util.Log;
import nu.fw.jeti.util.Preferences;
import nu.fw.jeti.util.Utils;


/**
 * @author E.S. de Boer
 *
 */
public class Plugin implements Plugins , ConnectionPacketReceiver, XMPP, CallbackHandler
{
	public final static String VERSION = "0.1";
	public final static String DESCRIPTION = "xmpp.XMPP_implementation";
	public final static String MIN_JETI_VERSION = "0.7.5";
	public final static String NAME = "xmpp";
	public final static String ABOUT = "by E.S. de Boer";
	private Connect connect;
	private LoginInfo loginInfo;
	private String connectionId;
	private JID jid;
	private State state = State.TLS;
	private boolean secure=false;
	private SaslClient saslClient;
	
	enum State {TLS,SASL,COMPRESSION,BIND}
			
	public static void init(Backend backend)
	{
	
		backend.addExtensionHandler(StartTls.NAMESPACE,new StartTLSExtensionHandler());
		backend.addExtensionHandler("urn:ietf:params:xml:ns:xmpp-sasl",new SaslMechanismHandler());
		backend.addExtensionHandler("urn:ietf:params:xml:ns:xmpp-bind",new IQBindHandler());
		backend.addExtensionHandler("urn:ietf:params:xml:ns:xmpp-session",new IQSessionHandler());
		
	}
		
	public ConnectionPacketReceiver getConnectionPacketReceiver(LoginInfo loginInfo,Connect connect)
	{
		state = State.TLS;
		secure = false;
		jid=null;
		this.loginInfo = loginInfo;
		this.connect = connect;
		return this;
	}
	
	public String resolveXMPPDomain(String server)
	{
		return DnsSrv.resolveXMPPDomain(server);
	}
	
	public Map<String,PacketHandler> getXMPPHandlers()
	{
		Map<String,PacketHandler> handlers = new HashMap<String,PacketHandler>(); 
		handlers.put("stream:features",new FeaturesHandler());
		handlers.put("proceed",new StartTlsPacketHandler(false));
		handlers.put("failure",new StartTlsPacketHandler(true));
		handlers.put("success",new SaslHandler(true));
		handlers.put("auth",new SaslHandler());
		handlers.put("challenge",new ChallengeHandler());
		if(PluginsInfo.isPluginLoaded("compression"))
		{
			nu.fw.jeti.plugins.compression.Plugin.addHandler(handlers);
		}
		return handlers;
	}
	
	public void connected(String connectionId,String xmppVersion) 
	{
		this.connectionId = connectionId;
		System.out.println("connected");
	}
	
	private void doTls()
	{
		System.out.println("doingtls");
		new Thread() {
			public void run()
			{
				secure=true;
				System.out.println("secure set"  + secure);
				secure = connect.startTls(Plugin.this);
			}
		}.start();
		throw new UnsupportedOperationException("end xmlparser");
	}
	
	//TODO send SASL abort on abort
	private void startSasl(SaslMechanisms mechanisms)
	{
		if(Preferences.getBoolean("xmpp", "anonymousLogin", false))
		{
			if(mechanisms.getMechanisms().contains("ANONYMOUS"))
			{
				System.out.println("sending anon");
				connect.sendWhileConnecting(new SaslPacket("ANONYMOUS"));
			}
			else connect.sendLoginError("No Anonymous login possible");
		}
		else if(Utils.isAtleastJava(5))
		{
			String[] m = mechanisms.getMechanisms().toArray(
					new String[mechanisms.getMechanisms().size()]);
			try {
				 SaslClient sc = new SaslClient(javax.security.sasl.Sasl.createSaslClient(m
							, null, "xmpp", loginInfo.getServer(), null,this));
				 byte[] response = (sc.hasInitialResponse() 
						 ? sc.evaluateChallenge(new byte[0]) : null);
				 String res = null;
				 System.out.println("res made");
				 if(response!=null)res =Base64.encodeBytes(response);
				 saslClient = sc;
				 System.out.println("ready to send");
				 connect.sendWhileConnecting(new SaslPacket(sc.getMechanismName(),res));
				 System.out.println("auth packet send");
			}catch(Exception e)
			{
				e.printStackTrace();
				connect.sendLoginError(e.getMessage());
			}
		}
		else if(mechanisms.getMechanisms().contains("PLAIN"))
		{
			System.out.println("secure read"  + secure);
			if(!secure)
			{
				int option = JOptionPane.showConfirmDialog(null
						,I18N.gettext("main.loginstatus.Sending_password_as_plain_text_over_an_unencrypted_connection,_continue?"),"Plain text",JOptionPane.YES_NO_OPTION);
				if (option == JOptionPane.NO_OPTION)
				{
					connect.sendLoginError("Sending password in plain not allowed");
					return;
				}
			}
			String ret = loginInfo.getUsername() + "@" + loginInfo.getServer() +
	        		'\0' +loginInfo.getUsername()+'\0'+loginInfo.getPassword();
			connect.sendWhileConnecting(new SaslPacket("PLAIN",Base64.encodeBytes(ret.getBytes())));
		}
	}
	
	public void respond(ChallengePacket challenge)
	{
		if(saslClient==null)
		{
			connect.sendLoginError("Sasl challenge failed, no saslclient");
			return;
		}
		try {
			final byte[] response = saslClient.evaluateChallenge(Base64.decode(challenge.getChallenge()));
			 connect.sendWhileConnecting(new Packet()
	         {
				public void appendToXML(StringBuffer xml)
				{
					xml.append("<response xmlns='urn:ietf:params:xml:ns:xmpp-sasl'>");
					if(response!=null)xml.append(Base64.encodeBytes(response));
					xml.append("</response>");
				}
			});
			 
		}catch(Exception e)
		{
			e.printStackTrace();
			connect.sendLoginError(e.getMessage());
		}
	}
	
	public void handle(Callback[] callbacks)
	{
		for(int i=0;i<callbacks.length;i++)
		{
			Callback callback = callbacks[i];
			if (callback instanceof RealmCallback) {
				String def = ((RealmCallback) callback).getDefaultText();
				System.out.println(def);
				((RealmCallback) callback).setText(def);
				System.out.println("realm");
			}
			else if(callback instanceof NameCallback) {
				((NameCallback) callback).setName(loginInfo.getUsername());
				System.out.println("simple name");
				/*
				during authentication the initiating entity SHOULD
			       provide as the simple username its sending domain (IP address or
			       fully qualified domain name as contained in a domain identifier)*/
			}
			else if(callback instanceof PasswordCallback) {
				System.out.println("passw");
				((PasswordCallback) callback).setPassword(loginInfo.getPassword().toCharArray());
			}
			else
			{
				connect.sendLoginError("Sasl negotion failed");
				System.err.println(callback.toString());
			}
		}

	}
	
	private void doBind()
	{
		System.out.println("binding");
		if(Preferences.getBoolean("xmpp", "anonymousLogin", false))
		{
			connect.sendWhileConnecting(new InfoQuery("get",new IQBind()));
		}
		else connect.sendWhileConnecting(new InfoQuery("set",new IQBind(loginInfo.getResource())));
	}
	
	public void outputDeath() {System.out.println("output death");}
	
	public void inputDeath() {System.out.println("input death");}
	
	public void receivePackets(Packet packet)
	{
		if (packet instanceof StreamError) connect.streamError((StreamError)packet);
		else if(connect.isAborted())return;
		else if(packet instanceof Features)
		{
			doFeatures((Features)packet);
		}
		else if(packet instanceof StartTlsPacket)
		{
			StartTlsPacket tlsp = (StartTlsPacket)packet;
			if(tlsp.hasFailed())
			{
				if(state == State.SASL)
				{//TLS failed
					//TODO ask user to try again or try without TLS or use old
					//or try again, then try old ssl on 5223, then ask to connect without tls
					connect.sendLoginError("TLS negotion failed");
				}
				else if (state == State.COMPRESSION) 
				{//compression failed
					connect.sendLoginError("Compression negotion failed");
				}
				else if (state == State.BIND) 
				{//SASL failed
					connect.sendLoginError("Wrong password");
					String error = tlsp.getError();
					if("incorrect-encoding".equals(error))
					{//wrong base64
						connect.sendLoginError("Bug in Jeti, please report");
						Log.error("Sasl incorrect-encoding");
					}
					else if("aborted".equals(error))
					{//user aborted
						connect.sendLoginError("aborted");
					}
					else if("not-authorized".equals(error))
					{
						connect.sendUnauthorized();		
					}
					else if("invalid-authzid".equals(error))
					{//send by google on wrong passw
						connect.sendUnauthorized();		
					}
					else connect.sendLoginError(error);
/*
				   
				   o  <invalid-mechanism/> -- The initiating entity did not provide a
				      mechanism or requested a mechanism that is not supported by the
				      receiving entity; sent in reply to an <auth/> element.

				   o  <mechanism-too-weak/> -- The mechanism requested by the initiating
				      entity is weaker than server policy permits for that initiating
				      entity; sent in reply to a <response/> element or an <auth/>
				      element with initial response data.

				      o  <temporary-auth-failure/> -- The authentication failed because of
				      a temporary error condition within the receiving entity; sent in
				      reply to an <auth/> element or <response/> element.
					*/
					
				}
				else 
				{
					Log.xmlReceivedError("Something other then tls or sasl failed");
					return;
				}
			}
			else doTls();
		}
		else if(packet instanceof SaslPacket)
		{
			connect.startSasl(this);
		}
		else if(packet instanceof ChallengePacket)
		{
			respond((ChallengePacket)packet);
		}
		else if (packet instanceof InfoQuery)
		{
			InfoQuery infoquery = (InfoQuery)packet; 
			IQExtension iq = infoquery.getIQExtension();
			if(iq instanceof IQBind)
			{
				if (infoquery.getType().equals("result"))
				{
					jid = ((IQBind)iq).getJID();
					System.out.println(jid);
					connect.sendWhileConnecting(new InfoQuery(null,"set","sessionStart",new IQSession()));
				}
				else if(infoquery.getType().equals("error"))
				{
					String error = infoquery.getXMPPError().getFirstXMPPError().getError();
					if(error.equals("bad-request "))
					{//wrong resource, try otherone (not allowed by string prep)
						//make one, improve
						connect.sendWhileConnecting(new InfoQuery("get",new IQBind()));
					}
					else if(error.equals("not-allowed"))
					{//resource limit reached
						connect.sendLoginError("Resource limit reached, logoff a resource");
					}
					else if(error.equals("conflict"))
					{//resource in use choose new
//						make one, improve
						connect.sendWhileConnecting(new InfoQuery("get",new IQBind()));
					}
				}
			}
			else if(infoquery.getID().equals("sessionStart"))
			{
				if (infoquery.getType().equals("result"))
				{
					connect.authenticated(jid);
				}
				else if(infoquery.getType().equals("error"))
				{
					String error = infoquery.getXMPPError().getFirstXMPPError().getError();
					if(error.equals("internal-server-error"))
					{//internal server error, try again, retry limit?
						connect.sendLoginError("Server error, try again");
					}
					else if(error.equals("forbidden"))
					{//not allowed to make session, try non sasl
						connect.sendLoginError("Not allowed to create session");
					}
					else if(error.equals("cancel"))
					{//resource conflict, choose different resource
//						make one, improve
						connect.sendWhileConnecting(new InfoQuery("get",new IQBind()));
					}
				}
 			}
		}
		else if(PluginsInfo.isPluginLoaded("compression")){
			nu.fw.jeti.plugins.compression.Plugin.receivePackets(packet,connect,this);
		//	throw new RuntimeException();
		}
	}
	
	//if java 1.4 && sasl && only plain try to login with old
	private void doFeatures(Features f) {
		System.out.println("features " + f + " state" + state);
		if(state == State.TLS && f.hasFeature(StartTls.class))
		{
			state = State.SASL;
			connect.sendWhileConnecting(new StartTlsPacket());
		}
		else if(state == State.TLS || state == State.SASL)
		{
			state = State.BIND;
			if(f.hasFeature(SaslMechanisms.class)) startSasl((SaslMechanisms)f.getFeature(SaslMechanisms.class));
			else connect.connected(connectionId,null);//no SASL login with old jabber auth
		}
		else if(state == State.BIND && PluginsInfo.isPluginLoaded("compression"))
		{
			if(nu.fw.jeti.plugins.compression.Plugin.checkFeature(f, connect)){
				state = State.COMPRESSION;
			}
			else {
				state = State.COMPRESSION;
				doFeatures(f);
			}
		}
		else if(f.hasFeature(IQBind.class))doBind();
		else
		{
			if(state == State.COMPRESSION) 
			{//server doesn't support bind so make own resource
				JID jid = new JID(loginInfo.getUsername().toLowerCase(),loginInfo.getServer() ,loginInfo.getResource());
				connect.authenticated(jid);
			}
			else connect.connected(connectionId, null);//no features so login with old
		}
	}
	
		
	public static void unload(Backend backend)
	{
	
		//backend.removeExtensionHandler("http://jeti.tk/cam");
	}
	
	public void unload()
	{
	
	
	}
}
/*
 * Overrides for emacs
 * Local variables:
 * tab-width: 4
 * End:
 */
