/* 
 *	Jeti, a Java Jabber client, Copyright (C) 2001 E.S. de Boer  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	For questions, comments etc, 
 *	use the website at http://jeti.jabberstudio.org
 *  or mail/IM me at jeti@jabber.org
 */

//created 23-10-2005

package nu.fw.jeti.plugins.xmppuri;

import java.awt.Color;
import java.awt.Toolkit;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.*;
import java.util.List;

import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.text.*;

import nu.fw.jeti.backend.Start;
import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.jabber.JIDStatus;
import nu.fw.jeti.jabber.UnknownJIDStatus;
import nu.fw.jeti.jabber.elements.IQRegister;
import nu.fw.jeti.jabber.elements.InfoQuery;
import nu.fw.jeti.plugins.Plugins;
import nu.fw.jeti.plugins.PluginsInfo;
import nu.fw.jeti.plugins.Translator;
import nu.fw.jeti.plugins.Word;
import nu.fw.jeti.plugins.links.Browser;
import nu.fw.jeti.ui.AddContact;
import nu.fw.jeti.ui.SendMessage;
import nu.fw.jeti.util.I18N;
import nu.fw.jeti.util.Log;


/**
 * @author E.S. de Boer
 * @version 1.0
 */

public class Plugin implements Plugins, Translator
{
	public final static String VERSION = "0.1";
	public final static String DESCRIPTION = "xmppuri.Enable_xmpp_links";
	public final static String MIN_JETI_VERSION = "0.7.2";
	public final static String NAME = "xmppuri";
	public final static String ABOUT = "by E.S. de Boer";
	private static Server server;
	private SimpleAttributeSet linkAttributeSet = new SimpleAttributeSet();
	
	public Plugin()
	{
		StyleConstants.setForeground(linkAttributeSet, Color.green);
		StyleConstants.setUnderline(linkAttributeSet, true);
	}
	
	 public static void init(Backend backend)
	 {
		 server = new Server(backend);
	 }
	
	 public void init(final JTextComponent text)
	 {
		text.addMouseListener(new MouseAdapter()
		{
			public void mousePressed(MouseEvent e)
			{
				if (SwingUtilities.isLeftMouseButton(e))
				{
					int location = text.viewToModel(e.getPoint());
					//System.out.println(location);
					StyledDocument doc = (StyledDocument) text.getDocument();
					AttributeSet set = doc.getCharacterElement(location).getAttributes();
					String xmpp = (String)set.getAttribute("xmpp");
					if(xmpp !=null)
					{
						try
						{
							server.process(xmpp);
						} catch (Exception e1)
						{
							JOptionPane.showMessageDialog(text,"xmpp processing failed " +  e1.getMessage());
						}
					}
				}
			}
		});
    }
	    
	    
    public void translate(List wordList)
	{
		for(int i=0;i<wordList.size();i++) 
	    {
			Word word = (Word)wordList.get(i);
	         String token = word.word;
	         if(token.startsWith("xmpp:"))
	         {
	        	 linkAttributeSet.addAttribute("xmpp", token);
	        	 word.addAttributes(linkAttributeSet);
	         }
	    }
	}
	
	public void unload()
	{
		
	}
	
	public static void unload(Backend backend)
	{
		server.unload();
	}
}
/*
 * Overrides for emacs
 * Local variables:
 * tab-width: 4
 * End:
 */
