/* 
 *	Jeti, a Java Jabber client, Copyright (C) 2001 E.S. de Boer  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	For questions, comments etc, 
 *	use the website at http://jeti.jabberstudio.org
 *  or mail/IM me at jeti@jabber.org
 *  
 *  Created on 25-okt-2005
 */

package nu.fw.jeti.plugins.xmppuri;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.URLDecoder;

import javax.swing.JOptionPane;

import nu.fw.jeti.jabber.Backend;
import nu.fw.jeti.jabber.JID;
import nu.fw.jeti.jabber.JIDStatus;
import nu.fw.jeti.jabber.UnknownJIDStatus;
import nu.fw.jeti.jabber.elements.IQRegister;
import nu.fw.jeti.jabber.elements.InfoQuery;
import nu.fw.jeti.plugins.PluginsInfo;
import nu.fw.jeti.ui.AddContact;
import nu.fw.jeti.ui.SendMessage;
import nu.fw.jeti.util.I18N;
import nu.fw.jeti.util.Log;

public class Server
{
	private Backend backend;
	private volatile boolean running = true;
	private Thread server;
			
//	the exception of stringprep verification, the conversion of syntax-
//	   relevant US-ASCII characters (e.g., "?"), and the conversion of
//	   percent-encoded octets from the "inodeid", "ihost", and "iresid"
//	   rules into their character equivalents (e.g., "%20" into the US-ASCII
//	   space character), an XMPP IRI can be constructed directly by
//	   prepending "xmpp:" to an XMPP address.  Furthermore, an XMPP IRI can
//	   be converted into URI syntax by adhering to the procedure specified
//	   in Section 3.1 of [IRI] and an XMPP URI can be converted into IRI
//	   syntax by adhering to the procedure specified in Section 3.2 of
//	   [IRI], thus ensuring interoperability with applications that are able
//	   to process URIs but unable to process IRIs.
			
	public Server(Backend backend) 
	{//TODO stop thread on unload
		this.backend = backend;
		server = new Thread()
		{
			public void run()
			{
				try {
					
					ServerSocket s = new ServerSocket(45943,10,InetAddress.getLocalHost());
					while(running)
					{
						Socket socket = s.accept();
						String command = new BufferedReader(new InputStreamReader(socket.getInputStream())).readLine();
						socket.close();
						try
						{
							Server.this.backend.getMainWindow().setVisible(true);
							process(command);
						} catch (Exception e)
						{
							Log.notParsedXML("Link processing failed " +  e.getMessage());
						}					
					}
				}catch(IOException e)
				{
					e.printStackTrace();
				}
			}
		};
		server.start();
	}
	
	public void process(String command) throws Exception
	{
//		TODO im and pres protocols
		System.out.println(command);
		if(command.startsWith("xmpp:")) command = command.substring(5,command.length());
		JID authJID=null;
		if(command.startsWith("//"))
		{//auth component
			command = command.substring(2,command.length());
			String authJidText=null;
			int questionIndex = command.indexOf("?");
			int slashIndex = command.indexOf("/");
			if(slashIndex<questionIndex)
			{
				authJidText =command.substring(0,slashIndex);
				command = command.substring(slashIndex+1,command.length());
			}
			else if(questionIndex!=-1)
			{
				authJidText =command.substring(0,questionIndex);
				command = command.substring(questionIndex,command.length());
			}
			else
			{
				authJidText =command;
				command = "";
			}
			authJidText = URLDecoder.decode(authJidText,"UTF-8");
			try
			{
				authJID = JID.checkedJIDFromString(authJidText);
			} catch (InstantiationException e)
			{
				Log.notParsedXML("Wrong JID from link " +  e.getMessage());
			}
		}
		//JID part
		String jidText=null;
		int questionIndex = command.indexOf("?");
		if(questionIndex!=-1)
		{
			jidText = command.substring(0,questionIndex);
			command = command.substring(questionIndex+1,command.length());
		}
		else
		{
			jidText =command;
			command = "";
		}
		jidText = URLDecoder.decode(jidText,"UTF-8");
		JID jid = null;
		try
		{
			jid = JID.checkedJIDFromString(jidText);
		} catch (InstantiationException e)
		{
			Log.notParsedXML("Wrong JID from link " +  e.getMessage());
		}
		String queryText=null;
		if(command.length()!=0)
		{//Query part
			int hashIndex = command.indexOf("#");
			if(hashIndex!=-1)
			{
				queryText = command.substring(0,hashIndex);
				command = command.substring(hashIndex+1,command.length());
			}
			else queryText = command;
		}//add fragment part? (#)
		
		//TODO login with authJID if not yet logged in
		System.out.println(authJID);
		System.out.println(jid);
		System.out.println(queryText);
		
		if(jid!=null)
		{
			JIDStatus jidStatus = backend.getJIDStatus(jid);
			if(jidStatus==null)jidStatus = new UnknownJIDStatus(jid);
			if(queryText==null)
			{
				backend.getMain().chat(jidStatus);
			}
			else 
			{
				if(queryText.startsWith("message"))
				{
					messageAction(queryText, jidStatus);
				}
				else if(queryText.startsWith("roster"))
				{
					rosterAction(jid, queryText);
				}
				else if(queryText.startsWith("join"))
				{
					groupchatAction(jid);
				}
				else if(queryText.startsWith("invite;"))
				{//If the joining user is not yet in the room, the application MUST send two stanzas: the first to join the room and the second to invite the other individual. If the joining user is in the room already, the application shall send only the invitation stanza.
					//TODO make this
					if(true)return;
					if(PluginsInfo.isPluginLoaded("groupchat"))
					{//todo open groupchat + check if already open
						queryText = queryText.substring(7,queryText.length());
						String[] keyvalues = queryText.split(";");
						for(int i=0;i<keyvalues.length;i++)
						{
							String keyvalue = keyvalues[i];
							String value = getValue(keyvalue);
							if(value!=null)
							{
								if(keyvalue.startsWith("jid"))
								{
									try
									{//TODO invite user
										JID invitee = JID.checkedJIDFromString(value);
									} catch (InstantiationException e)
									{
										// TODO Auto-generated catch block
										e.printStackTrace();
									}
								}
							}
						}
					}
					else JOptionPane.showMessageDialog(backend.getMainWindow(),I18N.gettext("xmppuri.Please_enable_the_groupchat_plugin_to_groupchat"));
				}
				else if(queryText.startsWith("file"))
				{
					if(PluginsInfo.isPluginLoaded("filetransfer"))
					{
						new nu.fw.jeti.plugins.filetransfer.SendFileWindow(backend,jid).setVisible(true);
					}
					else JOptionPane.showMessageDialog(backend.getMainWindow(),I18N.gettext("xmppuri.Please_enable_the_filetransfer_plugin_to_send_files"));
				}
				else if(queryText.startsWith("register"))
				{
					backend.send(new InfoQuery(jid,"get",new IQRegister()));
				}
			}
		}
	}

	private void groupchatAction(JID jid)
	{
		if(PluginsInfo.isPluginLoaded("groupchat"))
		{
			if(jid.getResource()!=null)
			{
				nu.fw.jeti.plugins.groupchat.Plugin.startGroupchat(jid,backend);
			}
			String nick = JOptionPane.showInputDialog(
					backend.getMainWindow(),I18N.gettext(
							"groupchat.Please_enter_your_nickname"));
			if(!nick.equals("")) {
				JID j = new JID(jid.getUser(),jid.getUser(), nick);
				nu.fw.jeti.plugins.groupchat.Plugin.startGroupchat(j,backend);
		    }
		}
		else JOptionPane.showMessageDialog(backend.getMainWindow(),I18N.gettext(
			"xmppuri.Please_enable_the_groupchat_plugin_to_groupchat"));
	}

	private void rosterAction(JID jid, String queryText) throws UnsupportedEncodingException
	{
		String name=null;
		String group=null;
		if(queryText.startsWith("roster;"))
		{
			queryText = queryText.substring(7,queryText.length());
			String[] keyvalues = queryText.split(";");
			for(int i=0;i<keyvalues.length;i++)
			{
				String keyvalue = keyvalues[i];
				String value = getValue(keyvalue);
				if(value!=null)
				{
					if(keyvalue.startsWith("name"))name = value;
					else if(keyvalue.startsWith("group"))group=value;
				}
			}
		}
		new AddContact(jid,name,group,backend).setVisible(true);
	}

	private void messageAction(String queryText, JIDStatus jidStatus) throws UnsupportedEncodingException
	{
		String subject=null;
		String body = null;
		String type=null;
		if(queryText.startsWith("message;"))
		{
			queryText = queryText.substring(8,queryText.length());
			String[] keyvalues = queryText.split(";");
			for(int i=0;i<keyvalues.length;i++)
			{
				String keyvalue = keyvalues[i];
				String value = getValue(keyvalue);
				if(value!=null)
				{
					if(keyvalue.startsWith("subject"))subject = value;
					else if(keyvalue.startsWith("body"))body=value;
					else if(keyvalue.startsWith("type"))type=value;
				}
			}
		}
		if("chat".equals(type))backend.getMain().chat(jidStatus);
		else new SendMessage(backend,jidStatus,subject,body).setVisible(true);
	}
	
	private String getValue(String keyvalue) throws UnsupportedEncodingException
	{
		int seperator = keyvalue.indexOf("=");
		if(seperator==-1) return null;
		String value = keyvalue.substring(seperator+1,keyvalue.length());
		return URLDecoder.decode(value,"UTF-8");
	}
	
	public void unload()
	{
		running = false;
		server.interrupt();
	}
}
